Package: mboost
Title: Model-Based Boosting
Version: 2.9-11
Date: 2024-08-19
Authors@R: c(person("Torsten", "Hothorn", role = c("cre", "aut"), 
                email = "Torsten.Hothorn@R-project.org",
                comment = c(ORCID = "0000-0001-8301-0471")),
             person("Peter", "Buehlmann", role = "aut",
                    comment = c(ORCID = "0000-0002-1782-6015")),
             person("Thomas", "Kneib", role = "aut",
                    comment = c(ORCID = "0000-0003-3390-0972")),
             person("Matthias", "Schmid", role = "aut",
                    comment = c(ORCID = "0000-0002-0788-0317")),
             person("Benjamin", "Hofner", role = "aut",
	            comment = c(ORCID = "0000-0003-2810-3186")),
             person("Fabian", "Otto-Sobotka", role = "ctb",
                    comment = c(ORCID = "0000-0002-9874-1311")),
             person("Fabian", "Scheipl", role = "ctb",
                    comment = c(ORCID = "0000-0001-8172-3603")),
             person("Andreas", "Mayr", role = "ctb",
                    comment = c(ORCID = "0000-0001-7106-9732")))
Description: Functional gradient descent algorithm
  (boosting) for optimizing general risk functions utilizing
  component-wise (penalised) least squares estimates or regression
  trees as base-learners for fitting generalized linear, additive
  and interaction models to potentially high-dimensional data.
  Models and algorithms are described in <doi:10.1214/07-STS242>,
  a hands-on tutorial is available from <doi:10.1007/s00180-012-0382-5>.
  The package allows user-specified loss functions and base-learners.
Depends: R (>= 3.2.0), methods, stats, parallel, stabs (>= 0.5-0)
Imports: Matrix, survival (>= 3.2-10), splines, lattice, nnls,
        quadprog, utils, graphics, grDevices, partykit (>= 1.2-1)
Suggests: TH.data, MASS, fields, BayesX, gbm, mlbench, RColorBrewer,
        rpart (>= 4.0-3), randomForest, nnet, testthat (>= 0.10.0),
        kangar00
License: GPL-2
BugReports: https://github.com/boost-R/mboost/issues
URL: https://github.com/boost-R/mboost
NeedsCompilation: yes
Packaged: 2024-08-19 09:29:51 UTC; hothorn
Author: Torsten Hothorn [cre, aut] (<https://orcid.org/0000-0001-8301-0471>),
  Peter Buehlmann [aut] (<https://orcid.org/0000-0002-1782-6015>),
  Thomas Kneib [aut] (<https://orcid.org/0000-0003-3390-0972>),
  Matthias Schmid [aut] (<https://orcid.org/0000-0002-0788-0317>),
  Benjamin Hofner [aut] (<https://orcid.org/0000-0003-2810-3186>),
  Fabian Otto-Sobotka [ctb] (<https://orcid.org/0000-0002-9874-1311>),
  Fabian Scheipl [ctb] (<https://orcid.org/0000-0001-8172-3603>),
  Andreas Mayr [ctb] (<https://orcid.org/0000-0001-7106-9732>)
Maintainer: Torsten Hothorn <Torsten.Hothorn@R-project.org>
Repository: CRAN
Date/Publication: 2024-08-22 14:10:05 UTC
Built: R 4.6.0; x86_64-apple-darwin20; 2026-04-27 15:10:35 UTC; unix
Archs: mboost.so.dSYM

import(methods)
import(stats)
import(stabs)

import(Matrix)
import(parallel)
importFrom(survival, Surv, survfit)
importFrom(splines, bs, splineDesign)
importFrom(lattice, levelplot, barchart)
importFrom(nnls, nnls)
importFrom(quadprog, solve.QP)
importFrom(utils, packageDescription)
importFrom("graphics", "abline", "axis", "box", "lines", "matlines", "matplot", "plot", "points", "polygon")
importFrom("grDevices", "hcl", "rgb")
importFrom("partykit", "ctree_control", "extree_data", "extree_fit", "fitted_node")

export(glmboost,
       gamboost,
       blackboost,
       mboost,
       boost_control, mstop, Family,
       GaussReg, Gaussian, GaussClass, Laplace, Binomial, Poisson, GammaReg, QuantReg,
       ExpectReg, NBinomial, PropOdds, Weibull, Loglog, Lognormal, AUC, mboost_fit,
       Huber, AdaExp, Gehan, Cindex, CoxPH, Hurdle, Multinomial, RCG, FP, IPCweights,
       cvrisk, cv, 
       bbs, bols, bspatial, brandom, btree, bss, bns, brad, bkernel, bmono, bmrf, buser,
       survFit, selected, selected.mboost,
       nuisance, "%+%", "%X%", "%O%", extract, risk, "mstop<-",
       fitted.mboost, plot.glmboost, plot.mboost, lines.mboost, plot.cvrisk,
       predict.mboost, predict.glmboost,
       stabsel.mboost, stabsel_parameters.mboost,
       confint.mboost, confint.glmboost, plot.mboost.ci, lines.mboost.ci, print.glmboost.ci,
       downstream.test,
       varimp, plot.varimp, as.data.frame.varimp,
       mboost_intern)
       ###, basesel, fitsel)
exportClasses("boost_family")
exportMethods("show")

S3method(print, glmboost)
S3method(plot, glmboost)
S3method(glmboost, formula)
S3method(glmboost, matrix)
S3method(glmboost, default)
S3method(print, mboost)
S3method(summary, mboost)
S3method(print, summary.mboost)
S3method(plot, mboost)
S3method(lines, mboost)
S3method(plot, survFit)
S3method(coef, mboost)
S3method(coef, glmboost)
S3method(hatvalues, mboost)
S3method(hatvalues, gamboost)
S3method(hatvalues, glmboost)
S3method(fitted, mboost)
S3method(predict, mboost)
S3method(predict, glmboost)
S3method(AIC, mboost)
S3method(logLik, mboost)
S3method(resid, mboost)
S3method(variable.names, mboost)
S3method(variable.names, glmboost)
S3method(print, mboost)
S3method(print, cvrisk)
S3method(print, gbAIC)
S3method(plot, gbAIC)
S3method(plot, cvrisk)
S3method(mstop, gbAIC)
S3method(mstop, cvrisk)
S3method(mstop, mboost)
S3method(nuisance, mboost)
S3method(survFit, mboost)
S3method("[", mboost)
S3method(dpp, blg)
S3method(model.frame, blg)
S3method(model.frame, mboost)
S3method(fit, bl)
S3method(coef, bm_lin)
S3method(coef, bm_cwlin)
S3method(selected, mboost)
S3method(cvrisk, mboost)
# S3method(selected, glmboost)
S3method(update, mboost)
S3method(extract, mboost)
S3method(extract, glmboost)
S3method(extract, blackboost)
S3method(extract, blg)
S3method(extract, bl_lin)
S3method(extract, bl_tree)
S3method(residuals, mboost)
S3method(risk, mboost)
S3method(confint, mboost)
S3method(confint, glmboost)
S3method(plot, mboost.ci)
S3method(lines, mboost.ci)
S3method(print, glmboost.ci)
S3method(stabsel, mboost)
S3method(stabsel_parameters, mboost)
S3method(varimp, mboost)
S3method(plot, varimp)
S3method(as.data.frame, varimp)

useDynLib(mboost)

library("mboost")
load("india.Rdata")

inds <- paste("w", 1:50, sep="")
quantiles <- c(0.05, 0.1, 0.5)
pred1 <- pred2 <- pred3 <- matrix(0, ncol=4, nrow=length(inds))
colnames(pred1) <- colnames(pred2) <- colnames(pred3) <- c("add","vcm","stump","tree")
add1.selected <- add2.selected <- add3.selected <- list()
vcm1.selected <- vcm2.selected <- vcm3.selected <- list()
add1.stabsel <- add2.stabsel <- add3.stabsel <- matrix(NA, ncol=length(inds), nrow=210000)
add1.its <- add2.its <- add3.its <- rep(0, length(inds))
vcm1.its <- vcm2.its <- vcm3.its <- rep(0, length(inds))
vcm1.stabsel <- vcm2.stabsel <- vcm3.stabsel <- matrix(NA, ncol=length(inds), nrow=210000)

loss <- function(y, f, tau)
{
    sum(tau*(y - f)*((y - f) >= 0) - (1-tau)*(y - f)*((y - f) < 0))
}

# read the predictions

for(i in 1:length(inds))
{
    print(i)
    indiapred <- india[india[,inds[i]]==3,]
    indiapred <- cbind(indiapred, ff = 0)
    names(indiapred)[71] <- "FALSE"
    
    load(paste("additive/model",i,"_0.05.Rdata",sep=""))
    pred <- predict(model, newdata=indiapred)
    pred1[i,1] <- loss(indiapred$stunting, pred, tau=0.05)
    add1.selected[[i]] <- factor(selected(model), levels=1:28, labels=names(model$baselearner))
    add1.stabsel[1:mstop(model),i] <- selected(model) 
    add1.its[i] <- mstop(model)
    
    load(paste("additive/model",i,"_0.1.Rdata",sep=""))
    pred <- predict(model, newdata=indiapred)
    pred2[i,1] <- loss(indiapred$stunting, pred, tau=0.1)
    add2.selected[[i]] <- factor(selected(model), levels=1:28, labels=names(model$baselearner))
    add2.stabsel[1:mstop(model),i] <- selected(model) 
    add2.its[i] <- length(selected(model))
    
    load(paste("additive/model",i,"_0.5.Rdata",sep=""))
    pred <- predict(model, newdata=indiapred)
    pred3[i,1] <- loss(indiapred$stunting, pred, tau=0.5)
    add3.selected[[i]] <- factor(selected(model), levels=1:28, labels=names(model$baselearner))
    add3.stabsel[1:mstop(model),i] <- selected(model) 
    add3.its[i] <- length(selected(model))
    
    
    load(paste("VCM/model",i,"_0.05.Rdata",sep=""))
    pred <- predict(model, newdata=indiapred)
    pred1[i,2] <- loss(indiapred$stunting, pred, tau=0.05)
    vcm1.selected[[i]] <- factor(selected(model), levels=1:54, labels=names(model$baselearner))
    vcm1.stabsel[1:mstop(model),i] <- selected(model) 
    vcm1.its[i] <- length(selected(model))
    
    load(paste("VCM/model",i,"_0.1.Rdata",sep=""))
    pred <- predict(model, newdata=indiapred)
    pred2[i,2] <- loss(indiapred$stunting, pred, tau=0.1)
    vcm2.selected[[i]] <- factor(selected(model), levels=1:54, labels=names(model$baselearner))
    vcm2.stabsel[1:mstop(model),i] <- selected(model) 
    vcm2.its[i] <- length(selected(model))
    
    load(paste("VCM/model",i,"_0.5.Rdata",sep=""))
    pred <- predict(model, newdata=indiapred)
    pred3[i,2] <- loss(indiapred$stunting, pred, tau=0.5)
    vcm3.selected[[i]] <- factor(selected(model), levels=1:54, labels=names(model$baselearner))
    vcm3.stabsel[1:mstop(model),i] <- selected(model) 
    vcm3.its[i] <- length(selected(model))
    
    
    load(paste("stumps/model",i,"_0.05.Rdata",sep=""))
    pred1[i,3] <- loss(indiapred$stunting, pred, tau=0.05)
    
    load(paste("stumps/model",i,"_0.1.Rdata",sep=""))
    pred2[i,3] <- loss(indiapred$stunting, pred, tau=0.1)
    
    load(paste("stumps/model",i,"_0.5.Rdata",sep=""))
    pred3[i,3] <- loss(indiapred$stunting, pred, tau=0.5)
    
    
    load(paste("trees/model",i,"_0.05.Rdata",sep=""))
    pred1[i,4] <- loss(indiapred$stunting, pred, tau=0.05)
    
    load(paste("trees/model",i,"_0.1.Rdata",sep=""))
    pred2[i,4] <- loss(indiapred$stunting, pred, tau=0.1)
    
    load(paste("trees/model",i,"_0.5.Rdata",sep=""))
    pred3[i,4] <- loss(indiapred$stunting, pred, tau=0.5)
}



# plotting of effects in the additive model

preddata <- preddatafm <- list()
plotdata.add1 <- plotdata.add2 <- plotdata.add3 <- list()
plotdata.vcm1 <- plotdata.vcm2 <- plotdata.vcm3 <- list()

varnames <- c("cage", "breastfeeding", "mbmi", "mage", "medu", "edupartner",
              "csex", "ctwin", "cbirthorder", "munemployed", "mreligion",
              "mresidence", "deadchildren", "wealth", "electricity", "radio",
              "television", "refrigerator", "bicycle", "motorcycle", "car")

refval <- list()
for(i in 1:length(varnames))
{
    print(i)
    if(is.numeric(india[,varnames[i]]))
        refval[[i]] <- mean(india[,varnames[i]])
    if(is.factor(india[,varnames[i]]))
        refval[[i]] <- unique(india[,varnames[i]])[which.max(table(india[,varnames[i]]))]
}

for(i in 1:length(varnames))
{
    print(i)
    vals <- unique(india[,varnames[i]])
    preddata[[i]] <- india[1:length(vals),]
    for(j in 1:length(varnames))
    {
        if(j==i)
            preddata[[i]][,varnames[j]] <- vals
        else
            preddata[[i]][,varnames[j]] <- rep(refval[[j]], length(vals))
    }
    preddata[[i]] <- cbind(preddata[[i]], ff = 0)
    names(preddata[[i]])[71] <- "FALSE"
    
    preddatafm[[i]] <- preddata[[i]]
    preddatafm[[i]][,"csex"] <- rep(unique(india[,"csex"])[which.min(table(india[,"csex"]))], length(vals))
    
    
    plotdata.add1[[i]] <- data.frame(vals, matrix(0, nrow=length(vals), ncol=length(inds)))
    names(plotdata.add1[[i]]) <- c(varnames[i],paste("f", 1:length(inds), sep=""))
    
    plotdata.add2[[i]] <- data.frame(vals, matrix(0, nrow=length(vals), ncol=length(inds)))
    names(plotdata.add1[[i]]) <- c(varnames[i],paste("f", 1:length(inds), sep=""))
    
    plotdata.add3[[i]] <- data.frame(vals, matrix(0, nrow=length(vals), ncol=length(inds)))
    names(plotdata.add1[[i]]) <- c(varnames[i],paste("f", 1:length(inds), sep=""))
    
    
    plotdata.vcm1[[i]] <- data.frame(vals, matrix(0, nrow=length(vals), ncol=length(inds)))
    names(plotdata.vcm1[[i]]) <- c(varnames[i],paste("fm", 1:length(inds), sep=""))
    plotdata.vcm1[[length(varnames)+i]] <- data.frame(vals, matrix(0, nrow=length(vals), ncol=length(inds)))
    names(plotdata.vcm1[[length(varnames)+i]]) <- c(varnames[i],paste("ff", 1:length(inds), sep=""))
    
    plotdata.vcm2[[i]] <- data.frame(vals, matrix(0, nrow=length(vals), ncol=length(inds)))
    names(plotdata.vcm2[[i]]) <- c(varnames[i],paste("fm", 1:length(inds), sep=""))
    plotdata.vcm2[[length(varnames)+i]] <- data.frame(vals, matrix(0, nrow=length(vals), ncol=length(inds)))
    names(plotdata.vcm2[[length(varnames)+i]]) <- c(varnames[i],paste("ff", 1:length(inds), sep=""))
    
    plotdata.vcm3[[i]] <- data.frame(vals, matrix(0, nrow=length(vals), ncol=length(inds)))
    names(plotdata.vcm3[[i]]) <- c(varnames[i],paste("fm", 1:length(inds), sep=""))
    plotdata.vcm3[[length(varnames)+i]] <- data.frame(vals, matrix(0, nrow=length(vals), ncol=length(inds)))
    names(plotdata.vcm3[[length(varnames)+i]]) <- c(varnames[i],paste("ff", 1:length(inds), sep=""))
}

for(i in 1:length(inds))
{
    print(i)
    load(paste("additive/model",i,"_0.05.Rdata",sep=""))
    for(j in 1:length(varnames))
    {
        plotdata.add1[[j]][,(i+1)] <- predict(model, newdata=preddata[[j]])
    }
    
    load(paste("additive/model",i,"_0.1.Rdata",sep=""))
    for(j in 1:length(varnames))
    {
        plotdata.add2[[j]][,(i+1)] <- predict(model, newdata=preddata[[j]])
    }
    
    load(paste("additive/model",i,"_0.5.Rdata",sep=""))
    for(j in 1:length(varnames))
    {
        plotdata.add3[[j]][,(i+1)] <- predict(model, newdata=preddata[[j]])
    }
    
    
    load(paste("VCM/model",i,"_0.05.Rdata",sep=""))
    for(j in 1:length(varnames))
    {
        plotdata.vcm1[[j]][,(i+1)] <- predict(model, newdata=preddata[[j]])
        plotdata.vcm1[[length(varnames)+j]][,(i+1)] <- predict(model, newdata=preddatafm[[j]])
    }
    
    load(paste("VCM/model",i,"_0.1.Rdata",sep=""))
    for(j in 1:length(varnames))
    {
        plotdata.vcm2[[j]][,(i+1)] <- predict(model, newdata=preddata[[j]])
        plotdata.vcm2[[length(varnames)+j]][,(i+1)] <- predict(model, newdata=preddatafm[[j]])
    }
    
    load(paste("VCM/model",i,"_0.5.Rdata",sep=""))
    for(j in 1:length(varnames))
    {
        plotdata.vcm3[[j]][,(i+1)] <- predict(model, newdata=preddata[[j]])
        plotdata.vcm3[[length(varnames)+j]][,(i+1)] <- predict(model, newdata=preddatafm[[j]])
    }
}

save("pred1", "pred2", "pred3", "plotdata.add1", "plotdata.add2",
     "plotdata.add3", "plotdata.vcm1", "plotdata.vcm2", "plotdata.vcm3",
     "add1.selected", "add2.selected", "add3.selected", 
     "vcm1.selected", "vcm2.selected", "vcm3.selected", 
     "add1.its", "add2.its", "add3.its", "vcm1.its", "vcm2.its", "vcm3.its", 
     "add1.stabsel", "add2.stabsel", "add3.stabsel", 
     "vcm1.stabsel", "vcm2.stabsel", "vcm3.stabsel", 
     "varnames", file="plotresults.Rdata")

library("mboost")

### bodyfat benchmarks

data("bodyfat", package = "mboost")
bffm <- DEXfat ~ age + waistcirc + hipcirc + elbowbreadth + kneebreadth +
      anthro3a + anthro3b + anthro3c + anthro4 - 1

### center independent variables
indep <- names(bodyfat)[names(bodyfat) != "DEXfat"]
bodyfat[indep] <- lapply(bodyfat[indep], function(x) x - mean(x))

### setup benchmark experiments with B = 100 bootstrap samples
n <- nrow(bodyfat)
set.seed(290875)
bs <- rmultinom(100, n, rep(1, n)/n)

mod1 <- glmboost(bffm, data = bodyfat)

grid <- seq(from = 2, to = 100, by = 2)
boob <- cvrisk(mod1, folds = bs, grid = grid)

mod2 <- glmboost(bffm, data = bodyfat, control = boost_control(nu = 0.2))
boobfull <- cvrisk(mod1, folds = bs, grid = 5000)

bffm <- DEXfat ~ hipcirc + kneebreadth + anthro3a
mod3 <- glmboost(bffm, data = bodyfat, control = boost_control(nu = 0.2)) 
boobrest <- cvrisk(mod3, folds = bs, grid = 5000)

save(boob, boobfull, boobrest, grid, file = "bodyfat_benchmarks.rda")

### curve estimation benchmarks

dgp <- function(n = 100) {
    x <- sort(runif(n) - 0.5)
    ytrue <- 0.8 * x + sin(6 * x)
    data.frame(x = x, y = ytrue + rnorm(n, sd = sqrt(2)), ytrue = ytrue)
}

nsim <- 100
dfree <- seq(from = 2, to = 40, by = 2)
mstops <- seq(from = 5, to = 1005, by = 10)

mseSS <- matrix(0, nrow = nsim, ncol = length(dfree))
mseB <- matrix(0, nrow = nsim, ncol = length(mstops))
se <- GaussReg()@risk

for (i in 1:nsim) {
    print(i)
    learn <- dgp()
    mseSS[i,] <- sapply(dfree, function(d) {
        se(learn$ytrue, predict(smooth.spline(x = learn$x, y = learn$y, df = d), 
                           x = learn$x)$y)/nrow(learn)
        })
    mod <- mboost(y ~ bbs(x, df = 2.5), data = learn, 
                  control = boost_control(mstop = max(mstops)))
    
    fm <- predict(mod, agg = "cumsum")[,mstops]
    mseB[i,] <- apply(fm, 2, function(f) se(learn$ytrue, f)) / nrow(learn)
}

save(dfree, mstops, mseSS, mseB, file = "curve_estimation.rda")

### WPBC glm benchmarks

### Wisconsin prognostic breast cancer data, without missing values
data("wpbc")
wpbc <- wpbc[complete.cases(wpbc),]

indep <- names(wpbc)[!(names(wpbc) %in% c("time", "status"))]
wpbc[indep] <- lapply(wpbc[indep], function(x) x - mean(x))

### setup benchmark experiments with B = 100 bootstrap samples
n <- nrow(wpbc)
set.seed(290875)
bs <- rmultinom(100, n, rep(1, n)/n)

mod <- glmboost(status ~ ., data = wpbc[,-2], family = Binomial())

boob <- c()
grid <- seq(from = 5, to = 500, by = 5)

boob <- cvrisk(mod, folds = bs, grid = grid)

goob <- c()

flink <- function(x)
    pmin(abs(x), 18) * sign(x)

### negative binom. log-lik / n for glm(...)
for (j in 1:ncol(bs)) {
    b <- bs[,j]
    print(j)
    y <- c(-1, 1)[as.integer(wpbc$status)]
    foo <- function(f) 
        Binomial()@risk(y, f, as.numeric(b == 0)) / sum(b == 0)
    object <- glm(status ~  ., data = wpbc[,-2], subset = b > 0, 
                  family = binomial(), weights = b)
    p <- predict(object, newdata = wpbc[,-2], type = "link")
    goob <- c(goob, foo(flink(p)))
}

soob <- c()

flink <- function(x)
    pmin(abs(x), 18) * sign(x)

### negative binom. log-lik / n for step(glm(...))
for (j in 1:ncol(bs)) {
    b <- bs[,j]
    print(j)
    foo <- function(f) 
        Binomial()@risk(y, f, as.numeric(b == 0)) / sum(b == 0)
    object <- step(glm(status ~  ., data = wpbc[,-2], subset = b > 0, 
                       family = binomial(), weights = b),
                       trace = 0)
    p <- predict(object, newdata = wpbc[,-2], type = "link")
    soob <- c(soob, foo(flink(p)))
}



save(bs, goob, soob, boob, file = "wpbc_benchmarks.rda")

### WPBC survival benchmarks

### Wisconsin prognostic breast cancer data, without missing values
data("wpbc")
wpbc <- wpbc[complete.cases(wpbc),]
iw <- IPCweights(Surv(wpbc$time, wpbc$status == "R"))
wpbc3 <- wpbc[,colnames(wpbc) != "status"]

wpbc3 <- wpbc3[iw > 0,]
iw <- iw[iw > 0]

### setup benchmark experiments with B = 100 bootstrap samples
n <- nrow(wpbc3)
set.seed(290875)
bs <- rmultinom(10, n, rep(1, n)/n)

mod <- glmboost(log(time) ~ ., data = wpbc3)

grid <- seq(from = 5, to = 1000, by = 10)

boob <- cvrisk(mod, folds = bs, grid = grid)

save(boob, grid, file = "wpbc_survivalbenchmarks.rda")

extractfit <- function(inb, fm, its)
{
    
    blsstr <- labels(terms(fm))
    attach(india)
    bls <- list()
    for(i in 1:length(blsstr))
        bls[[i]] <- eval(parse(text=blsstr[i]))
    detach(india)
    
    res <- list()
    res$blsstr <- blsstr
    res$its <- its
    res$coefs <- list()
    res$fits <- list()
    res$ensemble <- list()
    res$meanfits <- list()
    res$meancoefs <- list()
    
    for(i in 1:length(its))
    {
        coefs <- list()
        for(k in 1:length(bls))
            coefs[[k]] <- rep(0, dim(bls[[k]])[[2]])
        
        for(k in 1:its[i])
        {
            coefs[[inb$ensemble[k]]] <- coefs[[inb$ensemble[k]]] + inb$coef[[k]]
        }
        
        fits <- list()
        for(k in 1:length(bls))
            fits[[k]] <- bls[[k]] %*% coefs[[k]]
        
        meanfits <- list()
        for(k in 1:length(bls))
        {
            meanfits[[k]] <- fits[[k]] + inb$offset
            for(l in (1:length(bls))[-k])
            {
                if(!is.factor(india[,get("xname", environment(attr(bls[[l]], "dpp")))]))
                {
                    desX <- get("newX", environment(attr(bls[[l]], "dpp")))
                    meanx <- mean(india[,get("xname", environment(attr(bls[[l]], "dpp")))])
                    meanfits[[k]] <- meanfits[[k]] + (desX(meanx)[1,,drop=FALSE] %*% coefs[[l]])[1,1]
                }
                else
                    meanfits[[k]] <- meanfits[[k]] + coefs[[l]][1]
            }
        }
        
        meancoefs <- list()
        for(k in 1:length(bls))
        {
            meancoefs[[k]] <- coefs[[k]] + inb$offset
            for(l in (1:length(bls))[-k])
            {
                if(!is.factor(india[,get("xname", environment(attr(bls[[l]], "dpp")))]))
                {
                    desX <- get("newX", environment(attr(bls[[l]], "dpp")))
                    meanx <- mean(india[,get("xname", environment(attr(bls[[l]], "dpp")))])
                    meancoefs[[k]] <- meancoefs[[k]] + (desX(meanx)[1,,drop=FALSE] %*% coefs[[l]])[1,1]
                }
                else
                    meancoefs[[k]] <- meancoefs[[k]] + coefs[[l]][1]
            }
        }
        
        res$ensemble[[i]] <- table(factor(inb$ensemble[1:its[i]], levels=1:length(blsstr), labels=blsstr))
        res$coefs[[i]] <- coefs
        res$fits[[i]] <- fits
        res$meanfits[[i]] <- meanfits
        res$meancoefs[[i]] <- meancoefs
    }
    
    return(res)
}

plotnonpar <- function(name="cage", index=1, label="age of the child", fit)
{
    nits <- length(fit$its)
    greyseq <- seq(0.9, 0, length=nits)
    
    ind <- (1:length(india[,name]))[!duplicated(india[,name])]
    o <- order(india[ind,name])
    ind <- ind[o]
    india[ind,name]
    
    plot(india[ind,name], fit$fits[[nits]][[index]][ind], type="l", xlab=label, ylab="")
    for(i in 1:nits)
        lines(india[ind,name], fit$fits[[i]][[index]][ind], col=grey(greyseq[i]))
    
    return(invisible())
}

plotnonpar2 <- function(name="cage", index=1, label="age of the child", fits)
{
    nits <- length(fits[[1]]$its)
    ind <- (1:length(india[,name]))[!duplicated(india[,name])]
    o <- order(india[ind,name])
    ind <- ind[o]
    
    xplot <- india[ind,name]
    fitplot <- matrix(0, length(fits), length(xplot))
    for(i in 1:length(fits))
        fitplot[i,] <-  fits[[i]]$fits[[nits]][[index]][ind]
    rfit <- range(fitplot)
    
    plot(xplot, fitplot[1,], type="l", xlab=label, ylab="", ylim=c(rfit[1],rfit[2]))
    for(i in 2:length(fits))
        lines(xplot, fitplot[i,], lty=i)
    
    return(invisible())
}

plotnonpar3 <- function(name="cage", index=1, label="age of the child", fits)
{
    nits <- length(fits[[1]]$its)
    ind <- (1:length(india[,name]))[!duplicated(india[,name])]
    o <- order(india[ind,name])
    ind <- ind[o]
    
    xplot <- india[ind,name]
    fitplot <- matrix(0, length(fits), length(xplot))
    for(i in 1:length(fits))
        fitplot[i,] <-  fits[[i]]$meanfits[[nits]][[index]][ind]
    rfit <- range(fitplot)
    
    plot(xplot, fitplot[1,], type="l", xlab=label, ylab="", ylim=c(rfit[1],rfit[2]))
    for(i in 2:length(fits))
        lines(xplot, fitplot[i,], lty=i)
    
    return(invisible())
}


plotpar <- function(name = "csex", index = 3, label="sex of the child", fit)
{
    labels <- levels(india[,name])[-1]
    
    nits <- length(fit$its)
    mat <- matrix(0,nits,length(fit$coefs[[nits]][[index]])-1)
    for(i in 1:nits)
        mat[i,] <- fit$coefs[[i]][[index]][-1]
    
    plot(fit$its, mat[,1], ylim=range(mat), type="l", xlab=label, ylab="")
    
    if(dim(mat)[2]>1)
    {
        for(i in 2:(dim(mat)[2]))
        {
            lines(fit$its, mat[,i])
        }
    }
    axis(4, at=mat[nits,], labels)
    
    return(invisible())
}

plotpar2 <- function(name = "csex", index = 3, label="sex of the child", fits)
{
    labels <- factor(levels(india[,name])[-1])
    
    nits <- length(fits[[1]]$its)
    mat <- matrix(0,length(fits),length(fits[[1]]$coefs[[nits]][[index]])-1)
    for(i in 1:length(fits))
        mat[i,] <- fits[[i]]$coefs[[nits]][[index]][-1]
    
    plot(labels, mat[1,], ylim=range(mat), ylab=label)
    for(i in 2:length(fits))
        plot(labels, mat[i,], lty=i, add=TRUE)
    return(invisible())
}

plotpar3 <- function(name = "csex", index = 3, label="sex of the child", fits)
{
    labels <- factor(levels(india[,name]))
    
    nits <- length(fits[[1]]$its)
    mat <- matrix(0,length(fits),length(fits[[1]]$meancoefs[[nits]][[index]]))
    for(i in 1:length(fits))
        mat[i,] <- fits[[i]]$meancoefs[[nits]][[index]]
    
    plot(labels, mat[1,], ylim=range(mat), ylab=label)
    for(i in 2:length(fits))
        plot(labels, mat[i,], lty=i, add=TRUE)
    return(invisible())
}

library("foreign")
library("car")

indiaraw <- read.dta("IAKR51FL.DTA")
vars <- character(0)
nams <- character(0)

# Response related variables:
# hw2      "Weight in kilograms (1 decimal)"
# hw3      "Height in centimeters (1 decimal)"
# hw70     "Ht/A Standard deviations (according to WHO)" = Stunting
# hw71     "Wt/A Standard deviations (according to WHO)" = Wasting
# hw72     "Wt/Ht Standard deviations (according to WHO)" = Underweight
# hw73     "BMI Standard deviations (according to WHO)"
# hw5      "Ht/A Standard deviations" = old definition of Stunting
# hw8      "Wt/A Standard deviations" = old definition of Wasting
# hw11     "Wt/Ht Standard deviations" = old definition of Underweight

indiaraw$hw2 <- indiaraw$hw2/10
indiaraw$hw3 <- indiaraw$hw3/10
indiaraw$cbmi <- indiaraw$hw2/((indiaraw$hw3/100)^2)

vars <- c(vars, c("hw2", "hw3", "hw70", "hw71", "hw72", "hw73", "hw5", "hw8",
                  "hw11","cbmi"))
nams <- c(nams, c("cweight", "cheight", "stunting", "wasting", "underweight",
                  "cbmiscore", "stuntingold", "wastingold", "underweightold",
                  "cbmi"))

# v008     "Date of interview (CMC)"
# b3       "Date of birth (CMC)"
# m5       "Months of breastfeeding"
# b4       "Sex of child"
# bord     "Birth order number"
# b0       "Child is twin"

indiaraw$cage <- indiaraw$v008-indiaraw$b3
indiaraw$m5[indiaraw$m5==94] <- 0
indiaraw$m5[indiaraw$m5>90] <- NA
indiaraw$ctwin <- recode(indiaraw$b0, "c('1st of multiple','2nd of multiple','3rd of multiple','4th of multiple','5th of multiple')='twin'")

vars <- c(vars, c("cage", "m5", "b4", "ctwin", "bord"))
nams <- c(nams, c("cage", "breastfeeding", "csex", "ctwin", "cbirthorder"))


# v445     "Body mass index for respondent"
# v437     "Respondent's weight (kilos-1d)"
# v438     "Respondent's height (cms-1d)"
# b3       "Date of birth (CMC)"
# v011     "Date of birth (CMC)"
# v133     "Education in single years"
# v715     "Partner's education-single years"
# v717     "Respondent's occupation"
# v130     "Religion"
# v025     "Type of place of residence"
# v206     "Sons who have died"
# v207     "Daughters who have died"

indiaraw$v445 <- indiaraw$v445/100
indiaraw$v437 <- indiaraw$v437/10
indiaraw$v438 <- indiaraw$v438/10
indiaraw$mage <- round((indiaraw$b3-indiaraw$v011)/12, 0)
indiaraw$munemployed <- as.numeric(indiaraw$v717)
indiaraw$munemployed[indiaraw$munemployed>1] <- 2
indiaraw$munemployed <- factor(indiaraw$munemployed, labels=c("unemployed","employed"))
#indiaraw$cunemployed <- recode(indiaraw$v717, "c('prof., tech., manag.','clerical','sales','agric-self employed','agric-employee','household & domestic','services','skilled & unskilled manual','[for nfhs-3 skilled and unskilled manual combined]','unskilled','don\'t know')='working'")
indiaraw$mreligion <- recode(indiaraw$v130, "c('buddhist/neo-buddhist','jain','jewish','parsi/zoroastrian','no religion','donyi polo')='other'")
indiaraw$deadchildren <- indiaraw$v206 + indiaraw$v207

vars <- c(vars, c("v445", "v437", "v438", "mage", "v133", "v715", "munemployed", "mreligion", "v025", "deadchildren"))
nams <- c(nams, c("mbmi", "mweight", "mheight", "mage", "medu", "edupartner", "munemployed", "mreligion", "mresidence", "deadchildren"))


# v190     "Wealth index"
# v119     "Has electricity"
# v120     "Has radio"
# v121     "Has television"
# v122     "Has refrigerator"
# v123     "Has bicycle"
# v124     "Has motorcycle/scooter"
# v125     "Has car"
vars <- c(vars, c("v190", "v119", "v120", "v121", "v122", "v123", "v124", "v125"))
nams <- c(nams, c("wealth", "electricity", "radio", "television", "refrigerator", "bicycle", "motorcycle", "car"))


india <- indiaraw[,vars]
names(india) <- nams
rm(indiaraw)


#######
# plausibility checks

hist(india$cweight)
india$cweight[india$cweight>25] <- NA
hist(india$cheight)
india$cheight[india$cheight>140] <- NA
hist(india$stunting)
india$stunting[india$stunting>2000] <- NA
hist(india$wasting)
india$wasting[india$wasting>2000] <- NA
hist(india$underweight)
india$underweight[india$underweight>2000] <- NA

hist(india$cbmi)
india$cbmi[india$cbmi>26] <- NA
india$cbmi[india$cbmi<10] <- NA
plot(india$cbmi, india$stunting)

hist(india$cage)
plot(india$cage, india$stunting)

table(india$breastfeeding)
plot(india$breastfeeding, india$stunting)
sum(india$breastfeeding > india$cage, na.rm=T)

table(india$csex)
plot(india$csex, india$stunting)

table(india$ctwin)
plot(india$ctwin, india$stunting)

table(india$cbirthorder)
india$cbirthorder[india$cbirthorder>5] <- 5
india$cbirthorder <- factor(india$cbirthorder)
plot(india$cbirthorder, india$stunting)

hist(india$mbmi)
india$mbmi[india$mbmi>40] <- NA
plot(india$mbmi, india$stunting)

hist(india$mweight)
india$mweight[india$mweight>120] <- NA
india$mweight[india$mweight<20] <- NA
plot(india$mweight, india$stunting)

hist(india$mheight)
india$mheight[india$mheight>185] <- NA
india$mheight[india$mheight<123] <- NA
plot(india$mheight, india$stunting)

hist(india$mage)
plot(india$mage, india$stunting)

hist(india$medu)
plot(india$medu, india$stunting)

hist(india$edupartner)
india$edupartner[india$edupartner>80] <- NA
plot(india$edupartner, india$stunting)

table(india$munemployed)
plot(india$munemployed, india$stunting)

table(india$mreligion)
plot(india$mreligion, india$stunting)

table(india$mresidence)
plot(india$mresidence, india$stunting)

table(india$deadchildren)
india$deadchildren[india$deadchildren>3] <- 3
india$deadchildren <- factor(india$deadchildren)
plot(india$deadchildren, india$stunting)

table(india$wealth)
plot(india$wealth, india$stunting)

table(india$electricity)
india$electricity[india$electricity=="not de jure resident"] <- NA
india$electricity<- factor(india$electricity, labels=c("no","yes"))
plot(india$electricity, india$stunting)

table(india$radio)
india$radio[india$radio=="not de jure resident"] <- NA
india$radio<- factor(india$radio, labels=c("no","yes"))
plot(india$radio, india$stunting)

table(india$television)
india$television[india$television=="not de jure resident"] <- NA
india$television<- factor(india$television, labels=c("no","yes"))
plot(india$television, india$stunting)

table(india$refrigerator)
india$refrigerator[india$refrigerator=="not de jure resident"] <- NA
india$refrigerator<- factor(india$refrigerator, labels=c("no","yes"))
plot(india$refrigerator, india$stunting)

table(india$bicycle)
india$bicycle[india$bicycle=="not de jure resident"] <- NA
india$bicycle<- factor(india$bicycle, labels=c("no","yes"))
plot(india$bicycle, india$stunting)

table(india$motorcycle)
india$motorcycle[india$motorcycle=="not de jure resident"] <- NA
india$motorcycle<- factor(india$motorcycle, labels=c("no","yes"))
plot(india$motorcycle, india$stunting)

table(india$car)
india$car[india$car=="not de jure resident"] <- NA
india$car<- factor(india$car, labels=c("no","yes"))
plot(india$car, india$stunting)

india <- india[complete.cases(india),]

india$intercept <- 1

center <- function(x)
    return(x-mean(x))

india$cagec <- center(india$cage)
india$magec <- center(india$mage)
india$mbmic <- center(india$mbmi)
india$meduc <- center(india$medu)
india$breastfeedingc <- center(india$breastfeeding)
india$eduparterc <- center(india$edupartner)

# create random partitions of the data set
set.seed(12345)

help <- rep(1:3, each=nrow(india)/3)
india$w1 <- sample(help)
india$w2 <- sample(help)
india$w3 <- sample(help)
india$w4 <- sample(help)
india$w5 <- sample(help)
india$w6 <- sample(help)
india$w7 <- sample(help)
india$w8 <- sample(help)
india$w9 <- sample(help)
india$w10 <- sample(help)
india$w11 <- sample(help)
india$w12 <- sample(help)
india$w13 <- sample(help)
india$w14 <- sample(help)
india$w15 <- sample(help)
india$w16 <- sample(help)
india$w17 <- sample(help)
india$w18 <- sample(help)
india$w19 <- sample(help)
india$w20 <- sample(help)
india$w21 <- sample(help)
india$w22 <- sample(help)
india$w23 <- sample(help)
india$w24 <- sample(help)
india$w25 <- sample(help)
india$w26 <- sample(help)
india$w27 <- sample(help)
india$w28 <- sample(help)
india$w29 <- sample(help)
india$w30 <- sample(help)
india$w31 <- sample(help)
india$w32 <- sample(help)
india$w33 <- sample(help)
india$w34 <- sample(help)
india$w35 <- sample(help)
india$w36 <- sample(help)
india$w37 <- sample(help)
india$w38 <- sample(help)
india$w39 <- sample(help)
india$w40 <- sample(help)
india$w41 <- sample(help)
india$w42 <- sample(help)
india$w43 <- sample(help)
india$w44 <- sample(help)
india$w45 <- sample(help)
india$w46 <- sample(help)
india$w47 <- sample(help)
india$w48 <- sample(help)
india$w49 <- sample(help)
india$w50 <- sample(help)

write.table(india, "india.raw", col.names=TRUE, row.names=FALSE, sep=" ", quote=FALSE)
save("india", file="india.Rdata")

load("plotresults.Rdata")

mystabsel <- function(mat, q=10, FWER=0.05, bl)
{
    p <- length(bl)
    cutoff <- min(0.9, (q^2/(FWER * p) + 1)/2)
    topq <- rep(0, ncol(mat))
    for(i in 1:ncol(mat))
    {
        print(i)
        if(length(unique(mat[!is.na(mat[,i]),i])) < q)
            topq[i] <- sum(!is.na(mat[,i]))
        else 
        {
            topq[i] <- q
            while(length(unique(mat[1:(topq[i]),i])) < q)
            {
                topq[i] <- topq[i]+1
            }
        }
    }
    topbl <- rep(0, p)
    for(i in 1:p)
    {
        for(j in 1:ncol(mat))
        {
            if(i %in% mat[1:(topq[j]),j])
                topbl[i] <- topbl[i] + 1
        }
    }  
    topbl <- topbl / ncol(mat)
    topblind <- 1*(topbl >= cutoff)
    return(list(topbl=topbl, topblind=topblind))
}

# additive models
q <- 10
FWER <- 0.05
bl <- levels(add1.selected[[1]])

add1.topbl <- mystabsel(add1.stabsel, q=q, FWER=FWER, bl=bl)
add2.topbl <- mystabsel(add2.stabsel, q=q, FWER=FWER, bl=bl)
add3.topbl <- mystabsel(add3.stabsel, q=q, FWER=FWER, bl=bl)

bl[add1.topbl$topblind==1]
bl[add2.topbl$topblind==1]
bl[add3.topbl$topblind==1]

# VCMs

q <- 10
FWER <- 0.05
bl <- levels(vcm1.selected[[1]])

vcm1.topbl <- mystabsel(vcm1.stabsel, q=q, FWER=FWER, bl=bl)
vcm2.topbl <- mystabsel(vcm2.stabsel, q=q, FWER=FWER, bl=bl)
vcm3.topbl <- mystabsel(vcm3.stabsel, q=q, FWER=FWER, bl=bl)

bl[vcm1.topbl$topblind==1]
bl[vcm2.topbl$topblind==1]
bl[vcm3.topbl$topblind==1]

# rescale results to match definition of the Z-Score

plotdata.add1[[1]][,-1] <- plotdata.add1[[1]][,-1]/100
plotdata.add3[[1]][,-1] <- plotdata.add3[[1]][,-1]/100

plotdata.add1[[2]][,-1] <- plotdata.add1[[2]][,-1]/100
plotdata.add3[[2]][,-1] <- plotdata.add3[[2]][,-1]/100

plotdata.add1[[4]][,-1] <- plotdata.add1[[4]][,-1]/100
plotdata.add3[[4]][,-1] <- plotdata.add3[[4]][,-1]/100

plotdata.add1[[7]][,-1] <- plotdata.add1[[7]][,-1]/100
plotdata.add3[[7]][,-1] <- plotdata.add3[[7]][,-1]/100

plotdata.add1[[8]][,-1] <- plotdata.add1[[8]][,-1]/100
plotdata.add3[[8]][,-1] <- plotdata.add3[[8]][,-1]/100

plotdata.add1[[14]][,-1] <- plotdata.add1[[14]][,-1]/100
plotdata.add3[[14]][,-1] <- plotdata.add3[[14]][,-1]/100

plotdata.vcm1[[1]][,-1] <- plotdata.vcm1[[1]][,-1]/100
plotdata.vcm3[[1]][,-1] <- plotdata.vcm3[[1]][,-1]/100

plotdata.vcm1[[length(varnames)+1]][,-1] <- plotdata.vcm1[[length(varnames)+1]][,-1]/100
plotdata.vcm3[[length(varnames)+1]][,-1] <- plotdata.vcm3[[length(varnames)+1]][,-1]/100

# parallel coordinate plots of the prediction risks

pdf("india_predictions.pdf", height = 4)

library("lattice")
cv <- data.frame(error = c(as.vector(pred1),
                           as.vector(pred2),
                           as.vector(pred3)),
                 algo = rep(factor(colnames(pred1), levels = colnames(pred1),
                                   labels = c("additive", "VCM", "stumps", "trees")), 
                            rep(nrow(pred1), ncol(pred1))),
                 quant = rep(factor(c("5% Quantile", "10% Quantile", "Median"),
                                    levels = c("5% Quantile", "10% Quantile", "Median"),
                                    labels = c("5% Quantile", "10% Quantile", "Median")),
                             rep(length(pred1), 3)))

mypanel <- function(x, y, ...) {
    panel.bwplot(x = x, y = y, ...)
    z <- as.data.frame(matrix(y, ncol = 4))
    apply(z, 1, function(x)
        llines(x = 1:4, y = x, col = rgb(0.1, 0.1, 0.1, 0.1)))
}
print(bwplot(error ~ algo | quant, data = cv, ylab = "CV risk", 
             scales = list(y = list(relation = "sliced"), x = list(rot = 25)), 
             layout = c(3, 1), panel = mypanel, fill="grey", pch = "|",
             par.settings = list(plot.symbol = list(col=1,pch=20, cex=0.7),
                                 box.rectangle = list(col=1),
                                 box.umbrella = list(lty=1, col=1)),
             strip= strip.custom(bg="white")))

dev.off()

# nonlinear effects from the aditive model shown in the main part of the paper

pdf("india_additive_nonlinear.pdf", width=8, height=10.5)
mat <- matrix(0, 6, 4)
mat[1,] <- c(0, 0.5, 2/3, 1)
mat[2,] <- c(0.5, 1, 2/3, 1)
mat[3,] <- c(0, 0.5, 1/3, 2/3)
mat[4,] <- c(0.5, 1, 1/3, 2/3)
mat[5,] <- c(0, 0.5, 0, 1/3)
mat[6,] <- c(0.5, 1, 0, 1/3)
split.screen(mat)

i <- 1
screen(1)
par(mai=c(1.0,0.5,0.5,0.1))

col <- rgb(0.1, 0.1, 0.1, 0.1)
pr <- range(plotdata.add1[[i]][,-1])
ind <- order(plotdata.add1[[i]][,1])
plot(plotdata.add1[[i]][ind,1], plotdata.add1[[i]][ind,2], ylim=pr,
     axes=TRUE, type="l", xlab="age of the child in months", ylab="", main="5% Quantile",
     col=col, cex.lab=1.)
for(j in 2:ncol(plotdata.add1[[i]]))
    lines(plotdata.add1[[i]][ind,1], plotdata.add1[[i]][ind,j], col=col)

screen(2)
par(mai=c(1.0,0.5,0.5,0.1))

pr <- range(plotdata.add3[[i]][,-1])
ind <- order(plotdata.add3[[i]][,1])
plot(plotdata.add3[[i]][ind,1], plotdata.add3[[i]][ind,2], ylim=pr,
     axes=TRUE, type="l", xlab="age of the child in months", ylab="", main="Median",
     col=col, cex.lab=1.)
for(j in 2:ncol(plotdata.add3[[i]]))
    lines(plotdata.add3[[i]][ind,1], plotdata.add3[[i]][ind,j], col=col)

i <- 2
screen(3)
par(mai=c(1.0,0.5,0.5,0.1))

pr <- range(plotdata.add1[[i]][,-1])
ind <- order(plotdata.add1[[i]][,1])
plot(plotdata.add1[[i]][ind,1], plotdata.add1[[i]][ind,2], ylim=pr,
     axes=TRUE, type="l", xlab="duration of breastfeeding in months", ylab="", main="5% Quantile",
     col=col, cex.lab=1.)
for(j in 2:ncol(plotdata.add1[[i]]))
    lines(plotdata.add1[[i]][ind,1], plotdata.add1[[i]][ind,j], col=col)

screen(4)
par(mai=c(1.0,0.5,0.5,0.1))

pr <- range(plotdata.add3[[i]][,-1])
ind <- order(plotdata.add3[[i]][,1])
plot(plotdata.add3[[i]][ind,1], plotdata.add3[[i]][ind,2], ylim=pr,
     axes=TRUE, type="l", xlab="duration of breastfeeding in months", ylab="", main="Median",
     col=col, cex.lab=1.)
for(j in 2:ncol(plotdata.add3[[i]]))
    lines(plotdata.add3[[i]][ind,1], plotdata.add3[[i]][ind,j], col=col)

i <- 4
screen(5)
par(mai=c(1.0,0.5,0.5,0.1))

pr <- range(plotdata.add1[[i]][,-1])
ind <- order(plotdata.add1[[i]][,1])
plot(plotdata.add1[[i]][ind,1], plotdata.add1[[i]][ind,2], ylim=pr,
     axes=TRUE, type="l", xlab="age of the mother in years", ylab="", main="5% Quantile",
     col=col, cex.lab=1.)
for(j in 2:ncol(plotdata.add1[[i]]))
    lines(plotdata.add1[[i]][ind,1], plotdata.add1[[i]][ind,j], col=col)

screen(6)
par(mai=c(1.0,0.5,0.5,0.1))

pr <- range(plotdata.add3[[i]][,-1])
ind <- order(plotdata.add3[[i]][,1])
plot(plotdata.add3[[i]][ind,1], plotdata.add3[[i]][ind,2], ylim=pr,
     axes=TRUE, type="l", xlab="age of the mother in years", ylab="", main="Median",
     col=col, cex.lab=1.)
for(j in 2:ncol(plotdata.add3[[i]]))
    lines(plotdata.add3[[i]][ind,1], plotdata.add3[[i]][ind,j], col=col)

close.screen(all = TRUE)
dev.off()

# parametric effects from the additive model shown in the main part of the paper

pdf("india_additive_categorical.pdf", width=8, height=10.5)
mat <- matrix(0, 6, 4)
mat[1,] <- c(0, 0.5, 2/3, 1)
mat[2,] <- c(0.5, 1, 2/3, 1)
mat[3,] <- c(0, 0.5, 1/3, 2/3)
mat[4,] <- c(0.5, 1, 1/3, 2/3)
mat[5,] <- c(0, 0.5, 0, 1/3)
mat[6,] <- c(0.5, 1, 0, 1/3)
split.screen(mat)

i <- 7
screen(1)
par(mai=c(0.5,0.5,0.5,0.1))

pr <- range(plotdata.add1[[i]][,-1])
ind <- order(plotdata.add1[[i]][,1])

tmp <- t(plotdata.add1[[i]][,-1])
colnames(tmp) <- levels(plotdata.add1[[i]][,1])
boxplot(tmp, main="5% Quantile", cex.lab=1., cex.axis=0.9, col = "grey", 
        pars = list(outcol = "black", outpch = 19, outcex = 0.7))
out <- apply(tmp, 1, function(x) lines(1:ncol(tmp), x, col = col))

screen(2)
par(mai=c(0.5,0.5,0.5,0.1))

tmp <- t(plotdata.add3[[i]][,-1])
colnames(tmp) <- levels(plotdata.add3[[i]][,1])
boxplot(tmp, main="Median", cex.lab=1., cex.axis=0.9, col = "grey", 
        pars = list(outcol = "black", outpch = 19, outcex = 0.7))
out <- apply(tmp, 1, function(x) lines(1:ncol(tmp), x, col = col))

i <- 8
screen(3)
par(mai=c(0.5,0.5,0.5,0.1))

tmp <- t(plotdata.add1[[i]][,-1])
colnames(tmp) <- levels(plotdata.add1[[i]][,1])
boxplot(tmp, main="5% Quantile", cex.lab=1., cex.axis=0.9, col = "grey", 
        pars = list(outcol = "black", outpch = 19, outcex = 0.7))
out <- apply(tmp, 1, function(x) lines(1:ncol(tmp), x, col = col))

screen(4)
par(mai=c(0.5,0.5,0.5,0.1))

tmp <- t(plotdata.add3[[i]][,-1])
colnames(tmp) <- levels(plotdata.add3[[i]][,1])
boxplot(tmp, main="Median", cex.lab=1., cex.axis=0.9, col = "grey", 
        pars = list(outcol = "black", outpch = 19, outcex = 0.7))
out <- apply(tmp, 1, function(x) lines(1:ncol(tmp), x, col = col))

i <- 14
screen(5)
par(mai=c(0.5,0.5,0.5,0.1))

tmp <- t(plotdata.add1[[i]][,-1])
colnames(tmp) <- as.character(plotdata.add1[[i]][,1])
tmp <- tmp[, levels(plotdata.add1[[i]][,1])]
boxplot(tmp, main="5% Quantile", cex.lab=1., cex.axis=0.9, col = "grey", 
        pars = list(outcol = "black", outpch = 19, outcex = 0.7))
out <- apply(tmp, 1, function(x) lines(1:ncol(tmp), x, col = col))

screen(6)
par(mai=c(0.5,0.5,0.5,0.1))

tmp <- t(plotdata.add3[[i]][,-1])
colnames(tmp) <- as.character(plotdata.add3[[i]][,1])
tmp <- tmp[, levels(plotdata.add3[[i]][,1])]
boxplot(tmp, main="Median", cex.lab=1., cex.axis=0.9, col = "grey", 
        pars = list(outcol = "black", outpch = 19, outcex = 0.7))
out <- apply(tmp, 1, function(x) lines(1:ncol(tmp), x, col = col))

close.screen(all = TRUE)
dev.off()

# nonlinear effects from the VCM shown in the main paper

pdf("india_vcm_nonlinear.pdf", width=8, height=7)
mat <- matrix(0, 4, 4)
mat[1,] <- c(0, 0.5, 0.5, 1)
mat[2,] <- c(0.5, 1, 0.5, 1)
mat[3,] <- c(0, 0.5, 0, 0.5)
mat[4,] <- c(0.5, 1, 0, 0.5)
split.screen(mat)

i <- 1
screen(1)
par(mai=c(1.0,0.5,0.5,0.1))

pr <- range(cbind(plotdata.vcm1[[i]][,-1],
                  plotdata.vcm1[[length(varnames)+i]][,-1]))
ind <- order(plotdata.vcm1[[i]][,1])
plot(plotdata.vcm1[[i]][ind,1], plotdata.vcm1[[i]][ind,2], ylim=pr,
     axes=TRUE, type="l", xlab="age of the child in months", ylab="", main="5% Quantile (boys)",
     col=col, cex.lab=1.)
for(j in 2:ncol(plotdata.vcm1[[i]]))
    lines(plotdata.vcm1[[i]][ind,1], plotdata.vcm1[[i]][ind,j], col=col)


screen(2)
par(mai=c(1.0,0.5,0.5,0.1))

pr <- range(cbind(plotdata.vcm3[[i]][,-1],
                  plotdata.vcm3[[length(varnames)+i]][,-1]))
ind <- order(plotdata.vcm3[[i]][,1])
plot(plotdata.vcm3[[i]][ind,1], plotdata.vcm3[[i]][ind,2], ylim=pr,
     axes=TRUE, type="l", xlab="age of the child in months", ylab="", main="Median (boys)",
     col=col, cex.lab=1.)
for(j in 2:ncol(plotdata.vcm3[[i]]))
    lines(plotdata.vcm3[[i]][ind,1], plotdata.vcm3[[i]][ind,j], col=col)

screen(3)
par(mai=c(1.0,0.5,0.5,0.1))

pr <- range(cbind(plotdata.vcm3[[i]][,-1],
                  plotdata.vcm3[[length(varnames)+i]][,-1]))
pr <- range(plotdata.vcm1[[length(varnames)+i]][,-1])
ind <- order(plotdata.vcm1[[length(varnames)+i]][,1])
plot(plotdata.vcm1[[length(varnames)+i]][ind,1], plotdata.vcm1[[length(varnames)+i]][ind,2], ylim=pr,
     axes=TRUE, type="l", xlab="age of the child in months", ylab="", main="5% Quantile (girls)",
     col=col, cex.lab=1.)
for(j in 2:ncol(plotdata.vcm1[[length(varnames)+i]]))
    lines(plotdata.vcm1[[length(varnames)+i]][ind,1], plotdata.vcm1[[length(varnames)+i]][ind,j], col=col)

screen(4)
par(mai=c(1.0,0.5,0.5,0.1))

pr <- range(cbind(plotdata.vcm3[[i]][,-1],
                  plotdata.vcm3[[length(varnames)+i]][,-1]))
pr <- range(plotdata.vcm3[[length(varnames)+i]][,-1])
ind <- order(plotdata.vcm3[[length(varnames)+i]][,1])
plot(plotdata.vcm3[[length(varnames)+i]][ind,1], plotdata.vcm3[[length(varnames)+i]][ind,2], ylim=pr,
     axes=TRUE, type="l", xlab="age of the child in months", ylab="", main="Median (girls)",
     col=col, cex.lab=1.)
for(j in 2:ncol(plotdata.vcm3[[length(varnames)+i]]))
    lines(plotdata.vcm3[[length(varnames)+i]][ind,1], plotdata.vcm3[[length(varnames)+i]][ind,j], col=col)

close.screen(all = TRUE)
dev.off()

# Figures for the additive model from the eSupplement

load("plotresults.Rdata")

for(i in 1:21)
{
    plotdata.add1[[i]][,-1] <- plotdata.add1[[i]][,-1]/100
    plotdata.add2[[i]][,-1] <- plotdata.add2[[i]][,-1]/100
    plotdata.add3[[i]][,-1] <- plotdata.add3[[i]][,-1]/100
}

varlab <- c("age of the child in months",
            "duration of breastfeeding in months",
            "body mass index of the mother",
            "age of the mother in years",
            "education years of the mother",
            "education years of the mother's partner",
            "sex of the child",
            "twin birth",
            "position in the birth order",
            "unemployment of the mother",
            "religion of the mother",
            "area of residence",
            "no. of dead children",
            "wealth index",
            "electrictiy",
            "radio",
            "television",
            "refrigerator",
            "bicycle",
            "motorcycle",
            "car")

plotfnc <- function(offset=1)
{
    mat <- matrix(0, 6, 4)
    mat[1,] <- c(0, 0.5, 2/3, 1)
    mat[2,] <- c(0.5, 1, 2/3, 1)
    mat[3,] <- c(0, 0.5, 1/3, 2/3)
    mat[4,] <- c(0.5, 1, 1/3, 2/3)
    mat[5,] <- c(0, 0.5, 0, 1/3)
    mat[6,] <- c(0.5, 1, 0, 1/3)
    split.screen(mat)
    
    if(is.numeric(plotdata.add1[[offset]][,1]))
    {
        col <- rgb(0.1, 0.1, 0.1, 0.1)
        
        screen(1)
        par(mai=c(1.0,0.5,0.5,0.1))
        
        pr <- range(plotdata.add1[[offset]][,-1])
        ind <- order(plotdata.add1[[offset]][,1])
        
        plot(plotdata.add1[[offset]][ind,1], plotdata.add1[[offset]][ind,2], ylim=pr,
             axes=TRUE, type="l", xlab=varlab[offset], ylab="", main="5% Quantile",
             col=col, cex.lab=1.)
        for(j in 2:ncol(plotdata.add1[[offset]]))
            lines(plotdata.add1[[offset]][ind,1], plotdata.add1[[offset]][ind,j], col=col)
        
        screen(3)
        par(mai=c(1.0,0.5,0.5,0.1))
        
        pr <- range(plotdata.add2[[offset]][,-1])
        ind <- order(plotdata.add2[[offset]][,1])
        
        plot(plotdata.add2[[offset]][ind,1], plotdata.add2[[offset]][ind,2], ylim=pr,
             axes=TRUE, type="l", xlab=varlab[offset], ylab="", main="10% Quantile",
             col=col, cex.lab=1.)
        for(j in 2:ncol(plotdata.add2[[offset]]))
            lines(plotdata.add2[[offset]][ind,1], plotdata.add2[[offset]][ind,j], col=col)
        
        screen(5)
        par(mai=c(1.0,0.5,0.5,0.1))
        
        pr <- range(plotdata.add3[[offset]][,-1])
        ind <- order(plotdata.add3[[offset]][,1])
        
        plot(plotdata.add3[[offset]][ind,1], plotdata.add3[[offset]][ind,2], ylim=pr,
             axes=TRUE, type="l", xlab=varlab[offset], ylab="", main="Median",
             col=col, cex.lab=1.)
        for(j in 2:ncol(plotdata.add3[[offset]]))
            lines(plotdata.add3[[offset]][ind,1], plotdata.add3[[offset]][ind,j], col=col)
    }
    else
    {
        col <- rgb(0.1, 0.1, 0.1, 0.1)
        
        screen(1)
        par(mai=c(1.0,0.5,0.5,0.1))
        
        pr <- range(plotdata.add1[[offset]][,-1])
        ind <- order(plotdata.add1[[offset]][,1])
        
        tmp <- t(plotdata.add1[[offset]][,-1])
        colnames(tmp) <- as.character(plotdata.add1[[offset]][,1])
        tmp <- tmp[, levels(plotdata.add1[[offset]][,1])]
        boxplot(tmp, main="5% Quantile", cex.lab=1., cex.axis=0.9, col = "grey", 
                pars = list(outcol = "black", outpch = 19, outcex = 0.7), xlab=varlab[offset])
        out <- apply(tmp, 1, function(x) lines(1:ncol(tmp), x, col = col))
        
        screen(3)
        par(mai=c(1.0,0.5,0.5,0.1))
        
        pr <- range(plotdata.add2[[offset]][,-1])
        ind <- order(plotdata.add2[[offset]][,1])
        
        tmp <- t(plotdata.add2[[offset]][,-1])
        colnames(tmp) <- as.character(plotdata.add2[[offset]][,1])
        tmp <- tmp[, levels(plotdata.add2[[offset]][,1])]
        boxplot(tmp, main="10% Quantile", cex.lab=1., cex.axis=0.9, col = "grey", 
                pars = list(outcol = "black", outpch = 19, outcex = 0.7), xlab=varlab[offset])
        out <- apply(tmp, 1, function(x) lines(1:ncol(tmp), x, col = col))
        
        screen(5)
        par(mai=c(1.0,0.5,0.5,0.1))
        
        pr <- range(plotdata.add3[[offset]][,-1])
        ind <- order(plotdata.add3[[offset]][,1])
        
        tmp <- t(plotdata.add3[[offset]][,-1])
        colnames(tmp) <- as.character(plotdata.add3[[offset]][,1])
        tmp <- tmp[, levels(plotdata.add3[[offset]][,1])]
        boxplot(tmp, main="Median", cex.lab=1., cex.axis=0.9, col = "grey", 
                pars = list(outcol = "black", outpch = 19, outcex = 0.7), xlab=varlab[offset])
        out <- apply(tmp, 1, function(x) lines(1:ncol(tmp), x, col = col))
    }
    
    if(is.numeric(plotdata.add1[[offset+1]][,1]))
    {
        col <- rgb(0.1, 0.1, 0.1, 0.1)
        
        screen(2)
        par(mai=c(1.0,0.5,0.5,0.1))
        
        pr <- range(plotdata.add1[[offset+1]][,-1])
        ind <- order(plotdata.add1[[offset+1]][,1])
        
        plot(plotdata.add1[[offset+1]][ind,1], plotdata.add1[[offset+1]][ind,2], ylim=pr,
             axes=TRUE, type="l", xlab=varlab[offset+1], ylab="", main="5% Quantile",
             col=col, cex.lab=1.)
        for(j in 2:ncol(plotdata.add1[[offset+1]]))
            lines(plotdata.add1[[offset+1]][ind,1], plotdata.add1[[offset+1]][ind,j], col=col)
        
        screen(4)
        par(mai=c(1.0,0.5,0.5,0.1))
        
        pr <- range(plotdata.add2[[offset+1]][,-1])
        ind <- order(plotdata.add2[[offset+1]][,1])
        
        plot(plotdata.add2[[offset+1]][ind,1], plotdata.add2[[offset+1]][ind,2], ylim=pr,
             axes=TRUE, type="l", xlab=varlab[offset+1], ylab="", main="10% Quantile",
             col=col, cex.lab=1.)
        for(j in 2:ncol(plotdata.add2[[offset+1]]))
            lines(plotdata.add2[[offset+1]][ind,1], plotdata.add2[[offset+1]][ind,j], col=col)
        
        screen(6)
        par(mai=c(1.0,0.5,0.5,0.1))
        
        pr <- range(plotdata.add3[[offset+1]][,-1])
        ind <- order(plotdata.add3[[offset+1]][,1])
        
        plot(plotdata.add3[[offset+1]][ind,1], plotdata.add3[[offset+1]][ind,2], ylim=pr,
             axes=TRUE, type="l", xlab=varlab[offset+1], ylab="", main="Median",
             col=col, cex.lab=1.)
        for(j in 2:ncol(plotdata.add3[[offset+1]]))
            lines(plotdata.add3[[offset+1]][ind,1], plotdata.add3[[offset+1]][ind,j], col=col)
    }
    else
    {
        col <- rgb(0.1, 0.1, 0.1, 0.1)
        
        screen(2)
        par(mai=c(1.0,0.5,0.5,0.1))
        
        pr <- range(plotdata.add1[[offset+1]][,-1])
        ind <- order(plotdata.add1[[offset+1]][,1])
        
        tmp <- t(plotdata.add1[[offset+1]][,-1])
        colnames(tmp) <- as.character(plotdata.add1[[offset+1]][,1])
        tmp <- tmp[, levels(plotdata.add1[[offset+1]][,1])]
        boxplot(tmp, main="5% Quantile", cex.lab=1., cex.axis=0.9, col = "grey", 
                pars = list(outcol = "black", outpch = 19, outcex = 0.7), xlab=varlab[offset+1])
        out <- apply(tmp, 1, function(x) lines(1:ncol(tmp), x, col = col))
        
        screen(4)
        par(mai=c(1.0,0.5,0.5,0.1))
        
        pr <- range(plotdata.add2[[offset+1]][,-1])
        ind <- order(plotdata.add2[[offset+1]][,1])
        
        tmp <- t(plotdata.add2[[offset+1]][,-1])
        colnames(tmp) <- as.character(plotdata.add2[[offset+1]][,1])
        tmp <- tmp[, levels(plotdata.add2[[offset+1]][,1])]
        boxplot(tmp, main="10% Quantile", cex.lab=1., cex.axis=0.9, col = "grey", 
                pars = list(outcol = "black", outpch = 19, outcex = 0.7), xlab=varlab[offset+1])
        out <- apply(tmp, 1, function(x) lines(1:ncol(tmp), x, col = col))
        
        screen(6)
        par(mai=c(1.0,0.5,0.5,0.1))
        
        pr <- range(plotdata.add3[[offset+1]][,-1])
        ind <- order(plotdata.add3[[offset+1]][,1])
        
        tmp <- t(plotdata.add3[[offset+1]][,-1])
        colnames(tmp) <- as.character(plotdata.add3[[offset+1]][,1])
        tmp <- tmp[, levels(plotdata.add3[[offset+1]][,1])]
        boxplot(tmp, main="Median", cex.lab=1., cex.axis=0.9, col = "grey", 
                pars = list(outcol = "black", outpch = 19, outcex = 0.7), xlab=varlab[offset+1])
        out <- apply(tmp, 1, function(x) lines(1:ncol(tmp), x, col = col))
    }
    
    close.screen(all = TRUE)
}

pdf("india_additive_appendix1.pdf", width=8, height=10.5)
plotfnc(1)
dev.off()

pdf("india_additive_appendix2.pdf", width=8, height=10.5)
plotfnc(3)
dev.off()

pdf("india_additive_appendix3.pdf", width=8, height=10.5)
plotfnc(5)
dev.off()

pdf("india_additive_appendix4.pdf", width=8, height=10.5)
plotfnc(7)
dev.off()

pdf("india_additive_appendix5.pdf", width=8, height=10.5)
plotfnc(9)
dev.off()

pdf("india_additive_appendix6.pdf", width=8, height=10.5)
plotfnc(11)
dev.off()

pdf("india_additive_appendix7.pdf", width=8, height=10.5)
plotfnc(13)
dev.off()

pdf("india_additive_appendix8.pdf", width=8, height=10.5)
plotfnc(15)
dev.off()

pdf("india_additive_appendix9.pdf", width=8, height=10.5)
plotfnc(17)
dev.off()

pdf("india_additive_appendix10.pdf", width=8, height=10.5)
plotfnc(19)
dev.off()

pdf("india_additive_appendix11.pdf", width=8, height=10.5)
plotfnc(21)
dev.off()

library("mboost")
library("party")

load("india.Rdata")

#######
# VCM #
#######

quantiles <- c(0.05, 0.1, 0.5)
inds <- paste("w", 1:50, sep="")

fmvcm <- stunting ~ bols(intercept, intercept=FALSE) + 
    bols(cage) + bbs(cage, center=TRUE, df=1, knots = 20) +
    bols(breastfeeding) + bbs(breastfeeding, center=TRUE, df=1, knots=20) +
    bols(mbmi) + bbs(mbmi, center=TRUE, df=1, knots=20) +
    bols(mage) + bbs(mage, center=TRUE, df=1, knots=20) +
    bols(medu) + bbs(medu, center=TRUE, df=1, knots=20) +
    bols(edupartner) + bbs(edupartner, center=TRUE, df=1, knots=20) +
    bols(csex) +
    bols(ctwin) +
    bols(cbirthorder) +
    bols(munemployed) +
    bols(mreligion) +
    bols(mresidence) +
    bols(deadchildren) +
    bols(wealth) +
    bols(electricity) +
    bols(radio) +
    bols(television) +
    bols(refrigerator) +
    bols(bicycle) +
    bols(motorcycle) +
    bols(car) + 
    bols(cage, by=csex) + bbs(cage, center=TRUE, df=1, knots = 20, by=csex) +
    bols(breastfeeding, by=csex) + bbs(breastfeeding, center=TRUE, df=1, knots=20, by=csex) +
    bols(mbmi, by=csex) + bbs(mbmi, center=TRUE, df=1, knots=20, by=csex) +
    bols(mage, by=csex) + bbs(mage, center=TRUE, df=1, knots=20, by=csex) +
    bols(medu, by=csex) + bbs(medu, center=TRUE, df=1, knots=20, by=csex) +
    bols(edupartner, by=csex) + bbs(edupartner, center=TRUE, df=1, knots=20, by=csex) +
    bols(ctwin, by=csex) +
    bols(cbirthorder, by=csex) +
    bols(munemployed, by=csex) +
    bols(mreligion, by=csex) +
    bols(mresidence, by=csex) +
    bols(deadchildren, by=csex) +
    bols(wealth, by=csex) +
    bols(electricity, by=csex) +
    bols(radio, by=csex) +
    bols(television, by=csex) +
    bols(refrigerator, by=csex) +
    bols(bicycle, by=csex) +
    bols(motorcycle, by=csex) +
    bols(car, by=csex) 

for(i in 1:length(inds))
{
    indiafit <- india[india[,inds[i]]<3,]
    indiapred <- india[india[,inds[i]]==3,]
    wfit <- as.numeric(indiafit[,inds[i]]==1)
    
    for(j in 1:length(quantiles))
    {
        it <- 10000
        inc <- 10000
        maxit <- 200000
        bc <- boost_control(mstop = it, nu=0.25, trace = TRUE, risk = "oob")
        
        model <- gamboost(fmvcm,
                          data = indiafit,
                          control = bc,
                          weights = wfit,
                          family = QuantReg(tau = quantiles[j])
        )
        
        risk <- model$risk()                    
        while( ((risk[it-inc+1] - risk[it])>=0.05) &
               (it<=maxit) ) 
        {
            it <- it + inc
            model[it]
            risk <- model$risk()                    
        }
        
        save(model, file=paste("VCM/model",i,"_",quantiles[j],".Rdata",sep=""))
        rm(model)
        gc()
    }
}

library("mboost")
set.seed(290875)


data("bodyfat", package = "TH.data")

object <- glmboost(DEXfat ~ ., data = bodyfat, center = TRUE)
a <- stabsel(object, cutoff = 0.9, q = 2)
a
plot(a)

object <- gamboost(DEXfat ~ ., data = bodyfat)
a <- stabsel(object, cutoff = 0.9, q = 2)
a
plot(a)

###########################################################
##
## This code replicates the analysis presented in 
## 
##   N. Fenske, T. Kneib, and T. Hothorn (2011), 
##   Identifying risk factors for severe childhood malnutrition by 
##   boosting additive quantile regression.
##   Journal of the American Statistical Association, 106:494-510.
## 
###########################################################
## 
## Note: Requires Data File IAKR51FL.DTA 
## (Children`s recode for the 2005/2006 India DHS in Stata format)
## Freely available from www.measuredhs.com for research purposes
## (-> Data -> Available Datasets -> India)
##
###########################################################

# Preprocessing of the raw data
source("india_preproc.R", echo=TRUE)

# Fit additive quantile regressions
source("india_additive.R", echo=TRUE)

# Fit VCM quantile regressions
source("india_vcm.R", echo=TRUE)

# Fit quantile regressions trees
source("india_blackboost.R", echo=TRUE)

# Fit additive quantile regression stumps
source("india_stumps.R", echo=TRUE)

# Summarise results for plotting
# (all algorithms without rqss)
source("india_summary.R", echo=TRUE)
# and create plots
source("india_plots.R", echo=TRUE)


# Fit additive quantile regression by rqss
source("india_rqss.R", echo=TRUE)

# And plot results for empirical risk results for all algorithms
source("india_rqssResults.R", echo=TRUE)

#-----------------------------------------------------------
# Explore results for empirical risk with all algorithms 
# including rqss
#-----------------------------------------------------------


#-----------------------------------------------------------
# Path, Packages, source functions
#-----------------------------------------------------------

library("quantreg")
library("lattice")
library("mboost")

load("india.Rdata")
load("plotresults.Rdata")

source("india_rqss_lambdaOptFunc.R")

#-----------------------------------------------------------
# Quantiles and Indices
#-----------------------------------------------------------
inds <- paste("w", 1:50, sep="")
quantiles <- c(0.05, 0.1, 0.5)

pred1 <- pred2 <- pred3 <- matrix(0, ncol=5, nrow=length(inds))
colnames(pred1) <- colnames(pred2) <- colnames(pred3) <- c("rqss", "add","vcm","stump","tree")

lambda1rqss <- lambda2rqss <- lambda3rqss <- matrix(NA, nrow=length(inds), ncol=6)

predAdditive1 <- predAdditive2 <- predAdditive3 <- matrix(NA, ncol=50, nrow=nrow(india)/3)
predVCM1 <- predVCM2 <-predVCM3 <- matrix(NA, ncol=50, nrow=nrow(india)/3)
nTest <- rep(0, length(inds))

# write cvRisks into a vector
for(i in 1:length(inds)){
    print(i)
    
    indiaTrain <- india[india[,inds[i]]==1,]
    
    indiaTestOrg <- india[india[,inds[i]]==3,] 
    
    # Add variable for prediction with mboost
    indiaTestOrg <- cbind(indiaTestOrg, ff = 0)
    names(indiaTestOrg)[ncol(indiaTestOrg)] <- "FALSE"
    
    # Indices of convex hull of the test data
    indiaTestInd <- convexHullInd(dataOrg=indiaTestOrg, dataRef=indiaTrain)
    indiaTest <- convexHull(dataOrg=indiaTestOrg, dataRef=indiaTrain)
    nTest[i] <- nrow(indiaTest)
    
    #----------------
    # Prediction rqss
    load(paste("rqssObjects/model",i,"_0.05.Rdata",sep=""))
    pred <- predict(model, newdata=indiaTest)
    pred1[i,1] <- loss(indiaTest$stunting, pred, tau=0.05)
    lambda1rqss[i,] <- model$lambdas
    
    load(paste("rqssObjects/model",i,"_0.1.Rdata",sep=""))
    pred <- predict(model, newdata=indiaTest)
    pred2[i,1] <- loss(indiaTest$stunting, pred, tau=0.1)
    lambda2rqss[i,] <- model$lambdas
    
    load(paste("rqssObjects/model",i,"_0.5.Rdata",sep=""))
    pred <- predict(model, newdata=indiaTest)
    pred3[i,1] <- loss(indiaTest$stunting, pred, tau=0.5)
    lambda3rqss[i,] <- model$lambdas
    
    #---------------
    # Prediction additive
    load(paste("additive/model",i,"_0.05.Rdata",sep=""))
    predAdditive1[,i] <- predict(model, newdata=indiaTestOrg)
    pred1[i,2] <- loss(indiaTest$stunting, predAdditive1[indiaTestInd==0,i], tau=0.05)
    
    load(paste("additive/model",i,"_0.1.Rdata",sep=""))
    predAdditive2[,i] <- predict(model, newdata=indiaTestOrg)
    pred2[i,2] <- loss(indiaTest$stunting, predAdditive2[indiaTestInd==0,i], tau=0.1)
    
    load(paste("additive/model",i,"_0.5.Rdata",sep=""))
    predAdditive3[,i] <- predict(model, newdata=indiaTestOrg)
    pred3[i,2] <- loss(indiaTest$stunting, predAdditive3[indiaTestInd==0,i], tau=0.5)
    
    #---------------
    # Prediction VCM
    load(paste("VCM/model",i,"_0.05.Rdata",sep=""))
    predVCM1[,i] <- predict(model, newdata=indiaTestOrg)
    pred1[i,3] <- loss(indiaTest$stunting, predVCM1[indiaTestInd==0,i], tau=0.05)
    
    load(paste("VCM/model",i,"_0.1.Rdata",sep=""))
    predVCM2[,i] <- predict(model, newdata=indiaTestOrg)
    pred2[i,3] <- loss(indiaTest$stunting, predVCM2[indiaTestInd==0,i], tau=0.1)
    
    load(paste("VCM/model",i,"_0.5.Rdata",sep=""))
    predVCM3[,i] <- predict(model, newdata=indiaTestOrg)
    pred3[i,3] <- loss(indiaTest$stunting, predVCM3[indiaTestInd==0,i], tau=0.5)
    
    #---------------
    # Prediction Stumps
    load(paste("stumps/model",i,"_0.05.Rdata",sep=""))
    
    # -> The prediction on the test data was directly calculated after
    # calculating the model. Therefore not any more necessary here
    pred1[i,4] <- loss(indiaTest$stunting, pred[indiaTestInd==0], tau=0.05)
    
    load(paste("stumps/model",i,"_0.1.Rdata",sep=""))
    pred2[i,4] <- loss(indiaTest$stunting, pred[indiaTestInd==0], tau=0.1)
    
    load(paste("stumps/model",i,"_0.5.Rdata",sep=""))
    pred3[i,4] <- loss(indiaTest$stunting, pred[indiaTestInd==0], tau=0.5)
    
    
    load(paste("trees/model",i,"_0.05.Rdata",sep=""))
    pred1[i,5] <- loss(indiaTest$stunting, pred[indiaTestInd==0], tau=0.05)
    
    load(paste("trees/model",i,"_0.1.Rdata",sep=""))
    pred2[i,5] <- loss(indiaTest$stunting, pred[indiaTestInd==0], tau=0.1)
    
    load(paste("trees/model",i,"_0.5.Rdata",sep=""))
    pred3[i,5] <- loss(indiaTest$stunting, pred[indiaTestInd==0], tau=0.5)
    
    
}


save(pred1, pred2, pred3, file="CVRisks.Rdata")
save(predAdditive1, predAdditive2, predAdditive3, file="PredAdditive.Rdata")
save(predVCM1, predVCM2, predVCM3, file="PredVCM.Rdata")

#load("CVRisks.Rdata")

# Change the indices according to performance
pred1 <- pred1[,c(2,1,5,4,3)]
pred2 <- pred2[,c(2,1,5,4,3)]
pred3 <- pred3[,c(2,1,5,4,3)]



#-----------------------------------------------------------
# Boxplots for Empirical Risks 
#-----------------------------------------------------------

#pdf("IndiaRisksAll.pdf", width=8, height=6.5)

cv <- data.frame(error = c(as.vector(pred1),
                           as.vector(pred2),
                           as.vector(pred3)),
                 algo = rep(factor(colnames(pred1), levels = colnames(pred1),
                                   labels = c("additive", "rqss", "trees", "stumps", "VCM")), 
                            rep(nrow(pred1), ncol(pred1))),
                 quant = rep(factor(c("5% Quantile", "10% Quantile", "Median"),
                                    levels = c("5% Quantile", "10% Quantile", "Median"),
                                    labels = c("5% Quantile", "10% Quantile", "Median")),
                             rep(length(pred1), 3)))

mypanel <- function(x, y, ...) {
    panel.bwplot(x = x, y = y, ...)
    z <- as.data.frame(matrix(y, ncol = 5))
    apply(z, 1, function(x)
        llines(x = 1:5, y = x, col = rgb(0.1, 0.1, 0.1, 0.1)))
}
print(bwplot(error ~ algo | quant, data = cv, ylab = "CV risk", 
             scales = list(y = list(relation = "sliced"), x = list(rot = 25)), 
             layout = c(3, 1), panel = mypanel, fill="grey", pch = "|",
             par.settings = list(plot.symbol = list(col=1,pch=20, cex=0.7),
                                 box.rectangle = list(col=1),
                                 box.umbrella = list(lty=1, col=1)),
             strip= strip.custom(bg="white")))
#dev.off()


#-----------------------------------------------------------
# Boxplots for Optimized Lambda Parameters rqss
#-----------------------------------------------------------

lims <- range(c(lambda1rqss, lambda2rqss, lambda3rqss))

#pdf("IndiaLambdas.pdf", width=8, height=6)
par(mfrow=c(1,3), las=3, mar=c(8,4,2,1))
boxplot(lambda1rqss[,1:6],axes=FALSE, ylab="lambda", xlab="",  main="0.05", ylim=range(lims))
box()
axis(2)
axis(1, at=1:6, labels=c("cage", "breastfeeding", "mbmi", "mage", "medu", "edupartner"))

boxplot(lambda2rqss[,1:6],axes=FALSE, ylab="lambda", xlab="",  main="0.1", ylim=range(lims))
box()
axis(2)
axis(1, at=1:6, labels=c("cage", "breastfeeding", "mbmi", "mage", "medu", "edupartner"))

boxplot(lambda3rqss[,1:6],axes=FALSE, ylab="lambda", xlab="",  main="0.5", ylim=range(lims))
box()
axis(2)
axis(1, at=1:6, labels=c("cage", "breastfeeding", "mbmi", "mage", "medu", "edupartner"))

#dev.off()


# Number of observations in the test data
#pdf("IndiaNTest.pdf")
boxplot(nTest)
#dev.off()

print(summary(nTest))


#-----------------------------------------------------------
# Boxplots for empirical risks without rqss
#-----------------------------------------------------------

load("PredAdditive.Rdata")
load("PredVCM.Rdata")

inds <- paste("w", 1:50, sep="")
quantiles <- c(0.05, 0.1, 0.5)

pred1 <- pred2 <- pred3 <- matrix(0, ncol=4, nrow=length(inds))
colnames(pred1) <- colnames(pred2) <- colnames(pred3) <- c("add","vcm","stump","tree")

for(i in 1:length(inds)){
    print(i)
    
    indiaTrain <- india[india[,inds[i]]==1,]
    
    indiaTest <- india[india[,inds[i]]==3,] 
    
    indiaTest <- cbind(indiaTest, ff = 0)
    names(indiaTest)[ncol(indiaTest)] <- "FALSE"
    
    #---------------
    # Prediction additive
    pred1[i,1] <- loss(indiaTest$stunting, predAdditive1[,i], tau=0.05)
    pred2[i,1] <- loss(indiaTest$stunting, predAdditive2[,i], tau=0.1)   
    pred3[i,1] <- loss(indiaTest$stunting, predAdditive3[,i], tau=0.5)
    
    #---------------
    # Prediction VCM
    pred1[i,2] <- loss(indiaTest$stunting, predVCM1[,i], tau=0.05)
    pred2[i,2] <- loss(indiaTest$stunting, predVCM2[,i], tau=0.1)
    pred3[i,2] <- loss(indiaTest$stunting, predVCM3[,i], tau=0.5)
    
    #---------------
    # Prediction Stumps
    load(paste("stumps/model",i,"_0.05.Rdata",sep=""))
    pred1[i,3] <- loss(indiaTest$stunting, pred, tau=0.05)
    
    load(paste("stumps/model",i,"_0.1.Rdata",sep=""))
    pred2[i,3] <- loss(indiaTest$stunting, pred, tau=0.1)
    
    load(paste("stumps/model",i,"_0.5.Rdata",sep=""))
    pred3[i,3] <- loss(indiaTest$stunting, pred, tau=0.5)
    
    load(paste("trees/model",i,"_0.05.Rdata",sep=""))
    pred1[i,4] <- loss(indiaTest$stunting, pred, tau=0.05)
    
    load(paste("trees/model",i,"_0.1.Rdata",sep=""))
    pred2[i,4] <- loss(indiaTest$stunting, pred, tau=0.1)
    
    load(paste("trees/model",i,"_0.5.Rdata",sep=""))
    pred3[i,4] <- loss(indiaTest$stunting, pred, tau=0.5)
    
    
}

save(pred1, pred2, pred3, file="CVRisksTotal.Rdata")

pred1 <- pred1[,c(1,4,3,2)]
pred2 <- pred2[,c(1,4,3,2)]
pred3 <- pred3[,c(1,4,3,2)]

#pdf("IndiaRisksWithoutRqss.pdf", width=8, height=6.5)

cv <- data.frame(error = c(as.vector(pred1),
                           as.vector(pred2),
                           as.vector(pred3)),
                 algo = rep(factor(colnames(pred1), levels = colnames(pred1),
                                   labels = c("additive", "trees", "stumps", "VCM")), 
                            rep(nrow(pred1), ncol(pred1))),
                 quant = rep(factor(c("5% Quantile", "10% Quantile", "Median"),
                                    levels = c("5% Quantile", "10% Quantile", "Median"),
                                    labels = c("5% Quantile", "10% Quantile", "Median")),
                             rep(length(pred1), 3)))

mypanel <- function(x, y, ...) {
    panel.bwplot(x = x, y = y, ...)
    z <- as.data.frame(matrix(y, ncol = 4))
    apply(z, 1, function(x)
        llines(x = 1:4, y = x, col = rgb(0.1, 0.1, 0.1, 0.1)))
}
print(bwplot(error ~ algo | quant, data = cv, ylab = "CV risk", 
             scales = list(y = list(relation = "sliced"), x = list(rot = 25)), 
             layout = c(3, 1), panel = mypanel, fill="grey", pch = "|",
             par.settings = list(plot.symbol = list(col=1,pch=20, cex=0.7),
                                 box.rectangle = list(col=1),
                                 box.umbrella = list(lty=1, col=1)),
             strip= strip.custom(bg="white")))
#dev.off()

### R code from vignette source 'mboost_tutorial.Rnw'

###################################################
### code chunk number 1: init
###################################################
## set up R session
options(prompt = "R> ", continue = "+  ", width = 80)
pd <- packageDescription("mboost")
if (any(is.na(pd))){
    install.packages("mboost", repos = "http://cran.at.r-project.org")
    pd <- packageDescription("mboost")
}
if (compareVersion(pd$Version, "2.1-0") < 0){ # must be mboost 2.1-X or newer
    warning("Current version of mboost is installed!")
    install.packages("mboost", repos = "http://cran.at.r-project.org")
}
options(digits = 3)
require("mboost")
set.seed(190781)
### make graphics directory if not existing
if (!file.exists("graphics"))
    dir.create("graphics")


###################################################
### code chunk number 2: setup
###################################################
library("mboost")                         ## load package
data("bodyfat", package = "TH.data")      ## load data


###################################################
### code chunk number 3: mod_garcia
###################################################
## Reproduce formula of Garcia et al., 2005
lm1 <- lm(DEXfat ~ hipcirc  + kneebreadth + anthro3a, data = bodyfat)
coef(lm1)


###################################################
### code chunk number 4: mod_garcia_boosted
###################################################
## Estimate same model by glmboost
glm1 <- glmboost(DEXfat ~ hipcirc  + kneebreadth + anthro3a, data = bodyfat)
coef(glm1, off2int=TRUE)    ## off2int adds the offset to the intercept


###################################################
### code chunk number 5: mod_glmboost
###################################################
glm2 <- glmboost(DEXfat ~ ., data = bodyfat)


###################################################
### code chunk number 6: mod_glmboost2
###################################################
preds <- names(bodyfat[, names(bodyfat) != "DEXfat"])        ## names of predictors
fm <- as.formula(paste("DEXfat ~", paste(preds, collapse = "+")))  ## build formula
fm


###################################################
### code chunk number 7: coef_glmboost
###################################################
coef(glm2,        ## usually the argument 'which' is used to specify single base-
     which = "")  ## learners via partial matching; With which = "" we select all.


###################################################
### code chunk number 8: plot_glmboost
###################################################
plot(glm2, off2int = TRUE)    ## default plot, offset added to intercept
## now change ylim to the range of the coefficients without intercept (zoom-in)
plot(glm2, ylim = range(coef(glm2, which = preds)))


###################################################
### code chunk number 9: coefsplot1
###################################################
## same as first plot from chunk "plot_glmboost" but with better margins
par(mar = c(4, 4, 0, 6) + 0.1)
plot(glm2, main="", off2int=TRUE)


###################################################
### code chunk number 10: coefsplot2
###################################################
## same as second plot from chunk "plot_glmboost" but with better margins
par(mar = c(4, 4, 0, 6) + 0.1)
plot(glm2, ylim = range(coef(glm2, which = preds)), main = "")


###################################################
### code chunk number 11: mod_gamboost
###################################################
## now an additive model with the same variables as lm1
gam1 <- gamboost(DEXfat ~ bbs(hipcirc) + bbs(kneebreadth) + bbs(anthro3a),
                 data = bodyfat)


###################################################
### code chunk number 12: plot_gamboost
###################################################
par(mfrow = c(1,3))    ## 3 plots in one device
plot(gam1)             ## get the partial effects


###################################################
### code chunk number 13: partialplots
###################################################
## same as plot from chunk "plot_gamboost" but with better margins
par(mfrow=c(1,3))
par(mar = c(4, 4, 0, 0) + 0.1)
plot(gam1)


###################################################
### code chunk number 14: mod_gamboost2
###################################################
## every predictor enters the model via a bbs() base-learner (i.e., as smooth effect)
gam2 <- gamboost(DEXfat ~ ., baselearner = "bbs", data = bodyfat,
                 control = boost_control(trace = TRUE))


###################################################
### code chunk number 15: mod_gamboost2_refit
###################################################
## refit model to supress trace in cvrisk
## this is only required for the output in the PDF
gam2 <- gamboost(DEXfat ~ ., baselearner = "bbs", data = bodyfat)


###################################################
### code chunk number 16: mod_gamboost2_ctd
###################################################
set.seed(123)          ## set seed to make results reproducible
cvm <- cvrisk(gam2)    ## default method is 25-fold bootstrap cross-validation
## if package 'multicore' is not available this will trigger a warning
cvm
plot(cvm)    ## get the paths


###################################################
### code chunk number 17: cvpaths
###################################################
## same as plot from chunk "mod_gamboost2_ctd"with better margins
par(mar = c(4, 4, 0, 0) + 0.1)
plot(cvm, main = "")


###################################################
### code chunk number 18: mstop
###################################################
mstop(cvm)    ## extract the optimal mstop


###################################################
### code chunk number 19: mstop_ctd
###################################################
gam2[ mstop(cvm) ]    ## set the model automatically to the optimal mstop


###################################################
### code chunk number 20: mstop_save
###################################################
m_cvm <- mstop(cvm)


###################################################
### code chunk number 21: mod_gamboost2_refit2
###################################################
## refit model to get trace again
## this is only required for the output in the PDF
gam2 <- gamboost(DEXfat ~ ., baselearner = "bbs", data = bodyfat,
                 control = boost_control(trace = TRUE))
gam2[ mstop(cvm) ]


###################################################
### code chunk number 22: coef_gamboost_save
###################################################
n_coef <- length(coef(gam2))
coef_string <-
    paste(c("zero", "one", "two", "three", "four",
            "five", "six", "seven", "eight", "nine")[n_coef + 1],
          if (n_coef == 1) "predictor" else "predictors")


###################################################
### code chunk number 23: coef_gamboost
###################################################
names(coef(gam2)) ## displays the selected base-learners at iteration "mstop(cvm)"
## To see that nothing got lost we now increase mstop to 1000:
gam2[1000, return = FALSE]  # return = FALSE just supresses "print(gam2)"


###################################################
### code chunk number 24: coef_gamboost2
###################################################
names(coef(gam2))  ## displays the selected base-learners, now at iteration 1000


###################################################
### code chunk number 25: quantreg_gamboost
###################################################
## Same model as glm1 but now with QuantReg() family
glm3 <- glmboost(DEXfat ~ hipcirc + kneebreadth + anthro3a, data = bodyfat,
                 family = QuantReg(tau = 0.5), control = boost_control(mstop = 500))
coef(glm3, off2int = TRUE)


###################################################
### code chunk number 26: loss
###################################################
loss =  function(y, f) tau * (y - f) * ((y - f) >= 0) +
                       (tau - 1) * (y - f) * ((y - f) < 0)


###################################################
### code chunk number 27: ngradient
###################################################
ngradient = function(y, f, w = NULL)  tau * ((y - f) >= 0) +
                    (tau- 1) * ((y - f) < 0)


###################################################
### code chunk number 28: OurQR
###################################################
OurQuantReg <- function(tau = 0.5){    ## function to include dependency on tau
  Family(                              ## applying the Family function
         loss = function(y, f)                               ## loss as above
                    tau       * (y - f) * ((y - f) >= 0) +
                    (tau - 1) * (y - f) * ((y - f) < 0) ,
         ngradient = function(y, f, w = NULL)                ## ngradient as above
                    tau * ((y - f) >= 0) + (tau - 1) * ((y - f) < 0),
         offset = function(y, w = NULL)                      ## median as offset
                    quantile(y, p = 0.5),
         name = "Our new family for quantile regression" )}
OurQuantReg()


###################################################
### code chunk number 29: ourQR_gamboost
###################################################
## Same model as glm3 but now with our new family
glm3b <- glmboost(DEXfat ~ hipcirc + kneebreadth + anthro3a, data = bodyfat,
                  family = OurQuantReg(tau = 0.5),
                  control = boost_control(mstop = 500))
identical(coef(glm3b), coef(glm3))


###################################################
### code chunk number 30: QR_hipcirc
###################################################
glm4a <- glmboost(DEXfat ~ hipcirc, family = OurQuantReg(tau = 0.05), data = bodyfat,
                  control = boost_control(mstop = 2000))
glm4b <- glmboost(DEXfat ~ hipcirc, family = OurQuantReg(tau = 0.5),  data = bodyfat,
                  control = boost_control(mstop = 2000))
glm4c <- glmboost(DEXfat ~ hipcirc, family = OurQuantReg(tau = 0.95), data = bodyfat,
                  control = boost_control(mstop = 2000))


###################################################
### code chunk number 31: plot_QR_hipcirc
###################################################
ord <- order(bodyfat$hipcirc)    ## order the data to avoid problems when plotting
plot(bodyfat$hipcirc[ord], bodyfat$DEXfat[ord])                   ## observed data
lines(bodyfat$hipcirc[ord], fitted(glm4a)[ord], lty = 2, lwd = 2) ## 0.05 quantile
lines(bodyfat$hipcirc[ord], fitted(glm4b)[ord], lty = 1, lwd = 2) ## median
lines(bodyfat$hipcirc[ord], fitted(glm4c)[ord], lty = 2, lwd = 2) ## 0.95 quantile


###################################################
### code chunk number 32: quantregbodyfat
###################################################
## same plot but with better margings and parameters
par(mar = c(4, 4, 0, 0) + 0.1)
plot(bodyfat$hipcirc[ord], bodyfat$DEXfat[ord])
lines(bodyfat$hipcirc[ord], fitted(glm4a)[ord], lty=2,lwd=2)
lines(bodyfat$hipcirc[ord], fitted(glm4b)[ord], lty=1, lwd=2)
lines(bodyfat$hipcirc[ord], fitted(glm4c)[ord], lty=2, lwd=2)


###################################################
### code chunk number 33: init_figs
###################################################
################################################################################
## the following chunks produce the graphics that are NOT part of the bodyfat ##
## example                                                                    ##
################################################################################


###################################################
### code chunk number 34: init
###################################################
red <- rgb(103,0,31, max = 255) ## define red color


###################################################
### code chunk number 35: center_false
###################################################
## load library mboost
library("mboost")
library("RColorBrewer")
## define colors
cols <- paste(brewer.pal(11,"RdBu")[c(10,2)], "E6", sep="")

## create graphics to show importance of centering
set.seed(1907)
x <- rnorm(50, mean = 5)
y <- rnorm(50, mean = x, sd = 0.3)
## subtract offset as usual in boosting
y <- y - mean(y)

par(ps = 8, cex=1, cex.lab=1, mar=c(3.1,3,0.5,0.1), mgp=c(2,1,0))
xrange <- range(0,x)
plot(y ~ x, xlim = xrange, cex = 1,
     xlab = "x", ylab = "negative gradient in step m = 1",
     pch = 20,
     col = rgb(0.5, 0.5, 0.5, alpha = 0.8))
abline(h = 0, col = "gray", lwd = 0.5)
abline(v = 0, col = "gray", lwd = 0.5)
abline(lm(y ~ x -1), col = cols[1], lwd = 1.5)
points(0, 0, col = cols[2], lwd = 1.5)
points(mean(x), mean(y), col = cols[2], pch = 3, lwd = 1.5)
legend(0.1, 2.35,  legend = c("origin", "center of data", "base-learner"),
       pch = c(21, 3, -1), lty = c(-1, -1, 1),
       col = c(cols[2], cols[2], cols[1]), lwd = 1.5, bg = "white", bty = "n")


###################################################
### code chunk number 36: center_true
###################################################
## create graphics to show importance of centering
set.seed(1907)
x <- rnorm(50, mean = 5)
y <- rnorm(50, mean = x, sd = 0.3)
## subtract offset as usual in boosting
y <- y - mean(y)

## figure with centering of x
mx <- mean(x)
xrange <- range(0,x)
x <- x - mean(x)

par(ps = 8, cex=1, cex.lab=1, mar=c(3.1,3,0.5,0.1), mgp=c(2,1,0))
plot(y ~ x, xlim = xrange - mx, cex = 1,
     xlab = "x (centered)", ylab = "negative gradient in step m = 1",
     pch = 20,
     col = rgb(0.5, 0.5, 0.5, alpha = 0.8))
abline(h = 0, col = "gray", lwd = 0.5)
abline(v = 0, col = "gray", lwd = 0.5)
abline(lm(y ~ x -1), col = cols[1], lwd = 1.5)
points(0, 0, col = cols[2], lwd = 1.5)
points(mean(x), mean(y), col = cols[2], pch = 3, lwd = 1.5)


###################################################
### code chunk number 37: bolsx1
###################################################
### Simulate some data
set.seed(1907)
n <- 100
x1 <- rnorm(n)
x2 <- as.factor(sample(0:3, n, replace = TRUE))
y <- 0.5 * x1 +  rnorm(n)
mod <- gamboost(y ~ bols(x1), control = boost_control(mstop = 25))
par(mar = c(4, 4, 0, 0) + 0.1)
plot(sort(x1), (0.5 * x1)[order(x1)], type = "l", lwd = 2,
     xlab = expression(x[1]), ylab = expression(f(x[1])))
lines(sort(x1), fitted(mod, which = 1)[order(x1)], col = red,
      lwd = 2, type = "b", pch = 20)
legend("topleft", c("true effect", "model"),
       lty = c(1, 1), pch = c(-1, 20), merge = TRUE,
       lwd = 2,
       col = c("black", red), bty = "n")


###################################################
### code chunk number 38: bolsx2
###################################################
beta <- c(0, -1, 0.5, 3)
y <- drop(model.matrix(~ x2) %*% beta + rnorm(n, sd = 0.3))
mod <- gamboost(y ~ bols(x2), control = boost_control(mstop = 50))
par(mar = c(4, 4, 0, 0) + 0.1)
betaPred <- coef(mod)[[1]]
betaPred[1] <- betaPred[1] + attr(coef(mod), "offset")
betaTrue <- c(0, -1, 0.5, 3)
plot(1:4, betaPred, type = "n", xaxt = "n",
     xlab = expression(x[2]), ylab = expression(f(x[2])),
     xlim = c(0.5, 4.5), ylim = c(-1.5, 3.5))
axis(1, at = 1:4, labels = expression(x[2]^(1), x[2]^(2), x[2]^(3), x[2]^(4)))
for (i in 1:4)
    lines(i + c(-0.38, 0, 0.38), rep(betaPred[i], each = 3),
          lwd = 2, col = red, type = "b", pch = 20)
for (i in 1:4)
    lines(i + c(-0.4, 0.4), rep(betaTrue[i], each = 2),
          lwd = 2, col = "black")
legend("topleft", c("true effect", "model"),
       pch = c(-1,20), lty = c(1, 1), lwd = 2,
       col = c("black", red), bty = "n")


###################################################
### code chunk number 39: bbsx1
###################################################
set.seed(1907)
n <- 100
x1 <- rnorm(n)
x2 <- rnorm(n) + 0.25 * x1
x3 <- as.factor(sample(0:1, n, replace = TRUE))
x4 <- gl(4, 25)
y <- 3 * sin(x1) + x2^2 + rnorm(n)

## specify knot locations for bbs(x2, ...)
knots.x2 <- quantile(x2, c(0.25, 0.5, 0.75))

## fit the model
mod <- gamboost(y ~ bbs(x1, knots = 20, df = 4) +
                bbs(x2, knots = knots.x2, df = 5) +
                bols(x3) + bols(x4))

## plot partial effects
par(mar = c(4, 4, 0, 0) + 0.1)
plot(sort(x1), (3 * sin(x1))[order(x1)], type = "l", lwd = 2,
     xlab = expression(x[1]), ylab =  expression(f[1](x[1])))
lines(sort(x1), fitted(mod, which = 1)[order(x1)], col = red, lwd = 2,
      type = "b", pch = 20)
legend("topleft", c("true effect", "model"),
       lty = c(1, 1), pch = c(-1,20), merge = TRUE, lwd = 2,
       col = c("black", red), bty = "n")


###################################################
### code chunk number 40: bbsx2
###################################################
par(mar = c(4, 4, 0, 0) + 0.1)
plot(sort(x2), (x2^2)[order(x2)], type = "l", lwd = 2,
     xlab = expression(x[2]), ylab =  expression(f[2](x[2])))
## offset needed to conserve correct level (otherwise center both effects around
## zero to make them comparable)
lines(sort(x2), fitted(mod, which = 2)[order(x2)] + mod$offset, col = red, lwd = 2,
      type = "b", pch = 20)


###################################################
### code chunk number 41: cyclic1
###################################################
### example for cyclic spline
set.seed(1907)
true_f <- function(x)
    cos(x) + 0.25 * sin(4 * x)
x <- runif(150, 0, 2 * pi)
y <- rnorm(150, mean = true_f(x), sd=0.1)
newX <- seq(0, 2*pi, length = 200)
mod <- gamboost(y ~ bbs(x, knots = 20, degree = 4))
mod[3000]
mod_cyclic <- gamboost(y ~ bbs(x, cyclic=TRUE, knots = 20,
                               degree = 4, boundary.knots=c(0, 2*pi)))
mod_cyclic[3000]

par(mar = c(4, 4, 0, 0) + 0.1)
plot(x,y, ylab = "f(x)",
     pch = 20, xlim = c(0, 7),
     col = rgb(0.5, 0.5, 0.5, alpha = 0.8))
lines(newX, predict(mod, data.frame(x = newX)),
      col = "black", lwd = 2)
lines(newX + 2 * pi, predict(mod, data.frame(x = newX)),
      col = "black", lwd = 2)
legend("bottomleft", c("cyclic = FALSE"),
       lty = c(1), col = c("black"), lwd = 2, bty = "n")


###################################################
### code chunk number 42: cyclic2
###################################################
par(mar = c(4, 4, 0, 0) + 0.1)
plot(x, y, ylab = "f(x)",
     pch = 20, xlim = c(0, 7),
     col = rgb(0.5, 0.5, 0.5, alpha = 0.8))
lines(newX, predict(mod_cyclic, data.frame(x = newX)),
      col = red, lwd = 2)
lines(newX + 2 * pi, predict(mod_cyclic, data.frame(x = newX)),
      col = red, lwd = 2)
abline(v = 2 * pi, col = "gray", lty = "dashed", lwd = 2)
legend("bottomleft", c("cyclic = TRUE"),
       lty = c(1), col = c(red), lwd = 2, bty = "n")


###################################################
### code chunk number 43: bspatial1
###################################################
set.seed(1907)
x1 <- runif(250,-pi,pi)
x2 <- runif(250,-pi,pi)
y <- sin(x1) * sin(x2) + rnorm(250, sd = 0.4)
modspa <- gamboost(y ~ bspatial(x1, x2, knots = list(x1 = 12, x2 = 12)))
# possible to specify different knot mesh for x1 and x2:
# modspa <- gamboost(y ~ bspatial(x1, x2, knots = list(x1 = 12, x2 = 20)))

library(lattice)
library(RColorBrewer)
## set up colorscheme
nCols <- 50
cols <- colorRampPalette(rev(brewer.pal(11, "RdBu")), space = "Lab",
                         interpolate = "spline")(nCols)
## make new data on a grid:
nd <- expand.grid(x1 = seq(-pi, pi, length = 100),
                  x2 = seq(-pi, pi, length = 100))
preds <- predict(modspa, newdata = nd)
breaks <- seq(min(preds), max(preds), length = nCols + 1)
print(levelplot(preds ~ nd$x1 + nd$x2,
          xlab = expression(x[1]),
          ylab = expression(x[2]),
          at = breaks,
          col.regions = cols, cex = 0.7,
          colorkey = list(space = "left", at = breaks, width = 1)))


###################################################
### code chunk number 44: bspatial2
###################################################
x1 <- x2 <- seq(-pi, pi, length = 50)
nd <- expand.grid(x1 = x1,
                  x2 = x2)
preds <- predict(modspa, newdata = nd)
z <- matrix(preds, ncol = length(x1))
nrz <- nrow(z)
ncz <- ncol(z)
jet.colors <- colorRampPalette(paste(brewer.pal(11,"RdBu")[c(10,2)],
                                     "E6", sep=""))
nbcol <- 10
color <- jet.colors(nbcol)
zfacet <- z[-1, -1] + z[-1, -ncz] + z[-nrz, -1] + z[-nrz, -ncz]
facetcol <- cut(zfacet, nbcol)
par(mar=c(1, 0.1, 0.1, 0.1), mgp = c(1.8,1,0))
persp(x1, x2, z,
      theta = 45, phi = 30, expand = 0.5,
      col = color[facetcol], ticktype = "detailed",
      nticks = 3,
      xlab = "x1", ylab = "x2")


###################################################
### code chunk number 45: families
###################################################
pdf("./graphics/fig-family.pdf", width = 5, height = 4)
par(mar = c(4, 4, 0, 0) + 0.1)
## Gaussian
x <- seq(-2, 2, length = 200)
plot(x,x^2, type = "l", ylab = expression(rho), xlab = expression(y-f))

## Laplace
x <- seq(-2, 2, length = 200)
lossL1 <- function(x, param) abs(x)
mp <- 0.5
dat <- data.frame(x = rep(x, length(mp)),
                  y = as.numeric(sapply(mp, function(z) lossL1(x, param = z))),
                  param = rep(mp, each = length(x)))
dat$tau <- factor(dat$param)
plot(x, dat$y, type = "l", ylab = expression(rho), xlab = expression(y-f))

## Huber
x <- seq(-2, 2, length = 200)
lossH <- function(x, param) ifelse(abs(x) <= param, (1/2) * x^2, param * (abs(x) - param / 2))
mp <- c(0.2, 0.5, 1, 2, 10)
dat <- data.frame(x = x, sapply(mp, function(z) lossH(x, param = z)))
plot(x, dat$X1, type = "l", ylim = c(0,2), ylab = expression(rho),
     xlab = expression(y-f))
legend(0, 2, xjust = 0.5, legend = paste(mp), title = expression(delta),
       lty = 1, col = c(1,3,4,5,6), cex = 0.6, box.col = "gray")
for(i in 1:(length(mp)-1)){
    lines(x, dat[,i+2], col = i+2)
}

## Quantile
x <- seq(-2, 2, length = 200)
lossL1 <- function(x, param) ifelse(x >= 0, param  * x, (param - 1) * x)
mp <- c(5:9 / 10)
dat <- data.frame(x = x, sapply(mp, function(z) lossL1(x, param = z)))
plot(x, dat$X1, type = "l", ylab = expression(rho), ylim = c(0,2),
     xlab = expression(y-f))
legend(0,2, xjust = 0.5, legend = paste(mp), title = expression(tau),
       lty = 1, col = c(1,3,4,5,6), cex = 0.6, box.col = "gray")
for(i in 1:(length(mp)-1)){
    lines(x, dat[,i+2], col = i+2)
}

## Binary
x <- seq(-2, 2, length = 200)
dat <- data.frame(x = x, Binomial = log(1+exp(-2*x), base=2), AdaExp = exp(-x))
plot(x, dat$Binomial, type = "l", lty = 1, col = 1, xlab = expression(tilde(y) * f),
     ylab = expression(rho), ylim = c(0,7.5))
lines(x, dat$AdaExp, lty = 2, col = 2)
legend("topright", legend = c("Binomial","AdaExp"), xjust = 0.5, title = "loss",
       lty = 1:2, col = 1:2, cex = 0.8, bty = "n")
dev.off()

### R code from vignette source 'SurvivalEnsembles.Rnw'

###################################################
### code chunk number 1: setup
###################################################
source("setup.R")
if (!require("TH.data"))
    stop("cannot attach package ", sQuote("TH.data"))
if (!require("rpart"))
    stop("cannot attach package ", sQuote("rpart"))
if (!require("survival"))
    stop("cannot attach package ", sQuote("survival"))
if (!require("partykit"))
    stop("cannot attach package ", sQuote("partykit"))

set.seed(290875)
CEX <- 0.85
options(digits = 3)

### mean difference plots
mdplot <- function(obs, pred, main = "", ...) {

    m <- (obs + pred)/2
    d <- obs - pred
    plot(m, d, xlab = "(Observed + Predicted) / 2",
         ylab = "Observed - Predicted", main =
         main, cex.axis = CEX, cex.main = CEX, cex.lab = CEX, ...)
    abline(h = 0, lty = 3)
}


###################################################
### code chunk number 2: loaddata
###################################################
### load data. See `mboost/inst/readAML_Bullinger.R' for
### how the data were generated from the raw data.
load(file.path(path.package(package = "TH.data"), "rda", "AML_Bullinger.rda"))


###################################################
### code chunk number 3: AML-dpp
###################################################
### compute IPC weights
AMLw <- IPCweights(Surv(clinical$time, clinical$event))

### risk score
risk <- rep(0, nrow(clinical))
rlev <- levels(clinical[, "Cytogenetic.group"])
risk[clinical[, "Cytogenetic.group"] %in% rlev[c(7,8,4)]] <- "low"
risk[clinical[, "Cytogenetic.group"] %in% rlev[c(5, 9)]] <- "intermediate"
risk[clinical[, "Cytogenetic.group"] %in% rlev[-c(4,5, 7,8,9)]] <- "high"
risk <- as.factor(risk)

### set-up learning sample
AMLlearn <- cbind(clinical[, c("time", "Sex", "Age", "LDH", "WBC",
                        "FLT3.aberration.", "MLL.PTD", "Tx.Group.")],
               risk = risk,
               iexpressions[, colnames(iexpressions) %in% selgenes[["Clone.ID"]]])
cc <- complete.cases(AMLlearn)
AMLlearn <- AMLlearn[AMLw > 0 & cc,]
AMLw <- AMLw[AMLw > 0 & cc]


###################################################
### code chunk number 4: AML-RF
###################################################
### controls for tree growing
ctrl <- ctree_control(testtype = "Teststatistic", 
                      teststat = "maximum", mincriterion = .1, minsplit = 5)
### was: cforest_control(mincriterion = 0.1, mtry = 5, minsplit = 5, ntree = 250)

### fit random forest for censored data (warnings are OK here)
AMLrf <- cforest(log(time) ~ ., data = AMLlearn, control = ctrl,
                 weights = AMLw, mtry = 5, ntree = 250, 
                 perturb = list(replace = TRUE, fraction = 0.632))


###################################################
### code chunk number 5: AML-boost
###################################################
AMLl2b <- glmboost(I(log(time)) ~ ., data = AMLlearn, weights = AMLw,
                    control = boost_control(mstop = 5000))


###################################################
### code chunk number 6: AML-AIC
###################################################
### AIC criterion
plot(aic <- AIC(AMLl2b))


###################################################
### code chunk number 7: AML-fitted
###################################################
### restrict number of boosting iterations and inspect selected variables
AMLl2b <- AMLl2b[mstop(aic)]
cAML <- coef(AMLl2b)
cAML[abs(cAML) > 0]

### fitted values
AMLprf <- predict(AMLrf, newdata = AMLlearn)
AMLpb <- predict(AMLl2b, newdata = AMLlearn)


###################################################
### code chunk number 8: Figure1
###################################################
Mmod <- sum(AMLw * log(AMLlearn$time))/sum(AMLw )
par(mai = par("mai") * c(0.7, 0.8, 0.7, 0.6))
layout(matrix(1:4, ncol = 2))

mdplot(log(AMLlearn$time), AMLprf, main = "Random Forest",
       cex = AMLw / 4, ylim = c(-4, 4), xlim = c(0, 7))

plot(log(AMLlearn$time), AMLprf, cex = AMLw / 4,
       ylim = range(log(AMLlearn$time)), ylab = "Predicted", xlab = "Observed",
       main = "Random Forest",  cex.axis = CEX, cex.main = CEX, cex.lab = CEX)
abline(h = Mmod, lty = 2)

mdplot(log(AMLlearn$time), AMLpb,
        cex = AMLw / 4, main = "Boosting", ylim = c(-4, 4), xlim = c(0, 7))

plot(log(AMLlearn$time), AMLpb, cex = AMLw / 4,
       ylim = range(log(AMLlearn$time)), ylab = "Predicted", xlab = "Observed",
       main = "Boosting",  cex.axis = CEX, cex.main = CEX, cex.lab = CEX)
abline(h = Mmod, lty = 2)


###################################################
### code chunk number 9: GBSG2-dpp
###################################################
### attach data
data("GBSG2", package = "TH.data")

### IPC weights
GBSG2w <- IPCweights(Surv(GBSG2$time, GBSG2$cens))

### set-up learning sample
GBSG2learn <- cbind(GBSG2[,-which(names(GBSG2) %in% c("time", "cens"))],
               ltime = log(GBSG2$time))
n <- nrow(GBSG2learn)


###################################################
### code chunk number 10: GBSG2-models
###################################################
### linear model
LMmod <- lm(ltime ~ . , data = GBSG2learn, weights = GBSG2w)
LMerisk <- sum((GBSG2learn$ltime - predict(LMmod))^2*GBSG2w) / n

### regression tree
pos <- GBSG2w > 0
TRmod <- rpart(ltime ~ . , data = GBSG2learn, weights = GBSG2w, 
               subset = pos)
TRerisk <- sum((GBSG2learn$ltime[pos] - predict(TRmod))^2*GBSG2w[pos]) / n

### tree controls
ctrl <- ctree_control(testtype = "Teststatistic", 
                      teststat = "maximum", mincriterion = qnorm(.95), 
                      minsplit = 5)
### was: cforest_control(mincriterion = qnorm(0.95), mtry = 5,
###                      minsplit = 5, ntree = 100)


### fit random forest for censored data (warnings are OK here)
RFmod <- cforest(ltime ~ . , data = GBSG2learn, weights = GBSG2w,
                 control = ctrl, mtry = 5, ntree = 100, 
                 perturb = list(replace = TRUE, 
                     fraction = 0.632 * sum(GBSG2w > 0)))

### fit L2 boosting for censored data
L2Bmod <- glmboost(ltime ~ ., data = GBSG2learn, weights = GBSG2w,
                   control = boost_control(mstop = 250))

### with Huber loss function
L2BHubermod <- glmboost(ltime ~ ., data = GBSG2learn, weights = GBSG2w,
                        family = Huber(d = log(2)))


###################################################
### code chunk number 11: GBSG2-AIC
###################################################
plot(aic <- AIC(L2Bmod))


###################################################
### code chunk number 12: GBSG2-fitted
###################################################
GBSG2Hp <- predict(L2BHubermod, newdata = GBSG2learn)
L2Berisk <- sum((GBSG2learn$ltime - predict(L2Bmod, newdata = GBSG2learn))^2*GBSG2w) / n
RFerisk <- sum((GBSG2learn$ltime - predict(RFmod, newdata = GBSG2learn))^2*GBSG2w) / n


###################################################
### code chunk number 13: Figure3
###################################################
lim <- c(4,9)
mylwd <- 0.5
par(mai = par("mai") * c(0.7, 0.8, 0.7, 0.6))

layout(matrix(1:4, ncol = 2))
Mmod <- sum(GBSG2w * GBSG2learn$ltime)/sum(GBSG2w)

mdplot(GBSG2learn$ltime, predict(LMmod), cex = GBSG2w / 4, main = "Linear Model",
       ylim = c(-3, 3), xlim = c(5, 8))
mdplot(GBSG2learn$ltime[pos], predict(TRmod), cex = GBSG2w / 4, main = "Tree",
       ylim = c(-3, 3), xlim = c(5, 8))
mdplot(GBSG2learn$ltime, predict(RFmod, newdata = GBSG2learn), cex = GBSG2w / 4,
       main = "Random Forest", ylim = c(-3, 3), xlim = c(5, 8))
mdplot(GBSG2learn$ltime, predict(L2Bmod, newdata = GBSG2learn), cex = GBSG2w / 4,
       main = "Boosting", ylim = c(-3, 3), xlim = c(5, 8))


###################################################
### code chunk number 14: Figure5
###################################################
RFpr <- predict(RFmod, newdata = GBSG2learn)
L2Bpr <- predict(L2Bmod, newdata = GBSG2learn)
ylim <- range(c(RFpr[GBSG2w > 0], L2Bpr[GBSG2w > 0]))
mydf <- 4
par(mai = par("mai") * c(0.7, 0.8, 0.4, 0.6))
layout(matrix(1:4, ncol = 2))
plot(GBSG2learn$pnodes, RFpr, cex = GBSG2w/4, xlim = c(0,40), lwd = mylwd,
     xlab = "Nr. positive lymph nodes", ylim = ylim, ylab =
     expression(hat(Y)), cex.axis = CEX, cex.main = CEX, cex.lab = CEX,)
lines(smooth.spline(GBSG2learn$pnodes, RFpr, GBSG2w/4, df = mydf))
plot(GBSG2learn$age, RFpr, cex = GBSG2w/4, xlab = "Age",
     ylab = expression(hat(Y)), ylim = ylim, lwd = mylwd, cex.axis = CEX,
     cex.main = CEX, cex.lab = CEX)
lines(smooth.spline(GBSG2learn$age, RFpr, GBSG2w/4, df = mydf))
plot(GBSG2learn$estrec, RFpr, cex = GBSG2w/4, xlab = "Estrogen receptor",
     ylab = expression(hat(Y)), ylim = ylim, lwd = mylwd, cex.axis = CEX,
     cex.main = CEX, cex.lab = CEX)
lines(smooth.spline(GBSG2learn$estrec, RFpr, GBSG2w/4, df = mydf))
indx <- which(GBSG2learn$progrec < 100)
plot(GBSG2learn$progrec[indx], RFpr[indx], cex = GBSG2w[indx]/4,
     xlab = "Progesterone receptor (< 100 fmol / l)",
     ylab = expression(hat(Y)), ylim = ylim, lwd = mylwd, cex.axis = CEX,
     cex.main = CEX, cex.lab = CEX)
lines(smooth.spline(GBSG2learn$progrec[indx], RFpr[indx], GBSG2w[indx]/4, df = mydf))


###################################################
### code chunk number 15: Figure6
###################################################
par(mai = par("mai") * c(0.7, 0.8, 0.4, 0.6))
layout(matrix(1:4, ncol = 2))
plot(GBSG2learn$pnodes, L2Bpr, cex = GBSG2w/4, xlim = c(0,40),
     ylab = expression(hat(Y)),
     xlab = "Nr. positive lymph nodes", ylim = ylim, lwd = mylwd, cex.axis =
     CEX, cex.main = CEX, cex.lab = CEX)
lines(smooth.spline(GBSG2learn$pnodes, L2Bpr, GBSG2w/4, df = mydf))
plot(GBSG2learn$age, L2Bpr, cex = GBSG2w/4, xlab = "Age",
     ylab = expression(hat(Y)),
     ylim = ylim, lwd = mylwd, cex.axis = CEX, cex.main = CEX, cex.lab =
     CEX)
lines(smooth.spline(GBSG2learn$age, L2Bpr, GBSG2w/4, df = mydf))
plot(GBSG2learn$estrec, L2Bpr, cex = GBSG2w/4, xlab = "Estrogen receptor",
     ylab = expression(hat(Y)), ylim = ylim, lwd = mylwd, cex.axis = CEX,
     cex.main = CEX, cex.lab = CEX)
lines(smooth.spline(GBSG2learn$estrec, L2Bpr, GBSG2w/4, df = mydf))
indx <- which(GBSG2learn$progrec < 100)
plot(GBSG2learn$progrec[indx], L2Bpr[indx], cex = GBSG2w[indx]/4,
     xlab = "Progesterone receptor (< 100 fmol / l)",
     ylab = expression(hat(Y)), ylim = ylim, lwd = mylwd, cex.axis = CEX,
     cex.main = CEX, cex.lab = CEX)
lines(smooth.spline(GBSG2learn$progrec[indx], L2Bpr[indx], GBSG2w[indx]/4, df = mydf))


###################################################
### code chunk number 16: Figure7
###################################################
Mmod <- sum(GBSG2w * GBSG2learn$ltime)/sum(GBSG2w)
par(mai = par("mai") * c(0.7, 0.8, 0.7, 0.6))
layout(matrix(1:4, ncol = 2))

yl <- range(c(GBSG2Hp[GBSG2w > 0], L2Bpr[GBSG2w > 0]))
mdplot(GBSG2learn$ltime, GBSG2Hp, main = "Huber Loss",
       cex = GBSG2w / 4, ylim = c(-3, 3), xlim = c(5, 8))

plot(GBSG2learn$ltime, GBSG2Hp, cex = GBSG2w / 4,
       xlim = range(GBSG2learn$ltime[GBSG2w > 0]), ylim = yl, ylab = "Predicted", xlab = "Observed",
       main = "Huber Loss",  cex.axis = CEX, cex.main = CEX, cex.lab = CEX)

mdplot(GBSG2learn$ltime, L2Bpr,
        cex = GBSG2w / 4, main = "Quadratic Loss", ylim = c(-3, 3), xlim = c(5, 8))

plot(GBSG2learn$ltime, L2Bpr, cex = GBSG2w / 4,
       xlim = range(GBSG2learn$ltime[GBSG2w > 0]), ylim = yl, ylab = "Predicted", xlab = "Observed",
       main = "Quadratic Loss",  cex.axis = CEX, cex.main = CEX, cex.lab = CEX)

###############################################
###
###	Data preprocessing for data published in 
###
###     Use of Gene-Expression Profiling to Identify Prognostic
###     Subclasses in Adult Acute Myloid Leukemia
###
###     Bullinger et al. (2004), New England Journal of Medicine 350 (16),
###     1605-1616.
###
###############################################

if (!require("pamr"))
    stop("cannot attach package ", sQuote("pamr"))

### file as downloaded from http://www.ncbi.nlm.nih.gov/geo, 
### accession number GSE425
### use Format: SOFT to download this file
txt <- readLines(url("http://www.imbe.med.uni-erlangen.de/~hothorn/GSE425.txt"))

### separate clinical and expression data and selected cDNA's
writeLines(txt[121:237], con = "clinical.txt")
writeLines(txt[265:6548], con = "expressions.txt")
writeLines(txt[6788:6937], con = "selections.txt")

### clinical data
clinical <- read.table("clinical.txt", header = TRUE, sep = "\t",
                       strip.white = TRUE)
file.remove("clinical.txt")
rownames(clinical) <- gsub(" ", ".", as.character(clinical$Sample))
clinical$Sample <- NULL

### Study: Treatment Protocol 
### (HD98A between 16 and 60 years, HD98B > 60 years)
summary(clinical$Study)

### Sample.source. (BM: Bone marrow, PB: peripheral blood)
summary(clinical$Sample.source.)

### Cytogenetic.group (Chromosomal abnormalities)
summary(clinical$Cytogenetic.group)

### Karyotype ?
summary(clinical$Karyotype)

### FISH ?
summary(clinical$FISH)

### FLT3.aberration. -> mutation
summary(clinical$FLT3.aberration.)

### MLL.PTD  -> mutation
summary(clinical$MLL.PTD) 

### Sex
summary(clinical$Sex)

### Age (in years)
names(clinical) <- gsub("Age..years..", "Age", names(clinical))
summary(clinical$Age)

### Preceding.Malignancy.
summary(clinical$Preceding.Malignancy.)

### WBC: white blood cell count
names(clinical) <- gsub("WBC..x1000.ul..", "WBC", names(clinical))
summary(clinical$WBC)

### PB.Blasts (blood?)
names(clinical) <- gsub("PB.Blasts......", "PB.Blasts", names(clinical))
summary(clinical$PB.Blasts)

### BM.Blasts (bone marrow?)
names(clinical) <- gsub("BM.Blasts.....", "BM.Blasts", names(clinical))
summary(clinical$BM.Blasts)

### LDH
names(clinical) <- gsub("LDH..U.l..", "LDH", names(clinical))
summary(clinical$LDH)

### FAB.subtype. (French-American-British classification)
summary(clinical$FAB.subtype.)

### Tx.group: Treatment
### HSCT hematopoietic stem-cell transplantation, 
### IC intensive chemotherapy, AUTO autologous HSCT, and ALLO allogeneic HSCT.
summary(clinical$Tx.Group.)

### Remission.status.
### CR complete response, PR partial response, RD refractory disease, R randomization
summary(clinical$Remission.status.)

### Censoring Status
clinical$event <- clinical$Status == "dead"
clinical$Status <- NULL
summary(clinical$event)

### Survival Time (in days)
clinical$time <- clinical[["Overall.Survival..days.."]]
clinical[["Overall.Survival..days.."]] <- NULL
### hm, time = 0 induces problems
clinical$time[clinical$time == 0] <- 0.1
summary(clinical$time)

### = 333
median(clinical$time)
### = 611
median(clinical$time[!clinical$event])

### Training.Test.Set. (for Bullinger paper only)
summary(clinical$Training.Test.Set.)

clinical_predictors <- names(clinical)[-c(1, 2, 17:20)]
dim(clinical)

### expression data
expressions <- read.table("expressions.txt", header = TRUE, sep = "\t", 
                           quote = "", comment.char = "", 
                           colClasses = c(rep("character", 2), rep("numeric", 119)))
file.remove("expressions.txt")
CLID <- expressions$CLID

expressions <- t(expressions[,-(1:2)])
colnames(expressions) <- CLID

expressions <- expressions[match(rownames(clinical), rownames(expressions)),]

dim(expressions)

### impute missing values, use package `pamr'
iexpressions <- t(pamr.knnimpute(list(x = t(expressions)))$x)

### Selected genes
selgenes <- read.table("selections.txt", header = TRUE, sep = "\t",
                       quote = "", comment.char = "", na.string = "", 
                       colClasses = rep("character", 3))[,-1]
file.remove("selections.txt")
selgenes <- selgenes[!is.na(selgenes$Clone.ID),]
dim(selgenes)
summary(selgenes)
sum(!is.na(selgenes$Gene.Symbol)) ### = 133

### no expression levels?
selgenes[!selgenes$Clone.ID %in% colnames(expressions),]

### save data
save(clinical, clinical_predictors, expressions, iexpressions, selgenes,
     file = "AML_Bullinger.Rda")

library("mboost")
data("birds", package = "TH.data")

# define characteristics of the boosting algorithm
bcr <- boost_control(mstop=200, trace=TRUE)

# estimation of a purely linear GLM
fm <- SG5 ~ bols(GST) + bols(DBH) + bols(AOT) + bols(AFS) + bols(DWC) +
            bols(LOG)
sp <- gamboost(fm, data = birds, family = Poisson(), control = bcr)

# extract and plot AIC curve against iteration index and determine stopping
# iteration
birdsaic <- AIC(sp, "classical")
plot(birdsaic)
ms <- mstop(birdsaic)

# selection frequencies of the model terms
table(sp$xselect()[1:ms])

# estimated coefficients
coef(sp[ms])

# re-define boosting iterations
bcr <- boost_control(mstop=500, trace=TRUE)

# Variable selection in a GLM without spatial component
fm <- SG4 ~ bols(GST) + bols(DBH) + bols(AOT) + bols(AFS) + bols(DWC) +
            bols(LOG)
sp <- gamboost(fm, data = birds, family = Poisson(), control = bcr)
table(sp$xselect())
coef(sp, which=1:6)

# Variable selection in a GLM with high df spatial component
fm <- SG4 ~ bols(GST) + bols(DBH) + bols(AOT) + bols(AFS) + bols(DWC) +
            bols(LOG) + bspatial(x_gk, y_gk, df=5, differences=1, knots=c(12,12))
sp <- gamboost(fm, data = birds, family = Poisson(), control = bcr)
table(sp$xselect())
coef(sp, which=1:6)

# Variable selection in a GLM with small df spatial component
fm <- SG4 ~ bols(GST) + bols(DBH) + bols(AOT) + bols(AFS) + bols(DWC) +
            bols(LOG) + bspatial(x_gk, y_gk, df=1, differences=1, knots=c(12,12), center=TRUE)
sp <- gamboost(fm, data = birds, family = Poisson(), control = bcr)
table(sp$xselect())
coef(sp, which=1:6)


# Geoadditive regression model without centering
fm <- SG5 ~ bbs(GST) + bbs(DBH) + bbs(AOT) + bbs(AFS) + bbs(DWC) +
            bbs(LOG) + bspatial(x_gk, y_gk, df=4, differences=1, knots=c(12,12))
sp <- gamboost(fm, data = birds, family = Poisson(), control = bcr)
plot(sp)

# Geoadditive regression model with centering

fm <- SG5 ~ bols(GST) + bbs(GST, df=1, center=TRUE) +
            bols(AOT) + bbs(AOT, df=1, center=TRUE) +
            bols(AFS) + bbs(AFS, df=1, center=TRUE) +
            bols(DWC) + bbs(DWC, df=1, center=TRUE) +
            bols(LOG) + bbs(LOG, df=1, center=TRUE) +
            bspatial(x_gk, y_gk, df=1, differences=1, knots=c(12,12),
              center=TRUE)
sp <- gamboost(fm, data = birds, family = Poisson(), control = bcr)
plot(sp)


# re-define boosting iterations
bcr <- boost_control(mstop=200, trace=TRUE)

# transform covariates to [0,1]
birds$GST <- (birds$GST-min(birds$GST))/(max(birds$GST)-min(birds$GST))
birds$AOT <- (birds$AOT-min(birds$AOT))/(max(birds$AOT)-min(birds$AOT))
birds$AFS <- (birds$AFS-min(birds$AFS))/(max(birds$AFS)-min(birds$AFS))
birds$DWC <- (birds$DWC-min(birds$DWC))/(max(birds$DWC)-min(birds$DWC))
birds$LOG <- (birds$LOG-min(birds$LOG))/(max(birds$LOG)-min(birds$LOG))

# Space-varying coefficient models (with centered spatial effects)
fm <- SG5 ~ bols(GST) + bspatial(x_gk, y_gk, by = GST, df=1, differences=1,
              knots=c(12, 12), center=TRUE) +
            bols(AOT) + bspatial(x_gk, y_gk, by = AOT, df=1, differences=1,
              knots=c(12, 12), center=TRUE) +
            bols(AFS) + bspatial(x_gk, y_gk, by = AFS, df=1, differences=1,
              knots=c(12, 12), center=TRUE) +
            bols(DWC) + bspatial(x_gk, y_gk, by = DWC, df=1, differences=1,
              knots=c(12, 12), center=TRUE) +
            bols(LOG) + bspatial(x_gk, y_gk, by = LOG, df=1, differences=1,
              knots=c(12, 12), center=TRUE) +
            bspatial(x_gk, y_gk, df=1, differences=1, knots=c(12, 12),
              center=TRUE)
sp <- gamboost(fm, data = birds, family = Poisson(), control = bcr)
plot(sp, which = "GST")
plot(sp, which = "AOT")
plot(sp, which = "AFS")
plot(sp, which = "DWC")
plot(sp, which = "LOG")

library("mboost")
library("party")

load("india.Rdata")

##############
# Blackboost #
##############

quantiles <- c(0.05, 0.1, 0.5)
inds <- paste("w", 1:50, sep="")

features <- names(india)[c(11:16, 19:33)]
fmtree <- paste(features, collapse = " + ")
fmtree <- as.formula(paste("stunting", fmtree, sep=" ~ "))

for(i in 1:length(inds))
{
    indiafit <- india[india[,inds[i]]<3,]
    indiapred <- india[india[,inds[i]]==3,]
    wfit <- as.numeric(indiafit[,inds[i]]==1)
    
    for(j in 1:length(quantiles))
    {
        it <- 1000
        inc <- 1000
        maxit <- 50000
        bc <- boost_control(mstop = it, nu=0.25, trace = TRUE, risk = "oob")
        tc <- ctree_control(maxdepth = 5, savesplitstats = FALSE)
        
        model <- blackboost(fmtree, 
                            data = indiafit, 
                            control = bc,
                            tree_controls = tc,
                            weights = wfit,
                            family = QuantReg(tau = quantiles[j])
        )
        
        risk <- model$risk()                    
        while( ((risk[it-inc+1] - risk[it])>=0.05) &
               (it<=maxit) ) 
        {
            it <- it + inc
            model[it]
            risk <- model$risk()                    
        }
        
        pred <- predict(model[which.min(risk)], newdata=indiapred)
        save(pred, file=paste("trees/model",i,"_",quantiles[j],".Rdata",sep=""))
    }
}

#============================================================================
# Function lambdaOpt()
#============================================================================
# Aim: Finding an optimal lambda for rqss (this function is minimized by optim or optimize)
#
# Input:
# --------
# lambdas     Vector of lambda values
# modeldata   Training data from generateData()
# evaldata    Test data lying in the convex hull of modeldata
# tau         Quantile of interest
# --------
# Output:
# --------
# Empirical risk when rqss model is estimated on modeldata and
# evaluated on evaldata
#

lambdaOpt <- function(lambdas, fitdata, evaldata, tau){
    
    # Model formula
    form <- buildFormula(lambdas=lambdas)
    
    # Generate rqss object
    rqssLambda <- rqss(form, data=fitdata, tau=tau)
    
    # Prediction with test data  
    predLambda <- predict(rqssLambda, newdata=evaldata)
    
    # Calculate empirical risk
    CVrisk <-loss(evaldata$stunting, predLambda, tau=tau)
    
    return(CVrisk)
    
}

#==============================================================================
# Check function
#==============================================================================
rho <- function(u, tau){u*(tau - (u<0))}

#==============================================================================
# Risk
#==============================================================================
loss <- function(yTrue,yHat,tau){sum(rho(yTrue-yHat,tau))/length(yTrue)}

#======================================================================
# Function buildFormula()
#======================================================================
# Aim: Building formula for function rqss()

buildFormula <- function(lambdas){
    
    return(as.formula(paste("stunting ~ qss(cage,lambda =", lambdas[1],") + qss(breastfeeding, lambda =", lambdas[2],
                            ") + qss(mbmi, lambda =", lambdas[3],") + qss(mage, lambda =", lambdas[4],") + qss(medu, lambda =", lambdas[5],
                            ") + qss(edupartner, lambda =", lambdas[6],") + csex + ctwin + cbirthorder + munemployed + mreligion + mresidence
      + deadchildren + wealth + electricity + radio + television + refrigerator + bicycle + motorcycle + car", sep="")))
}



#============================================================================
# Function convexHull()
#============================================================================
# Aim: Reduce data set such as lying in the convex hull of reference data
#
# Input:
# --------
# dataOrg     Origninal data, here: valdata
# dataRef     Reference data, here: traindata
# --------
# Output:
# --------
# reduced data set where each covariate lies in the range of the
# covariates from modeldata
#

convexHull <- function(dataOrg, dataRef){
    
    conNames <- c("cage", "breastfeeding", "mbmi", "mage", "medu", "edupartner")
    indMat <- matrix(0, nrow=nrow(dataOrg), ncol=6)
    
    for(i in 1:length(conNames)){
        indMat[,i] <- as.numeric(dataOrg[,conNames[i]] <= min(dataRef[,conNames[i]]) | dataOrg[,conNames[i]] >= max(dataRef[,conNames[i]]))
    }
    indVec <- apply(indMat, MARGIN=1, sum)
    
    # Observations with indVec>0 are removed from the dataOrg
    evaldata <- dataOrg[indVec==0,]
    
    return(evaldata=evaldata)
    
}


convexHullInd <- function(dataOrg, dataRef){
    
    conNames <- c("cage", "breastfeeding", "mbmi", "mage", "medu", "edupartner")
    indMat <- matrix(0, nrow=nrow(dataOrg), ncol=6)
    
    for(i in 1:length(conNames)){
        indMat[,i] <- as.numeric(dataOrg[,conNames[i]] <= min(dataRef[,conNames[i]]) | dataOrg[,conNames[i]] >= max(dataRef[,conNames[i]]))
    }
    indVec <- apply(indMat, MARGIN=1, sum)
    
    # Observations with indVec>0 are removed from the dataOrg
    #evaldata <- dataOrg[indVec==0,]
    
    return(indVec=indVec)
    
}

library("mboost")
library("party")

load("india.Rdata")

##################
# Additive Model #
##################

quantiles <- c(0.05, 0.1, 0.5)

inds <- paste("w", 1:50, sep="")

fmadd <- stunting ~ bols(intercept, intercept=FALSE) + 
    bols(cage) + bbs(cage, center=TRUE, df=1, knots = 20) +
    bols(breastfeeding) + bbs(breastfeeding, center=TRUE, df=1, knots=20) +
    bols(mbmi) + bbs(mbmi, center=TRUE, df=1, knots=20) +
    bols(mage) + bbs(mage, center=TRUE, df=1, knots=20) +
    bols(medu) + bbs(medu, center=TRUE, df=1, knots=20) +
    bols(edupartner) + bbs(edupartner, center=TRUE, df=1, knots=20) +
    bols(csex) +
    bols(ctwin) +
    bols(cbirthorder) +
    bols(munemployed) +
    bols(mreligion) +
    bols(mresidence) +
    bols(deadchildren) +
    bols(wealth) +
    bols(electricity) +
    bols(radio) +
    bols(television) +
    bols(refrigerator) +
    bols(bicycle) +
    bols(motorcycle) +
    bols(car)

for(i in 1:50)
{
    indiafit <- india[india[,inds[i]]<3,]
    indiapred <- india[india[,inds[i]]==3,]
    wfit <- as.numeric(indiafit[,inds[i]]==1)
    
    for(j in 1:length(quantiles))
    {
        it <- 10000
        inc <- 10000
        maxit <- 200000
        bc <- boost_control(mstop = it, nu=0.25, trace = TRUE, risk = "oob")
        
        model <- gamboost(fmadd,
                          data = indiafit,
                          control = bc,
                          weights = wfit,
                          family = QuantReg(tau = quantiles[j])
        )
        
        risk <- model$risk()                    
        while( ((risk[it-inc+1] - risk[it])>=0.05) &
               (it<=maxit) ) 
        {
            it <- it + inc
            model[it]
            risk <- model$risk()                    
        }
        
        save(model, file=paste("additive/model",i,"_",quantiles[j],".Rdata",sep=""))
        rm(model)
        gc()
    }
}

#-----------------------------------------------------------
# Path, Packages, source functions
#-----------------------------------------------------------

library("quantreg")
load("india.Rdata")
source("india_rqss_lambdaOptFunc.R")

#-----------------------------------------------------------
# Quantiles and Indices
#-----------------------------------------------------------

quantiles <- c(0.05, 0.1, 0.5)
inds <- paste("w", 1:50, sep="")


for(i in 1:length(inds)){
    
    indiaTrain <- india[india[,inds[i]]==1,]
    
    indiaTuning <- india[india[,inds[i]]==2,] 
    indiaTuning <- convexHull(dataOrg=indiaTuning, dataRef=indiaTrain)
    
    indiaTest <- india[india[,inds[i]]==3,] 
    indiaTest <- convexHull(dataOrg=indiaTest, dataRef=indiaTrain)
    
    
    for(j in 1:length(quantiles)){
        
        
        # Starting parameters for lambda equal to 100:
        #optimLambda <- optim(rep(100,6), fn=lambdaOpt, fitdata=indiaTrain, 
        #                     evaldata=indiaTuning, tau=quantiles[j], lower=0)                     
        
        # Next try: Starting parameters equal to 10:
        optimLambda <- optim(rep(10,6), fn=lambdaOpt, fitdata=indiaTrain, 
                             evaldata=indiaTuning, tau=quantiles[j], lower=0) 
        lambdaMin <- optimLambda$par
        
        formule <- buildFormula(lambdas=lambdaMin)
        model <- rqss(formule, data=indiaTrain, tau=quantiles[j])
        
        # Results with starting parameters for lambda equal to 100:
        save(model, file=paste("rqssObjects/model",i,"_",quantiles[j],".Rdata",sep=""))
        rm(model)
        gc()
        
    }
}

options(digits = 3)

library("mboost")

set.seed(290875)

df <- function(n = 100) {
    x <- matrix(runif(n * 10), ncol = 10)
    y <- x %*% c(1:3, rep(0, 7)) + rnorm(n)
    data.frame(y = y, x)
}

mydf <- df()

w <- c(rep(0, 50), rep(1, 50))
mod1 <- glmboost(y ~ ., data = mydf, weights = w, center = FALSE)
cf1 <- coef(mod1)

### hat matrix: fast for linear models
H <- attr(hatvalues(mod1), "hatmatrix")
print(max(abs(H %*% (mydf$y - weighted.mean(mydf$y, w)) - fitted(mod1) + weighted.mean(mydf$y, w))) <
          sqrt(.Machine$double.eps))

mb <- function(x) bols(x, intercept = FALSE)
mydf$int <- rep(1, nrow(mydf))
## used to be: mod2 <- gamboost(y ~ ., data = mydf, weights = w, baselearner = mb) ## but doesn't work as .mboost functions deprecated.
mod2 <- gamboost(y ~ ., data = mydf, weights = w, baselearner = mb)
cf2 <- coef(mod2)
print(all(round(unlist(cf2), 5) %in% round(cf1, 5)))

### hat matrix: less faster but in C
H <- attr(hatvalues(mod2), "hatmatrix")
print(max(abs(H %*% (mydf$y - weighted.mean(mydf$y, w)) - fitted(mod2) + weighted.mean(mydf$y, w))) <
          sqrt(.Machine$double.eps))

### hat matrix: directly in R
# <FIXME> this is non-sense </FIXME>
#mod2$family <- Laplace()
#H <- attr(hatvalues(mod2), "hatmatrix")
#print(max(abs(H %*% (mydf$y - weighted.mean(mydf$y, w)) - fitted(mod2) + weighted.mean(mydf$y, w))) <
#          sqrt(.Machine$double.eps))

library("mboost")

load("india.Rdata")

source("india_helpfunc.R")

QuantileReg <- function(q = 0.25) {
   Family(
   ngradient = function(y, f, w = 1) q*((y-f)>0) - (1-q)*((y-f)<0) + 0*((y-f)==0) ,
   loss = function(y, f) q*(y-f)*((y-f)>=0) - (1-q)*(y-f)*((y-f)<0) ,
   offset = function(y, w = rep(1, length(y))) quantile(y[w==1], q),
   check_y = function(y) {
     if (!is.numeric(y) || !is.null(dim(y)))
         stop("response is not a numeric vector but ", sQuote("family = QuantileReg()"))
     TRUE
     },
   name = "Quantile Regression",
   )
 }

fm <- stunting ~ bbs(cage, center=FALSE, df=5, knots = 20) +
  bbs(breastfeeding, center=FALSE, df=5, knots=20) +
  bols(csex, center=FALSE) +
  bols(ctwin, center=FALSE) +
  bols(cbirthorder, center=FALSE) +
  bbs(mbmi, center=FALSE, df=5, knots=20) +
  bbs(mage, center=FALSE, df=5, knots=20) +
  bbs(medu, center=FALSE, df=5, knots=20) +
  bbs(edupartner, center=FALSE, df=5, knots=20) +
  bols(munemployed, center=FALSE) +
  bols(mreligion, center=FALSE) +
  bols(mresidence, center=FALSE) +
  bols(deadchildren, center=FALSE) +
  bols(wealth, center=FALSE) +
  bols(electricity, center=FALSE) +
  bols(radio, center=FALSE) +
  bols(television, center=FALSE) +
  bols(refrigerator, center=FALSE) +
  bols(bicycle, center=FALSE) +
  bols(motorcycle, center=FALSE) +
  bols(car, center=FALSE)

inb1 <- gamboost(fm,
                    data = india,
                    control = boost_control(mstop = 200000, nu=0.25, trace = TRUE, save_ensembless=FALSE, risk = "oob"),
                    weights = india$cv,
                    family = QuantileReg(0.05))
mstop1 <- which.min(inb1$risk)
its1 <- round(seq(from=1000, to=mstop1, length=20))
fit1 <- extractfit(inb1, fm, its1)

inb2 <- gamboost(fm,
                    data = india,
                    control = boost_control(mstop = 200000, nu=0.25, trace = TRUE, save_ensembless=FALSE, risk = "oob"),
                    weights = india$cv,
                    family = QuantileReg(0.1))
mstop2 <- which.min(inb2$risk)
its2 <- round(seq(from=1000, to=mstop2, length=20))
fit2 <- extractfit(inb2, fm, its2)

inb3 <- gamboost(fm,
                    data = india,
                    control = boost_control(mstop = 200000, nu=0.25, trace = TRUE, save_ensembless=FALSE, risk = "oob"),
                    weights = india$cv,
                    family = QuantileReg(0.5))
mstop3 <- which.min(inb3$risk)
its3 <- round(seq(from=1000, to=mstop3, length=20))
fit3 <- extractfit(inb3, fm, its3)

save(inb1, fit1, inb2, fit2, inb3, fit3, file="india_results.Rdata")

pdf("india_results1.pdf", paper="a4", width=8, height=12)
par(mfrow=c(3,2))
plotnonpar("cage", 1, "age of the child", fit1)
plotnonpar("breastfeeding", 2, "duration of breastfeeding", fit1)
plotnonpar("mbmi", 6, "body mass index of the mother", fit1)
plotnonpar("mage", 7, "age of the mother", fit1)
plotnonpar("medu", 8, "years of education (mother)", fit1)
plotnonpar("edupartner", 9, "years of education (partner)", fit1)

plotpar("csex", 3, "sex of the child", fit1)
plotpar("ctwin", 4, "twin", fit1)
plotpar("cbirthorder", 5, "birth order", fit1)
plotpar("munemployed", 10, "unemployment of the mother", fit1)
plotpar("mreligion", 11, "religion", fit1)
plotpar("mresidence", 12, "place of residence", fit1)
plotpar("deadchildren", 13, "no. of dead children", fit1)
plotpar("wealth", 14, "wealth index", fit1)
plotpar("electricity", 15, "electricity", fit1)
plotpar("radio", 16, "radio", fit1)
plotpar("television", 17, "television", fit1)
plotpar("refrigerator", 18, "refrigerator", fit1)
plotpar("bicycle", 19, "bicycle", fit1)
plotpar("motorcycle", 20, "motorcycle", fit1)
plotpar("car", 21, "car", fit1)
dev.off()

pdf("india_results2.pdf", paper="a4", width=8, height=12)
par(mfrow=c(3,2))
plotnonpar("cage", 1, "age of the child", fit2)
plotnonpar("breastfeeding", 2, "duration of breastfeeding", fit2)
plotnonpar("mbmi", 6, "body mass index of the mother", fit2)
plotnonpar("mage", 7, "age of the mother", fit2)
plotnonpar("medu", 8, "years of education (mother)", fit2)
plotnonpar("edupartner", 9, "years of education (partner)", fit2)

plotpar("csex", 3, "sex of the child", fit2)
plotpar("ctwin", 4, "twin", fit2)
plotpar("cbirthorder", 5, "birth order", fit2)
plotpar("munemployed", 10, "unemployment of the mother", fit2)
plotpar("mreligion", 11, "religion", fit2)
plotpar("mresidence", 12, "place of residence", fit2)
plotpar("deadchildren", 13, "no. of dead children", fit2)
plotpar("wealth", 14, "wealth index", fit2)
plotpar("electricity", 15, "electricity", fit2)
plotpar("radio", 16, "radio", fit2)
plotpar("television", 17, "television", fit2)
plotpar("refrigerator", 18, "refrigerator", fit2)
plotpar("bicycle", 19, "bicycle", fit2)
plotpar("motorcycle", 20, "motorcycle", fit2)
plotpar("car", 21, "car", fit2)
dev.off()

pdf("india_results3.pdf", paper="a4", width=8, height=12)
par(mfrow=c(3,2))
plotnonpar("cage", 1, "age of the child", fit3)
plotnonpar("breastfeeding", 2, "duration of breastfeeding", fit3)
plotnonpar("mbmi", 6, "body mass index of the mother", fit3)
plotnonpar("mage", 7, "age of the mother", fit3)
plotnonpar("medu", 8, "years of education (mother)", fit3)
plotnonpar("edupartner", 9, "years of education (partner)", fit3)

plotpar("csex", 3, "sex of the child", fit3)
plotpar("ctwin", 4, "twin", fit3)
plotpar("cbirthorder", 5, "birth order", fit3)
plotpar("munemployed", 10, "unemployment of the mother", fit3)
plotpar("mreligion", 11, "religion", fit3)
plotpar("mresidence", 12, "place of residence", fit3)
plotpar("deadchildren", 13, "no. of dead children", fit3)
plotpar("wealth", 14, "wealth index", fit3)
plotpar("electricity", 15, "electricity", fit3)
plotpar("radio", 16, "radio", fit3)
plotpar("television", 17, "television", fit3)
plotpar("refrigerator", 18, "refrigerator", fit3)
plotpar("bicycle", 19, "bicycle", fit3)
plotpar("motorcycle", 20, "motorcycle", fit3)
plotpar("car", 21, "car", fit3)
dev.off()

boost_control <- function(mstop = 100, nu = 0.1,
                          risk = c("inbag", "oobag", "none"),
                          stopintern = FALSE,
                          center = TRUE, trace = FALSE) {

   risk <- match.arg(risk)
   stopintern <- stopintern & (risk == "oobag")
   RET <- list(mstop = mstop, nu = nu,
               risk = risk, stopintern = stopintern,
               center = center, trace = trace)
   class(RET) <- c("boost_control")
   RET
}

### workhorse for fitting matrix-response (Ridge-penalized) baselearners
### Y = kronecker(X2, X1)
### see Currie, Durban, Eilers (2006, JRSS B)
bl_lin_matrix <- function(blg, Xfun, args) {

    mf <- blg$get_data()
    index <- blg$get_index()
    vary <- blg$get_vary()

    newX <- function(newdata = NULL, prediction = FALSE) {
        if (!is.null(newdata)) {
            mf <- check_newdata(newdata, blg, mf, to.data.frame = FALSE)
        }
        ## this argument is currently only used in X_bbs --> bsplines
        args$prediction <- prediction
        return(Xfun(mf, vary, args))
    }
    X <- newX()
    K <- X$K
    X <- X$X
    c1 <- ncol(X$X1)
    c2 <- ncol(X$X2)
    n1 <- nrow(X$X1)
    n2 <- nrow(X$X2)

    G <- function(x) {
        one <- matrix(rep(1, ncol(x)), nrow = 1)
        suppressMessages(
            ret <- kronecker(x, one) * kronecker(one, x)
            )
        ret
    }

    dpp <- function(weights) {

        if (!is.null(attr(X$X1, "deriv")) || !is.null(attr(X$X2, "deriv")))
            stop("fitting of derivatives of B-splines not implemented")

        W <- matrix(weights, nrow = n1, ncol = n2)

        ### X = kronecker(X2, X1)
        XtX <- crossprod(G(X$X1), W) %*% G(X$X2)
        mymatrix <- matrix
        if (is(XtX, "Matrix")) mymatrix <- Matrix
        XtX <- array(XtX, c(c1, c1, c2, c2))
        XtX <- mymatrix(aperm(XtX, c(1, 3, 2, 4)), nrow = c1 * c2)

        ### If lambda was given in both baselearners, we
        ### directly multiply the marginal penalty matrices by lambda
        ### and then compute the total penalty as the kronecker sum.
        ### args$lambda is NA in this case and we don't compute
        ### the corresponding df's (unlike bl_lin)
        if (is.null(args$lambda)) {

            ### <FIXME>: is there a better way to feed XtX into lambdadf?
            lambdadf <- df2lambda(X = diag(rankMatrix(X$X1, method = 'qr', warn.t = FALSE) *
                                           rankMatrix(X$X2, method = 'qr', warn.t = FALSE)),
                                  df = args$df, lambda = args$lambda,
                                  dmat = K, weights = weights, XtX = XtX)
            ### </FIXME>
            lambda <- lambdadf["lambda"]
            K <- lambda * K
        } else {
            lambdadf <- args[c("lambda", "df")]
        }
        ### note: K already contains the lambda penalty parameter(s)
        XtX <- XtX + K

        ### nnls
        constr <- (!is.null(attr(X$X1, "constraint"))) +
                  (!is.null(attr(X$X2, "constraint")))

        if (constr == 2)
            stop("only one dimension may be subject to constraints")
        constr <- constr > 0

        ## dense matrizes should be coerced to class matrix and
        ## handled in the standard way
        if (is(XtX, "Matrix") && is(XtX, "sparseMatrix")) {
            XtXC <- Cholesky(forceSymmetric(XtX))
            mysolve <- function(y) {
                Y <- matrix(y, nrow = n1) * W
                if (constr)
                    return(nnls2D(X, as(XtXC, "matrix"), Y))
                XWY <- as.vector(crossprod(X$X1, Y) %*% X$X2)
                solve(XtXC, XWY)  ## special solve routine from
                                  ## package Matrix
                }
        } else {
            if (is(XtX, "Matrix")) {
                ## coerce Matrix to matrix
                XtX <- as(XtX, "matrix")
            }
            mysolve <- function(y) {
                Y <- matrix(y, nrow = n1) * W
                if (constr)
                    return(nnls2D(X, as(XtX, "matrix"), Y))
                XWY <- crossprod(X$X1, Y) %*% X$X2
                solve(XtX, matrix(as(XWY, "matrix"), ncol = 1))
            }
        }

        cfprod <- function(b) tcrossprod(X$X1 %*% b, X$X2)

        fit <- function(y) {
            coef <- as(mysolve(y), "matrix")
            if (nrow(coef) != c1) coef <- matrix(as.vector(coef), nrow = c1)
            f <- cfprod(coef)
            f <- as(f, "matrix")
            if (options("mboost_Xmonotone")$mboost_Xmonotone) {
                md <- apply(f, 1, function(x) min(diff(x)))
                if (any(md < -(.Machine$double.eps)^(1/3))) {
                    coef <- matrix(0, nrow = nrow(coef), ncol = ncol(coef))
                    f <- matrix(0, nrow = nrow(f), ncol = ncol(f))
                }
            }
            ret <- list(model = coef,
                        fitted = function() as.vector(f))
            class(ret) <- c("bm_lin", "bm")
            ret
        }

        ### <FIXME> check for large n, option?
        hatvalues <- function() {
            return(NULL)
        }
        ### </FIXME>

        ### actually used degrees of freedom (trace of hat matrix)
        df <- function() lambdadf

        ### prepare for computing predictions
        predict <- function(bm, newdata = NULL, aggregate = c("sum", "cumsum", "none")) {
            cf <- lapply(bm, function(x) x$model)
            if(!is.null(newdata)) {
                index <- NULL
                X <- newX(newdata, prediction = TRUE)$X
            }
            ncfprod <- function(b)
                as.vector(as(tcrossprod(X$X1 %*% b, X$X2), "matrix"))
            aggregate <- match.arg(aggregate)
            pr <- switch(aggregate, "sum" = {
                cf2 <- 0
                for (b in cf) cf2 <- cf2 + b
                ncfprod(cf2)
            },
            "cumsum" = {
                cf2 <- 0
                ret <- c()
                for (b in cf) {
                    cf2 <- cf2 + b
                    ret <- cbind(ret, ncfprod(cf2))
                }
                ret
            },
            "none" = {
                ret <- c()
                for (b in cf) {
                    ret <- cbind(ret, ncfprod(b))
                }
                ret
            })
            return(pr)
        }

        Xnames <- outer(colnames(X$X1), colnames(X$X2), paste, sep = "_")
        ret <- list(fit = fit, hatvalues = hatvalues,
                    predict = predict, df = df,
                    Xnames = as.vector(Xnames))
        class(ret) <- c("bl_kronecker", "bl_lin", "bl")
        return(ret)

    }
    return(dpp)
}

### kronecker product of two baselearners
### bbs(x1) %O% bbs(x2) means that
### X = kronecker(X2, X1) and x1 varies fastest
"%O%" <- function(bl1, bl2) {

    if (is.list(bl1) && !inherits(bl1, "blg"))
        return(lapply(bl1, "%X%", bl2 = bl2))

    if (is.list(bl2) && !inherits(bl2, "blg"))
        return(lapply(bl2, "%X%", bl1 = bl1))

    cll <- paste(bl1$get_call(), "%O%",
                 bl2$get_call(), collapse = "")
    stopifnot(inherits(bl1, "blg"))
    stopifnot(inherits(bl2, "blg"))

    mf1 <- model.frame(bl1)
    mf2 <- model.frame(bl2)
    stopifnot(!any(colnames(mf1) %in%
                   colnames(mf2)))
    mf <- c(mf1, mf2)
    stopifnot(all(complete.cases(mf[[1]])))
    stopifnot(all(complete.cases(mf[[2]])))

    index <- NULL

    vary <- ""

    ret <- list(model.frame = function()
                    return(mf),
                get_call = function(){
                    cll <- deparse(cll, width.cutoff=500L)
                    if (length(cll) > 1)
                        cll <- paste(cll, collapse="")
                    cll
                },
                get_data = function() mf,
                get_index = function() index,
                get_vary = function() vary,
                get_names = function() names(mf),
                ## <FIXME> Is this all we want to change if we set names here?
                set_names = function(value) attr(mf, "names") <<- value)
                ## </FIXME>
    class(ret) <- "blg"

    args1 <- environment(bl1$dpp)$args
    args2 <- environment(bl2$dpp)$args
    l1 <- args1$lambda
    l2 <- args2$lambda
    if (xor(is.null(l1), is.null(l2)))
        stop("lambda needs to be given in both baselearners combined with ",
             sQuote("%O%"))
    if (!is.null(l1) && !is.null(l2)) {
        ### there is no common lambda!
        args <- list(lambda = NA, df = NA)
    } else {
        args <- list(lambda = NULL,
            df = ifelse(is.null(args1$df), 1, args1$df) *
                 ifelse(is.null(args2$df), 1, args2$df))
    }

    Xfun <- function(mf, vary, args) {

        newX1 <- environment(bl1$dpp)$newX
        newX2 <- environment(bl2$dpp)$newX

        X1 <- newX1(as.data.frame(mf[bl1$get_names()]),
                    prediction = args$prediction)
        K1 <- X1$K
        X1 <- X1$X
        if (!is.null(l1)) K1 <- l1 * K1
        MATRIX <- options("mboost_useMatrix")$mboost_useMatrix
        if (MATRIX & !is(X1, "Matrix"))
            X1 <- Matrix(X1)
        if (MATRIX & !is(K1, "Matrix"))
            K1 <- Matrix(K1)

        X2 <- newX2(as.data.frame(mf[bl2$get_names()]),
                    prediction = args$prediction)
        K2 <- X2$K
        X2 <- X2$X
        if (!is.null(l2)) K2 <- l2 * K2
        if (MATRIX & !is(X2, "Matrix"))
            X2 <- Matrix(X2)
        if (MATRIX & !is(K2, "Matrix"))
            K2 <- Matrix(K2)
        suppressMessages(
            K <- kronecker(K2, diag(ncol(X1))) +
                kronecker(diag(ncol(X2)), K1)
            )
        list(X = list(X1 = X1, X2 = X2), K = K)
    }

    ret$dpp <- bl_lin_matrix(ret, Xfun = Xfun, args = args)

    return(ret)
}

##
## cross-validation (bootstrap, k-fold cv etc.) of empirical risk
## for boosting algorithms
##

cvrisk <- function(object, ...)
    UseMethod("cvrisk")

cvrisk.mboost <- function (object, folds = cv(model.weights(object)),
                           grid = 0:mstop(object), papply = mclapply,
                           fun = NULL, mc.preschedule = FALSE,
                           ...) {
    
    papply <- match.fun(papply)
    weights <- model.weights(object)
    if (any(weights == 0))
        warning("zero weights")
    if (is.null(folds)) {
        folds <- rmultinom(25, length(weights), weights/sum(weights))
        attr(folds, "type") <- "25-fold bootstrap"
    } else {
        stopifnot(is.matrix(folds) && nrow(folds) == length(weights))
    }
    fitfct <- object$update
    oobrisk <- matrix(0, nrow = ncol(folds), ncol = length(grid))
    if (!is.null(fun))
        stopifnot(is.function(fun))
    fam_name <- object$family@name
    call <- deparse(object$call)
    if (is.null(fun)) {
        dummyfct <- function(weights, oobweights) {
            mod <- fitfct(weights = weights, oobweights = oobweights)
            mstop(mod) <- max(grid)
            ## return all risk values in grid (+ 1 as 0 is included)
            risk(mod)[grid + 1]
        }
        if (fam_name == "Cox Partial Likelihood" && all(colSums(folds == 0) == 1))
            stop("Leave-one-out cross-validation cannot be used with ", sQuote("family = CoxPH()"))
    } else { ## !is.null(fun)
        dummyfct <- function(weights, oobweights) {
            mod <- fitfct(weights = weights, oobweights = oobweights)
            mod[max(grid)]
            ## make sure dispatch works correctly
            class(mod) <- class(object)
            fun(mod)
        }
    }

    ## use case weights as out-of-bag weights (but set inbag to 0)
    OOBweights <- matrix(rep(weights, ncol(folds)), ncol = ncol(folds))
    OOBweights[folds > 0] <- 0
    if (identical(papply, mclapply)) {
        oobrisk <- papply(1:ncol(folds),
                          function(i) try(dummyfct(weights = folds[, i],
                                                   oobweights = OOBweights[, i]),
                                          silent = TRUE),
                          mc.preschedule = mc.preschedule,
                          ...)
    } else {
        oobrisk <- papply(1:ncol(folds),
                          function(i) try(dummyfct(weights = folds[, i],
                                                   oobweights = OOBweights[, i]),
                                          silent = TRUE),
                          ...)
    }
    ## if any errors occured remove results and issue a warning
    if (any(idx <- sapply(oobrisk, is.character))) {
        warning(sum(idx), " fold(s) encountered an error. ",
                "Results are based on ", ncol(folds) - sum(idx),
                " folds only.\n",
                "Original error message(s):\n",
                sapply(oobrisk[idx], function(x) x))
        oobrisk[idx] <- NA
    }
    if (!is.null(fun))
        return(oobrisk)
    oobrisk <- t(as.data.frame(oobrisk))
    oobrisk <- oobrisk / colSums(OOBweights)
    colnames(oobrisk) <- grid
    rownames(oobrisk) <- 1:nrow(oobrisk)
    attr(oobrisk, "risk") <- fam_name
    attr(oobrisk, "call") <- call
    attr(oobrisk, "mstop") <- grid
    attr(oobrisk, "type") <- ifelse(!is.null(attr(folds, "type")),
                                    attr(folds, "type"), "user-defined")
    class(oobrisk) <- "cvrisk"
    oobrisk
}

print.cvrisk <- function(x, ...) {
    cat("\n\t Cross-validated", attr(x, "risk"), "\n\t",
        attr(x, "call"), "\n\n")
    print(colMeans(x, na.rm = TRUE))
    cat("\n\t Optimal number of boosting iterations:", mstop(x), "\n")
    return(invisible(x))
}

plot.cvrisk <- function(x, xlab = "Number of boosting iterations",
                        ylab = attr(x, "risk"),
                        ylim = range(x), main = attr(x, "type"), ...) {
    
    ## force evaluation of attributes (to not evaluate them in the wrong situation)
    force(ylab)
    force(main)
    
    mstops <- attr(x, "mstop")

    x <- x[, apply(x, 2, function(y) all(!is.na(y))), drop = FALSE]
    cm <- colMeans(x)
    plot(1:ncol(x), cm, ylab = ylab, ylim = ylim,
         type = "n", lwd = 2, xlab = xlab,
         main = main, axes = FALSE, ...)
    out <- apply(x, 1, function(y) lines(1:ncol(x),y, col = "lightgrey"))
    rm(out)
    ms <- which.min(cm)
    lines(c(ms, ms), c(min(c(0, ylim[1] * ifelse(ylim[1] < 0, 2, 0.5))), cm[ms]),
          lty = 2)
    lines(1:ncol(x), cm, type = "l")
    axis(1, at = 1:ncol(x), labels = mstops)
    axis(2)
    box()
}

mstop.cvrisk <- function(object, ...)
    attr(object, "mstop")[which.min(colSums(object, na.rm = TRUE))]

cv <- function(weights, type = c("bootstrap", "kfold", "subsampling"),
               B = ifelse(type == "kfold", 10, 25),
               prob = 0.5, strata = NULL) {

    type <- match.arg(type)
    n <- length(weights)

    if (is.null(strata)) strata <- gl(1, n)
    if (!is.factor(strata)) stop(sQuote("strata"), " must be a factor")
    folds <- matrix(0, nrow = n, ncol = B)

    ### <FIXME> handling of weights needs careful documentation </FIXME>
    for (s in levels(strata)) {
        indx <- which(strata == s)
        folds[indx,] <- switch(type,
                               "bootstrap" = cvboot(length(indx), B = B, weights[indx]),
                               "kfold" = cvkfold(length(indx), k = B) * weights[indx],
                               "subsampling" = cvsub(length(indx), prob = prob, B = B) * weights[indx])
    }
    attr(folds, "type") <- paste(B, "-fold ", type, sep = "")
    return(folds)
}


cvboot <- function(n, B, weights)
    rmultinom(B, n, weights / sum(weights))

cvkfold <- function(n, k) {
    #if (k > n / 2) stop("k > n/2")
    fl <- floor(n/k)
    folds <- c(rep(c(rep(0, fl), rep(1, n)), k - 1),
               rep(0, n * k - (k - 1) * (fl + n)))
    matrix(folds, nrow = n)[sample(1:n),, drop = FALSE]
}

cvsub <- function(n, prob, B) {
    k <- floor(n * prob)
    indx <- rep(c(0, 1), c(n - k, k))
    replicate(B, sample(indx))[sample(1:n),, drop = FALSE]
}

survFit <- function(object, ...)
    UseMethod("survFit")

survFit.mboost <- function(object, newdata = NULL, ...)
{

    n <- length(w <- model.weights(object))
    if (!isTRUE(all.equal(w,rep(1,n))))
        stop("survFit cannot (yet) deal with weights")
    y <- object$response
    stopifnot(inherits(y, "Surv"))

    ord <- order(y[,1])
    time <- y[ord,1]
    event <- y[ord,2]
    n.event <- aggregate(event, by = list(time), sum)[,2]

    pr <- predict(object)
    linpred <- (pr - mean(pr))[ord]
    R <- rev(cumsum(rev(exp(linpred))))
    R[R<1e-6] <- Inf

    d <- c(1,diff(time)) != 0
    R <- R[d]
    H <- cumsum(1/R*n.event)
    time <- time[d]

    devent <- n.event > 0
    if (!is.null(newdata)){
        S <- exp( tcrossprod( -H, exp(as.numeric(predict(object,
        newdata=newdata)- mean(predict(object)))) ))[devent,]
        colnames(S) <- rownames(newdata)
    } else S <- matrix(exp(-H)[devent], ncol = 1)

    ret <- list(surv = S, time = time[devent], n.event = n.event[devent])
    class(ret) <- "survFit"
    ret
}

plot.survFit <- function(x, xlab = "Time", ylab = "Probability", ...) {
    plot(x$time, rep(1, length(x$time)), ylim = c(0, 1), type = "n",
         ylab = ylab, xlab = xlab, ...)
    tmp <- apply(rbind(1,x$surv), 2, function(s) lines(c(0,x$time), s,
                 type = "s"))
}


### inverse probability of censoring weights
### see van der Laan & Robins (2003)
IPCweights <- function(x, maxweight = 5) {

    if (!extends(class(x), "Surv"))
        stop(sQuote("x"), " is not a Surv object")

    event <- x[,2]
    x[,2] <- 1 - event
    km <- survfit(x ~ 1)
    Ghat <- getsurv(km, times = x[,1]) ## see github issue #54
    Ghat[event == 0] <- 1
    w <- event / Ghat
    w[w > maxweight] <- maxweight
    w
}

### extract survival probabilities
### taken from ipred:::getsurv
### DO NOT TOUCH HERE
getsurv <- function(obj, times)
{
    # get the survival probability for times from KM curve j'

    if (!inherits(obj, "survfit")) stop("obj is not of class survfit")
    # <FIXME: methods may have problems with that>
    class(obj) <- NULL
    # </FIXME>
    lt <- length(times)
    nsurv <- times

    # if the times are the same, return the km-curve

    if(length(times) == length(obj$time)) {
        if (all(times == obj$time)) return(obj$surv)
    }

    # otherwise get the km-value for every element of times separatly

    inside <- times %in% obj$time
    for (i in (1:lt)) {
        if (inside[i])
            nsurv[i] <- obj$surv[obj$time == times[i]]
        else  {
            less <- obj$time[obj$time < times[i]]
            if (length(less) == 0)
                nsurv[i] <- 1
            else
                nsurv[i] <- obj$surv[obj$time == max(less)]
        }
    }
    nsurv
}

### P-spline base-learner with (monotonicity) constraints
bmono <- function(..., constraint = c("increasing", "decreasing",
                                      "convex", "concave", "none",
                                      "positive", "negative"),
                  type = c("quad.prog", "iterative"),
                  by = NULL, index = NULL, knots = 20, boundary.knots = NULL,
                  degree = 3, differences = 2, df = 4,
                  lambda = NULL, lambda2 = 1e6, niter = 10,
                  intercept = TRUE, contrasts.arg = "contr.treatment",
                  boundary.constraints = FALSE,
                  cons.arg = list(lambda = 1e6, n = NULL, diff_order = NULL)){
                                  #type = c("none", "constant", "linear"))) {

    if (!is.null(lambda)) df <- NULL

    cll <- match.call()
    cll[[1]] <- as.name("bmono")

    mf <- list(...)

    ## change this to use lists as for knots
    if (!is.list(constraint)) {
        constraint <- match.arg(constraint)
    } else {
        c.args <- eval(formals(sys.function())[["constraint"]])
        constraint <- lapply(constraint, match.arg, choices = c.args)
    }
    if (length(mf) > 1){
        if(length(mf) != 2)
            stop("Only one or two covariates can be specified in",
                 sQuote("bmono"))
        if (length(constraint) == 1)
            constraint <- list(constraint, constraint)
    }
    ##

    type = match.arg(type)

    if (length(mf) == 1 && (is.matrix(mf[[1]]) || is.data.frame(mf[[1]]))) {
        mf <- as.data.frame(mf[[1]])
    } else {
        mf <- as.data.frame(mf)
        cl <- as.list(match.call(expand.dots = FALSE))[2][[1]]
        colnames(mf) <- sapply(cl, function(x) deparse(x))
    }
    stopifnot(is.data.frame(mf))
    if (!(all(sapply(mf, is.numeric))) && !all(sapply(mf, is.ordered))) {
        stop("cannot compute ", sQuote("bmono"),
             ": all variables must be numeric or ordered")
    }
    vary <- ""
    if (!is.null(by)){
        stopifnot(is.numeric(by) || (is.factor(by) && nlevels(by) == 2))
        mf <- cbind(mf, by)
        colnames(mf)[ncol(mf)] <- vary <- deparse(substitute(by))
    }

    CC <- all(Complete.cases(mf))
    if (!CC)
        warning("base-learner contains missing values;\n",
                "missing values are excluded per base-learner, ",
                "i.e., base-learners may depend on different",
                " numbers of observations.")
    ### option
    DOINDEX <- (nrow(mf) > options("mboost_indexmin")[[1]] ||
                is.factor(mf[[1]]))
    if (is.null(index)) {
        if (!CC || DOINDEX) {
            ### try to remove duplicated observations or
            ### observations with missings
            index <- get_index(mf)
            mf <- mf[index[[1]],,drop = FALSE]
            index <- index[[2]]
        }
    }

    ret <- list(model.frame = function()
                    if (is.null(index)) return(mf) else
                        return(mf[index,,drop = FALSE]),
                get_call = function(){
                    cll <- deparse(cll, width.cutoff=500L)
                    if (length(cll) > 1)
                        cll <- paste(cll, collapse="")
                    cll
                },
                get_data = function() mf,
                get_index = function() index,
                get_vary = function() vary,
                get_names = function() colnames(mf),
                set_names = function(value) {
                    if(length(value) != length(colnames(mf)))
                        stop(sQuote("value"), " must have same length as ",
                             sQuote("colnames(mf)"))
                    for (i in 1:length(value)){
                        cll[[i+1]] <<- as.name(value[i])
                    }
                    attr(mf, "names") <<- value
                })
    class(ret) <- "blg"
    if (!is.factor(mf[[1]])){
        args <- hyper_bbs(mf, vary, knots = knots,
                          boundary.knots =  boundary.knots,
                          degree = degree, differences = differences,
                          df = df, lambda = lambda, center = FALSE)
        args$constraint <- constraint
        args$type <- type
        args$lambda2 <- lambda2
        args$niter <- niter
        args$boundary.constraints <- boundary.constraints
        if(boundary.constraints) {
            if (is.null(cons.arg$n)){
                ## use 10% of the knots on each side per default
                cons.arg$n <- sapply(args$knots,
                                     function(x) rep(length(x$knots), 2)) * 0.1
            } else {
                if(length(cons.arg$n) != 2 && !is.list(cons.arg$n)){
                    cons.arg$n <- rep(cons.arg$n, 2)
                }
                ## <fixme> was passiert bei bivariatem bmono? </fixme>
            }
            ## diff_order for boundary constraints
            if(is.null(cons.arg$diff_order)){
                ## use same difference order as defined by "constraint":
                ## <FIXME> args$constraint may be a list of length 2 for spatial effects
                cons.arg$diff_order <- differences(args$constraint)
            }
            if(is.null(cons.arg$lambda)){
                cons.arg$lambda <- 1e6
            }
        }
        args$cons.arg <- cons.arg
        ret$dpp <- bl_mono(ret, Xfun = X_bbs, args = args)
    } else {
        args <- hyper_ols(df = df, lambda = lambda,
                          intercept = intercept,
                          contrasts.arg = contrasts.arg)
        args$constraint <- constraint
        args$type <- type
        args$lambda2 <- lambda2
        args$niter <- niter
        ## <FIXME> Was machen wir bei kateg. Effekten? Da muesste das doch auch gehen!
        args$boundary.constraints <- boundary.constraints
        args$cons.arg$n <- cons.arg$n
        ret$dpp <- bl_mono(ret, Xfun = X_ols, args = args)
    }
    return(ret)
}

bl_mono <- function(blg, Xfun, args) {
    mf <- blg$get_data()
    index <- blg$get_index()
    vary <- blg$get_vary()

    newX <- function(newdata = NULL, prediction = FALSE) {
        if (!is.null(newdata)) {
            mf <- check_newdata(newdata, blg, mf)
        }
        ## this argument is currently only used in X_bbs --> bsplines
        args$prediction <- prediction
        return(Xfun(mf, vary, args))
    }
    X <- newX()
    K <- X$K
    X <- X$X

    if (length(args$constraint) == 1) {
        D <- V <- lambda2 <- vector(mode = "list", length = 2)
        ## set up difference matrix
        D[[1]] <- differences(args$constraint, ncol(X))

        if (is.factor(mf[[1]]) && args$intercept) {
            D[[1]][1,1] <- 0
        }

        if (!is.factor(mf[[1]]) && args$boundary.constraints) {
            ## set up boundary constraints
            cons.arg <- args$cons.arg
            idx <- rep(0, ncol(X) - cons.arg$diff_order)
            if (cons.arg$n[1] == 0) {
                lower <- 0
            } else {
                lower <- 1:cons.arg$n[1]
            }
            if (cons.arg$n[2] == length(idx)) {
                upper <- 0
            } else {
                upper <- length(idx) - 1:cons.arg$n[2] + 1
            }
            idx[c(lower, upper)] <- 1
            V3 <- diag(idx)
            D3 <- V3 %*% diff(diag(ncol(X)), differences = cons.arg$diff_order)
        }

        V[[1]] <- matrix(0, ncol = nrow(D[[1]]), nrow =  nrow(D[[1]]))
        lambda2[[1]] <- ifelse(args$constraint == "none", 0, args$lambda2)
        lambda2[[2]] <- 0
        if (args$boundary.constraints) {
            lambda3 <- cons.arg$lambda
        } else {
            lambda3 <- 0
        }
    }
    if (length(args$constraint) == 2) {
        if (is.factor(mf[[1]]))
            stop(paste("Bivariate monotonic effects currently not",
                       "implemented for ordered factors"))
        ## ncol1 = length(knots[[1]]) + degree + 1
        ## ncol2 = length(knots[[2]]) + degree + 1
        ## ncol(X) = ncol1 * ncol2
        ncoli <- lapply(args$knots, function(x)
                        length(x$knots) + args$degree + 1)
        stopifnot(ncoli[[1]] * ncoli[[2]] == ncol(X))

        D <- V <- lambda2 <- vector(mode = "list", length =2)
        ## set up difference matrices
        D <- differences(args$constraint, ncoli)
        idx <- !sapply(D, is.null)
        V[idx] <- lapply(D[idx], function(m) matrix(0, nrow(m), nrow(m)))

        if (length(args$lambda2) == 1) {
            lambda2[[1]] <- lambda2[[2]] <- args$lambda2
        } else {
            lambda2 <- args$lambda2
        }
        ## set lambda2 = 0 if no constraint is used
        if (any(none <- args$constraint == "none"))
            lambda2[none] <- 0
        if (any(none <- lambda2 == 0))
            args$constraint[none] <- "none"
        ## <FIXME> Boundary constraints for bivariate smooths are currently not
        ## implemented
        if (args$boundary.constraints)
            warning("Boundary constraints for bivariate smooths",
                    "are currently not implemented")
        lambda3 <- 0
    }

    dpp <- function(weights) {
        weights[!Complete.cases(mf)] <- 0
        w <- weights
        if (!is.null(index))
            w <- .Call("R_ysum", as.double(weights), as.integer(index),
                       PACKAGE = "mboost")
        lambdadf <- df2lambda(X, df = args$df, lambda = args$lambda,
                              dmat = K, weights = w)
        lambda <- lambdadf["lambda"]
        XtX <- crossprod(X * w, X)
        XtX <- XtX + lambda * K

        if (args$type == "iterative") {
            fct <- define_solver(lambda2, lambda3, X)
            ## deparsing and parsing again needed to tidy-up code.
            mysolve <- eval(parse(text = deparse(eval(parse(text = fct)))))
        }

        fit <- function(y) {
            if (!is.null(index)) {
                y <- .Call("R_ysum", as.double(weights * y), as.integer(index),
                           PACKAGE = "mboost")
            } else {
                y <- y * weights
            }

            if (args$type == "iterative") {
                for (i in 1:args$niter){
                    coef <- mysolve(y, V)
                    if (args$constraint[[1]] == "none")
                        break ## as there is no need to iterate

                    ## compare old and new V
                    tmp1 <- violations(D[[1]] %*% coef)
                    if (lambda2[[2]] != 0)
                        tmp2 <- violations(D[[2]] %*% coef)

                    if ( all( V[[1]] == tmp1 ) &&
                        ( lambda2[[2]] == 0 || all( V[[2]] == tmp2 ) ) )
                        break    # if both are equal: done!
                        #if (args$boundary.constraints &&
                        #    all( V[[1]][-idxB, -idxB] == tmp1[-idxB, -idxB]) )
                        #    break   # if both are equal (without V for boundary
                        #            # constraints): done!
                    V[[1]] <- tmp1
                        #if (args$boundary.constraints) {
                        #    V[[1]][idxFlat, idxFlat] <- diag(rep(1, length(idxFlat)))
                        #}
                    if (lambda2[[2]] != 0)
                        V[[2]] <- tmp2
                    if (i == args$niter)
                        warning("no convergence of coef in bmono\n",
                                "You could try increasing ", sQuote("niter"),
                                " or ", sQuote("lambda2"))
                }
            } else {  ## i.e. type == "quad.prog"
                if (lambda2[[2]] == 0) {
                    coef <- solveLSEI(XtX, crossprod(X, y), D = D[[1]])
                } else {
                    coef <- solveLSEI(XtX, crossprod(X, y), D = D)
                }
            }

            ret <- list(model = coef,
                        fitted = function() {
                            ret <- as.vector(X %*% coef)
                            if (is.null(index)) return(ret)
                            return(ret[index])
                        })
            class(ret) <- c("bm_lin", "bm")
            ret
        }

        hatvalues <- function() {
            warning("hatvalues might be a very poor approximation",
                    "for monotonic base-learners.")
            if(lambda2[[2]] == 0)
                pen2 <- lambda2[[1]] * crossprod(D[[1]], V[[1]] %*% D[[1]])
            else
                pen2 <- lambda2[[1]] * crossprod(D[[1]], V[[1]] %*% D[[1]]) +
                        lambda2[[2]] * crossprod(D[[2]], V[[2]] %*% D[[2]])
            ret <- as.matrix(tcrossprod(X %*% solve(XtX + pen2), X * w))
            if (is.null(index)) return(ret)
            return(ret[index,index])
        }

        ### actually used degrees of freedom (trace of hat matrix)
        df <- function() lambdadf

        ### prepare for computing predictions
        predict <- function(bm, newdata = NULL,
                            aggregate = c("sum", "cumsum", "none")) {
            cf <- sapply(bm, coef)
            if (!is.matrix(cf)) cf <- matrix(cf, nrow = 1)
            if(!is.null(newdata)) {
                index <- NULL
                ## Use sparse data represenation if data set is huge
                ## and a data.frame
                if (is.data.frame(newdata) && nrow(newdata) > options("mboost_indexmin")[[1]]) {
                    index <- get_index(newdata)
                    newdata <- newdata[index[[1]], , drop = FALSE]
                    index <- index[[2]]
                }
                X <- newX(newdata, prediction = TRUE)$X
            }
            aggregate <- match.arg(aggregate)
            pr <- switch(aggregate, "sum" =
                as(X %*% rowSums(cf), "matrix"),
            "cumsum" = {
                as(X %*% .Call("R_mcumsum", as(cf, "matrix"),
                               PACKAGE = "mboost"), "matrix")
            },
            "none" = as(X %*% cf, "matrix"))
            if (is.null(index))
                return(pr[, , drop = FALSE])
            return(pr[index, , drop = FALSE])
        }

        ret <- list(fit = fit, hatvalues = hatvalues,
                    predict = predict, df = df,
                    Xnames = colnames(X))
        class(ret) <- c("bl_lin", "bl")
        return(ret)

    }
    return(dpp)
}

violations <- function(diffs)
    diag(c(as.numeric(diffs)) <= 0)

define_solver <- function(lambda2, lambda3, X) {
    ## define function as text and eval(parse()) later.
    l2txt <- "+ lambda2[[2]] * crossprod(D[[2]], V[[2]] %*% D[[2]])"
    l3txt <- "+ lambda3 * crossprod(D3, V3 %*% D3)"
    fct <- c("function(y, V) {",
             "    XtXC <- Cholesky(forceSymmetric(XtX +",
             "       lambda2[[1]] * crossprod(D[[1]], V[[1]] %*% D[[1]])",
             ## add if lambda2[[2]] != 0
             ifelse(lambda2[[2]] != 0, l2txt,""),
             ## add if lambda3 != 0
             ifelse(lambda3 != 0, l3txt,""),
             "                                   ))",
             "    solve(XtXC, crossprod(X, y))",
             "}"
             )

    if (!is(X, "Matrix")) {
        ## some lines must be replaced in order to solve directly
        fct[2] <- '    solve(XtX +'
        fct[6] <- "          , crossprod(X, y)"
    }
    fct
}

###
# User-specified base-learner with quadratic penalty
buser <- function(X, K = NULL,  by = NULL, index = NULL, df = 4, lambda = NULL){

    ## TODO:
    ## index should be available
    ## check if CC and index work

    if (!is.null(lambda)) df <- NULL
    cll <- match.call()
    cll[[1]] <- as.name("buser")

    mf <- as.data.frame(X) ##<FIXME> is this correct this way?

    vary <- ""
    if (!is.null(by)){
        #stopifnot(is.data.frame(mf))
        mf <- cbind(mf, by)
        colnames(mf)[ncol(mf)] <- vary <- deparse(substitute(by))
    }

    CC <- all(Complete.cases(mf))
    if (!CC)
        warning("base-learner contains missing values;\n",
                "missing values are excluded per base-learner, ",
                "i.e., base-learners may depend on different",
                " numbers of observations.")
    ### option
    DOINDEX <- (nrow(mf) > options("mboost_indexmin")[[1]])
    if (is.null(index)) {
        if (!CC || DOINDEX) {
            index <- get_index(mf)
            mf <- mf[index[[1]],,drop = FALSE]
            index <- index[[2]]
        }
    }

    ret <- list(model.frame = function()
                    if (is.null(index)) return(mf) else return(mf[index,,drop = FALSE]),
                get_call = function(){
                    cll <- deparse(cll, width.cutoff=500L)
                    if (length(cll) > 1)
                        cll <- paste(cll, collapse="")
                    cll
                },
                get_data = function() mf,
                get_index = function() index,
                get_vary = function() vary,
                get_names = function() colnames(mf),
                set_names = function(value) {
                    if(length(value) != length(colnames(mf)))
                        stop(sQuote("value"), " must have same length as ",
                             sQuote("colnames(mf)"))
                    for (i in 1:length(value)){
                        cll[[i+1]] <<- as.name(value[i])
                    }
                    attr(mf, "names") <<- value
                })
    class(ret) <- "blg"

    ret$dpp <- bl_lin(ret, Xfun = X_user,
                      args = hyper_user(mf, vary, K, df, lambda))
    return(ret)
}


### model.matrix for arbitrary user-specified base-learners
X_user <- function(mf, vary, args) {
    X <- mf
    K <- args$K
    if (vary != "") {
        by <- model.matrix(as.formula(paste("~", vary, collapse = "")),
                           data = as.data.frame(mf))[ , -1, drop = FALSE]
        X <- X[,colnames(mf) != vary]
        DM <- lapply(1:ncol(by), function(i) {
            ret <- X * by[, i]
            colnames(ret) <- paste(colnames(ret), colnames(by)[i], sep = ":")
            ret
        })
        X <- do.call("cbind", DM)
        if (ncol(by) > 1)
            suppressMessages(K <- kronecker(diag(ncol(by)), K))
    }
    X <- as.matrix(X)
    # contr <- NULL ##<FIXME> Do we need this?
    return(list(X = X, K = K))
}

### hyper parameters for arbitrary user-specified base-learners
hyper_user <- function(mf, vary, K, df, lambda) {
    if (is.null(K)){
        K <- diag(ncol(mf[,colnames(mf) != vary]))
        lambda <- 0
        df <- NULL
    }
    list(df = df, lambda = lambda, K = K)
}

options(digits = 3)

tol2 <- .Machine$double.eps^(1/3)

.all.equal <- function(...) isTRUE(all.equal(..., check.environment = FALSE))

library("mboost")
attach(asNamespace("mboost"), warn.conflicts = FALSE)
library("MASS")
library("Matrix")

set.seed(290875)

### dgp
n <- 20000
xn <- round(runif(n), 3)
xn[sample(1:n)[1:(n / 100)]] <- NA
xf <- gl(4, n / 4)
xf[sample(1:n)[1:(n / 100)]] <- NA
z1 <- sample(gl(2, n / 2))
z1[sample(1:n)[1:(n / 100)]] <- NA
z2 <- round(runif(n), 3)
z2[sample(1:n)[1:(n / 100)]] <- NA
w <- rpois(n, lambda = 2)
y <- 2 * xn + rnorm(n)
y[is.na(y)] <- rnorm(sum(is.na(y)))

testfun <- function(m1, m2) {
    ret <- c(max(abs(coef(m1) - coef(m2))),
      max(abs(fitted(m1) - fitted(m2)()), na.rm = TRUE))
    if (any(ret > sqrt(.Machine$double.eps)))
        return(ret)
}

### numeric x with intercept
m1 <- lm(y ~ xn, weights = w, na.action = na.exclude)
m2 <- fit(dpp(bols(xn), w), y)
testfun(m1, m2)

### numeric x without intercept
m1 <- lm(y ~ xn - 1, weights = w, na.action = na.exclude)
m2 <- fit(dpp(bols(xn, intercept = FALSE), w), y)
testfun(m1, m2)

### factor x with intercept
m1 <- lm(y ~ xf, weights = w, na.action = na.exclude)
m2 <- fit(dpp(bols(xf), w), y)
testfun(m1, m2)

### factor x without intercept
tmp <- model.matrix(~ xf)[,-1] ## build model matrix without first row
mm <- matrix(NA, ncol = ncol(tmp), nrow = length(y))
mm[!is.na(xf),] <- tmp ## build model matrix with missings
m1 <- lm(y ~ mm - 1, weights = w, na.action = na.exclude)
m2 <- fit(dpp(bols(xf, intercept = FALSE), w), y)
testfun(m1, m2)

### factor x with "contr.dummy"
m1 <- lm(y ~ xf - 1, weights = w, na.action = na.exclude)
m2 <- fit(dpp(bols(xf, contrasts.arg = "contr.dummy"), w), y)
testfun(m1, m2)

### contrasts
m1 <- lm(y ~ xf, weights = w, contrasts = list(xf = "contr.sum"), na.action = na.exclude)
m2 <- fit(dpp(bols(xf, contrasts.arg = list(xf = "contr.sum")), w), y)
testfun(m1, m2)

### multiple x
m1 <- lm(y ~ xn + xf, weights = w, na.action = na.exclude)
m2 <- fit(dpp(bols(xn, xf), w), y)
testfun(m1, m2)

### interaction with binary factor
xtmp <- (z1 == "2") * xn
m1 <- lm(y ~ xtmp - 1, weights = w, na.action = na.exclude)
m2 <- fit(dpp(bols(xn, by = z1, intercept = FALSE), w), y)
testfun(m1, m2)

### interaction with numeric variable
m1 <- lm(y ~ z2:xn - 1, weights = w, na.action = na.exclude)
m2 <- fit(dpp(bols(z2, by = xn, intercept = FALSE), w), y)
testfun(m1, m2)

### ridge
one <- rep(1, n)
cf1 <- coef(lm.ridge(y ~ one + xn - 1, lambda = 2))
cf2 <- coef(fit(dpp(bols(one, xn, lambda = 2, intercept = FALSE), rep(1, n)), y))
max(abs(cf1 - cf2))
cf1 <- coef(lm.ridge(y ~ xf - 1, lambda = 2))
cf2 <- coef(fit(dpp(bols(xf, lambda = 2, intercept = FALSE), rep(1, n)), y))
max(abs(cf1 - cf2))

### matrix (here with missing values)
cf1 <- coef(mod <- lm(y ~ xn + xf * z1 - 1, weights = w, y = TRUE, x = TRUE))
tX <- mod$x
tw <- mod$weights
ty <- mod$y
cf2 <- coef(fit(dpp(bols(tX), weights = tw), ty))
max(abs(cf1 - cf2)) < tol2

### ridge again with matrix interface
tX <- matrix(runif(1000), ncol = 10)
ty <- rnorm(100)
tw <- rep(1, 100)

### compute & check df
op <- options(mboost_dftraceS = TRUE)
la <- df2lambda(tX, df = 2, dmat = diag(ncol(tX)), weights = tw)["lambda"]
truedf <- sum(diag(tX %*% solve(crossprod(tX * tw, tX) + la * diag(ncol(tX))) %*% t(tX * tw)))
abs(truedf - 2) < tol2

one <- rep(1, ncol(tX))
cf1 <- coef(lm.ridge(ty ~ . - 1, data = as.data.frame(tX), lambda = la))
cf2 <- coef(fit(dpp(bols(tX, df = 2), weights = tw), ty))
max(abs(cf1 - cf2))
# I think bols is better and thus right
sum((ty - tX %*% cf1)^2) + la * sum(cf1^2)
sum((ty - tX %*% cf2)^2) + la * sum(cf2^2)
options(op)

### now with other df-definition:
op <- options(mboost_dftraceS = FALSE)
la <- df2lambda(tX, df = 2, dmat = diag(ncol(tX)), weights = tw)["lambda"]
H <- tX %*% solve(crossprod(tX * tw, tX) + la * diag(ncol(tX))) %*% t(tX * tw)
truedf <- sum(diag(2*H - tcrossprod(H,H)))
abs(truedf - 2) < tol2
options(op)

# check df with weights
op <- options(mboost_dftraceS = TRUE)
tw <- rpois(100, 2)
la <- df2lambda(tX, df = 2, dmat = diag(ncol(tX)), weights = tw)["lambda"]
truedf <- sum(diag(tX %*% solve(crossprod(tX * tw, tX) + la * diag(ncol(tX))) %*% t(tX * tw)))
abs(truedf - 2) < tol2

### check df2lambda for P-splines (Bug spotted by B. Hofner)
set.seed(1907)
x <- runif(100, min = -1, max = 3)
## extract lambda from base-learner
lambda <- bbs(x, df = 4)$dpp(rep(1, length(x)))$df()["lambda"]
X <- get("X", envir = environment(bbs(x, df = 4)$dpp))
K <- get("K", envir = environment(bbs(x, df = 4)$dpp))
truedf <- sum(diag(X %*%  solve(crossprod(X,X) + lambda * K) %*% t(X)))
abs(truedf - 4) < tol2

### check accuracy of df2lambda
data("bodyfat", package="TH.data")
diff_df <- matrix(NA, nrow=8, ncol=ncol(bodyfat))
rownames(diff_df) <- paste("df", 3:10)
colnames(diff_df) <- names(bodyfat)
for (i in 3:10){
    for (j in 1:ncol(bodyfat)){
        lambda <- bbs(bodyfat[[j]], df = i)$dpp(rep(1, nrow(bodyfat)))$df()["lambda"]
        diff_df[i-2,j] <- bbs(bodyfat[[j]], lambda = lambda)$dpp(rep(1, nrow(bodyfat)))$df()["df"] - i
    }
}
all(diff_df < tol2)
options(op)

### check degrees of freedom for design matrices without full rank:
x <- sample(1:3, 100, replace = TRUE)
X <- extract(bbs(x))
rankMatrix(X)
## df2lambda:
df2lambda(X, df = NULL, lambda = 0, weights = rep(1, 100))[["df"]] == 3
(res <- df2lambda(X, df = 4, weights = rep(1, 100)))

### componentwise
cf2 <- coef(fit(dpp(bolscw(cbind(1, xn)), weights = w), y))
cf1 <- coef(lm(y ~ xn - 1, weights = w))
max(abs(cf1 - max(cf2))) < tol2

cf2 <- coef(fit(dpp(bolscw(matrix(xn, nc = 1)), weights = w), y))
cf1 <- coef(lm(y ~ xn - 1, weights = w))
max(abs(cf1 - max(cf2))) < tol2

### componentwise with matrix
n <- 200
m <- 10000
x <- rnorm(n * m)
x[abs(x) < 2] <- 0
X <- Matrix(data = x, ncol = m, nrow = n)
beta <- rpois(ncol(X), lambda = 1)
y <- X %*% beta + rnorm(nrow(X))
w <- rep(1, nrow(X)) ###rpois(nrow(X), lambda = 1)
f1 <- dpp(bolscw(X), weights = w)$fit
f1(y)$model

### varying coefficients
x1 <- runif(n, max = 2)
x2 <- sort(runif(n, max = 2 * pi))
y <- sin(x2) * x1 + rnorm(n)
w <- rep(1, n)

d <- dpp(bbs(x2, by = x1, df = 4), w)
f <- fit(d, y)
f2 <- d$predict(list(f), newdata = data.frame(x1 = 1, x2 = x2))

max(abs(sin(x2) - f2))

### bols and bbs; matrix interfaces
n <- 10000
x <- runif(n, min = 0, max = 2*pi)
y <- sin(x) + rnorm(n, sd = 0.1)
w <- rpois(n, lambda = 1)
x[sample(1:n)[1:(n / 100)]] <- NA
h <- hyper_bbs(data.frame(x = x), vary = "")
X <- X_bbs(data.frame(x = x), vary = "", h)$X
f1 <- fit(dpp(bbs(x, df = ncol(X)), w), y)
f2 <- fit(dpp(bols(X, df = ncol(X)), w), y)
max(abs(coef(f1) - coef(f2))) < tol2

.all.equal(get_index(data.frame(x, x)), get_index(X))
.all.equal(get_index(data.frame(x)), get_index(X))

### check handling of missings for cyclic effects
h <- hyper_bbs(data.frame(x = x), vary = "", cyclic = TRUE)
X <- X_bbs(data.frame(x = x), vary = "", h)$X
all(is.na(X[is.na(x),]))
all(!is.na(X[!is.na(x),]))

### combinations and tensor products of base-learners

set.seed(29)
n <- 1000
x1 <- rnorm(n)
x2 <- rnorm(n)
x3 <- rnorm(n)
f <- gl(4, 25)
y <- rnorm(n)
ndf <- data.frame(x1 = x1[1:10], x2 = x2[1:10], f = f[1:10])

### spatial
m1 <- gamboost(y ~ bbs(x1) %X% bbs(x2))
m2 <- gamboost(y ~ bspatial(x1, x2, df = 16))
max(abs(predict(m1) - predict(m2))) < tol2
max(abs(predict(m1, newdata = ndf) - predict(m2, newdata = ndf))) < tol2

### spatio-temporal
m1 <- gamboost(y ~ bbs(x1, knots = 6) %X% bbs(x2, knots = 6) %X% bbs(x3, knots = 6))
m2 <- gamboost(y ~ (bbs(x1, knots = 6) + bbs(x2, knots = 6)) %X% bbs(x3, knots = 6))

### varying numeric
m1 <- gamboost(y ~ bbs(x1) %X% bols(x2, intercept = FALSE, lambda = 0))
m2 <- gamboost(y ~ bbs(x1, by = x2, df = 4))
max(abs(predict(m1) - predict(m2))) < tol2
max(abs(predict(m1, newdata = ndf) - predict(m2, newdata = ndf))) < tol2

### varying factor
m1 <- gamboost(y ~ bbs(x1) %X% bols(f, df = 5, contrasts.arg = "contr.dummy"))
coef(m1)
predict(m1, newdata = ndf)

### cbind
m1 <- gamboost(y ~ bols(x1, intercept = FALSE, df = 1) %+%
                   bols(x2, intercept = FALSE, df = 1))
m2 <- gamboost(y ~ bols(x1, x2, intercept = FALSE, df = 2))
max(abs(predict(m1) - predict(m2))) < tol2
max(abs(predict(m1, newdata = ndf) - predict(m2, newdata = ndf))) < tol2

### yeah
m1 <- gamboost(y ~ (bols(x1, intercept = FALSE, df = 1) %+%
                    bols(x2, intercept = FALSE, df = 1)) %X% bols(f, df = 4) +
                    bbs(x1) + bspatial(x1, x2))


### test bmono with categorical covariates
set.seed(781)
x <- as.ordered(gl(4, 50))
y <- rnorm(200, mean = rep(c(1,3,2,4), each = 50), sd = 1)
mod1 <- gamboost(y ~ bols(x))
diff(coef(mod1)[[1]])
mod2 <- gamboost(y ~ bmono(x, lambda2 = 10^15))
abs(coef(mod1)[[1]] - coef(mod2)[[1]])[c(1,4)] < 1e-5
all(diff(coef(mod2)[[1]]) > - tol2)

### test bmono for tensor-product splines
x1 <- runif(100, min = -2, max = 3)
x2 <- runif(100, min = -2, max = 3)
f <- function(x1, x2)
    0.5 + x1^2 * (x2 + 3)
y <- rnorm(100, mean = f(x1, x2), sd = 0.5)
mod1 <- mboost(y ~ bspatial(x1, x2, df = 6, knots = list(x1 = 10, x2 = 5)),
               control = boost_control(mstop = 50))
mod21 <- mboost(y ~ bmono(x1, x2, df = 6, knots = list(x1 = 10, x2 = 5),
                          lambda2 = list(x1 = 10e6, x2 = 0)),
                control = boost_control(mstop = 50))
mod22 <- mboost(y ~ bmono(x1, x2, df = 6, knots = list(x1 = 10, x2 = 5),
                          lambda2 = list(x1 = 0, x2 = 10e6)),
                control = boost_control(mstop = 50))

diff_order <- 1
D1 <- kronecker(diff(diag(10 + 3 + 1), differences = diff_order),
                diag(5 + 3 + 1))
D2 <- kronecker(diag(10 + 3 + 1), diff(diag(5 + 3 + 1),
                              differences = diff_order))

beta <- coef(mod1)[[1]]
round(sum((D1 %*% beta)[D1 %*% beta <= 0]), 3)
round(sum((D2 %*% beta)[D2 %*% beta <= 0]), 3)

beta <- coef(mod21)[[1]]
round(sum((D1 %*% beta)[D1 %*% beta <= 0]), 3)
round(sum((D2 %*% beta)[D2 %*% beta <= 0]), 3)
all(D1 %*% beta > - 2e-05)

beta <- coef(mod22)[[1]]
round(sum((D1 %*% beta)[D1 %*% beta <= 0]), 3)
round(sum((D2 %*% beta)[D2 %*% beta <= 0]), 3)
all(D2 %*% beta > - 2e-05)

#nd <- expand.grid(sort(x1), sort(x2))
#names(nd) <- c("x1", "x2")
#contour(sort(x1), sort(x2),
#        z = matrix(predict(mod1, newdata = nd), ncol = length(x1)))
#contour(sort(x1), sort(x2),
#        z = matrix(predict(mod21, newdata = nd), ncol = length(x1)),
#        col = "red", add = TRUE)
#contour(sort(x1), sort(x2),
#        z = matrix(predict(mod22, newdata = nd), ncol = length(x1)),
#        col = "green", add = TRUE)

### %O%: kronecker product of baselearners for matrix-valued
### responses
x1 <- 1:10/10
x2 <- 11:17/17

X1 <- cbind(1, x1, x1^2)
colnames(X1) <- paste("X1", 1:ncol(X1), sep = "_")
X2 <- cbind(1, x2, x2^2, x2^3)
colnames(X2) <- paste("X2", 1:ncol(X2), sep = "_")

x <- expand.grid(x1, x2)
colnames(x) <- c("x1", "x2")
B1 <- with(x, cbind(1, x1, x1^2))
colnames(B1) <- paste("B1", 1:ncol(B1), sep = "_")
B2 <- with(x, cbind(1, x2, x2^2, x2^3))
colnames(B2) <- paste("B2", 1:ncol(B2), sep = "_")

X <- kronecker(X2, X1)
w <- rep(1, nrow(X))

B <- kronecker(matrix(1, ncol = ncol(B2)), B1) *
     kronecker(B2, matrix(1, ncol = ncol(B1)))

tol3 <- 1 / 10000
max(abs(X - B)) < tol3

K1 <- diag(ncol(B1))
K2 <- crossprod(diff(diag(ncol(B2)), diff = 2))

b1 <- buser(B2, K = diag(ncol(B2))) %X%
            buser(B1, K = diag(ncol(B1)))

max(abs(extract(b1, "design") - B)) < tol3
b1d <- b1$dpp(w)

b2 <- buser(X1, K = diag(ncol(X1))) %O%
      buser(X2, K = diag(ncol(X2)))
b2d <- b2$dpp(w)

max(abs(get("XtX", environment(b1d$fit)) -
        get("XtX", environment(b2d$fit)))) < tol3

y <- runif(nrow(X))

max(abs(b1d$fit(y)$model - as.vector(b2d$fit(y)$model))) < 1/1000

m1 <- b1d$fit(y)
p1 <- b1d$predict(list(m1, m1))

m2 <- b2d$fit(y)
p2 <- b2d$predict(list(m2, m2))

max(abs(p1 - p2)) < tol3


x1 <- runif(200)
x2 <- runif(60)
y <- rnorm(length(x1) * length(x2))
d <- expand.grid(x1, x2)
d$y <- y
w <- as.vector(rmultinom(1, length(y), rep(1 / length(y), length(y))))

m1 <- mboost(y ~ bbs(Var2, df = 3, knots = 5) %X%
                 bbs(Var1, df = 3, knots = 7),
             data = d, weights = w)
m2 <- mboost(y ~ bbs(x1, df = 3, knots = 7) %O%
                 bbs(x2, df = 3, knots = 5),
             weights = w)

max(abs(fitted(m1) - fitted(m2))) < tol3

max(abs(coef(m1)[[1]] - coef(m2)[[1]])) < tol3

max(abs(m1$predict() - m2$predict())) < tol3

p1 <- predict(m1, newdata = expand.grid(Var1 = c(0.2, 0.5), Var2 = c(0.7, 0.3)))
p2 <- predict(m2, newdata = data.frame(x1 = c(0.2, 0.5), x2 = c(0.7, 0.3)))

max(abs(p1 - p2)) < tol3

### large data set with ties
nunique <- 100
xindex <- sample(1:nunique, 1000000, replace = TRUE)
x <- runif(nunique)
y <- rnorm(length(xindex))
w <- rep.int(1, length(xindex))
### brute force computations
op <- options()
options(mboost_indexmin = Inf, mboost_useMatrix = FALSE)
## data pre-processing
b1 <- bbs(x[xindex])$dpp(w)
## model fitting
c1 <- b1$fit(y)$model
options(op)
### automatic search for ties, faster
b2 <- bbs(x[xindex])$dpp(w)
c2 <- b2$fit(y)$model
### manual specification of ties, even faster
b3 <- bbs(x, index = xindex)$dpp(w)
c3 <- b3$fit(y)$model
.all.equal(c1, c2)
.all.equal(c1, c3)

### new T spline monotonicity
library("lattice")

options(mboost_useMatrix = FALSE)
x <- sort(runif(100, max = 2))
y <- sin(x) + rnorm(100, sd = .1) + 10
layout(matrix(1:3, nc = 3))
plot(x, y)
m1 <- mboost(y ~ bbs(x))
lines(x, fitted(m1))
plot(x, y)
m2 <- mboost(y ~ bbs(x, constraint = "increasing"))
lines(x, fitted(m2))
plot(x, -y)
m3 <- mboost(I(-y) ~ bbs(x, constraint = "decreasing"))
lines(x, fitted(m3))


### penalty problem -- penalize differences???
bl <- bbs(x, constraint = "increasing", lambda = 100)
lines(x, bl$dpp(rep(1, length(y)))$fit(y)$fitted(), col = "red")

x1 <- seq(from = -3, to = 3, by = .1)
x2 <- seq(from = 0, to = 2 * pi, by = .1)
m2 <- sin(x2)
y <- sapply(m2, function(m) pnorm(x1, mean = m))
x <- expand.grid(x1 = x1, x2 = x2)
y <- x$y <- as.vector(y) + runif(nrow(x), min = -.3, max = .3)

wireframe(y ~ x1 + x2, data = x)

m1 <- mboost(y ~ bbs(x1) %O% bbs(x2))
x$p1 <- fitted(m1)
wireframe(p1 ~ x1 + x2, data = x)

m2 <- mboost(y ~ bbs(x1, constraint = "increasing", df = 10) %O% bbs(x2))
x$p2 <- fitted(m2)
wireframe(p2 ~ x1 + x2, data = x)

m3 <- mboost(I(-y) ~ bbs(x1, constraint = "decreasing", df = 10) %O% bbs(x2))
x$p3 <- fitted(m3)
wireframe(p3 ~ x1 + x2, data = x)


### check brandom
x1 <- rnorm(100)
x2 <- rnorm(100)
z1 <- as.factor(sample(1:10, 100, TRUE))
z2 <- as.factor(sample(1:10, 100, TRUE))
Zm <- model.matrix(~ z1 - 1)
Z <- as.data.frame(Zm)

extract(brandom(z1))
extract(brandom(z1, by = x2))
extract(brandom(Zm))
## probably non-sense but ok...
extract(brandom(Z))
## not really useful but might be ok
extract(brandom(z1, z2))
## should throw an error
try(extract(brandom(x1, by = x2, intercept = FALSE)))

## check if one can specify either df or lambda
round(extract(brandom(z1, df = 3)$dpp(rep(1, 100)), what = "lambda"), 2)
round(extract(brandom(z1, df = 3)$dpp(rep(1, 100)), what = "df"), 2)
round(extract(brandom(z1, lambda = 50.39)$dpp(rep(1, 100)), what = "lambda"), 2)
round(extract(brandom(z1, lambda = 50.39)$dpp(rep(1, 100)), what = "df"), 2)


### check if data beyond boundary knots is permitted
set.seed(1234)
x <- rnorm(100)
y <- sin(x) + rnorm(100, sd = 0.1)
plot(x, y, xlim = c(-3, 5))
## should not work:
try(mod <- mboost(y ~ bbs(x, boundary.knots = c(-1, 1))))
try(mod <- mboost(y ~ bbs(x, cyclic = TRUE, boundary.knots = c(-1, 1))))
## now fit models and check linear extrapolation
mod <- mboost(y ~ bbs(x))
tail(pr <- predict(mod, newdata = data.frame(x = seq(-3, 5, by = 0.1))))
lines(seq(-3, 5, by = 0.1), pr)
## now with bmono
mod <- mboost(y ~ bmono(x))
tail(pr2 <- predict(mod, newdata = data.frame(x = seq(-3, 5, by = 0.1))))
lines(seq(-3, 5, by = 0.1), pr2, col = "red")
## check same with cyclic splines
mod <- mboost(y ~ bbs(x, cyclic = TRUE))
try(predict(mod, newdata = data.frame(x = seq(-3, 5, by = 0.1))))

## make sure bols works for factors with unobserved levels breaks
## (https://github.com/boost-R/mboost/issues/47)
x <- rnorm(100)
z <- factor(sample(1:5, 100, replace = TRUE), levels = 1:6)
z2 <- factor(sample(1:2, 100, replace = TRUE), levels = 0:5)
z3 <- factor(sample(1:2, 100, replace = TRUE))
y <- rnorm(100)
mod <- mboost(y ~ bols(z3)) # no warning
# warnings but it works
mod <- mboost(y ~ bols(z))
mod <- mboost(y ~ bols(x, by = z))
mod <- mboost(y ~ bols(z2, by = z))
mod <- mboost(y ~ bols(z2, by = z3))

### bkernel
### removed in 2.9-10 because kangar00 often fails to check
#if (require("kangar00")) {
#    tmpdir <- tempdir()
#    wd <- setwd(tmpdir)
#    download.file("https://downloads.hindawi.com/journals/cmmm/2017/6742763.f2.zip", 
#                  destfile = "bkernel.zip", extra = "--no-check-certificate", 
#                  method = "wget", quiet = TRUE)
#    unzip("bkernel.zip")
#    txt <- readLines("Kernel_Boosting_example_code.R")
#    writeLines(txt[-c(1:10, 149:length(txt))], con = "run.R")
#    source("run.R", echo = FALSE)
#    setwd(wd)
#}

### the classical tree-based baselearner; stumps by default
### (also fits an additive model)
btree <- function(..., by = NULL, nmax = Inf,
    tree_controls = partykit::ctree_control(
        teststat = "quad",
        testtype = "Teststatistic",
#        splittest = TRUE,
        mincriterion = 0,
        minsplit = 10,
        minbucket = 4,
        maxdepth = 1, 
        saveinfo = FALSE))
{

    cll <- match.call()
    cll[[1]] <- as.name("btree")

    ctrl <- tree_controls
    mf <- list(...)
    if (length(mf) == 1 && is.data.frame(mf[[1]])) {
        mf <- mf[[1]]
    } else {
        mf <- as.data.frame(mf)
        cl <- as.list(match.call(expand.dots = FALSE))[2][[1]]
        colnames(mf) <- sapply(cl, function(x) as.character(x))
    }

    vary <- ""
    if (!is.null(by)){
        vary <- deparse(substitute(by))
        stopifnot(is.factor(by) || all(by %in% 0:1))
        if (is.factor(by)) {
            stopifnot(nlevels(by) == 2L)
            xlev <- levels(by)
            iby <- (0:1)[by]
        } else {
            iby <- by
        }
    } else {
        iby <- rep(1, NROW(mf))
    }

    mffun <- function() {
        if (vary == "") return(mf)
        ret <- cbind(mf, by)
        colnames(ret)[ncol(ret)] <- vary
        ret
    }

    ret <- list(model.frame = mffun,
                get_call = function(){
                    cll <- deparse(cll, width.cutoff=500L)
                    if (length(cll) > 1)
                        cll <- paste(cll, collapse="")
                    cll
                },
                get_names = function() colnames(mf),
                get_vary = function() vary,
                set_names = function(value) {
                    if(length(value) != length(colnames(mf)))
                        stop(sQuote("value"), " must have same length as ",
                             sQuote("colnames(mf)"))
                    for (i in 1:length(value)){
                        cll[[i+1]] <<- as.name(value[i])
                    }
                    attr(mf, "names") <<- value
                })
    class(ret) <- "blg"


    ret$dpp <- function(weights) {

        ### construct design matrix etc.
        y <- vector(length = nrow(mf), mode = "numeric")
        ### name for working response (different from any x)
        rname <- paste(sample(LETTERS, 25, replace = TRUE), collapse = "")
        fm <- as.formula(paste(rname, " ~ ", paste(colnames(mf), collapse = "+")))
        df <- mffun()
        df[[rname]] <- y
        d <- extree_data(fm, data = df, yx = "none", 
                         nmax = c(yx = Inf, z = nmax))
        Y <- NULL
        ytrafo <- function(subset, weights, info, estfun, object, ...) 
            list(estfun = Y, unweighted = TRUE) 
        mymf <- model.frame(d)
        tree_controls$update <- FALSE
        subset <- which(weights > 0 & iby == 1)
        weights <- weights * iby

        fitfun <- function(y) {
            if (!is.matrix(y)) y <- matrix(y, ncol = 1)
 
            assign("Y", y, envir = environment(ytrafo))
            tree <- extree_fit(data = d, trafo = ytrafo, converged = TRUE, 
                               partyvars = d$variables$z, subset = subset, 
                               weights = weights, ctrl = tree_controls, 
                               doFit = TRUE)$node
            where <- factor(fitted_node(tree, mymf))
            coef <- do.call("rbind", tapply(1:NROW(y), where, function(i)
                colSums(y[i,,drop = FALSE] * weights[i]) / sum(weights[i]), 
                simplify = FALSE))

            fitted <- function()
                return(coef[unclass(where),,drop = FALSE] * iby)

            predict <- function(newdata = NULL) {
                if (is.null(newdata)) newdata <- mffun()
                vmatch <- match(names(mymf), names(newdata))
                wh <- factor(fitted_node(tree, newdata, vmatch = vmatch), 
                             levels = levels(where), labels = levels(where))
                ret <- coef[unclass(wh),,drop = FALSE]
                if (vary != "") {
                    by <- newdata[, vary]
                    stopifnot(is.factor(by) || all(by %in% 0:1))
                    if (is.factor(by)) {
                        stopifnot(isTRUE(all.equal(levels(by), xlev)))
                        by <- (0:1)[by]
                    }
                    ret <- ret * by
                }
                return(ret)
            }

            ret <- list(model = tree, fitted = fitted, predict = predict)
            class(ret) <- c("bm_tree", "bm")
            ret
        }

        predict <- function(bm, newdata = NULL, aggregate = c("sum", "cumsum", "none")) {
            aggregate <- match.arg(aggregate)

            if (is.null(newdata)) 
                newdata <- mffun()
            if (!is.data.frame(newdata))
                newdata <- as.data.frame(newdata)

            pr <- 0
            for (i in 1:length(bm)) {
                pri <- bm[[i]]$predict(newdata)
                if (aggregate == "sum") {
                    pr <- pr + pri
                } else {
                    if (i > 1) {
                        pr <- cbind(pr, pri)
                    } else {
                        pr <- pri
                    }
                    if (aggregate == "cumsum")
                        if (i > 1) pr[,i] <- pr[,i] + pr[,i-1]
                }
            }
            return(pr)
        }

        ret <- list(fit = fitfun, predict = predict)
        class(ret) <- c("bl_tree", "bl")
        ret
    }
    return(ret)
}

### compute Ridge shrinkage parameter lambda from df
### or the other weightsay round
df2lambda <- function(X, df = 4, lambda = NULL, dmat = NULL, weights,
                      XtX = NULL) {

    stopifnot(xor(is.null(df), is.null(lambda)))
    if (!is.null(df)) {
        rank_X <- rankMatrix(X, method = 'qr', warn.t = FALSE)
        if (df >= rank_X) {
            if (df > rank_X)
                warning(sQuote("df"),
                        " too large:\n  Degrees of freedom cannot be larger",
                        " than the rank of the design matrix.\n",
                        "  Unpenalized base-learner with df = ",
                        rank_X, " used. Re-consider model specification.")
            return(c(df = df, lambda = 0))
        }
    }
    if (!is.null(lambda))
        if (lambda == 0)
            return(c(df = rankMatrix(X), lambda = 0))

    # Demmler-Reinsch Orthogonalization (cf. Ruppert et al., 2003,
    # Semiparametric Regression, Appendix B.1.1).

    ### there may be more efficient ways to compute XtX, but we do this
    ### elsewhere (e.g. in %O%)
    if (is.null(XtX)) 
        XtX <- crossprod(X * sqrt(weights))
    if (is.null(dmat)) {
        if(is(XtX, "Matrix")) diag <- Diagonal
        dmat <- diag(ncol(XtX))
    }
    ## avoid that XtX matrix is not (numerically) singular
    A <- XtX + dmat * options("mboost_eps")[[1]]
    ## make sure that A is also numerically positiv semi-definite
    A <- make_psd(as.matrix(A))
    ## make sure that A is also numerically symmetric
    if (is(A, "Matrix"))
        A <- forceSymmetric(A)
    Rm <- backsolve(chol(A), x = diag(ncol(XtX)))
    ## singular value decomposition without singular vectors
    d <- try(svd(crossprod(Rm, dmat) %*% Rm, nu=0, nv=0)$d)
    ## if unsucessfull try the same computation but compute singular vectors as well
    if (inherits(d, "try-error"))
        d <- svd(crossprod(Rm, dmat) %*% Rm)$d

    ### option
    if (options("mboost_dftraceS")[[1]]){
        ## df := trace(S)
        dfFun <- function(lambda) sum(1 / (1 + lambda * d))
    } else {
        ## df := trace(2S - S'S)
        dfFun <- function(lambda) 2 * sum( 1/(1+lambda*d) ) - sum( 1/(1+lambda*d)^2 )
    }

    if (!is.null(lambda))
        return(c(df = dfFun(lambda), lambda = lambda))
    if (df >= length(d)) return(c(df = df, lambda = 0))

    # search for appropriate lambda using uniroot
    df2l <- function(lambda)
        dfFun(lambda) - df

    ### option
    lambdaMax <- options("mboost_lambdaMax")[[1]]

    if (df2l(lambdaMax) > 0){
        if (df2l(lambdaMax) > sqrt(.Machine$double.eps))
            warning("lambda needs to be larger than ", lambdaMax, " for given ",
                    sQuote("df"), ";\n  setting lambda = ", lambdaMax,
                    " leeds to an deviation from ", sQuote("df"), " of ",
                    df2l(lambdaMax), ";\n  You can increase lambda_max via ",
                    sQuote("options(mboost_lambdaMax = value)"))
        return(c(df = df, lambda = lambdaMax))
    }
    lambda <- uniroot(df2l, c(0, lambdaMax), tol = sqrt(.Machine$double.eps))$root
    if (abs(df2l(lambda)) > sqrt(.Machine$double.eps))
        warning("estimated degrees of freedom differ from ", sQuote("df"),
                " by ", df2l(lambda))
    return(c(df = df, lambda = lambda))
}

### hyper parameters for ols baselearner
hyper_ols <- function(df = NULL, lambda = 0, intercept = TRUE,
                      contrasts.arg = "contr.treatment")
    list(df = df, lambda = lambda,
         intercept = intercept,
         contrasts.arg = contrasts.arg)

### model.matrix for ols baselearner
X_ols <- function(mf, vary, args) {

    if (isMATRIX(mf)) {
        X <- mf
        contr <- NULL
    } else {
        ### set up model matrix
        fm <- paste("~ ", paste(colnames(mf)[colnames(mf) != vary],
                    collapse = "+"), sep = "")
        fac <- sapply(mf[colnames(mf) != vary], is.factor)
        DUMMY <- FALSE
        if (any(fac)){
            if (!is.list(args$contrasts.arg)){
                ## first part needed to prevent warnings from calls such as
                ## contrasts.arg = contr.treatment(4, base = 1):
                if (DUMMY <- (is.character(args$contrasts.arg) &&
                    args$contrasts.arg == "contr.dummy")){ ## store on DUMMY for later use
                    if (!args$intercept)
                        stop('"contr.dummy" can only be used with ',
                             sQuote("intercept = TRUE"))
                    fm <- paste(fm, "-1")
                    args$contrasts.arg <- "contr.treatment"
                } 
                txt <- paste("list(", paste(colnames(mf)[colnames(mf) != vary][fac],
                                            "= args$contrasts.arg", collapse = ", "),")")
                args$contrasts.arg <- eval(parse(text=txt))
            } else {
                ## if contrasts are given as list check if "contr.dummy" is specified
                if (any(args$contrasts.arg == "contr.dummy"))
                    stop('"contr.dummy"',
                         " can only be used for all factors at the same time.\n",
                         "Use ", sQuote('contrasts.arg = "contr.dummy"'),
                         " to achieve this.")
            }
        } else {
            args$contrasts.arg <- NULL
        }
        X <- model.matrix(as.formula(fm), data = mf, contrasts.arg = args$contrasts.arg)
        if (DUMMY)
            attr(X, "contrasts") <- lapply(attr(X, "contrasts"),
                                           function(x) x <- "contr.dummy")
        contr <- attr(X, "contrasts")
        if (!args$intercept)
            X <- X[ , -1, drop = FALSE]
        MATRIX <- any(dim(X) > c(500, 50)) && any(fac)
        MATRIX <- MATRIX && options("mboost_useMatrix")$mboost_useMatrix
        if (MATRIX) {
            diag <- Diagonal
            if (!is(X, "Matrix"))
                X <- Matrix(X)
        }
        if (vary != "") {
            by <- model.matrix(as.formula(paste("~", vary, collapse = "")),
                               data = mf)[ , -1, drop = FALSE] # drop intercept
            if (nrow(X) != nrow(by)) 
                warning("The design matrix and the by argument imply a different number of rows: ", 
                        nrow(X), ", ", nrow(by))
            DM <- lapply(1:ncol(by), function(i) {
                ret <- X * by[, i]
                colnames(ret) <- paste(colnames(ret), colnames(by)[i], sep = ":")
                ret
            })
            X <- do.call("cbind", DM)
        }
    }
    ### <FIXME> penalize intercepts???
    ### set up penalty matrix
    ANOVA <- (!is.null(contr) && (length(contr) == 1)) && (ncol(mf) == 1)
    K <- diag(ncol(X))
    ### for ordered factors use difference penalty
    if (ANOVA && any(sapply(mf[, names(contr), drop = FALSE], is.ordered))) {
        K <- diff(diag(ncol(X) + 1), differences = 1)[, -1, drop = FALSE]
        if (vary != "" && ncol(by) > 1){       # build block diagonal penalty
            suppressMessages(K <- kronecker(diag(ncol(by)), K))
        }
        K <- crossprod(K)
    }
    ### </FIXME>
    if (is(X, "Matrix") && !is(K, "Matrix"))
        K <- Matrix(K)
    list(X = X, K = K)
}

### hyper parameters for P-splines baselearner (including tensor product P-splines)
hyper_bbs <- function(mf, vary, knots = 20, boundary.knots = NULL, degree = 3,
                      differences = 2, df = 4, lambda = NULL, center = FALSE,
                      cyclic = FALSE, constraint = "none", deriv = 0L) {

    knotf <- function(x, knots, boundary.knots) {
        if (is.null(boundary.knots))
            boundary.knots <- range(x, na.rm = TRUE)
        ## <fixme> At the moment only NULL or 2 boundary knots can be specified.
        ## Knot expansion is done automatically on an equidistand grid.</fixme>
        if ((length(boundary.knots) != 2) || !boundary.knots[1] < boundary.knots[2])
            stop("boundary.knots must be a vector (or a list of vectors) ",
                 "of length 2 in increasing order")
        if (length(knots) == 1) {
            knots <- seq(from = boundary.knots[1],
                         to = boundary.knots[2], length = knots + 2)
            knots <- knots[2:(length(knots) - 1)]
        }
        list(knots = knots, boundary.knots = boundary.knots)
    }
    nm <- colnames(mf)[colnames(mf) != vary]
    if (is.list(knots)) if(!all(names(knots) %in% nm))
        stop("variable names and knot names must be the same")
    if (is.list(boundary.knots)) if(!all(names(boundary.knots) %in% nm))
        stop("variable names and boundary.knot names must be the same")
    if (!identical(center, FALSE) && cyclic)
        stop("centering of cyclic covariates not yet implemented")
    ret <- vector(mode = "list", length = length(nm))
    names(ret) <- nm
    for (n in nm)
        ret[[n]] <- knotf(mf[[n]], if (is.list(knots)) knots[[n]] else knots,
                          if (is.list(boundary.knots)) boundary.knots[[n]]
                          else boundary.knots)
    if (cyclic & constraint != "none")
        stop("constraints not implemented for cyclic B-splines")
    stopifnot(is.numeric(deriv) & length(deriv) == 1)
    ## prediction is usually set in/by newX()
    list(knots = ret, degree = degree, differences = differences,
         df = df, lambda = lambda, center = center, cyclic = cyclic,
         Ts_constraint = constraint, deriv = deriv, prediction = FALSE)
}

### model.matrix for P-splines baselearner (including tensor product P-splines)
X_bbs <- function(mf, vary, args) {

    stopifnot(is.data.frame(mf))
    mm <- lapply(which(colnames(mf) != vary), function(i) {
        if (!args$cyclic) {
            X <- bsplines(mf[[i]],
                          knots = args$knots[[i]]$knots,
                          boundary.knots = args$knots[[i]]$boundary.knots,
                          degree = args$degree,
                          Ts_constraint = args$Ts_constraint,
                          deriv = args$deriv, extrapolation = args$prediction)
        } else { ## if cyclic spline
            X <- cbs(mf[[i]],
                     knots = args$knots[[i]]$knots,
                     boundary.knots = args$knots[[i]]$boundary.knots,
                     degree = args$degree,
                     deriv = args$deriv)
        }
        class(X) <- "matrix"
        return(X)
    })
    ### options
    MATRIX <- any(sapply(mm, dim) > c(500, 50)) || (length(mm) > 1)
    MATRIX <- MATRIX && options("mboost_useMatrix")$mboost_useMatrix
    if (MATRIX) {
        diag <- Diagonal
        for (i in 1:length(mm)){
            tmp <- attributes(mm[[i]])[c("degree", "knots", "Boundary.knots")]
            mm[[i]] <- Matrix(mm[[i]])
            attributes(mm[[i]])[c("degree", "knots", "Boundary.knots")] <- tmp
        }
    }
    if (length(mm) == 1) {
        X <- mm[[1]]
        if (vary != "") {
            by <- model.matrix(as.formula(paste("~", vary, collapse = "")),
                               data = mf)[ , -1, drop = FALSE] # drop intercept
            DM <- lapply(1:ncol(by), function(i) {
                ret <- X * by[, i]
                colnames(ret) <- paste(colnames(ret), colnames(by)[i], sep = ":")
                ret
            })
            X <- do.call("cbind", DM)
        }
        if (args$differences > 0){
            if (!args$cyclic) {
                K <- diff(diag(ncol(mm[[1]])), differences = args$differences)
            } else {
                ## cyclic P-splines
                differences <- args$differences
                K <- diff(diag(ncol(mm[[1]]) + differences),
                          differences = differences)
                tmp <- K[,(1:differences)]   # save first "differences" columns
                K <- K[,-(1:differences)]    # drop first "differences" columns
                indx <- (ncol(mm[[1]]) - differences + 1):(ncol(mm[[1]]))
                K[,indx] <- K[,indx] + tmp   # add first "differences" columns
            }
        } else {
            if (args$differences != 0)
                stop(sQuote("differences"), " must be an non-neative integer")
            K <- diag(ncol(mm[[1]]))
        }

        if (vary != "" && ncol(by) > 1){       # build block diagonal penalty
                suppressMessages(K <- kronecker(diag(ncol(by)), K))
        }
        if (!identical(args$center, FALSE)) {
            tmp <- attributes(X)[c("degree", "knots", "Boundary.knots")]
            center <- match.arg(as.character(args$center),
                                choices = c("TRUE", "differenceMatrix", "spectralDecomp"))
            if (center == "TRUE") center <- "differenceMatrix"
            X <- switch(center,
                ### L = t(D) in Section 2.3. of Fahrmeir et al. (2004, Stat Sinica)
                "differenceMatrix" = tcrossprod(X, K) %*% solve(tcrossprod(K)),
                ### L = \Gamma \Omega^1/2 in Section 2.3. of
                ### Fahrmeir et al. (2004, Stat Sinica)
                "spectralDecomp" = {
                    SVD <- eigen(crossprod(K), symmetric = TRUE)
                    ev <- SVD$vector[, 1:(ncol(X) - args$differences), drop = FALSE]
                    ew <- SVD$values[1:(ncol(X) - args$differences), drop = FALSE]
                    X %*% ev %*% diag(1/sqrt(ew))
                }
            )
            attributes(X)[c("degree", "knots", "Boundary.knots")] <- tmp
            K <- diag(ncol(X))
        } else {
            K <- crossprod(K)
        }
        if (!is.null(attr(X, "Ts_constraint"))) {
            D <- attr(X, "D")
            K <- crossprod(D, K) %*% D
        }
    }
    if (length(mm) == 2) {
        suppressMessages(
            X <- kronecker(mm[[1]], matrix(1, ncol = ncol(mm[[2]]))) *
                 kronecker(matrix(1, ncol = ncol(mm[[1]])), mm[[2]])
            )
        if (vary != "") {
            by <- model.matrix(as.formula(paste("~", vary, collapse = "")),
                               data = mf)[ , -1, drop = FALSE] # drop intercept
            DM <- X * by[,1]
            if (ncol(by) > 1){
                for (i in 2:ncol(by))
                    DM <- cbind(DM, (X * by[,i]))
            }
            X <- DM
            ### <FIXME> Names of X if by is given
        }
        if (args$differences > 0){
            if (!args$cyclic) {
                Kx <- diff(diag(ncol(mm[[1]])), differences = args$differences)
                Ky <- diff(diag(ncol(mm[[2]])), differences = args$differences)
            } else {
                ## cyclic P-splines
                differences <- args$differences
                Kx <- diff(diag(ncol(mm[[1]]) + differences),
                           differences = differences)
                Ky <- diff(diag(ncol(mm[[2]]) + differences),
                           differences = differences)

                tmp <- Kx[,(1:differences)]   # save first "differences" columns
                Kx <- Kx[,-(1:differences)]    # drop first "differences" columns
                indx <- (ncol(mm[[1]]) - differences + 1):(ncol(mm[[1]]))
                Kx[,indx] <- Kx[,indx] + tmp   # add first "differences" columns

                tmp <- Ky[,(1:differences)]   # save first "differences" columns
                Ky <- Ky[,-(1:differences)]    # drop first "differences" columns
                indx <- (ncol(mm[[2]]) - differences + 1):(ncol(mm[[2]]))
                Ky[,indx] <- Ky[,indx] + tmp   # add first "differences" columns
            }
        } else {
            if (args$differences != 0)
                stop(sQuote("differences"), " must be an non-neative integer")
            Kx <- diag(ncol(mm[[1]]))
            Ky <- diag(ncol(mm[[2]]))
        }

        Kx <- crossprod(Kx)
        Ky <- crossprod(Ky)
        suppressMessages(
            K <- kronecker(Kx, diag(ncol(mm[[2]]))) +
                kronecker(diag(ncol(mm[[1]])), Ky)
            )
        if (vary != "" && ncol(by) > 1){       # build block diagonal penalty
            suppressMessages(K <- kronecker(diag(ncol(by)), K))
        }
        if (!identical(args$center, FALSE)) {
            ### L = \Gamma \Omega^1/2 in Section 2.3. of Fahrmeir et al.
            ### (2004, Stat Sinica), always
            L <- eigen(K, symmetric = TRUE)
            L$vectors <- L$vectors[,1:(ncol(X) - args$differences^2), drop = FALSE]
            L$values <- sqrt(L$values[1:(ncol(X) - args$differences^2), drop = FALSE])
            L <- L$vectors %*% (diag(length(L$values)) * (1/L$values))
            X <- as(X %*% L, "matrix")
            K <- as(diag(ncol(X)), "matrix")
        }
    }
    if (length(mm) > 2)
        stop("not possible to specify more than two variables in ",
             sQuote("..."), " argument of smooth base-learners")

    ## compare specified degrees of freedom to dimension of null space
    if (!is.null(args$df)){
        rns <- ncol(K) - qr(as.matrix(K))$rank # compute rank of null space
        if (rns == args$df)
            warning( sQuote("df"), " equal to rank of null space ",
                    "(unpenalized part of P-spline);\n  ",
                    "Consider larger value for ", sQuote("df"),
                    " or set ", sQuote("center != FALSE"), ".", immediate.=TRUE)
        if (rns > args$df)
            stop("not possible to specify ", sQuote("df"),
                 " smaller than the rank of the null space\n  ",
                 "(unpenalized part of P-spline). Use larger value for ",
                 sQuote("df"), " or set ", sQuote("center != FALSE"), ".")
    }
    return(list(X = X, K = K))
}

### Linear baselearner, potentially Ridge-penalized (but not by default)
bols <- function(..., by = NULL, index = NULL, intercept = TRUE, df = NULL,
                 lambda = 0, contrasts.arg = "contr.treatment") {

    if (!is.null(df)) lambda <- NULL

    cll <- match.call()
    cll[[1]] <- as.name("bols")
    
    mf <- list(...)
    if (is.null(by)) {
        tmp <- mf
    } else {
        tmp <- c(mf, list(by))
    }
    if (length(unique(sapply(tmp, length))) > 1)
        warning("The elements in ... or by imply different number of rows: ", 
                paste(unique(sapply(tmp, length)), collapse = ", "))
    rm("tmp")
    
    ## check that center = TRUE/FALSE is not specified in ...
    if ("center" %in% names(mf) && 
        (length(mf[["center"]]) == 1 && is.logical(mf[["center"]])))
        stop(sQuote("bols(, center = TRUE/FALSE)"), " is deprecated. Please use ",
             sQuote("bols(, intercept = TRUE/FALSE)"), " instead.")
    
    if (length(mf) == 1 && ((isMATRIX(mf[[1]]) || is.data.frame(mf[[1]])) &&
                            ncol(mf[[1]]) > 1 )) {
        mf <- mf[[1]]
        ### spline bases should be matrices
        if (isMATRIX(mf) && !is(mf, "Matrix"))
            class(mf) <- "matrix"
    } else {
        mf <- as.data.frame(mf)
        cl <- as.list(match.call(expand.dots = FALSE))[2][[1]]
        colnames(mf) <- sapply(cl, function(x) as.character(x))
    }
    if(!intercept && !any(sapply(mf, is.factor)) &&
       !any(sapply(mf, function(x){uni <- unique(x);
                                   length(uni[!is.na(uni)])}) == 1)){
        ## if no intercept is used and no covariate is a factor
        ## and if no intercept is specified (i.e. mf[[i]] is constant)
        if (any(sapply(mf, function(x) abs(mean(x, na.rm=TRUE) / sd(x,na.rm=TRUE))) > 0.1))
            ## if covariate mean is not near zero
            warning("covariates should be (mean-) centered if ",
                    sQuote("intercept = FALSE"))
    }
    vary <- ""
    if (!is.null(by)){
        stopifnot(is.data.frame(mf))
        mf <- cbind(mf, by)
        colnames(mf)[ncol(mf)] <- vary <- deparse(substitute(by))
    }

    CC <- all(Complete.cases(mf))
    if (!CC)
        warning("base-learner contains missing values;\n",
                "missing values are excluded per base-learner, ",
                "i.e., base-learners may depend on different",
                " numbers of observations.")
    ### option
    DOINDEX <- is.data.frame(mf) &&
        (nrow(mf) > options("mboost_indexmin")[[1]] || is.factor(mf[[1]]))
    if (is.null(index)) {
        ### try to remove duplicated observations or
        ### observations with missings
        if (!CC || DOINDEX) {
            index <- get_index(mf)
            mf <- mf[index[[1]],,drop = FALSE]
            index <- index[[2]]
        }
    }
    
    ## check if factors with unobserved levels exist
    if (any(fac <- sapply(mf, is.factor))) {
        tmp <- droplevels(mf)
        if (!identical(tmp, mf)) {
            warning("Dropped unobserved factor levels")
            mf <- tmp
        } 
        rm("tmp")  
    }

    ret <- list(model.frame = function()
                    if (is.null(index)) return(mf) else return(mf[index,,drop = FALSE]),
                get_call = function(){
                    cll <- deparse(cll, width.cutoff=500L)
                    if (length(cll) > 1)
                        cll <- paste(cll, collapse="")
                    cll
                },
                get_data = function() mf,
                get_index = function() index,
                get_names = function() colnames(mf),
                get_vary = function() vary,
                set_names = function(value) {
                    if(length(value) != length(colnames(mf)))
                        stop(sQuote("value"), " must have same length as ",
                             sQuote("colnames(mf)"))
                    for (i in 1:length(value)){
                        cll[[i+1]] <<- as.name(value[i])
                    }
                    attr(mf, "names") <<- value
                })
    class(ret) <- "blg"

    ret$dpp <- bl_lin(ret, Xfun = X_ols, args = hyper_ols(
                      df = df, lambda = lambda,
                      intercept = intercept, contrasts.arg = contrasts.arg))
    return(ret)
}

### P-spline (and tensor-product spline) baselearner
bbs <- function(..., by = NULL, index = NULL, knots = 20, boundary.knots = NULL,
                degree = 3, differences = 2, df = 4, lambda = NULL, center = FALSE,
                cyclic = FALSE, constraint = c("none", "increasing", "decreasing"),
                deriv = 0) {

    if (!is.null(lambda)) df <- NULL

    cll <- match.call()
    cll[[1]] <- as.name("bbs")
    
    constraint <- match.arg(constraint)
    if (constraint != "none")
        warning("Using ", sQuote('bbs()'), ' with constraint != "none" is discouraged. Preferably use ', 
                sQuote('bmono()'), " instead.\n",
                "See section ", sQuote("Details"), " of ?bbs for more information.")

    mf <- list(...)
    if (is.null(by)) {
        tmp <- mf
    } else {
        tmp <- c(mf, list(by))
    }
    if (length(unique(sapply(tmp, length))) > 1)
        warning("The elements in ... or by imply different number of rows: ", 
                paste(unique(sapply(tmp, length)), collapse = ", "))
    rm("tmp")
    
    if (length(mf) == 1 && ((is.matrix(mf[[1]]) || is.data.frame(mf[[1]])) &&
                            ncol(mf[[1]]) > 1 )) {
        mf <- as.data.frame(mf[[1]])
    } else {
        mf <- as.data.frame(mf)
        cl <- as.list(match.call(expand.dots = FALSE))[2][[1]]
        colnames(mf) <- sapply(cl, function(x) deparse(x))
    }
    stopifnot(is.data.frame(mf))
    if(!(all(sapply(mf, is.numeric)))) {
        if (ncol(mf) == 1){
            warning("cannot compute ", sQuote("bbs"),
                    " for non-numeric variables; used ",
                    sQuote("bols"), " instead.")
            return(bols(mf, by = by, index = index))
        }
        stop("cannot compute bbs for non-numeric variables")
    }
    vary <- ""
    if (!is.null(by)){
        mf <- cbind(mf, by)
        colnames(mf)[ncol(mf)] <- vary <- deparse(substitute(by))
    }

    CC <- all(Complete.cases(mf))
    if (!CC)
        warning("base-learner contains missing values;\n",
                "missing values are excluded per base-learner, ",
                "i.e., base-learners may depend on different",
                " numbers of observations.")
    ### option
    DOINDEX <- (nrow(mf) > options("mboost_indexmin")[[1]])
    if (is.null(index)) {
        if (!CC || DOINDEX) {
            index <- get_index(mf)
            mf <- mf[index[[1]],,drop = FALSE]
            index <- index[[2]]
        }
    }

    ret <- list(model.frame = function()
                    if (is.null(index)) return(mf) else return(mf[index,,drop = FALSE]),
                get_call = function(){
                    cll <- deparse(cll, width.cutoff=500L)
                    if (length(cll) > 1)
                        cll <- paste(cll, collapse="")
                    cll
                },
                get_data = function() mf,
                get_index = function() index,
                get_vary = function() vary,
                get_names = function() colnames(mf),
                set_names = function(value) {
                    if(length(value) != length(colnames(mf)))
                        stop(sQuote("value"), " must have same length as ",
                             sQuote("colnames(mf)"))
                    for (i in 1:length(value)){
                        cll[[i+1]] <<- as.name(value[i])
                    }
                    attr(mf, "names") <<- value
                })
    class(ret) <- "blg"

    ret$dpp <- bl_lin(ret, Xfun = X_bbs,
                      args = hyper_bbs(mf, vary, knots = knots, boundary.knots =
                      boundary.knots, degree = degree, differences = differences,
                      df = df, lambda = lambda, center = center, cyclic = cyclic,
                      constraint = match.arg(constraint), deriv = deriv))
    return(ret)
}

### cyclic B-splines
### adapted version of mgcv::cSplineDes from S.N. Wood
cbs <- function (x, knots, boundary.knots, degree = 3, deriv = 0L) {

    if (any(x < boundary.knots[1], na.rm = TRUE) |
        any(x > boundary.knots[2], na.rm = TRUE))
        stop("some ", sQuote("x"), " values are beyond ",
             sQuote("boundary.knots"))

    nx <- names(x)
    x <- as.vector(x)
    ## handling of NAs
    nax <- is.na(x)
    if (nas <- any(nax))
        x <- x[!nax]

    knots <- c(boundary.knots[1], knots, boundary.knots[2])
    nKnots <- length(knots)
    ord <- degree + 1
    xc <- knots[nKnots - ord + 1]
    knots <- c(boundary.knots[1] -
               (boundary.knots[2] - knots[(nKnots - ord + 1):(nKnots - 1)]),
               knots)
    ind <- x > xc
    X <- splineDesign(knots, x, ord, derivs = rep(deriv, length(x)), outer.ok = TRUE)
    x[ind] <- x[ind] - boundary.knots[2] + boundary.knots[1]
    if (sum(ind)) {
        Xtmp <- splineDesign(knots, x[ind], ord, derivs = rep(deriv, length(x[ind])),
                             outer.ok = TRUE)
        X[ind, ] <- X[ind, ] + Xtmp
    }
    ## handling of NAs
    if (nas) {
        tmp <- matrix(NA, length(nax), ncol(X))
        tmp[!nax, ] <- X
        X <- tmp
    }
    ## add attributes
    attr(X, "degree") <- degree
    attr(X,"knots") <- knots
    attr(X,"boundary.knots") <- boundary.knots
    if (length(deriv) > 1 || deriv != 0)
        attr(X, "deriv") <- deriv
    dimnames(X) <- list(nx, 1L:ncol(X))
    return(X)
}

bsplines <- function(x, knots, boundary.knots, degree,
                     Ts_constraint = "none", deriv = 0L,
                     extrapolation = FALSE) {

    ## do not allow data beyond boundary knots while fitting
    if (!extrapolation && (any(x < boundary.knots[1], na.rm = TRUE) |
                               any(x > boundary.knots[2], na.rm = TRUE)))
        stop("some ", sQuote("x"), " values are beyond ",
             sQuote("boundary.knots"))

    ## allow extrapolation when predicting
    if (extrapolation <- extrapolation &&
        (any(x < boundary.knots[1], na.rm = TRUE) |
             any(x > boundary.knots[2], na.rm = TRUE))) {
        warning("Some ", sQuote("x"), " values are beyond ",
                sQuote("boundary.knots"), "; Linear extrapolation used.")
    }

    nx <- names(x)
    x <- as.vector(x)
    ## handling of NAs
    nax <- is.na(x)
    if (nas <- any(nax))
        x <- x[!nax]
    ## use equidistant boundary knots
    dx <- diff(boundary.knots)/(length(knots) + 1)
    bk_lower <- seq(boundary.knots[1] - degree * dx, boundary.knots[1],
                    length = degree + 1)
    bk_upper <- seq(boundary.knots[2], boundary.knots[2] + degree * dx,
                    length = degree + 1)
    ## complete knot mesh
    k <- c(bk_lower, knots, bk_upper)
    ## construct design matrix
    X <- splineDesign(k, x, degree + 1, derivs = rep(deriv, length(x)),
                      outer.ok = TRUE)

    ## code along the lines of mgcv::Predict.matrix.pspline.smooth
    if (extrapolation) {
        ## Build matrix to map coeficients to value (deriv = 0) and
        ## slope (deriv = 1) at end points.
        if (deriv != 0L) {
            warning("deriv != 0L; Linear extrapolation overwritten")
        } else {
              deriv <- c(0, 1, 0, 1)
          }
        D <- splineDesign(knots = k, x = rep(boundary.knots, each = 2),
                          ord = degree + 1, deriv)
        ## Add rows for linear extrapolation
        idx <- x < boundary.knots[1]
        if (any(idx, na.rm = TRUE))
            X[idx,] <- cbind(1, x[idx] - boundary.knots[1]) %*% D[1:2, ]
        idx <- x > boundary.knots[2]
        if (any(idx, na.rm = TRUE))
            X[idx,] <- cbind(1, x[idx] - boundary.knots[2]) %*% D[3:4, ]
    }

    ## handling of NAs
    if (nas) {
        tmp <- matrix(NA, length(nax), ncol(X))
        tmp[!nax, ] <- X
        X <- tmp
    }
    ### constraints; experimental
    D <- diag(ncol(X))
    D[lower.tri(D)] <- 1
    X <- switch(Ts_constraint, "none" = X,
                            "increasing" = X %*% D,
                            "decreasing" = -X %*% D)
    ## add attributes
    attr(X, "degree") <- degree
    attr(X, "knots") <- knots
    attr(X, "boundary.knots") <- list(lower = bk_lower, upper = bk_upper)
    if (Ts_constraint != "none")
        attr(X, "Ts_constraint") <- Ts_constraint
    if (Ts_constraint != "none")
        attr(X, "D") <- D
    if (length(deriv) > 1 || deriv != 0)
        attr(X, "deriv") <- deriv
    dimnames(X) <- list(nx, 1L:ncol(X))
    return(X)
}

### workhorse for fitting (Ridge-penalized) baselearners
bl_lin <- function(blg, Xfun, args) {

    mf <- blg$get_data()
    index <- blg$get_index()
    vary <- blg$get_vary()

    newX <- function(newdata = NULL, prediction = FALSE) {
        if (!is.null(newdata)) {
            mf <- check_newdata(newdata, blg, mf)
        }
        ## this argument is currently only used in X_bbs --> bsplines
        args$prediction <- prediction
        return(Xfun(mf, vary, args))
    }
    X <- newX()
    K <- X$K
    X <- X$X

    dpp <- function(weights) {

        if (!is.null(attr(X, "deriv")))
            stop("fitting of derivatives of B-splines not implemented")

        weights[!Complete.cases(mf)] <- 0
        w <- weights
        if (!is.null(index))
            w <- .Call("R_ysum", as.double(weights), as.integer(index), PACKAGE = "mboost")
        XtX <- crossprod(X * w, X)
        lambdadf <- df2lambda(X, df = args$df, lambda = args$lambda,
                              dmat = K, weights = w, XtX = XtX)
        lambda <- lambdadf["lambda"]
        XtX <- XtX + lambda * K

        ## matrizes of class dgeMatrix are dense generic matrices; they should
        ## be coerced to class matrix and handled in the standard way
        if (is(X, "Matrix") && !extends(class(XtX), "dgeMatrix")) {
            XtXC <- Cholesky(forceSymmetric(XtX))
            mysolve <- function(y) {
                if (is.null(attr(X, "Ts_constraint")))
                    return(solve(XtXC, crossprod(X, y)))  ## special solve routine from
                                                          ## package Matrix
                ### non-negative LS only at the moment
                return(nnls1D(as(XtX, "matrix"), as(X, "matrix"), y))
            }
        } else {
            if (is(X, "Matrix")) {
                ## coerce Matrix to matrix
                X <- as(X, "matrix")
                XtX <- as(XtX, "matrix")
            }
            mysolve <- function(y) {
                if (is.null(attr(X, "Ts_constraint")))
                    return(solve(XtX, crossprod(X, y)))
                ### non-negative LS only at the moment
                return(nnls1D(XtX, X, y))
            }
        }

        fit <- function(y) {
            if (!is.null(index)) {
                if (is.matrix(y)) {
                    y <- apply(y, 2, function(u) 
                        .Call("R_ysum", as.double(weights * u), as.integer(index),
                              PACKAGE = "mboost"))
                } else {
                    y <- .Call("R_ysum", as.double(weights * y), as.integer(index),
                               PACKAGE = "mboost")
                }
            } else {
                y <- y * weights
            }
            coef <- mysolve(y)
            ret <- list(model = coef,
                        fitted = function() {
                            ret <- as.vector(X %*% coef)
                            if (is.null(index)) return(ret)
                            return(ret[index])
                        })
            class(ret) <- c("bm_lin", "bm")
            ret
        }

        ### <FIXME> check for large n, option?
        hatvalues <- function() {
            ret <- as.matrix(tcrossprod(X %*% solve(XtX), X * w))
            if (is.null(index)) return(ret)
            return(ret[index, index])
        }
        ### </FIXME>

        ### actually used degrees of freedom (trace of hat matrix)
        df <- function() lambdadf

        ### prepare for computing predictions
        predict <- function(bm, newdata = NULL, aggregate = c("sum", "cumsum", "none")) {
            cf <- sapply(bm, coef)
            if (!is.matrix(cf))
                cf <- matrix(cf, nrow = 1)
            if(!is.null(newdata)) {
                index <- NULL
                ## Use sparse data represenation if data set is huge
                ## and a data.frame
                if (is.data.frame(newdata) && nrow(newdata) > options("mboost_indexmin")[[1]]) {
                    index <- get_index(newdata)
                    newdata <- newdata[index[[1]], , drop = FALSE]
                    index <- index[[2]]
                }
                X <- newX(newdata, prediction = TRUE)$X
            }
            ### when coef is actually a matrix
            ### coming from multidimensional gradients
            P <- 1L
            if (ncol(X) != nrow(cf)) {
                P <- nrow(cf) / ncol(X)
                X <- do.call("bdiag", list(X = X)[rep(1, P)])
            }
            aggregate <- match.arg(aggregate)
            pr <- switch(aggregate, "sum" = {
                ret <- as(X %*% rowSums(cf), "matrix")
                matrix(ret, ncol = P)
            },
            "cumsum" = {
                stopifnot(P == 1L)
                as(X %*% .Call("R_mcumsum", as(cf, "matrix"),
                               PACKAGE = "mboost"), "matrix")
            },
            "none" = {
                stopifnot(P == 1L)
                as(X %*% cf, "matrix")
            })
            if (is.null(index))
                return(pr[ , , drop = FALSE])
            return(pr[index, ,drop = FALSE])
        }

        ret <- list(fit = fit, hatvalues = hatvalues,
                    predict = predict, df = df,
                    Xnames = colnames(X))
        class(ret) <- c("bl_lin", "bl")
        return(ret)

    }
    return(dpp)
}

### tensor-product spline baselearner
bspatial <- function(..., df = 6) {
    cl <- cltmp <- match.call()
    if (is.null(cl$df)) cl$df <- df
    cl[[1L]] <- as.name("bbs")
    ret <- eval(cl, parent.frame())
    cltmp[[1]] <- as.name("bspatial")
    assign("cll", cltmp, envir = environment(ret$get_call))
    ret
}

### random-effects (Ridge-penalized ANOVA) baselearner
brandom <- function(..., by = NULL, index = NULL, df = 4, lambda = NULL,
                    contrasts.arg = "contr.dummy") {
    cl <- cltmp <- match.call()
    x <- list(...)
    ## drop further arguments to be passed to bols
    if (!is.null(names(x)))
        x <- x[names(x) == ""]

    if (!all(sapply(x, is.factor) |
             sapply(x, is.matrix) |
             sapply(x, is.data.frame)))
        stop(sQuote("..."), " must be a factor or design matrix in ",
             sQuote("brandom"))

    if (is.null(cl$df) && is.null(cl$lambda))
        cl$df <- df
    if (is.null(cl$contrasts.arg))
        cl$contrasts.arg <- contrasts.arg
    cl[[1L]] <- as.name("bols")
    ret <- eval(cl, parent.frame())
    cltmp[[1]] <- as.name("brandom")
    assign("cll", cltmp, envir = environment(ret$get_call))
    ret
}

### extract variables names from baselearner
names.blg <- function(x)
    x$get_names()

### extract data from baselearner
model.frame.blg <- function(formula, ...)
    formula$model.frame(...)

### extract coefficients
coef.bm_lin <- function(object, ...) {
    ret <- as.vector(object$model)
    names(ret) <- object$Xnames
    ret
}

### extract fitted values
fitted.bm <- function(object, ...)
    object$fitted()

### extract hatmatrix
hatvalues.bl_lin <- function(model, ...)
    model$hatvalues()

### data preprocessing (plug in weights)
dpp <- function(object, weights)
    UseMethod("dpp", object)

dpp.blg <- function(object, weights)
    object$dpp(weights)

### actually fit a baselearner to response y
fit <- function(object, y)
    UseMethod("fit", object)

fit.bl <- function(object, y)
    object$fit(y)

"%+%" <- function(bl1, bl2) {

    if (is.list(bl1) && !inherits(bl1, "blg"))
        return(lapply(bl1, "%+%", bl2 = bl2))

    if (is.list(bl2) && !inherits(bl2, "blg"))
        return(lapply(bl2, "%+%", bl1 = bl1))

    cll <- paste(bl1$get_call(), "%+%",
                 bl2$get_call(), collapse = "")
    stopifnot(inherits(bl1, "blg"))
    stopifnot(inherits(bl2, "blg"))

    if (nrow(model.frame(bl1)) != nrow(model.frame(bl2))) 
        warning("The design matrices of the two base-learners imply a different number of rows: ", 
                nrow(model.frame(bl1)), ", ", nrow(model.frame(bl2)))
    mf <- cbind(model.frame(bl1), model.frame(bl2))
    index1 <- bl1$get_index()
    index2 <- bl2$get_index()
    if (is.null(index1)) index1 <- 1:nrow(mf)
    if (is.null(index2)) index2 <- 1:nrow(mf)

    mfindex <- cbind(index1, index2)
    index <- NULL

    CC <- all(Complete.cases(mf))
    if (!CC)
        warning("base-learner contains missing values;\n",
                "missing values are excluded per base-learner, ",
                "i.e., base-learners may depend on different",
                " numbers of observations.")
    ### option
    DOINDEX <- (nrow(mf) > options("mboost_indexmin")[[1]])
    if (is.null(index)) {
        if (!CC || DOINDEX) {
            index <- get_index(mfindex)
            mf <- mf[index[[1]],,drop = FALSE]
            index <- index[[2]]
        }
    }

    vary <- ""

    ret <- list(model.frame = function()
                    if (is.null(index)) return(mf) else return(mf[index,,drop = FALSE]),
                get_call = function(){
                    cll <- deparse(cll, width.cutoff=500L)
                    if (length(cll) > 1)
                        cll <- paste(cll, collapse="")
                    cll
                },
                get_data = function() mf,
                get_index = function() index,
                get_vary = function() vary,
                get_names = function() colnames(mf),
                ## <FIXME> Is this all we want to change if we set names here?
                set_names = function(value) attr(mf, "names") <<- value)
                ## </FIXME>
    class(ret) <- "blg"

    args1 <- environment(bl1$dpp)$args
    args2 <- environment(bl2$dpp)$args
    l1 <- args1$lambda
    l2 <- args2$lambda
    if (!is.null(l1) && !is.null(l2)) {
        args <- list(lambda = 1, df = NULL)
    } else {
        args <- list(lambda = NULL,
            df = ifelse(is.null(args1$df), 0, args1$df) +
                 ifelse(is.null(args2$df), 0, args2$df))
    }

    Xfun <- function(mf, vary, args) {

        newX1 <- environment(bl1$dpp)$newX
        newX2 <- environment(bl2$dpp)$newX

        X1 <- newX1(mf[, bl1$get_names(), drop = FALSE],
                    prediction = args$prediction)
        K1 <- X1$K
        if (!is.null(l1)) K1 <- l1 * K1
        X1 <- X1$X

        X2 <- newX2(mf[, bl2$get_names(), drop = FALSE],
                    prediction = args$prediction)
        K2 <- X2$K
        if (!is.null(l2)) K2 <- l2 * K2
        X2 <- X2$X

        K <- matrix(0, ncol = ncol(K1) + ncol(K2),
                    nrow = nrow(K1) + nrow(K2))
        K[1:nrow(K1), 1:ncol(K1)] <- as.matrix(K1)
        K[-(1:nrow(K1)), -(1:ncol(K1))] <- as.matrix(K2)
        X <- cbind(as.matrix(X1), as.matrix(X2))
        MATRIX <- any(dim(X) > c(500, 50)) &&
                  options("mboost_useMatrix")$mboost_useMatrix
        if (MATRIX & !is(X, "Matrix"))
            X <- Matrix(X)
        if (MATRIX & !is(K, "Matrix"))
            K <- Matrix(K)
        list(X = X, K = K)
    }

    ret$dpp <- bl_lin(ret, Xfun = Xfun, args = args)

    return(ret)
}

"%X%" <- function(bl1, bl2) {

    if (is.list(bl1) && !inherits(bl1, "blg"))
        return(lapply(bl1, "%X%", bl2 = bl2))

    if (is.list(bl2) && !inherits(bl2, "blg"))
        return(lapply(bl2, "%X%", bl1 = bl1))

    cll <- paste(bl1$get_call(), "%X%",
                 bl2$get_call(), collapse = "")
    stopifnot(inherits(bl1, "blg"))
    stopifnot(inherits(bl2, "blg"))

    stopifnot(!any(colnames(model.frame(bl1)) %in%
                   colnames(model.frame(bl2))))
    if (nrow(model.frame(bl1)) != nrow(model.frame(bl2))) 
        warning("The design matrices of the two marginal base-learners imply a different number of rows: ", 
                nrow(model.frame(bl1)), ", ", nrow(model.frame(bl2)))
    mf <- cbind(model.frame(bl1), model.frame(bl2))
    index1 <- bl1$get_index()
    index2 <- bl2$get_index()
    if (is.null(index1)) index1 <- 1:nrow(mf)
    if (is.null(index2)) index2 <- 1:nrow(mf)

    mfindex <- cbind(index1, index2)
    index <- NULL

    CC <- all(Complete.cases(mf))
    if (!CC)
        warning("base-learner contains missing values;\n",
                "missing values are excluded per base-learner, ",
                "i.e., base-learners may depend on different",
                " numbers of observations.")
    ### option
    DOINDEX <- (nrow(mf) > options("mboost_indexmin")[[1]])
    if (is.null(index)) {
        if (!CC || DOINDEX) {
            index <- get_index(mfindex)
            mf <- mf[index[[1]],,drop = FALSE]
            index <- index[[2]]
        }
    }

    vary <- ""

    ret <- list(model.frame = function()
                    if (is.null(index)) return(mf) else return(mf[index,,drop = FALSE]),
                get_call = function(){
                    cll <- deparse(cll, width.cutoff=500L)
                    if (length(cll) > 1)
                        cll <- paste(cll, collapse="")
                    cll
                },
                get_data = function() mf,
                get_index = function() index,
                get_vary = function() vary,
                get_names = function() colnames(mf),
                ## <FIXME> Is this all we want to change if we set names here?
                set_names = function(value) attr(mf, "names") <<- value)
                ## </FIXME>
    class(ret) <- "blg"

    args1 <- environment(bl1$dpp)$args
    args2 <- environment(bl2$dpp)$args
    l1 <- args1$lambda
    l2 <- args2$lambda
    if (!is.null(l1) && !is.null(l2)) {
        args <- list(lambda = 1, df = NULL)
    } else {
        args <- list(lambda = NULL,
            df = ifelse(is.null(args1$df), 1, args1$df) *
                 ifelse(is.null(args2$df), 1, args2$df))
    }

    Xfun <- function(mf, vary, args) {

        newX1 <- environment(bl1$dpp)$newX
        newX2 <- environment(bl2$dpp)$newX

        X1 <- newX1(mf[, bl1$get_names(), drop = FALSE],
                    prediction = args$prediction)
        K1 <- X1$K
        X1 <- X1$X
        if (!is.null(l1)) K1 <- l1 * K1
        MATRIX <- options("mboost_useMatrix")$mboost_useMatrix
        if (MATRIX & !is(X1, "Matrix"))
            X1 <- Matrix(X1)
        if (MATRIX & !is(K1, "Matrix"))
            K1 <- Matrix(K1)

        X2 <- newX2(mf[, bl2$get_names(), drop = FALSE],
                    prediction = args$prediction)
        K2 <- X2$K
        X2 <- X2$X
        if (!is.null(l2)) K2 <- l2 * K2
        if (MATRIX & !is(X2, "Matrix"))
            X2 <- Matrix(X2)
        if (MATRIX & !is(K2, "Matrix"))
            K2 <- Matrix(K2)
        suppressMessages(
            X <- kronecker(X1, Matrix(1, ncol = ncol(X2),
                                  dimnames = list("", colnames(X2))),
                       make.dimnames = TRUE) *
                kronecker(Matrix(1, ncol = ncol(X1),
                              dimnames = list("", colnames(X1))),
                       X2, make.dimnames = TRUE)
            )
        suppressMessages(
            K <- kronecker(K1, diag(ncol(X2))) +
                 kronecker(diag(ncol(X1)), K2)
            )
        list(X = X, K = K)
    }

    ret$dpp <- bl_lin(ret, Xfun = Xfun, args = args)

    return(ret)
}

bns <- function(...)
    stop("Base-learner ",  sQuote("bns"), " has ben removed. Consider ",
         sQuote("bbs"), " instead.")

bss <- function(...)
    stop("Base-learner ",  sQuote("bss"), " has ben removed. Consider ",
         sQuote("bbs"), " instead.")

### Linear baselearner, potentially Ridge-penalized
bcobs <- function(..., index = NULL, lambda = 1, knots = 20,
                  constraint = c("none", "increase", "decrease",
                                 "convex", "concave", "periodic")) {

    cll <- match.call()
    cll[[1]] <- as.name("bcobs")
    cll <- deparse(cll)

    mf <- list(...)
    if (is.data.frame(mf[[1]])) {
        mf <- mf[[1]]
    } else {
        mf <- as.data.frame(mf)
        cl <- as.list(match.call(expand.dots = FALSE))[2][[1]]
        colnames(mf) <- sapply(cl, function(x) as.character(x))
    }

    x <- mf[[1]]
    boundary.knots <- range(x, na.rm = TRUE)
    if (length(knots) == 1) {
        knots <- seq(from = boundary.knots[1],
                     to = boundary.knots[2], length = knots + 2)
    }

    CC <- all(mboost:::Complete.cases(mf))
    ### option
    DOINDEX <- is.data.frame(mf) && (nrow(mf) > 10000 || is.factor(mf[[1]]))
    if (is.null(index)) {
        ### try to remove duplicated observations or
        ### observations with missings
        if (!CC || DOINDEX) {
            index <- mboost:::get_index(mf)
            mf <- mf[index[[1]],,drop = FALSE]
            index <- index[[2]]
        }
    }

    ret <- list(model.frame = function()
                    if (is.null(index)) return(mf) else return(mf[index,,drop = FALSE]),
                get_call = function() cll,
                get_data = function() mf,
                get_index = function() index,
                get_names = function() colnames(mf),
                get_vary = function() "",
                set_names = function(value) attr(mf, "names") <<- value)
    class(ret) <- "blg"

    ret$dpp <- function(weights) {

        weights[!mboost:::Complete.cases(mf)] <- 0
        w <- weights
        if (!is.null(index)) w <- as.vector(tapply(weights, index, sum))

        fit <- function(y) {
            if (!is.null(index)) {
                y <- as.vector(tapply(weights * y, index, sum))
            } else {
                y <- y * weights
            }
            model <- cobs(x = mf[[1]], y = y, w = w, constraint = constraint)
            ret <- list(model = model,
                        fitted = function() {
                            ret <- fitted(model)
                            if (is.null(index)) return(ret)
                            return(ret[index])
                        })
            class(ret) <- "bm"
            ret
        }

        ### prepare for computing predictions
        predict <- function(bm, newdata = NULL, aggregate = c("sum", "cumsum", "none")) {
            pr <- sapply(bm, function(mod) {
                if (is.null(newdata)) return(mod$fitted())
                cobs:::predict.cobs(mod$model, z = newdata[[colnames(mf)]])[,2]
            })
            aggregate <- match.arg(aggregate)
            pr <- switch(aggregate, "sum" =
                matrix(rowSums(pr), ncol = 1),
            "cumsum" = {
                M <- triu(crossprod(Matrix(1, nc = ncol(cf))))
                as(pr %*% M, "matrix")
            },
            "none" = pr)
            if (is.null(index)) return(pr[,,drop = FALSE])
            return(pr[index,,drop = FALSE])
        }

        ret <- list(fit = fit,
                    predict = predict)
        class(ret) <- "bl"
        return(ret)

    }
    return(ret)
}

library("mboost")
library("cobs")

data("bodyfat", package = "TH.data")

bb <- function(...) bcobs(..., constraint = "increase")
bmod <- mboost(DEXfat ~ . , data = bodyfat, baselearner = bb,
            control = boost_control(mstop = 50))
layout(matrix(1:9, nc = 3))
plot(bmod)
dev.off()

mod <- gamboost(DEXfat ~ ., data = bodyfat, control = boost_control(mstop = 50))
layout(matrix(1:9, nc = 3))
plot(mod)

### component-wise linear models baselearner
bolscw <- function(X) {

    ### formula parsing, data handling is done elsewhere
    stopifnot(isMATRIX(X))

    ### deal with missing values
    index <- NULL
    if (!all(cc <- Complete.cases(X))) {
        nd <- which(cc)
        index <- match(1:nrow(X), nd)
        X <- X[cc, , drop = FALSE]
    }

    ### we actually don't have raw data available
    ret <- list(model.frame = function() return(NULL),
                get_data = function() X,
                get_index = function() index,
                get_vary = function() NULL,
                get_names = function() colnames(X),
                set_names = function(value) attr(X, "colnames") <<- value)
    class(ret) <- "blg"

    ret$dpp <- function(weights) {

        weights <- weights[cc]
        xtx <- colSums(X^2 * weights, na.rm = TRUE)
        ### some columns may be zero
        xtx[xtx < .Machine$double.eps] <- 1
        sxtx <- sqrt(xtx)
        p <- ncol(X)
        ### use crossprod for estimation but not for Matrix objects
        ### since S4 dispatch is way to slow
        if (is(X, "Matrix")) {
            MPinvS <- t(X * weights) / sxtx
            est <- function(y) MPinvS %*% y
        } else {
            MPinvS <- NULL
            storage.mode(X) <- "double" ### crossprod is doing this anyway
            est <- function(y) crossprod(X, y * weights) / sxtx
        }

        fit <- function(y) {
            if (!is.null(index))
                y <- y[cc]

            mmu <- max(amu <- abs(mu <- est(y)))
            xselect <- which(as.logical(mmu == amu))[1]
            coef <- mu[xselect] / sxtx[xselect]
            ret <- list(model = c(coef = coef, xselect = xselect, p = p),
                        fitted = function() {
                            if (is.null(index)) return(coef * X[,xselect, drop = FALSE])
                            return(coef * X[index, xselect,drop = FALSE])
                        })
            class(ret) <- c("bm_cwlin", "bm_lin", "bm")
            return(ret)
        }

        predict <- function(bm, newdata = NULL, 
                            aggregate = c("sum", "cumsum", "none")) {

            aggregate <- match.arg(aggregate)
            cf <- switch(aggregate, "sum" = {
                cf <- rep(0, bm[[1]]$model["p"])
                for (i in 1:length(bm)) {
                    m <- bm[[i]]$model
                    cf[m[2]] <- cf[m[2]] + m[1]
                }
                cf
            },
            "cumsum" = {
                cf <- matrix(coef(bm[[1]], all = TRUE), ncol = 1)
                if (length(bm) > 2) {
                    for (i in 2:length(bm))
                        cf <- cbind(cf, cf[[i-1]] + coef(bm[[i]], all = TRUE))
                }
                cf
            },
            "none" = {
                cf <- matrix(coef(bm[[1]], all = TRUE), ncol = 1)
                if (length(bm) > 2) {
                    for (i in 2:length(bm))
                        cf <- cbind(cf, coef(bm[[i]], all = TRUE))
                }
                cf
            })

            if (!is.null(newdata)) {
                stopifnot(all(class(newdata) == class(X)))
                stopifnot(all(colnames(newdata) == colnames(X)))
                X <- newdata
                return(X %*% cf)
            }
            if (!is.null(index))
                return(X[index,,drop = FALSE] %*% cf)
            return(X %*% cf)
        }

        ret <- list(fit = fit, predict = predict, Xnames = colnames(X), 
                    MPinv = function() {
                        if (is.null(MPinvS)) MPinvS <<- t(X * weights) / sxtx
                        return(MPinvS / sxtx)
                    },
                    hatvalues = function() {
                        if (is.null(MPinvS)) MPinvS <<- t(X * weights) / sxtx
                        X %*% (MPinvS / sxtx)
                    })
        class(ret) <- c("bl_cwlin", "bl")
        return(ret)
    }
    return(ret)
}

coef.bm_cwlin <- function(object, all = FALSE, ...) {
    if (!all) return(object$model[1])
    cf <- numeric(object$model[3])
    cf[object$model[2]] <- object$model[1]
    cf
}

### extract hatmatrix
hatvalues.bl_cwlin <- function(model, ...)
    model$hatvalues()

mboost_intern <- function(...,
    fun = c("df2lambda", "hyper_bbs", "hyper_ols",
            "bl_lin", "bl_lin_matrix",
            "Complete.cases", "get_index", "isMATRIX",
            "cbs", "bsplines", "model.frame.blg", 
            "check_newdata", "do_trace", "rescale_weights", "nnls2D")) {

    fun <- match.arg(fun)
    do.call(fun, list(...))
}

varimp <- function(object, ...)
  UseMethod("varimp")


varimp.mboost <- function(object, ...) {  

  ### which baselearners/variables were selected in boosting steps
  learner_names <- if( !(inherits(object, "glmboost")) ) 
    names(object$baselearner) else variable.names(object)  
  learner_selected <- object$xselect()
  
  
  ### --------------------------------------------------
  ### compute inbag risk reduction for each step
  riskdiff <- -1 * diff(risk(object))
  
  ### compute empirical in-bag risk (according to output in cvrisk)
  riskdiff <- riskdiff / length(object$response)
  
  
  ### --------------------------------------------------
  ### explained risk attributed to baselearners
  explained <- sapply(seq_along(learner_names), FUN = function(i) {
    sum( riskdiff[which(learner_selected == i)] ) 
  })
  names(explained) <- learner_names
  
  
  ### --------------------------------------------------
  ### define new varimp-object
  class(explained) <- "varimp"
  
  # add selection frequencies to varimp-object
  attr(explained, "selfreqs") <- sapply(seq_along(learner_names), function(i) {
    mean(learner_selected == i)
  })
  
  # add variable names per baselearner to varimp-object
  var_names <- unname( variable.names(object) )
  # for identifiability sort variable names in interactions 
  # (not required for glmboost interactions)
  var_names <- sapply(strsplit(var_names, ", "), function(x) {
    do.call( function(...) paste(... , sep = ", " ), 
             as.list(x[order(x)]) ) })
  var_order <- order( sapply(var_names, function(i) {
    sum(explained[var_names == i]) 
  }) )
  
  # add names as ordered factor (for order in graphical output)
  attr(explained, "variable_names") <- ordered(var_names, levels = 
      unique(var_names[var_order]))
  
  return(explained)
}


as.data.frame.varimp <- function(x, row.names = NULL, optional = FALSE, ...) {
  data.frame(
    reduction = as.numeric(x), 
    # blearner as ordered factor (corresponding to variable(_names))
    blearner  = ordered(names(x), levels = unique(names(x)[order(x)])),
    variable  = attr(x, "variable_names"),
    selfreq   = attr(x, "selfreqs")
  )
}


plot.varimp <- function(x, percent = TRUE, type = c("variable", "blearner"), 
  blorder = c("importance", "alphabetical", "rev_alphabetical", "formula"), 
  nbars = 10L, maxchar = 20L, xlab = NULL, ylab = NULL, xlim, auto.key, ...) {
  
  ### --------------------------------------------------
  ### get arguments
  args <- as.list(match.call())  
    
  type <- match.arg(type) 
  blorder <- match.arg(blorder)   
  
  
  ### --------------------------------------------------
  ### check arguments
  if( !inherits(x, "varimp") ) 
    stop(paste(deparse(substitute(x)), "is not of class varimp."))
  
  if( !(is.logical(percent)) )
     stop("Parameter percent has of type logical.")
  
  if( !(type %in% c("blearner","variable")) )
     stop("Parameter type has to be 'blearner' or 'variable'.")
  
  if( !(is.numeric(nbars) && nbars > 0 && length(nbars) == 1) )
    stop("Parameter nbars has to be a positive integer.")
  
  if( !(is.numeric(maxchar) && maxchar > 0 && length(maxchar) == 1) )
    stop("Parameter maxchar has to be a positive integer.")
  
  if( !(blorder %in% c("importance", "alphabetical", "rev_alphabetical",
                       "formula")) )
    stop(paste("Parameter blorder has to be one of 'importance',",
      "'alphabetical', 'rev_alphabetical' or 'formula'."))
  
  
  ### --------------------------------------------------
  ### set plotting arguments
  # set range for x-axis
  if( !("xlim" %in% names(args)) ) {
    # special calculation to include the case that risk reduction is negative
    xlim <- c(sum(x*(x < 0)), sum(x*(x >= 0)))     
    if( percent ) xlim <- xlim / sum(abs(x))
  }  
  
  # set x-axis label and transform values to percentages if necessary
  if( percent ) {
    x <- x / sum(abs(x))
    # special handling if risk reduction is negative
    if( any(x < 0) ) 
      warning(paste("At least one risk reduction value is negative. Percental", 
        "reduction is thus calculated as 'reduction/sum(abs(reduction))'."))
  }
  
  # set axis labels
  xlab <- ifelse(is.null(xlab), ifelse(percent, "In-bag Risk Reduction (%)", 
    "In-bag Risk Reduction"), xlab) 
  ylab <- ifelse(is.null(ylab), ifelse(type == "variable", "Variables", 
    "Baselearner"), ylab) 
  
  
  
  ### --------------------------------------------------
  ### create data.frame for all values shown in barchart
  plot_data <- data.frame(x)

  
  ### --------------------------------------------------
  # specify baselearner order (if blorder != "importance", the default order set
  # via as.data.frame.varimp)
  plot_data[, "blearner"] <- switch(blorder,
    # importance is default, in this case no change
    importance       = plot_data[, "blearner"],
    # as blearner order is reverted again later, they are specified in 
    # reverted order here
    alphabetical     = ordered(names(x), rev(levels(ordered(names(x))))),
    rev_alphabetical = ordered(names(x)),
    formula          = ordered(names(x), rev(names(x)))
  )
  
  
  ### --------------------------------------------------
  ### if number of baselearners/variables exceeds nbars, aggregate some values
  if( nlevels(plot_data[, type]) > nbars ) {
    # logical vector indicating rows to be subsumed
    others <- !(plot_data[, type] %in% tail(levels(plot_data[, type]), nbars-1))    
    
    for( i in c("blearner","variable") ) {
      # add new level OTHER to factor variable blearner/variable
      plot_data[, i] <- 
        ordered(plot_data[, i], levels = c("other", levels(plot_data[, i])))
      # set baselearner/variable names to OTHER depending on nbars
      plot_data[others, i] <- "other"
    }    
    
    # add up risk reduction and selfreqs for OTHER
    plot_data[others, "reduction"] <- sum( plot_data[others, "reduction"] )
    plot_data[others, "selfreq"]   <- sum( plot_data[others, "selfreq"] )
    
    # use only number of observations corresponding to nbars
    plot_data <- plot_data[c(which(!others), which(others)[1]),] 
    
    # update ordered factors
    for( i in c("blearner","variable") ) 
      plot_data[, i] <- factor(plot_data[, i])
  }  
  
  
  ### --------------------------------------------------
  ### create bar labels
  # (maybe) truncate bar labels for baselearner or variable names
  trunc_levels <- sapply(levels(plot_data[, type]), 
    function(name) {
      paste(strtrim(name, maxchar), if( nchar(name) < maxchar ) "" else "..") 
    } 
  )  
  
  # additional warning message if bar labels are not distinct (after truncation)
  if( length(unique(trunc_levels)) != nlevels(plot_data[, type]) ) {
    warning(paste("'maxchar' set too small to distinguish", type, "labels.",
      "Different variables are probably summed up.",
      "Value for argument 'maxchar' should be incremented."))
  }
  levels(plot_data[, type]) <- trunc_levels
  
  # convert rounded selfreqs to character
  plot_data[, "selfreq"] <- 
    as.character(round(plot_data[, "selfreq"], digits = 2))  
  
  # for type = "variable" additionally accumulate selfreqs per variable 
  # in order of risk reduction of involved baselearners
  if( type == "variable" ) {    
    # reverse order of baselearners (larger stacks first)
    # (also compare line 128ff - setting of baselearner order)
    plot_data[, "blearner"] <- 
      ordered(plot_data[, "blearner"], rev(levels(plot_data[, "blearner"])))
    # sum up selfreqs
    plot_data[, "selfreq"] <- sapply(plot_data[, "variable"], function(i) {
      do.call( function(...) paste(..., sep = " + "), as.list( 
        plot_data[plot_data$variable == i, "selfreq"]
        [order(plot_data[plot_data$variable == i, "blearner"])] 
      ) )
    })
  }
  
  # add selfreqs to bar labels
  selfreq_labels = unlist( unique(plot_data[, c(type, "selfreq")])[2] )
  
  levels(plot_data[, type]) = paste0( levels(plot_data[, type]), "\n ",
    "sel. freq: ~", selfreq_labels[order(unique(plot_data[, type]))] )  
  

  ### --------------------------------------------------
  ### activate auto.key if bars are stacked (only other bars than 'others')
  if( !("auto.key" %in% names(args)) ) {
    auto.key <- nlevels(plot_data[, "variable"]) < nrow(plot_data)
  }
  
  
  ### --------------------------------------------------
  ### create final plot depending on type
  if( type == "variable" ) {
    barchart(variable ~ reduction, groups = plot_data[, "blearner"], 
      data = plot_data, horizontal = TRUE, xlab = xlab, ylab = ylab, xlim =xlim,
      scales = list(x = list(tck = c(1,0), at = seq(0,sum(x), length.out = 5))), 
      stack = TRUE, auto.key = auto.key, ...)
  } else {
    barchart(blearner ~ reduction, data = plot_data,
      horizontal = TRUE, xlab = xlab, ylab = ylab, xlim = xlim,
      scales = list(x = list(tck = c(1,0), at = seq(0,sum(x), length.out = 5))),
      ...)
  }
}  

print.varimp <- function(x, ...) {
    cat("Variable importance (fraction of in-bag risk reduction):\n\n")
    attr(x, "selfreqs") <- NULL
    attr(x, "variable_names") <- NULL
    class(x) <- "numeric"
    NextMethod()
}

### <FIXME>:
###          add link argument (family needs to be touched)
### </FIXME>
.predictmboost <- function(y, pr, type, nm, family) {
    if (!isMATRIX(pr)) {
        pr <- as.vector(pr)
        names(pr) <- nm
    }
    if (is.list(pr)) {
        rownames(pr) <- nm
        if (type != "link")
            warning("argument", sQuote("type"), " is ignored")
        return(pr)
    }
    if (type == "link") ret <- pr
    if (type == "response") ret <- family@response(pr)
    if (type == "class") {
        if (!is.factor(y)) stop("response is not a factor")
            ret <- factor(levels(y)[family@rclass(pr)], levels = levels(y))
        }
    names(ret) <- names(pr)
    return(ret)
}

predict.mboost <- function(object, newdata = NULL,
    type = c("link", "response", "class"), which = NULL,
    aggregate = c("sum", "cumsum", "none"), ...) {

    type <- match.arg(type)
    aggregate <- match.arg(aggregate)
    if (aggregate != "sum")
        stopifnot(type == "link")
    pr <- object$predict(newdata = newdata,
                         which = which, aggregate = aggregate)
    nm <- rownames(newdata)
    if (is.null(newdata)) 
        nm <- object$rownames
    if (length(pr) == 1 && length(nm) != length(pr))
        nm <- NULL
    if (is.list(pr)){
        RET <- lapply(pr, .predictmboost, y = object$response,
                      type = type, nm = nm, family = object$family)
        if (type == "link") attr(RET, "offset") <- attr(pr, "offset")
        return(RET)
    }
    RET <- .predictmboost(object$response, pr, type = type, nm = nm,
                          family = object$family)
    if (type != "link") attr(RET, "offset") <- NULL
    return(RET)
}

### extract coefficients
coef.mboost <- function(object, which = NULL,
    aggregate = c("sum", "cumsum", "none"), ...) {

    args <- list(...)
    if (length(args) > 0)
        warning("Arguments ", paste(names(args), sep = ", "), " unknown")
    if (grepl("Negative Binomial Likelihood (logit link)", object$family@name))
        message("\nNOTE: Coefficients from a Binomial model are half the size of ",
                "coefficients\n from a model fitted via ",
                "glm(... , family = 'binomial').\n",
                "See Warning section in ?coef.mboost\n")
    ### NOTE: This is only the case for type = "adaboost" and link = "logit" ! 
    ###       see github #63 and #65
    object$coef(which = which, aggregate = aggregate)
}

### compute boosting hat matrix and its trace
hatvalues.gamboost <- function(model, ...) {
    H <- model$hatvalues(...)
    n <- length(model$response)

    ### <FIXME> better checks
    L2 <- FALSE
    if (!extends(class(model$family), "boost_family_glm")) {
        warning("AIC might not be reasonable for family ", model$family@name)
        L2 <- TRUE
    } else {
        fW <- model$family@fW
    }
    ### </FIXME>
    if (checkL2(model) || L2) {
        op <- .Call("R_trace_gamboost", as.integer(n), H,
                    as.integer(model$xselect()), PACKAGE = "mboost")
    } else {
        fitm <- predict(model, aggregate = "cumsum")
        op <- bhatmat(n, H, model$xselect(), fitm, model$family@fW)
    }
    RET <- diag(op[[1]])
    attr(RET, "hatmatrix") <- op[[1]]
    attr(RET, "trace") <- op[[2]]
    RET
}

hatvalues.mboost <- function(model, ...) {
    stop("hatvalues are not implemented for models fitted via function ",
         sQuote("mboost"), ".\n Please use functions ", sQuote("gamboost"),
         " or ", sQuote("glmboost"), " instead.")
}

AIC.mboost <- function(object, method = c("corrected", "classical", "gMDL"),
                       df = c("trace", "actset"), ..., k = 2) {

    df <- match.arg(df)
    if (df == "actset" && !inherits(object, "glmboost")) {
        df <- "trace"
        warning("df = ", dQuote("actset"), " can only be used with ",
                sQuote("glmboost"), " models. df = ", dQuote("trace"),
                " is used instead.")
    }
    if (df == "trace") {
        hatval <- hatvalues(object)
        RET <- AICboost(object, method = method,
                        df = attr(hatval, "trace"), k = k)
    }
    if (df == "actset") {
        ### compute active set: number of non-zero coefficients
        ### for each boosting iteration
        xs <- object$xselect()
        xu <- sort(sapply(unique(xs), function(i) which(xs == i)[1]))
        xu <- c(xu, mstop(object) + 1)
        df <- rep(1:(length(xu) - 1), diff(xu))
        ### <FIXME>: offset = 0 may mean hat(offset) = 0 or
        ### no offset computed at all!
        if (object$offset != 0) df <- df + 1
        ### </FIXME>
        RET <- AICboost(object, method = method,
                        df = df, k = k)
    }
    return(RET)
}

AICboost <- function(object, method = c("corrected", "classical", "gMDL"), df, k = 2) {

    if (object$control$risk != "inbag")
        return(NA)
    method <- match.arg(method)

    if (checkL2(object) && method == "classical")
        stop("classical AIC method not implemented for Gaussian family")
    if (!checkL2(object) && method == "corrected")
        stop("corrected AIC method not implemented for non-Gaussian family")

    sumw <- sum(model.weights(object)[!is.na(fitted(object))])
    if (method == "corrected")
        AIC <- log(object$risk()[-1] / sumw) +
               (1 + df/sumw) / (1 - (df + 2)/sumw)

    ### loss-function is to be MINIMIZED, take -2 * logLik == 2 * risk
    if (method == "classical")
        AIC <- 2 * object$risk()[-1] + k * df
    if (method == "gMDL"){
        s <- object$risk()[-1]/(sumw - df)
        AIC <- log(s) + df/sumw * log((sum(object$response^2) - object$risk()[-1])
                      /(df * s))
        }
    mstop <- which.min(AIC)
    RET <- AIC[mstop]

    attr(RET, "mstop") <- which.min(AIC)
    attr(RET, "df") <- df
    attr(RET, "AIC") <- AIC
    attr(RET, "corrected") <- method == "corrected"

    class(RET) <- "gbAIC"
    return(RET)
}

print.gbAIC <- function(x, ...) {
    mstop <- mstop(x)
    df <- attr(x, "df")[mstop]
    attributes(x) <- NULL
    print(x)
    cat("Optimal number of boosting iterations:", mstop, "\n")
    cat("Degrees of freedom", paste("(for mstop = ", mstop, "):", sep = "",
                                    collapse = ""), df, "\n")
    invisible(x)
}

plot.gbAIC <- function(x, y = NULL, ...) {
    mstop <- mstop(x)
    class(x) <- NULL
    plot(attr(x, "AIC"), xlab = "Number of boosting iterations",
         ylab = ifelse(attr(x, "corrected"), "Corrected AIC", "AIC"),
         type = "l", ...)
    points(mstop, x)
    ylim <- list(...)$ylim
    if (!is.null(ylim)) {
        ymin <- ylim[1] * ifelse(ylim[1] < 0, 2, 0.5)
    } else {
        ymin <- x - x/2
    }
    lines(c(mstop, mstop),
          c(ymin, x), lty = 2)
}

mstop <- function(object, ...) UseMethod("mstop")

mstop.gbAIC <- function(object, ...) attr(object, "mstop")


### compute fitted values
fitted.mboost <- function(object, ...) {
    args <- list(...)
    if (length(args) == 0) {
        ret <- object$fitted()
        if (length(ret) == length(object$rownames))
            names(ret) <- object$rownames
    } else {
        if ("newdata" %in% names(args)) {
            args$newdata <- NULL
            warning("Argument ", sQuote("newdata"), " was  ignored. Please use ", 
                    sQuote("predict()"), " to make predictions for new data.")
        }
        args$object <- object
        ret <- do.call(predict, args)
    }
    ret
}

### residuals (the current negative gradient)
resid.mboost <- function(object, ...)
    object$resid()

logLik.mboost <- function(object, ...)
    object$logLik()

### restrict or enhance models to less/more
### boosting iterations.
### ATTENTION: x gets CHANGED!
"[.mboost" <- function(x, i, return = TRUE, ...) {
    if (!(length(i) == 1 && i >= 0))
        stop("Please provide a single non-negative number")
    x$subset(i)
    if (return) return(x)
    invisible(NULL)
}

"mstop<-" <- function(x, value) {
    return(x[value, return = TRUE])
}

mstop.mboost <- function(object, ...) object$mstop()

update.mboost <- function(object, weights, oobweights = NULL,
                          risk = NULL, trace = NULL, ...)
    object$update(weights = weights, oobweights = oobweights,
                  risk = risk, trace = trace)

model.frame.mboost <- function(formula, ...)
    formula$model.frame(...)

predict.glmboost <- function(object, newdata = NULL,
    type = c("link", "response", "class"), which = NULL,
    aggregate = c("sum", "cumsum", "none"), ...) {

    args <- list(...)
    if (length(args) > 0)
        warning("Arguments ", paste(names(args), sep = ", "), " unknown")

    aggregate <- match.arg(aggregate)

    ### X <- object$baselearner[[1]]$get_data()
    if (!is.null(newdata))
        newdata <- object$newX(newdata)

    ### <FIXME> implement predictions here </FIXME>
    ### which <- object$which(which, usedonly = nw <- is.null(which))
    ### cf <- coef(object, which = which, aggregate = aggregate)
    ### pr <- X[, which, drop = FALSE] %*% cf

    pr <- object$predict(newdata = newdata, which = which,
                         aggregate = aggregate)
    type <- match.arg(type)
    nm <- rownames(newdata)
    if (is.null(newdata)) 
        nm <- object$rownames
    if (length(pr) == 1 && length(nm) != length(pr))
        nm <- NULL
    if (is.list(pr))
        return(lapply(pr, .predictmboost, y = object$response,
                      type = type, nm = nm, family = object$family))
    .predictmboost(object$response, pr, type = type, nm = nm,
                   family = object$family)
}

coef.glmboost <- function(object, which = NULL,
    aggregate = c("sum", "cumsum", "none"), off2int = FALSE, ...) {

    args <- list(...)
    if (length(args) > 0)
        warning("Arguments ", paste(names(args), sep = ", "), " unknown")

    if (grepl("Negative Binomial Likelihood", object$family@name))
        message("\nNOTE: Coefficients from a Binomial model are half the size of ",
                "coefficients\n from a model fitted via ",
                "glm(... , family = 'binomial').\n",
                "See Warning section in ?coef.mboost\n")

    aggregate <- match.arg(aggregate)
    cf <- object$coef(which = which, aggregate = aggregate)
    offset <- object$offset
    
    if (is.null(cf))
        return(cf)

    ### intercept = hat(beta[1]) - bar(x) %*% hat(beta[-1])
    assign <- object$assign
    cm <- object$center
    INTERCEPT <- sum(assign == 0) == 1
    if (!INTERCEPT && off2int)
        warning(sQuote("object"), " does not contain an intercept, ",
                sQuote("off2int = TRUE"), " ignored.")
    if (INTERCEPT && any(cm != 0)) {
        intercept <- which(assign == 0)
        if (is.null(which)) {
            which <- object$which(usedonly = TRUE)
            if (!intercept %in% which) {
                which <- c(intercept, which)
                cf <- object$coef(which = which, aggregate = aggregate)
            }
        }
        which <- object$which(which)
        if (intercept %in% which) {
            if (all(which %in% object$which(usedonly = TRUE))) {
                cm <- cm[which]
                scf <- sapply(1:length(cf), function(i) cf[[i]] * cm[i])
                if (!is.matrix(scf)) scf <- matrix(scf, nrow = 1)
                cf[[intercept]] <- cf[[intercept]] - rowSums(scf)
            } else {
                cm <- cm[object$which(usedonly = TRUE)]
                tmp <- object$coef(which = object$which(usedonly = TRUE),
                                   aggregate = aggregate)
                scf <- sapply(1:length(tmp), function(i) tmp[[i]] * cm[i])
                if (!is.matrix(scf)) scf <- matrix(scf, nrow = 1)
                cf[[intercept]] <- cf[[intercept]] - rowSums(scf)
            }
        }
    }

    if (aggregate == "sum") cf <- unlist(cf)
    if (aggregate == "none") {
        attr(cf, "offset") <- offset
        if (off2int)
            warning(sQuote("off2int = TRUE"), " ignored for ",
                    sQuote("aggregate = \"none\""))
    } else {
        if (off2int & length(offset) == 1) {
            cf[[1]] <- cf[[1]] + offset
        } else {
            if (off2int)
                warning(sQuote("off2int = TRUE"),
                        " ignored for non-scalar offset")
            attr(cf, "offset") <- offset
        }
    }
    cf
}

hatvalues.glmboost <- function(model, ...) {

    args <- list(...)
    if (length(args) > 0)
        warning("Arguments ", paste(names(args), sep = ", "), " unknown")

    if (!checkL2(model)) return(hatvalues.gamboost(model))
    Xf <- t(model$basemodel[[1]]$MPinv()) * model$control$nu
    X <- model$baselearner[[1]]$get_data()
    op <- .Call("R_trace_glmboost", as(X, "matrix"), as(Xf, "matrix"),
                as.integer(model$xselect()),
                PACKAGE = "mboost")
    RET <- diag(op[[1]])
    attr(RET, "hatmatrix") <- op[[1]]
    attr(RET, "trace") <- op[[2]]
    RET
}

### methods: print
print.mboost <- function(x, ...) {

    cat("\n")
    cat("\t Model-based Boosting\n")
    cat("\n")
    if (!is.null(x$call)) {
        if(length(deparse(x$call$data)) > 20)
            x$call$data <- deparse(x$call$data, nlines = 1)
        cat("Call:\n", deparse(x$call), "\n\n", sep = "")
    }
    show(x$family)
    cat("\n")
    cat("Number of boosting iterations: mstop =", mstop(x), "\n")
    cat("Step size: ", x$control$nu, "\n")
    cat("Offset: ", x$offset, "\n")
    cat("Number of baselearners: ", length(variable.names(x)), "\n")
    cat("\n")
    invisible(x)
}

### methods: print
print.glmboost <- function(x, ...) {

    cat("\n")
    cat("\t Generalized Linear Models Fitted via Gradient Boosting\n")
    cat("\n")
    if (!is.null(x$call)) {
        if(length(deparse(x$call$data)) > 20)
            x$call$data <- deparse(x$call$data, nlines = 1)
        cat("Call:\n", deparse(x$call), "\n\n", sep = "")
    }
    show(x$family)
    cat("\n")
    cat("Number of boosting iterations: mstop =", mstop(x), "\n")
    cat("Step size: ", x$control$nu, "\n")
    cat("Offset: ", x$offset, "\n")
    cat("\n")
    cat("Coefficients: \n")
    cf <- coef(x)
    attr(x, "offset") <- NULL
    print(cf)
    cat("\n")
    invisible(x)
}

variable.names.mboost <- function(object, which = NULL, usedonly = FALSE, ...) {

    which <- object$which(which, usedonly = usedonly)

    args <- list(...)
    if (length(args) > 0)
        warning("Arguments ", paste(names(args), sep = ", "), " unknown")

    ### <FIXME> is pasting what we want?
    ret <- sapply(object$baselearner, function(x)
                  paste(x$get_names(), collapse = ", "))
    ### </FIXME>
    if (is.matrix(ret)) ret <- ret[, , drop = TRUE]
    ret[which]
}

variable.names.glmboost <- function(object, which = NULL, usedonly = FALSE, ...) {

    if (usedonly) {
        which <- object$which(usedonly = TRUE)
        ## if center = TRUE for model fitting intercept is implicitly selected
        center <- get("center", envir = environment(object$newX))
        if (center){
            intercept <- which(object$assign == 0)
            INTERCEPT <- sum(object$assign == 0) == 1
            if (INTERCEPT && !intercept %in% which)
                which <- c(intercept, which)
        }
    } else {
        which <- object$which(which, usedonly = usedonly)
    }

    args <- list(...)
    if (length(args) > 0)
        warning("Arguments ", paste(names(args), sep = ", "), " unknown")

    ret <- object$baselearner[[1]]$get_names()
    names(ret) <- ret
    ret[which]
}

selected.mboost <- function(object, ...)
    object$xselect()

summary.mboost <- function(object, ...) {

    ret <- list(object = object, selprob = NULL)
    xs <- selected(object)
    nm <- variable.names(object)
    if (length(xs) > 0) {
        selprob <- tabulate(xs, nbins = length(nm)) / length(xs)
        names(selprob) <- names(nm)
        selprob <- sort(selprob, decreasing = TRUE)
        ret$selprob <- selprob[selprob > 0]
    }
    class(ret) <- "summary.mboost"
    return(ret)
}

print.summary.mboost <- function(x, ...) {
    print(x$object)
    if (!is.null(x$selprob)) {
        cat("Selection frequencies:\n")
        print(x$selprob)
        cat("\n")
    } 
}

nuisance <- function(object)
    UseMethod("nuisance")

nuisance.mboost <- function(object)
    object$nuisance()[[mstop(object)]]



extract <- function(object, ...)
    UseMethod("extract")

extract.mboost <- function(object, what = c("design", "penalty", "lambda", "df",
                                   "coefficients", "residuals",
                                   "variable.names", "bnames", "offset",
                                   "nuisance", "weights", "index", "control"),
                           which = NULL, ...){
    what <- match.arg(what)
    which <- object$which(which, usedonly = is.null(which))
    if (what %in% c("design", "penalty", "lambda", "df", "index")){
        fun <- function(which)
            extract(object$basemodel[[which]], what = what, ...)
        ret <- lapply(which, fun)
        names(ret) <- extract(object, what = "bnames", which = which)
        return(ret)
    }
    switch(what,
           "coefficients" = return(coef(object, which = which)),
           "residuals" = return(residuals(object)),
           "variable.names" = return(variable.names(object, which)),
           "bnames" = return(get("bnames", envir = environment(object$update))[which]),
           "offset" = return(object$offset),
           "nuisance" = return(nuisance(object)),
           "weights" = return(model.weights(object)),
           "control" = return(object$control))
}

extract.glmboost <- function(object, what = c("design", "coefficients", "residuals",
                                     "variable.names", "offset",
                                     "nuisance", "weights", "control"),
                             which = NULL, asmatrix = FALSE, ...){
    what <- match.arg(what)
    center <- get("center", envir = environment(object$newX))
    if (is.null(which)) {
        which <- object$which(usedonly = TRUE)
        ## if center = TRUE for model fitting intercept is implicitly selected
        if (center){
            intercept <- which(object$assign == 0)
            INTERCEPT <- sum(object$assign == 0) == 1
            if (INTERCEPT && !intercept %in% which)
                which <- c(intercept, which)
        }
    } else {
        which <- object$which(which)
    }

    if (what == "design"){
        mat <- object$baselearner[[1]]$get_data()[,which]
        if (asmatrix)
            mat <- as.matrix(mat)
        return(mat)
    }

    switch(what,
           "coefficients" = return(coef(object, which = which)),
           "residuals" = return(residuals(object)),
           "variable.names" = return(variable.names(object, which)),
           "offset" = return(object$offset),
           "nuisance" = return(nuisance(object)),
           "weights" = return(model.weights(object)),
           "control" = return(object$control))
}

extract.blackboost <- function(object, ...)
    stop("function not yet implemented")

extract.blg <- function(object, what = c("design", "penalty", "index"),
                        asmatrix = FALSE, expand = FALSE, ...){
    what <- match.arg(what)
    #object <- object$dpp(rep(1, NROW(object$model.frame())))
    # return(extract(object, what = what,
    #               asmatrix = asmatrix, expand = expand))
    if (what == "design")
        mat <- get("X", envir = environment(object$dpp))
    if (what == "penalty")
        mat <- get("K", envir = environment(object$dpp))
    if (what == "index")
        return(get("index", envir = environment(object$dpp)))
    ## only applicable for design and penalty matrix
    if (asmatrix){
        mat <- as.matrix(mat)
    }
    if (expand && !is.null(indx <- extract(object, what = "index"))){
        a <- attributes(mat)
        mat <- mat[indx,]
        a[c("dim", "dimnames")] <- attributes(mat)[c("dim", "dimnames")]
        attributes(mat) <- a
    }
    return(mat)
}

extract.bl_lin <- function(object, what = c("design", "penalty", "lambda", "df",
                                   "weights", "index"),
                           asmatrix = FALSE, expand = FALSE,  ...){
    what <- match.arg(what)
    if (what == "design")
        mat <- get("X", envir = environment(object$fit))
    if (what == "penalty")
        mat <- get("K", envir = environment(object$fit))
    if (what %in% c("lambda", "df"))
        return(object$df()[what])
    if (what %in% c("weights", "index"))
        return(get(what, envir = environment(object$fit)))
    ## only applicable for design and penalty matrix
    if (asmatrix){
        mat <- as.matrix(mat)
    }
    if (expand && !is.null(indx <- extract(object, what = "index"))){
        a <- attributes(mat)
        mat <- mat[indx,]
        a[c("dim", "dimnames")] <- attributes(mat)[c("dim", "dimnames")]
        attributes(mat) <- a
    }
    return(mat)
}

extract.bl_tree <- function(object, what = c("design", "penalty", "lambda", "df",
                                    "weights", "index"),
                            ...){
    what <- match.arg(what)
    if (what == "weights"){
        return(get("weights", envir = environment(object$fit)))
    } else {
        warning(paste("model matrix, penalty matrix, lambda and index",
                      "do not exist for tree base-learners"))
        invisible(NULL)
    }
}

residuals.mboost <- function(object, ...){
    if(object$family@name == "Squared Error (Regression)")
        return(object$resid())
    stop(sQuote("residuals()"), " only implemented for ",
         sQuote("family = Gaussian()"))
}

risk <- function(object, ...)
    UseMethod("risk")

risk.mboost <- function(object, ...) {
    object$risk()
}

downstream.test <- function(object, ...) {
    if (object$family@name != "Ratio of Correlated Gammas (RCG)")
        stop("downstream tests currently only implemented for RCG family")
    if (!inherits(object, "glmboost"))
        stop("downstream tests currently only implemented for linear models, i.e., glmboost")
    
    # calculate Fisher information matrix
    alpha <- nuisance(object)[1]
    rho <- nuisance(object)[2]
    coefs <- coef(object, which = "")
    y <- object$response
    ## do we need the original design matrix (X1) or the centered design (X) matrix?
    X <- as.data.frame(extract(object, what = "design"))
    X1 <- data.frame(model.matrix(~ x1 + x2))
    
    Fisher <- environment(object$family@ngradient)[["Fisher"]](y, alpha, rho, X1, coefs)
    
    # downstream hypothesis tests
    p_value <- sapply(1:length(coefs), function(i) (1 - pnorm(abs(coefs[i]) / sqrt(solve(Fisher)[i, i]))) * 2)
    return(p_value)
}

bmrf <- function (..., by = NULL, index = NULL, bnd = NULL, df = 4,
                  lambda = NULL, center = FALSE) {
    if (!requireNamespace("BayesX"))
        stop("cannot load ", sQuote("BayesX"))

    cll <- match.call()
    cll[[1]] <- as.name("bmrf")
    cll <- deparse(cll)
    mf <- list(...)
    if (length(mf) == 1 && (is.matrix(mf[[1]]) || is.data.frame(mf[[1]]))) {
        mf <- as.data.frame(mf[[1]])
    }
    else {
        mf <- as.data.frame(mf)
        cl <- as.list(match.call(expand.dots = FALSE))[2][[1]]
        colnames(mf) <- sapply(cl, function(x) deparse(x))
    }
    stopifnot(is.data.frame(mf))
    if (!(all(sapply(mf, is.factor)))) {
        stop("cannot compute bmrf for numeric variables, region variable must be given as factor.")
    }
    vary <- ""
    if (!is.null(by)) {
        stopifnot(is.numeric(by) || (is.factor(by) && nlevels(by) ==
            2))
        mf <- cbind(mf, by)
        colnames(mf)[ncol(mf)] <- vary <- deparse(substitute(by))
    }
    CC <- all(complete.cases(mf))
    if (!CC)
        warning("base-learner contains missing values;\n",
                "missing values are excluded per base-learner, ",
                "i.e., base-learners may depend on different",
                " numbers of observations.")
    DOINDEX <- (nrow(mf) > 10000)
    if (is.null(index)) {
        if (!CC || DOINDEX) {
            index <- get_index(mf)
            mf <- mf[index[[1]], , drop = FALSE]
            index <- index[[2]]
        }
    }
    ret <- list(model.frame = function() if (is.null(index)) return(mf) else return(mf[index,
        , drop = FALSE]), get_call = function() cll, get_data = function() mf,
        get_index = function() index, get_vary = function() vary,
        get_names = function() colnames(mf), set_names = function(value) attr(mf,
            "names") <<- value)
    class(ret) <- "blg"
    ret$dpp <- bl_lin(ret, Xfun = X_bmrf, args = hyper_bmrf(mf,
        vary, bnd = bnd, df = df, lambda = lambda, center = center))
    return(ret)
}

hyper_bmrf <- function (mf, vary, bnd = NULL, df = 4, lambda = NULL,
                        center = FALSE) {
    if (is.null(bnd))
        stop("Neighbourhood relations must be given in matrix or boundary format.")
    else if (inherits(bnd, "bnd"))
        K <- BayesX::bnd2gra(bnd)
    else if (isMATRIX(bnd) &&
             nrow(bnd) == ncol(bnd) &&
             nlevels(mf[[1]]) <= nrow(bnd) &&
             sum(bnd) == 0 &&
             all(levels(mf[[1]]) %in% rownames(bnd)))
        K <- bnd
    else stop("Neighbourhood matrix not defined as stated in manual page.")
    ###    K <- as(K, "matrix")
    if (!is(K, "Matrix"))
        K <- Matrix(unclass(K))
    nm <- colnames(mf)[colnames(mf) != vary]
    list(K = K, bnd = bnd, pen = TRUE, df = df, lambda = lambda,
         center = center)
}

X_bmrf <- function (mf, vary, args) {
    K <- args$K
    districts <- rownames(K)
    X <- Diagonal(nrow(K))
    rownames(X) <- districts
    colnames(X) <- districts
    X <- X[as.character(mf[,1]),]
    # X <- matrix(0, nrow = nrow(mf), ncol = ncol(K))
    # for (i in 1:nrow(mf)) X[i, which(districts == mf[i, 1])] <- 1
    # MATRIX <- any(dim(X) > c(500, 50))
    # MATRIX <- MATRIX && options("mboost_useMatrix")$mboost_useMatrix
    # if (MATRIX) {
    #    diag <- Diagonal
    #     X <- Matrix(X)
    # }
    if (vary != "") {
        by <- model.matrix(as.formula(paste("~", vary, collapse = "")),
            data = mf)[, 2]
        X <- X * by
    }
    if (isTRUE(args$center)) {
        ### L = \Gamma \Omega^1/2 in Section 2.3. of Fahrmeir et al.
        ### (2004, Stat Sinica)
        SVD <- eigen(K)
        ev <- SVD$vectors[, -dim(SVD$vectors)[2]]
        ew <- SVD$values[-length(SVD$values)]
        X <- as(X %*% ev %*% diag(1/sqrt(ew)), "matrix")
        K <- as(diag(ncol(X)), "matrix")
    }
    return(list(X = X, K = K))
}

options(digits = 3)


.all.equal <- function(...) isTRUE(all.equal(..., check.environment = FALSE))

require("mboost")
require("Matrix")

set.seed(290875)

### predict did not return factor levels for blackboost models
dummy <- data.frame(y = gl(2, 100), x = runif(200))
pr <- predict(blackboost(y ~ x, data = dummy, family = Binomial()),
              newdata = dummy, type = "class")
print(is.factor(pr) && all(levels(pr) %in% levels(dummy$y)))

### predict for g{al}mboost.matrix did not work
ctrl <- boost_control(mstop = 10)
X <- cbind(int = 1, x = dummy$x)
gb <- glmboost(x = X, y = dummy$y, family = Binomial(), center = FALSE,
               control = ctrl)
print(.all.equal(predict(gb), predict(gb, newdata = X)))

if (FALSE) {
gb <- gamboost(x = X, y = dummy$y, family = Binomial(),
               control = ctrl)
print(.all.equal(predict(gb), predict(gb, newdata = X)))
}

### predict with center = TRUE in glmboost.matrix() did not work
gb <- glmboost(x = X, y = dummy$y, family = Binomial(),
               control = ctrl, center=TRUE)
p1 <- X %*% coef(gb)
print(max(abs(p1 - predict(gb, newdata = X))) < sqrt(.Machine$double.eps))

### blackboost _did_ touch the response, arg!

data("bodyfat", package = "TH.data")
ctrl <- boost_control(mstop = 500, nu = 0.01)
bb <- blackboost(DEXfat ~ ., data = bodyfat, control = ctrl)
n <- nrow(bodyfat)
bs <- rmultinom(3, n, rep(1, n) / n)
x <- seq(from = 10, to = 500, by = 10)
cv <- cvrisk(bb, bs, grid = x, papply = lapply)
ctrl$risk <- "oobag"
tmp <- blackboost(DEXfat ~ ., data = bodyfat, control = ctrl,
                 weights = bs[,3])

print(identical(max(abs(tmp$risk()[x + 1] / sum(bs[,3] == 0)  - cv[3,])), 0))

### center = TRUE and cvrisk were broken; same issue with masking original data

gb <- glmboost(DEXfat ~ ., data = bodyfat, center = TRUE)
cv1 <- cvrisk(gb, folds = bs, papply = lapply)
tmp <- glmboost(DEXfat ~ ., data = bodyfat, center = TRUE,
                control = boost_control(risk = "oobag"),
                weights = bs[,3])
print(identical(max(tmp$risk()[attr(cv1, "mstop") + 1] / sum(bs[,3] == 0) - cv1[3,]), 0))

### same problem, just another check

indep <- names(bodyfat)[names(bodyfat) != "DEXfat"]
cbodyfat <- bodyfat
cbodyfat[indep] <- lapply(cbodyfat[indep], function(x) x - mean(x))
bffm <- DEXfat ~ age + waistcirc + hipcirc + elbowbreadth + kneebreadth +
      anthro3a + anthro3b + anthro3c + anthro4

bf_glm_1 <- glmboost(bffm, data = cbodyfat, center = FALSE)
cv1 <- cvrisk(bf_glm_1, folds = bs, papply = lapply)
bf_glm_2 <- glmboost(bffm, data = bodyfat, center = TRUE)
cv2 <- cvrisk(bf_glm_2, folds = bs, papply = lapply)

print(mstop(cv1) == mstop(cv2))

if (FALSE){
### dfbase=1 was not working correctly for ssp
### spotted by Matthias Schmid <Matthias.Schmid@imbe.imed.uni-erlangen.de>
data("bodyfat", package = "TH.data")
ctrl <- boost_control(mstop = 100)
### COMMENT: Not using ssp here but P-splines
### Remove check!
ga <- gamboost(DEXfat ~ ., data = bodyfat, dfbase = 1, control = ctrl)
gl <- glmboost(DEXfat ~ ., data = bodyfat, center = TRUE, control = ctrl)
print(max(abs(predict(ga) - predict(gl))) < 1e-8)
AIC(gl)
}

if (FALSE) {
### prediction with matrices was broken for gamboost,
### spotted by <Max.Kuhn@pfizer.com>
x <- matrix(rnorm(1000), ncol = 10)
colnames(x) <- 1:ncol(x)
y <- rnorm(100)
fit <- gamboost(x = x, y = y, control = boost_control(mstop = 10))
a <- predict(fit, newdata = x[1:10,])
}

### AIC for centered covariates didn't work
y <- gl(2, 10)
xn <- rnorm(20)
xnm <- xn - mean(xn)
gc <- glmboost(y ~ xn, center = TRUE,
               family = Binomial())
g <- glmboost(y ~ xnm, center = FALSE, family = Binomial())
cgc <- coef(gc, off2int = TRUE)
cg <- coef(g, off2int = TRUE) - c(mean(xn) * coef(g)[2], 0)
names(cgc) <- NULL
names(cg) <- NULL
print(.all.equal(cgc, cg))
print(.all.equal(mstop(AIC(gc, "classical")),
                    mstop(AIC(g, "classical"))))

### fit ANCOVA models with centering
n <- 50
set.seed(290875)
x <- gl(3, n)
x1 <- sample(ordered(gl(3, n)))
z <- rnorm(length(x))
X <- model.matrix(~ x + z + x1)
eff <- X %*% (1:ncol(X))
y <- rnorm(length(x), mean = eff)

ctr <- list(list(x = "contr.treatment"), list(x = "contr.sum"),
            list(x = "contr.helmert"), list(x = "contr.SAS"))
mstop <- 2000
fm <- y ~ z:x + x + z:x1

for (cc in ctr) {
    modlm <- lm(fm, contrasts = cc)
    modT <- glmboost(fm, contrasts.arg = cc,
                     center = TRUE)[mstop]
    print(max(abs(coef(modlm) - coef(modT, off2int = TRUE)))
                      < .Machine$double.eps^(1/3))
    print(max(abs(hatvalues(modlm) - hatvalues(modT))) < 0.01)
    print(max(abs(predict(modlm) - predict(modT)))
                      < .Machine$double.eps^(1/3))
}

y <- factor(rbinom(length(x), size = 1,
                    prob = plogis(eff / max(abs(eff)) * 3)))
for (cc in ctr) {
    modlm <- glm(fm, contrasts = cc,
                 family = binomial())
    modT <- glmboost(fm, contrasts.arg = cc,
        center = TRUE, family = Binomial())[mstop]
    print(max(abs(coef(modlm) - coef(modT, off2int = TRUE) * 2))
                      < .Machine$double.eps^(1/3))
    print(max(abs(predict(modlm) - predict(modT) * 2))
                      < .Machine$double.eps^(1/3))
}


### check gamboost with weights (use weighted some of residuals
### for variable selection)
data("bodyfat", package = "TH.data")

set.seed(290875)
n <- nrow(bodyfat)
w <- numeric(n)
w <- rmultinom(1, n, rep(1, n) / n)[,1]
ctrl <- boost_control(mstop = 20)

mod1 <- glmboost(DEXfat ~ ., data = bodyfat, weights = w, center = FALSE)
aic1 <- AIC(mod1, "corrected")
attributes(aic1) <- NULL

mod2 <- glmboost(DEXfat ~ ., data = bodyfat[rep(1:n, w),], center = FALSE)
aic2 <- AIC(mod2, "corrected")
attributes(aic2) <- NULL

print(.all.equal(round(aic1, 3), round(aic2, 3)))

mod1 <- gamboost(DEXfat ~ ., data = bodyfat, weights = w, con = ctrl)
aic1 <- AIC(mod1, "corrected")
attributes(aic1) <- NULL

mod2 <- gamboost(DEXfat ~ ., data = bodyfat[rep(1:n, w),], con = ctrl)
aic2 <- AIC(mod2, "corrected")
attributes(aic2) <- NULL

print(.all.equal(round(aic1, 1), round(aic2, 1)))

mod1 <- blackboost(DEXfat ~ ., data = bodyfat, weights = w)
mod2 <- blackboost(DEXfat ~ ., data = bodyfat[rep(1:n, w),])

ratio <- mod1$risk() / mod2$risk()
print(ratio[1] > 0.95 && ratio[2] < 1.05)

if (FALSE){
### df <= 2
ctrl$risk <- "oobag"
mod1 <- gamboost(DEXfat ~ ., data = bodyfat, weights = w, con = ctrl, base = "bss", dfbase = 2)
mod2 <- gamboost(DEXfat ~ ., data = bodyfat, weights = w, con = ctrl, base = "bbs", dfbase = 2)
mod3 <- gamboost(DEXfat ~ ., data = bodyfat, weights = w, con = ctrl, base = "bols")
print(max(abs(predict(mod1) - predict(mod2))) < sqrt(.Machine$double.eps))
print(max(abs(predict(mod1) - predict(mod3))) < sqrt(.Machine$double.eps))
}

### check predictions of zero-weight observations (via out-of-bag risk)
mod1 <- gamboost(DEXfat ~ ., data = bodyfat, weights = w, con = ctrl, base = "bss")
mod2 <- gamboost(DEXfat ~ ., data = bodyfat, weights = w, con = ctrl, base = "bbs")
print(abs(coef(lm(mod1$risk() ~ mod2$risk() - 1)) - 1) < 0.01)

### not really a bug, a new feature; test fastp
df <- data.frame(y = rnorm(100), x = runif(100), z = runif(100))
eps <- sqrt(.Machine$double.eps)
s <- seq(from = 1, to = 100, by = 3)

x <- glmboost(y ~ ., data = df, center = FALSE)
for (i in s)
    print(max(abs(predict(x[i]) - predict(x[max(s)], agg = "cumsum")[,i])) < eps)

x <- gamboost(y ~ ., data = df)
for (i in s)
    print(max(abs(predict(x[i]) - predict(x[max(s)], agg = "cumsum")[,i])) < eps)

x <- blackboost(y ~ ., data = df)
for (i in s)
    print(max(abs(predict(x[i]) - predict(x[max(s)], agg = "cumsum")[,i])) < eps)

### make sure environment(formula) is used for evaluation
if (require("partykit")) {
    data("cars")
    ctl  <- boost_control(mstop = 100, trace = TRUE)
    tctl <- ctree_control(teststat = "max", testtype = "Teststat",
                          mincrit = 0, maxdepth = 5, saveinfo = FALSE)
    myfun <- function(cars, xx, zz){
        mboost(dist ~ btree(speed, tree_controls = zz),
               data = cars, control = xx)
    }
    try(mod <- myfun(cars, xx = ctl, zz = tctl))
}

### bbs with weights and expanded data
x <- runif(100)
y <- rnorm(length(x))
knots <- seq(from = 0.1, to = 0.9, by = 0.1)
w <- rmultinom(1, length(x), rep(1, length(x)) / length(x))[,1]
iw <- rep(1:length(x), w)

m1 <- bbs(x, knots = knots)$dpp(w)$fit(y)$model
m2 <- bbs(x[iw], knots = knots)$dpp(rep(1, length(iw)))$fit(y[iw])$model

print(max(abs(m1 - m2)) < sqrt(.Machine$double.eps))

### base learner handling
print(max(abs(fitted(gamboost(DEXfat ~ age, data = bodyfat)) -
                  fitted(gamboost(DEXfat ~ bbs(age), data = bodyfat)))) <
          sqrt(.Machine$double.eps))

### predict for matrix interface to glmboost
x <- matrix(runif(1000), ncol = 10)
y <- rowMeans(x) + rnorm(nrow(x))
mod <- glmboost(x = x, y = y, center = FALSE)
print(length(predict(mod, newdata = x[1:2,])) == 2)
try(predict(mod, newdata = as.data.frame(x[1:2,])))


### predict for varying coefficient models with categorical z
set.seed(1907)
x <- rnorm(100)
z <- gl(2,50)
y <- rnorm(100, sd = 0.1)
data <- data.frame(y=y, x=x, z=z)

model <- gamboost(y ~ bols(x, by=z), data = data)
print(!is.na(predict(model,data[1,-1])))

model <- gamboost(y ~ bbs(x, by=z), data = data)
print(!is.na(predict(model,data[1,-1])))

x <- as.factor(sample(1:10, 100, replace=TRUE))
data <- data.frame(y=y, x=x, z=z)
model <- gamboost(y ~ brandom(x, by=z), data = data)
print(!is.na(predict(model,data[1,-1])))

x1 <- rnorm(100)
x2 <- rnorm(100)
data <- data.frame(y=y, x1=x1, x2=x2, z=z)
model <- gamboost(y ~ bspatial(x1,x2, by=z), data = data)
print(!is.na(predict(model,data[1,-1])))

### bols with intercept = FALSE for categorical covariates was broken
x <- gl(2, 50)
y <- c(rnorm(50, mean = -1), rnorm(50, mean = 1))
print(length(coef(gamboost(y ~ bols(x, intercept=FALSE)))) == 1)

# check also for conntinuous x
x <- rnorm(100)
y <- c(rnorm(100, mean = 1 * x))
print(length(coef(gamboost(y ~ bols(x, intercept=FALSE)))) == 1)

### check interface of coef
set.seed(1907)
x1 <- rnorm(100)
int <- rep(1, 100)
y <- 3 * x1 + rnorm(100, sd=0.1)
dummy <- data.frame(y = y, int = int, x1 = x1)

gbm <- gamboost(y ~ bols(int, intercept=FALSE) +  bols(x1, intercept=FALSE) +
                bbs(x1, center=TRUE, df=1), data = dummy)

### check prediction if intercept=FALSE
gbm <- gamboost(y ~ bols(x1, intercept=FALSE), data = dummy)
print(!is.na(predict(gbm)) &
          max(abs(predict(gbm) - fitted(gbm))) < sqrt(.Machine$double.eps))

### check coef.glmboost
set.seed(1907)
x1 <- rnorm(100)
x2 <- rnorm(100)
y <- rnorm(100, mean= 3 * x1,sd=0.01)
linMod <- glmboost(y ~ x1 + x2, center = FALSE)
print(length(coef(linMod)) == 2)

### automated offset computation in Family
x <- rnorm(100)
y <- rnorm(100)
w <- drop(rmultinom(1, 100, rep(1 / 100, 100)))

G <- Gaussian()
fm <- Family(ngradient = G@ngradient, risk = G@risk)

m1 <- glmboost(y ~ x, family = G, center = FALSE)
m2 <- glmboost(y ~ x, family = fm, center = FALSE)
print(.all.equal(coef(m1), coef(m2)))

### formula evaluation
f <- function() {

    x <- rnorm(100)
    y <- rnorm(100)
    list(mboost(y ~ bbs(x)),
         gamboost(y ~ x),
         glmboost(y ~ x, center = FALSE),
         blackboost(y ~ x))
}
tmp <- f()

### both loss and risk given, spotted by
### Fabian Scheipl <fabian.scheipl@stat.uni-muenchen.de>
print(extends(class(Family(ngradient = function(y, f) y,
                               loss = function(y, w) sum(y),
                               risk = function(y, f, w) sum(y))),
                  "boost_family"))

### check coef with aggregate = "cumsum"
mod <- glmboost(DEXfat ~ ., data = bodyfat, center = FALSE)

print(max(abs(sapply(coef(mod, aggregate="cumsum"), function(x) x[,100])
                  - coef(mod))) < sqrt(.Machine$double.eps))

### get_index() was broken for factors
### (spotted by Juliane Schaefer <JSchaefer _at_ uhbs.ch>)
y <- factor(c(2, 1, 1, 2, 2, 2, 1, 1, 2, 2), levels = 1:2, labels = c("N", "S"))
x <- factor(c(2, 1, 2, 2, 2, 1, 1, 1, 2, 2), levels = 1:2,
            labels = c("No", "Yes"))
data <- data.frame(y,x)
m1 <- glm(y ~ x, data = data, family = binomial())
m2 <- glmboost(y ~ x, data = data, family = Binomial(),
               control = boost_control(mstop = 2000), center = FALSE)
m3 <- gamboost(y ~ bols(x), data = data, family = Binomial(),
               control = boost_control(mstop = 1000))
cf1 <- coef(m1)
a <- coef(m2); a[1] <- a[1] + m2$offset;
cf2 <- as.vector(a) * 2
b <- coef(m3); b[[1]][1] <- b[[1]][1] + m3$offset;
cf3 <- b[[1]] * 2

print(max(abs(cf1 - cf2)) < sqrt(.Machine$double.eps))
print(max(abs(cf1 - cf3)) < sqrt(.Machine$double.eps))
print(max(abs(predict(m2) - predict(m3))) < sqrt(.Machine$double.eps))

### bug in setting contrasts correctly
z <- as.ordered(gl(3, 10))
BL <- bols(z, contrasts.arg = "contr.treatment")$dpp
print(attr(get("X", envir = environment(BL)), "contrasts")$z == "contr.treatment")

### check dimensions
try(glmboost(x = matrix(1:10, nrow = 5), y = rnorm(4)))

### (slight) abuse of bolscw
X <- runif(1000)
X <- matrix(X, nrow = 100)
y <- rnorm(100)
glmboost(x = X, y = y)
b <- list(mboost:::bolscw(X))
mboost_fit(blg = b, response = y)
z <- rnorm(100)
b <- list(b1 = mboost:::bolscw(X), b2 = bbs(z))
mboost_fit(blg = b, response = y)
mod <- mboost_fit(blg = b, response = y)
cf <- coef(mod, which = 1)[[1]]
print(length(cf) == ncol(X))


### check interface of buser
### (K was fetched from global environment)
set.seed(1907)
x <- rnorm(100)
y <- rnorm(100, mean = x^2, sd = 0.1)
mod1 <- gamboost(y ~ bbs(x))
X1 <- extract(bbs(x))
K1 <- extract(bbs(x), "penalty")
mod2 <- gamboost(y ~ buser(X1, K1))
print(max(abs(predict(mod1) - predict(mod2))) < sqrt(.Machine$double.eps))


### check if mysolve works correctly
set.seed(1907)
X <- matrix(runif(1000), ncol = 10)
XM <- Matrix(X)
y <- rnorm(nrow(X))
# use interface for dense matrices from package Matrix
f1 <- fitted(gamboost(y ~ bols(XM)))
# use interface for matrices of class "matrix"
f2 <- fitted(gamboost(y ~ bols(X)))
print(max(abs(f1 - f2)) < sqrt(.Machine$double.eps))
## Now same with sparse matrix
X[X < .9] <- 0
XM <- Matrix(X)
# use interface for sparse matrices from package Matrix
f1 <- fitted(gamboost(y ~ bols(XM)))
# use interface for matrices of class "matrix"
f2 <- fitted(gamboost(y ~ bols(X)))
print(max(abs(f1 - f2)) < sqrt(.Machine$double.eps))

### check if bmrf works correctly
## (if not all locations are observed)
if (require("BayesX")) {
    germany <- read.bnd(system.file("examples/germany.bnd", package="BayesX"))
    districts <- attr(germany, "regions")
    set.seed(1907)
    regions <- factor(sample(districts, 400, replace = TRUE))
    B <- bmrf(regions, bnd = germany)
    X <- extract(B)
    K <- extract(B, what = "pen")
    Xs <- colSums(as.matrix(X))
    names(Xs) <- rownames(K)
    print(all(Xs[Xs > 0] == table(regions)[names(Xs[Xs > 0])]))

    ## check against old, slower code:
    Xold <- matrix(0, nrow = length(regions), ncol = ncol(K))
    for (i in 1:length(regions))
        Xold[i, which(districts == regions[i])] <- 1
    print(all(X == Xold))
}

## check handling of missing values
y <- yNa <- rnorm(100)
x1 <- rnorm(100)
x2 <- rnorm(100)
z1 <- as.factor(sample(1:10, 100, replace = TRUE))

yNa[1] <- NA
coef(mboost(yNa ~ x1))

yNa <- y
yNa[2] <- NaN
coef(mboost(yNa ~ x1))

x1[1] <- NA
z1[1] <- NA
mod <- mboost(y ~ bols(x1) + bbs(x1) + brandom(z1) +
                  bspatial(x1, x2) + brad(x1, x2, knots = 20) +
                  bmono(x1) +  buser(x1, K = 1, lambda = 0) + x2)

## check handling of lists without names
n <- 100
x1 <- rnorm(n)
x2 <- rnorm(n) + 0.25 * x1
y <- 3 * sin(x1) + x2^2 + rnorm(n)
spline1 <- bbs(x1, knots = 20, df = 4)
spline2 <- bbs(x2, knots = 10, df = 5)
mod1 <- mboost_fit(list(spline1, spline2), y)
mod2 <- mboost_fit(list(spline1), y)
mod3 <- mboost_fit(list(s1 = spline1, s2 = spline2), y)
names(coef(mod1))
names(coef(mod2))
names(coef(mod3))
extract(mod1, "bnames")
extract(mod2, "bnames")
extract(mod3, "bnames")

## check handling of newdata = list(), at least for common scenarios with lists
## [https://github.com/boost-R/mboost/issues/15]
data("bodyfat", package = "TH.data")
bf <- as.list(bodyfat)
mod <- mboost(DEXfat ~ bols(waistcirc) + bmono(hipcirc) + btree(age),
              data = bf)
## predict with data frame
nd <- bodyfat[1:2,]
pr1 <- predict(mod, newdata = nd)
## predict with list
nd <- as.list(bodyfat[1:2,])
pr2 <- predict(mod, newdata = nd)
print(pr1 == pr2)
## check plotting
nd <- list(waistcirc = 1, age = 1,
           hipcirc = seq(min(bf$hipcirc), max(bf$hipcirc), length = 100))
plot(mod, which = 2, newdata = nd)
nd <- data.frame(waistcirc = 1, age = 1,
                 hipcirc = seq(min(bf$hipcirc), max(bf$hipcirc), length = 100))
plot(mod, which = 2, newdata = nd)
lines(mod, which = 2, rug = TRUE)
## check user-specified labels
plot(mod, which = 2, newdata = nd, xlab = "hip circumference", ylab = "partial effect")

## check categorical variables
bodyfat$hip_cat <- cut(bodyfat$hipcirc, breaks = 2)
bodyfat$waist_cat <- cut(bodyfat$waistcirc, breaks = 2)
mod <- mboost(DEXfat ~ bols(waist_cat) + bols(hip_cat), data = bodyfat)
plot(mod, which = "hip_cat", ylim = c(-4,9))
plot(mod, which = "hip_cat", ylim = c(-4,9), xaxt = "n", xlab = "")
lines(mod, which = "waist_cat", col = "red")

mod <- mboost(DEXfat ~ bols(waistcirc) + bols(hip_cat) + bols(hip_cat, waistcirc),
              data = bodyfat)
plot(mod, which = "hip_cat")


## check if model fitting with very few knots works
## [https://github.com/boost-R/mboost/issues/30]
# kronecker product for matrix-valued responses
data("volcano", package = "datasets")
# estimate mean of image treating image as matrix
x1 <- 1:nrow(volcano)
x2 <- 1:ncol(volcano)
vol <- as.vector(volcano)
# do the model fit with default number of knots
mod <- mboost(vol ~ bbs(x1, df = 3, knots = 10)%O%
                bbs(x2, df = 3, knots = 10),
              control = boost_control(nu = 0.25))
# do the model fit with very few knots
mod <- mboost(vol ~ bbs(x1, df = 3, knots = 3) %O%
                bbs(x2, df = 3, knots = 3),
              control = boost_control(nu = 0.25))


### IPCweights problem, see github issue #54

if (require("survival")){
  x1 <- rnorm(100)
  x2 <- x1 + rnorm(100)
  X <- cbind(x1, x2)
  beta <- c(1, 0.5)
  survtime <- exp(X%*%beta)
  event <- rep(c(TRUE, FALSE), 50)
  
  ipcw1 <- IPCweights(Surv(survtime, event))
  ipcw2 <- IPCweights(Surv(as.numeric(survtime), event))
  summary(cbind(ipcw1, ipcw2))
  
  print(identical(ipcw1, ipcw2))
}

## make sure newdata is not used in fitted() but other arguments are:
data("bodyfat", package = "TH.data")
mod <- mboost(DEXfat ~ btree(age) + bols(waistcirc) + bbs(hipcirc),
              control = boost_control(mstop = 10),
              data = bodyfat)
fitted(mod, newdata = bodyfat)
print(.all.equal(fitted(mod, aggregate = "cumsum")[,10], 
                    fitted(mod), check.attributes = FALSE))
print(.all.equal(fitted(mod, aggregate = "cumsum"), 
                    fitted(mod, newdata = bodyfat, aggregate = "cumsum")))


## make sure weights are subset if missing values are present
weights <- sample(1:100, 100, replace=FALSE)
x <- rnorm(100)
y <- runif(100)
# create missing value
x[25] <- NA
myData <- data.frame(x=x, y=y)
## was broken:
gamboost(y ~ bols(x), data = myData, weights = weights, family = Gaussian())
mboost(y ~ bols(x), data = myData, weights = weights, family = Gaussian())
## was always working
glmboost(y ~ x, data = myData, weights = weights, family = Gaussian())
blackboost(y ~ x, data = myData, weights = weights, family = Gaussian())

### try to find duplicated entries / observations with missing values
### for more efficient memory handling
get_index <- function(x) {

    if (isMATRIX(x)) {
        ### handle missing values only
        cc <- Complete.cases(x)
        nd <- which(cc)
        index <- match(1:nrow(x), nd)
    } else {
        ### handle single variables (factors / numerics) factors
        if (length(x) == 1) {
            x <- x[[1]]
            nd <- which(!duplicated(x))
            nd <- nd[Complete.cases(x[nd])]
            index <- match(x, x[nd])
        ### go for data.frames with >= 2 variables
        } else {
            tmp <- do.call("paste", x)
            nd <- which(!duplicated(tmp))
            nd <- nd[Complete.cases(x[nd,])]
            index <- match(tmp, tmp[nd])
        }
    }
    return(list(nd, index))
}

rescale_weights <- function(w) {
    if (max(abs(w - floor(w))) < sqrt(.Machine$double.eps))
        return(w)
    return(w / sum(w) * sum(w > 0))
}

### check measurement scale of response for some losses
check_y_family <- function(y, family)
    family@check_y(y)

### check for negative gradient corresponding to L2 loss
### <FIXME> better check? </FIXME>
checkL2 <- function(object)
    isTRUE(all.equal(attributes(object$family)[1],
                     attributes(Gaussian())[1]))

### check for classical or Matrix matrices
isMATRIX <- function(x)
    is.matrix(x) || is(x, "Matrix")

### rows without missings in Matrices, matrices and data.frames
Complete.cases <- function(x) {
    if (isMATRIX(x))
        return(rowSums(is.na(x)) == 0)
    if (inherits(x, "GWASdata")) { 
        # object should not contain missing values anyway
        return(rep(TRUE, nrow(x@geno)))
    }
    complete.cases(x)
}

### fractional polynomials transformation
### all powers `p' of `x', all powers `p' of `x' times `log(x)' and `log(x)'
### see Sauerbrei & Royston (1999), JRSS A (162), 71--94
FP <- function(x, p = c(-2, -1, -0.5, 0.5, 1, 2, 3), scaling = TRUE) {
    xname <- deparse(substitute(x))
    if (is.logical(scaling)) {
        shift <- 0
        scale <- 1
        if (scaling) {
            if (min(x) <= 0) {
                z <- diff(sort(x))
                shift <- min(z[z > 0]) - min(x)
                shift <- ceiling(shift * 10)/10
            }
            range <- mean(x + shift)
            scale <- 10^(sign(log10(range)) * round(abs(log10(range))))
        }
        scaling <- c(shift, scale)
    } else {
        stopifnot(is.numeric(scaling) && length(scaling) == 2)
    }
    x <- (x + scaling[1]) / scaling[2]
    stopifnot(all(x > 0))

    Xp <- sapply(p, function(p) x^p)
    Xp <- cbind(Xp, log(x))
    Xp <- cbind(Xp, Xp * log(x))
    colnames(Xp) <- c(paste(xname, "^", p, sep = ""),
                      paste("log(", xname, ")", sep = ""),
                      paste("log(", xname, ")", xname, "^", p, sep = ""),
                      paste("log(", xname, ")^2", sep = ""))
    attr(Xp, "p") <- p
    attr(Xp, "scaling") <- scaling
    Xp
}

### trace boosting iterations
do_trace <- function(m, mstop, risk, step = options("width")$width / 2,
                     width = 1000) {
    m <- m - mstop
    if (m != width) {
        if ((m - 1) %/% step == (m - 1) / step) {
            mchr <- formatC(m+mstop, format = "d", width = nchar(width) + 1,
                            big.mark = "'")
            cat(paste("[", mchr, "] ",sep = ""))
        } else {
            if ((m %/% step != m / step)) {
                cat(".")
            } else {
                cat(" -- risk:", risk[m+mstop+1], "\n")
            }
        }
    } else {
        cat("\nFinal risk:", risk[m+mstop+1], "\n")
    }
}

bhatmat <- function(n, H, xselect, fitm, fW) {

    B <- matrix(0, nrow = n, ncol = n)
    I <- diag(n)
    tr <- numeric(length(xselect))

    for (m in 1:length(xselect)) {
        B <- B + (H[[xselect[m]]] * fW(fitm[,m])) %*% (I - B)
        tr[m] <- sum(diag(B))
    }
    list(hatmatrix = B, trace = tr)
}

nnls1D <- function(XtX, X, y) {
    my <- switch(attr(X, "Ts_constraint"),  "increasing" = {
        ### first column is intercept
        stopifnot(max(abs(X[,1,drop = TRUE] - 1)) < sqrt(.Machine$double.eps))
        min(y)
    }, "decreasing" = {
        stopifnot(max(abs(X[,1,drop = TRUE] + 1)) < sqrt(.Machine$double.eps))
        max(y)
    })
    y <- y - my
    cf <- nnls(XtX, crossprod(X, y))$x
    cf[1] <- cf[1] + switch(attr(X, "Ts_constraint"),  "increasing" = my,
                                                    "decreasing" = -my)
    cf
}

nnls2D <- function(X, XtX, Y) {

    constr <- which(c(!is.null(attr(X$X1, "Ts_constraint")),
                      !is.null(attr(X$X2, "Ts_constraint"))))
    if (length(constr) == 2)
            stop("only one dimension may be subject to constraints")
    Xc <- paste("X", constr, sep = "")
    my <- switch(attr(X[[Xc]], "Ts_constraint"),  "increasing" = {
        ### first column is intercept
        stopifnot(max(abs(X[[Xc]][,1,drop = TRUE] - 1)) < sqrt(.Machine$double.eps))
        min(Y)
    }, "decreasing" = {
        stopifnot(max(abs(X[[Xc]][,1,drop = TRUE] + 1)) < sqrt(.Machine$double.eps))
        max(Y)
    })
    Y <- Y - my

    XWY <- as.vector(crossprod(X$X1, Y) %*% X$X2)
    cf <- nnls(XtX, matrix(as(XWY, "matrix"), ncol = 1))$x
    cf <- matrix(cf, nrow = ncol(X$X1))
    if (constr == 1) cf[1,] <- cf[1,] + switch(attr(X[[Xc]], "Ts_constraint"),
                                               "increasing" = my,
                                               "decreasing" = -my)
    if (constr == 2) cf[,1] <- cf[,1] + switch(attr(X[[Xc]], "Ts_constraint"),
                                               "increasing" = my,
                                               "decreasing" = -my)
    cf
}

## function returns either differences or difference matrix
## needed in bmono
differences <- function(constraint, ncol = NULL) {
    if (length(constraint) == 1) {
        diff <- switch(constraint,
                       none = NULL,
                       positive = 0,
                       negative = 0,
                       increasing = 1,
                       decreasing = -1,
                       convex = 2,
                       concave = -2)
    } else {
        diff <- lapply(constraint, function(x)
                       switch(x,
                              none = NULL,
                              positive = 0,
                              negative = 0,
                              increasing = 1,
                              decreasing = -1,
                              convex = 2,
                              concave = -2))
    }
    if (is.null(ncol))
        return(diff)
    #### and thus quit function

    make_diffs <- function(diff, ncol) {
        if (is.null(diff))
            return(NULL)
        if (diff != 0) {
            D <- sign(diff) * diff(diag(ncol), differences = abs(diff))
        } else {
            D <- ifelse(constraint == "positive", 1, -1) * diag(ncol)
        }
        return(D)
    }

    if (length(diff) == 1)
        return(make_diffs(diff, ncol))
    #### and thus quit function

    D <- vector(mode = "list", length =2)
    tmpD <- make_diffs(diff[[1]], ncol[[1]])
    if (!is.null(tmpD))
        ## to make Matrix quiet
        D[[1]] <- suppressMessages(kronecker(tmpD, diag(ncol[[2]])))
    tmpD <- make_diffs(diff[[2]], ncol[[2]])
    if (!is.null(tmpD))
        ## to make Matrix quiet
        D[[2]] <- suppressMessages(kronecker(diag(ncol[[1]]), tmpD))
    return(D)
}


## least squares with equalities and inequalities
solveLSEI <- function(XtX, Xty, D = NULL) {
    if (is.null(D) || all(sapply(D, is.null)))
        return(solve(XtX, Xty))

    if (!isMATRIX(D)) ## i.e. D is a list with 2 entries
        D <- rbind(D[[1]], D[[2]])
    ## NOTE: Currently both constraints get the same weight
    cf <- solve.QP(Dmat = XtX, dvec = as.vector(Xty), Amat = t(D),
                   bvec = rep(0, nrow(D)))$solution
    cf
}

check_newdata <- function(newdata, blg, mf, to.data.frame = TRUE) {
    nm <- names(blg)
    if (!all(nm %in% names(newdata)))
        stop(sQuote("newdata"),
             " must contain all predictor variables,",
             " which were used to specify the model.")
    if (!any(class(newdata) %in% c("list", "data.frame")))
        stop(sQuote("newdata"), " must be either a data.frame or a list")
    if (any(duplicated(nm)))  ## removes duplicates
        nm <- unique(nm)

    cl1 <- sapply(newdata[nm], class)
    if (is.list(mf) || is.data.frame(mf)) {
        cl2 <- sapply(mf, class)
    } else {
        ## currently this is only needed for GWASdata
        cl2 <- class(mf)
        to.data.frame <- FALSE
    }

    if (!isTRUE(all.equal(cl1, cl2))) {
        # idx <- which(cl1 != cl2)
        idx <- which(!sapply(1:length(cl1),
                             function(i) all(cl1[[i]] == cl2[[i]])))
        ## classes can be different when one is integer and the other is double
        if (!all(sapply(newdata[nm][idx], is.numeric)) ||
            !all(sapply(mf[idx], is.numeric)))
            warning("Some variables in ", sQuote("newdata"),
                    " do not have the same class as in the original data set",
                    call. = FALSE)
    }

    ## subset data
    mf <- newdata[nm]
    if (is.list(mf) && to.data.frame)
        mf <- as.data.frame(mf)
    if (is.list(mf) && length(mf) == 1 && inherits(mf[[1]], "GWASdata"))
        mf <- mf[[1]]
    return(mf)
}

## make positive (semi-) definite matrix
make_psd <- function(x, eps = sqrt(.Machine$double.eps)) {
    lambda <- min(eigen(x, only.values = TRUE, symmetric = TRUE)$values)
    ## use some additional tolerance to ensure semipositive definite matrices
    lambda <- lambda - sqrt(.Machine$double.eps)
    # smallest eigenvalue negative = not semipositive definite
    if (lambda < -1e-10) {
        rho <- 1/(1-lambda)
        x <- rho * x + (1-rho) * diag(dim(x)[1])
        ## now check if it is really positive definite by recursively calling
        x <- make_psd(x)
    }
    return(x)
}

mboost_fit <- function(blg, response, weights = rep(1, NROW(response)),
                       offset = NULL, family = Gaussian(),
                       control = boost_control(),
                       oobweights = as.numeric(weights == 0)) {

    ### hyper parameters
    mstop <- 0
    risk <- control$risk
    nu <- control$nu
    trace <- control$trace
    stopintern <- control$stopintern
    if (is.numeric(stopintern)) {
        stopeps <- stopintern
        stopintern <- TRUE
    } else {
        stopeps <- 0
    }
    tracestep <- options("width")$width / 2

    ### extract negative gradient and risk functions
    ngradient <- family@ngradient
    riskfct <- family@risk

    ### weights not specified: unweighted problem
    if (is.null(weights)) 
        weights <- rep.int(1, NROW(response))
    
    ## oobweights not specified
    if (is.null(oobweights))
        oobweights <- as.numeric(weights == 0)
    
    ### handle missing responses (via zero weights)
    yna <- is.na(response)
    y <- response
    if (any(yna)) {
        weights[yna] <- 0
        y[is.na(y)] <- y[!yna][1] # use first non-missing value
        warning("response contains missing values;\n",
                " weights of corresponding observations are set to zero",
                " and thus these observations are not used for fitting")
    }
    y <- check_y_family(y, family)
    if (!family@weights(weights))
        stop(sQuote("family"), " is not able to deal with weights")

    ### rescale weights (because of the AIC criterion)
    ### <FIXME> is this correct with zero weights??? </FIXME>
    weights <- rescale_weights(weights)
    
    if (control$risk == "oobag") {
        triskfct <- function(y, f) riskfct(y, f, oobweights)
    } else {
        triskfct <- function(y, f) riskfct(y, f, weights)
    }

    ### set up the fitting functions
    bl <- lapply(blg, dpp, weights = weights)
    blfit <- lapply(bl, function(x) x$fit)
    fit1 <- blfit[[1]]

    if (identical("Negative Multinomial Likelihood", family@name)
        && ! all(vapply(bl, inherits, FALSE, what = "bl_kronecker")))
        stop(sQuote("family = Multinomial()"), " only works with Kronecker prodcut base-learners, ",
             "i.e., combined base-learners of the form ", sQuote("bl1 %O% bl2"), " fitted via ",
             sQuote("gamboost()"), " or ", sQuote("mboost()"),
             ".\n See ", sQuote("?Multinomial"), " for details.")
        
    xselect <- NULL
    ens <- vector(mode = "list", length = control$mstop)
    nuisance <- vector(mode = "list", length = control$mstop)

    ### initialized the boosting algorithm
    fit <- offset
    offsetarg <- offset
    if (is.null(offset))
        fit <- offset <- family@offset(y, weights)
    if (length(fit) == 1)
        fit <- rep(fit, NROW(y))
    u <- ustart <- ngradient(y, fit, weights)
    
    ### vector of empirical risks for all boosting iterations
    ### (either in-bag or out-of-bag)
    mrisk <- triskfct(y, fit)
    tsums <- numeric(length(bl))
    ss <- vector(mode = "list", length = length(bl))

    ### set up function for fitting baselearner(s)
    cwlin <- FALSE
    if (length(bl) > 1) {
        bnames <- names(bl)
        basefit <- function(u, m) {
           tsums <- rep(-1, length(bl))
           ### fit least squares to residuals _componentwise_
           for (i in 1:length(bl)) {
               ss[[i]] <<- sstmp <- blfit[[i]](y = u) ### try(fit(bl[[i]], y = u))
               ### if (inherits(ss[[i]], "try-error")) next
               tsums[i] <- mean.default(weights * ((sstmp$fitted()) - u)^2,
                                        na.rm = TRUE)
           }
           if (all(is.na(tsums)) || all(tsums < 0))
               stop("could not fit any base-learner in boosting iteration ", m)
           xselect[m] <<- which.min(tsums)
           return(ss[[xselect[m]]])
        }
    } else {
        cwlin <- inherits(bl[[1]], "bl_cwlin")
        if (cwlin) {
            bnames <- bl[[1]]$Xnames
            basefit <- function(u, m) {
                mod <- fit1(y = u)
                if(all(is.na(coef(mod))))
                    stop("could not fit any base-learner in boosting iteration ", m)
                xselect[m] <<- mod$model["xselect"]
                return(mod)
            }
        } else {
            bnames <- names(bl)
            basefit <- function(u, m) {
                xselect[m] <<- 1L
                mod <- fit1(y = u)
                if((!is.null(coef(mod)) && all(is.na(coef(mod)))) ||
                   all(is.na(mean.default(weights * ((mod$fitted()) - u)^2, na.rm = TRUE))))
                    stop("could not fit base-learner in boosting iteration ", m)
                return(mod)
            }
        }
    }

    ## if names are missing try to get these from the calls
    if (is.null(bnames) && !cwlin)
        names(blg) <- names(bl) <- bnames <- sapply(blg, function(x) x$get_call())


    ### set up a function for boosting
    boost <- function(niter) {
        for (m in (mstop + 1):(mstop + niter)) {

            ### fit baselearner(s)
            basess <- basefit(u, m)

            ### update step
            ### <FIXME> handle missing values!
            fit <<- fit + nu * basess$fitted()
            ### <FIXME>

            ### negative gradient vector, the new `residuals'
            u <<- ngradient(y, fit, weights)

            ### happened for family = Poisson() with nu = 0.1
            if (any(!is.finite(u[!is.na(u)])))
                stop("Infinite residuals: please decrease step-size nu in ",
                     sQuote("boost_control"))

            ### evaluate risk, either for the learning sample (inbag)
            ### or the test sample (oobag)
            mrisk[m + 1] <<- triskfct(y, fit)

            ### save the model
            ens[[m]] <<- basess
            nuisance[[m]] <<- family@nuisance()

            ### print status information
            ### print xselect???
            if (trace)
                do_trace(m, mstop = mstop, risk = mrisk,
                         step = tracestep, width = niter)

            ### internal stopping (for oobag risk only)
            if (stopintern) {
                if ((mrisk[m + 1] - mrisk[m]) > stopeps) break
            }
        }
        mstop <<- mstop + niter
        return(TRUE)
    }
    
    if (control$mstop > 0) {
        ### actually go for initial mstop iterations!
        tmp <- boost(control$mstop)
    }

    ### prepare a (very) rich objects
    RET <- list(baselearner = blg,          ### the baselearners (without weights)
                basemodel = bl,             ### the basemodels (with weights)
                offset = offset,            ### offset
                ustart = ustart,            ### first negative gradients
                control = control,          ### control parameters
                family = family,            ### family object
                response = response,        ### the response variable
                rownames =                  ### rownames of learning data
                    paste("obs", 1:length(weights), sep = ""),
                "(weights)" = weights,      ### weights used for fitting
                nuisance =
                    function() nuisance     ### list of nuisance parameters
    )

    ### update to new weights; just a fresh start
    RET$update <- function(weights = NULL, oobweights = NULL, risk = "oobag",
                           trace = NULL) {

        control$mstop <- mstop
        if (!is.null(risk))
            control$risk <- risk
        if (!is.null(trace))
            control$trace <- trace
        ### use user specified offset only (since it depends on weights otherwise)
        if (!is.null(offsetarg)) offsetarg <- offset
        mboost_fit(blg = blg, response = response, weights = weights,
                   offset = offsetarg, family = family, control = control,
                   oobweights = oobweights)
    }

    ### number of iterations performed so far
    RET$mstop <- function() mstop

    ### which basemodels have been selected so far?
    RET$xselect <- function() {
        if (mstop == 0)
            return(NULL)
        return(xselect[1:mstop])
    }

    ### current fitted values
    RET$fitted <- function() as.vector(fit)

    ### current negative gradient
    RET$resid <- function() u

    ### current risk fct.
    RET$risk <- function() {
        mrisk[1:(mstop + 1)]
    }

    ### negative risk (at current iteration)
    RET$logLik <- function() -mrisk[mstop + 1]

    ### figure out which baselearners are requested
    thiswhich <- function(which = NULL, usedonly = FALSE) {
        if (is.null(which)) which <- 1:length(bnames)
        if (is.character(which)) {
            i <- sapply(which, function(w) {
                wi <- grep(w, bnames, fixed = TRUE)
                if (length(wi) > 0) return(wi)
                return(NA)
            })
            if (any(is.na(i)))
                warning(paste(which[is.na(i)], collapse = ","), " not found")
            which <- i
        }
        ### return only those selected so far
        if (usedonly) which <- which[which %in% RET$xselect()]
        return(which)
    }
    RET$which <- thiswhich

    ### prepare for computing predictions in the following ways
    ### - for all selected baselearners (which = NULL) or chosen ones (selected or not)
    ### - aggregated ("sum"), the complete path over all
    ###   boosting iterations done so far ("cumsum") or
    ###   not aggregated at all ("none")
    ### - always returns a matrix
    RET$predict <- function(newdata = NULL, which = NULL,
                            aggregate = c("sum", "cumsum", "none")) {

        if (mstop == 0) {
            if (length(offset) == 1) {
                if (!is.null(newdata))
                    return(rep(offset, NROW(newdata)))
                return(rep(offset, NROW(y)))
            } 
            if (!is.null(newdata)) {
                warning("User-specified offset is not a scalar, ",
                        "thus it cannot be used for predictions when ",
                        sQuote("newdata"), " is specified.")
                return(rep(0, NCOL(newdata)))
            }
            return(offset)
        }
        if (!is.null(xselect))
            indx <- ((1:length(xselect)) <= mstop)
        which <- thiswhich(which, usedonly = nw <- is.null(which))
        if (length(which) == 0) return(NULL)

        aggregate <- match.arg(aggregate)

        pfun <- function(w, agg) {
            ix <- xselect == w & indx
            if (!any(ix))
                return(0)
            if (cwlin) w <- 1
            ret <- nu * bl[[w]]$predict(ens[ix],
                   newdata = newdata, aggregate = agg)
            if (agg == "sum") return(ret)
            m <- Matrix(0, nrow = nrow(ret), ncol = sum(indx))
            m[, which(ix)] <- ret
            m
        }

        pr <- switch(aggregate, "sum" = {
            pr <- lapply(which, pfun, agg = "sum")
            if (!nw){
                if (NCOL(pr[[1]]) == 1) {
                    pr <- do.call("cbind", pr)
                    colnames(pr) <- bnames[which]
                    attr(pr, "offset") <- offset
                }
                return(pr)
            } else {
                ## only if no selection of baselearners
                ## was made via the `which' argument
                ret <- Reduce("+", pr)
                if (length(offset) != 1 && !is.null(newdata)) {
                    warning("User-specified offset is not a scalar, ",
                            "thus it cannot be used for predictions when ",
                            sQuote("newdata"), " is specified.")
                } else {
                    ret <- ret + offset
                }
                return(ret)
            }
        }, "cumsum" = {
            if (!nw) {
                pr <- lapply(which, pfun, agg = "none")
                pr <- lapply(pr, function(x) {
                    .Call("R_mcumsum", as(x, "matrix"), PACKAGE = "mboost")
                })
                names(pr) <- bnames[which]
                attr(pr, "offset") <- offset
                return(pr)
            } else {
                pr <- 0
                for (i in 1:max(xselect)) pr <- pr + pfun(i, agg = "none")
                pr <- .Call("R_mcumsum", as(pr, "matrix"), PACKAGE = "mboost")
                if (length(offset) != 1 && !is.null(newdata)) {
                    warning(sQuote("length(offset) > 1"),
                            ": User-specified offset is not a scalar, ",
                            "thus offset not used for prediction when ",
                            sQuote("newdata"), " is specified")
                } else {
                    pr <- pr + offset
                }
                return(pr)
            }
         }, "none" = {
            if (!nw) {
                pr <- lapply(which, pfun, agg = "none")
                for (i in 1:length(pr)) pr[[i]] <- as(pr[[i]], "matrix")
                names(pr) <- bnames[which]
                attr(pr, "offset") <- offset
                return(pr)
            } else {
                pr <- 0
                for (i in 1:max(xselect)) pr <- pr + pfun(i, agg = "none")
                pr <- as(pr, "matrix")
                attr(pr, "offset") <- offset
                return(pr)
            }
         })
         return(pr)
    }

    ### extract a list of the model frames of the single baselearners
    RET$model.frame <- function(which = NULL) {
        which <- thiswhich(which, usedonly = is.null(which))
        ret <- lapply(blg[which], model.frame)
        names(ret) <- bnames[which]
        ret
    }

    ### update to a new number of boosting iterations mstop
    ### i <= mstop means less iterations than current
    ### i >  mstop needs additional computations
    ### updates take place in THIS ENVIRONMENT,
    ### some models are CHANGED!
    RET$subset <- function(i) {
        if (i <= mstop || i <= length(xselect)) {
            ## no need to recompute everything if mstop isn't changed
            if (i != mstop) {
                mstop <<- i
                fit <<- RET$predict()
                u <<- ngradient(y, fit, weights)
            }
        } else {
            ## if prior reduction of mstop,
            ## first increase mstop to old value first
            if (mstop != length(xselect)) {
                mstop <<- length(xselect)
                fit <<- RET$predict()
                u <<- ngradient(y, fit, weights)
            }
            ## now fit the rest
            tmp <- boost(i - mstop)
        }
    }

    ### if baselearners have a notion of coefficients,
    ### extract these either aggregated ("sum"),
    ### their coefficient path ("cumsum") or not
    ### aggregated at all ("none")
    RET$coef <- function(which = NULL, aggregate = c("sum", "cumsum", "none")) {

        if (!is.null(xselect)) 
            indx <- ((1:length(xselect)) <= mstop)
        which <- thiswhich(which, usedonly = is.null(which))
        if (length(which) == 0) return(NULL)

        aggregate <- match.arg(aggregate)
        cfun <- function(w) {
            ix <- (xselect == w & indx)
            cf <- numeric(mstop)
            if (!any(ix)) {
                if (!cwlin) {
                    nm <- bl[[w]]$Xnames
                    cf <- matrix(0, nrow = length(nm), ncol = mstop)
                }
            } else {
                if (inherits(ens[ix][[1]], "bm_cwlin") && !cwlin) {
                    cftmp <- sapply(ens[ix], coef, all = TRUE)
                } else {
                    cftmp <- sapply(ens[ix], coef)
                }
                nr <- NROW(cftmp)
                if (!is.matrix(cftmp)) nr <- 1
                cf <- matrix(0, nrow = nr, ncol = mstop)
                cf[, which(ix)] <- cftmp
            }
            if (!is.matrix(cf)) cf <- matrix(cf, nrow = 1)
            ## check if base-learner has coefficients
            if(any(sapply(cf, is.null))){
                ret <- NULL
            } else {
                ret <- switch(aggregate,
                              "sum" = rowSums(cf) * nu,
                              "cumsum" = {
                                  .Call("R_mcumsum", as(cf, "matrix") * nu,
                                        PACKAGE = "mboost")
                              },
                              "none" = nu * cf
                              )
            }
            ### set names, but not for bolscw base-learner
            if (!cwlin) {
                nm <- bl[[w]]$Xnames
                if (is.matrix(ret)) {
                    rownames(ret) <- nm
                } else {
                   names(ret) <- nm
               }
            }
            ret
        }
        ret <- lapply(which, cfun)
        names(ret) <- bnames[which]
        attr(ret, "offset") <- offset
        return(ret)
    }

    ### function for computing hat matrices of individual predictors
    RET$hatvalues <- function(which = NULL) {
        which <- thiswhich(which, usedonly = is.null(which))
        ### make sure each list element corresponds to one baselearner
        ### non-selected baselearners receive NULL
        ret <- vector(mode = "list", length = length(bl))
        ret[which] <- lapply(bl[which], function(b) hatvalues(b) * nu)
        names(ret) <- bnames
        ret
    }

    class(RET) <- "mboost"
    return(RET)
}

### main user interface function
### formula may contain unevaluated baselearners as well
### as additional variables
###     y ~ bols3(x1) + x2 + btree(x3)
### is evaluated as
###     y ~ bols3(x1) + baselearner(x2) + btree(x3)
### see mboost_fit for the dots
mboost <- function(formula, data = list(), na.action = na.omit, weights = NULL, 
                   offset = NULL, family = Gaussian(), control = boost_control(),
                   oobweights = NULL, baselearner = c("bbs", "bols", "btree", "bss", "bns"), 
                   ...) {


    ## We need at least variable names to go ahead
    if (length(formula[[3]]) == 1) {
        if (as.name(formula[[3]]) == ".") {
            formula <- as.formula(paste(deparse(formula[[2]]),
                "~", paste(names(data)[names(data) != all.vars(formula[[2]])],
                           collapse = "+"), collapse = ""))
        }
    }

    if (is.data.frame(data)) {
        if (!all(cc <- Complete.cases(data))) {
            ## drop cases with missing values in any of the specified variables:
            vars <- all.vars(formula)[all.vars(formula) %in% names(data)]
            data <- na.action(data[, vars])
            
            ## check if weights need to be removed as well
            if (!is.null(weights) && nrow(data) < length(weights)) {
                if (sum(cc) == nrow(data))
                    weights <- weights[cc]
            }
            ## check if oobweights need to be removed as well
            if (!is.null(oobweights) && nrow(data) < length(oobweights)) {
                if (sum(cc) == nrow(data))
                    oobweights <- oobweights[cc]
            }
        }
    } else {
        if (any(unlist(lapply(data, function(x) !all(Complete.cases(x))))))
            warning(sQuote("data"),
                    " contains missing values. Results might be affected. Consider removing missing values.")
    }

    if (is.character(baselearner)) {
        baselearner <- match.arg(baselearner)
        bname <- baselearner
        if (baselearner %in% c("bss", "bns")) {
            warning("bss and bns are deprecated, bbs is used instead")
            baselearner <- "bbs"
        }
        baselearner <- get(baselearner, mode = "function",
                           envir = parent.frame())
    } else {
        bname <- deparse(substitute(baselearner))
    }
    stopifnot(is.function(baselearner))

    ### instead of evaluating a model.frame, we evaluate
    ### the expressions on the rhs of formula directly
    "+" <- function(a,b) {
        cl <- match.call()
        ### got baselearner, fine!
        if (inherits(a, "blg")) a <- list(a)
        ### a single variable; compute baselearner
        if (!is.list(a)) {
            a <- list(baselearner(a))
            a[[1]]$set_names(deparse(cl[[2]]))
        }
        ### got baselearner, fine!
        if (inherits(b, "blg")) b <- list(b)
        ### a single variable, compute baselearner
        if (!is.list(b)) {
            b <- list(baselearner(b))
            b[[1]]$set_names(deparse(cl[[3]]))
        }
        ### join both baselearners in a list
        c(a, b)
    }
    ### set up all baselearners
    bl <- eval(as.expression(formula[[3]]),
               envir = c(as.list(data), list("+" = get("+"))),
               enclos = environment(formula))
    ### rhs was one single baselearner
    if (inherits(bl, "blg")) bl <- list(bl)
    ### rhs was one single variable
    if (!is.list(bl)) {
        bl <- list(baselearner(bl))
        bl[[1]]$set_names(deparse(formula[[3]]))
    }

    ### just a check
    stopifnot(all(sapply(bl, inherits, what = "blg")))

    ### assign calls as names of base learners
    names(bl) <- sapply(bl, function(x) x$get_call())

    ### get the response
    response <- eval(as.expression(formula[[2]]), envir = data,
                     enclos = environment(formula))
    
    ret <- mboost_fit(bl, response = response, weights = weights, 
                      offset = offset, family = family, 
                      control = control, oobweights = oobweights, ...)
    
    if (is.data.frame(data) && nrow(data) == length(response))
        ret$rownames <- rownames(data)
    else
        ret$rownames <- 1:NROW(response)
    ret$call <- match.call()
    ret
}

### nothing to do there
gamboost <- function(formula, data = list(), na.action = na.omit, weights = NULL, 
                     offset = NULL, family = Gaussian(), control = boost_control(),
                     oobweights = NULL, baselearner = c("bbs", "bols", "btree", "bss", "bns"), 
                     dfbase = 4, ...) {

    if (is.character(baselearner)) {
        baselearner <- match.arg(baselearner)
        if (baselearner %in% c("bss", "bns")) {
            warning("bss and bns are deprecated, bbs is used instead")
            baselearner <- "bbs"
        }
        baselearner <- get(baselearner, mode = "function",
                           envir = parent.frame())
    }
    stopifnot(is.function(baselearner))
    if (isTRUE(all.equal(baselearner, bbs)))
        baselearner <- function(...) bbs(as.data.frame(list(...)), df = dfbase)
    
    ret <- mboost(formula = formula, data = data, na.action = na.action, 
                  weights = weights, offset = offset, family = family, 
                  control = control, oobweights = oobweights,
                  baselearner = baselearner, ...)
    ret$call <- match.call()
    class(ret) <- c("gamboost", class(ret))
    ret
}


### just one single tree-based baselearner
blackboost <- function(formula, data = list(),
                       weights = NULL, na.action = na.pass,
                       offset = NULL, family = Gaussian(), 
                       control = boost_control(),
                       oobweights = NULL,
                       tree_controls = partykit::ctree_control(
                           teststat = "quad",
                           testtype = "Teststatistic",
#                           splittest = TRUE,
                           mincriterion = 0,
                           minsplit = 10,
                           minbucket = 4,
                           maxdepth = 2, saveinfo = FALSE),
                       ...) {

    ### get the model frame first
    cl <- match.call()
    mf <- match.call(expand.dots = FALSE)
    m <- match(c("formula", "data", "weights", "na.action"), names(mf), 0L)
    mf <- mf[c(1L, m)]
    mf$drop.unused.levels <- TRUE
    mf[[1L]] <- quote(stats::model.frame)
    mf <- eval(mf, parent.frame())
    response <- model.response(mf)
    weights <- model.weights(mf)
    ## drop outcome
    mf <- mf[,-1, drop = FALSE]
    ## drop weights
    mf$"(weights)" <- NULL
    bl <- list(btree(mf, tree_controls = tree_controls))
    ret <- mboost_fit(bl, response = response,  weights = weights, 
                      offset = offset, family = family, 
                      control = control, oobweights = oobweights, ...)
    ret$call <- cl
    ret$rownames <- rownames(mf)
    class(ret) <- c("blackboost", class(ret))
    ret
}

### fit a linear model componentwise
glmboost <- function(x, ...) UseMethod("glmboost", x)

glmboost.formula <- function(formula, data = list(), weights = NULL,
                             offset = NULL, family = Gaussian(),
                             na.action = na.pass, contrasts.arg = NULL,
                             center = TRUE, control = boost_control(), 
                             oobweights = NULL, ...) {

    ## We need at least variable names to go ahead
    if (length(formula[[3]]) == 1) {
        if (as.name(formula[[3]]) == ".") {
            formula <- as.formula(paste(deparse(formula[[2]]),
                "~", paste(names(data)[names(data) != all.vars(formula[[2]])],
                           collapse = "+"), collapse = ""))
        }
    }

    ### get the model frame first
    cl <- match.call()
    mf <- match.call(expand.dots = FALSE)
    m <- match(c("formula", "data", "weights", "na.action"), names(mf), 0L)
    mf <- mf[c(1L, m)]
    mf$drop.unused.levels <- TRUE
    mf$data <- data ## use cc data
    mf[[1L]] <- quote(stats::model.frame)
    mf <- eval(mf, parent.frame())

    ### center argument moved to this function
    if (!control$center) {
        center <- FALSE
        warning("boost_control(center = FALSE) is deprecated, use glmboost(..., center = FALSE)")
    }
    ### set up the model.matrix and center (if requested)
    X <- model.matrix(attr(mf, "terms"), data = mf,
                      contrasts.arg = contrasts.arg)
    assign <- attr(X, "assign")
    cm <- numeric(ncol(X))
    if (center) {
        if (!attr(attr(mf, "terms"), "intercept") == 1)
            warning("model with centered covariates does not contain intercept")
        cm <- colMeans(X, na.rm = TRUE)
        cm[assign == 0] <- 0
        X <- scale(X, center = cm, scale = FALSE)
    }
    ### this function will be used for predictions later
    newX <- function(newdata) {
        mf <- model.frame(delete.response(attr(mf, "terms")),
                          data = newdata, na.action = na.action)
        X <- model.matrix(delete.response(attr(mf, "terms")),
                          data = mf, contrasts.arg = contrasts.arg)
        scale(X, center = cm, scale = FALSE)
    }

    ### component-wise linear models baselearner
    bl <- list(bolscw(X))
    response <- model.response(mf)
    weights <- model.weights(mf)
    ret <- mboost_fit(bl, response = response, weights = weights, 
                      offset = offset, family = family, 
                      control = control, oobweights = oobweights, ...)
    ret$newX <- newX
    ret$assign <- assign
    ret$center <- cm
    ret$call <- cl
    ### need specialized method (hatvalues etc. anyway)
    ret$hatvalues <- function() {
        H <- vector(mode = "list", length = ncol(X))
        MPinv <- ret$basemodel[[1]]$MPinv()
        for (j in unique(ret$xselect()))
            H[[j]] <- (X[,j] %*% MPinv[j, ,drop = FALSE]) * control$nu
        H
    }
    ret$rownames <- rownames(mf)
    ### specialized method for model.frame
    ret$model.frame <- function(which = NULL) {
        if (!is.null(which))
            warning("Argument ", sQuote("which"), " is ignored")
        mf
    }
    ### save standard update function for re-use
    update <- ret$update
    ### needs a specialized update function as well
    ret$update <- function(weights = NULL, oobweights = NULL, risk = "oobag",
                           trace = NULL) {
        ## call standard update function
        res <- update(weights = weights, oobweights = oobweights, risk = risk,
                      trace = trace)
        ## now re-set all special arguments
        res$newX <- newX
        res$assign <- assign
        res$center <- cm
        res$call <- cl
        ### need specialized method (hatvalues etc. anyway)
        res$hatvalues <- function() {
            H <- vector(mode = "list", length = ncol(X))
            MPinv <- res$basemodel[[1]]$MPinv()
            for (j in unique(res$xselect()))
                H[[j]] <- (X[,j] %*% MPinv[j, ,drop = FALSE]) * control$nu
            H
        }
        res$rownames <- rownames(mf)
        ### specialized method for model.frame
        res$model.frame <- function(which = NULL) {
            if (!is.null(which))
                warning("Argument ", sQuote("which"), " is ignored")
            mf
        }
        class(res) <- c("glmboost", "mboost")
        res
    }
    class(ret) <- c("glmboost", "mboost")
    return(ret)
}

glmboost.matrix <- function(x, y, center = TRUE, weights = NULL,
                            offset = NULL, family = Gaussian(),
                            na.action = na.pass, control = boost_control(), 
                            oobweights = NULL, ...) {

    X <- x
    if (nrow(X) != NROW(y))
        stop("dimensions of ", sQuote("x"), " and ", sQuote("y"),
             " do not match")
    if (is.null(colnames(X)))
        colnames(X) <- paste("V", 1:ncol(X), sep = "")

    ## drop cases with missing values in any of the specified variables:
    if (any(!Complete.cases(cbind(X, y)))) {
        X <- na.action(X)
        if (!is.null(removed <- attr(X, "na.action"))) {
            y <- y[-removed]
        }
    }

    if (!control$center) {
        center <- FALSE
        warning("boost_control(center = FALSE) is deprecated, use glmboost(..., center = FALSE)")
    }
    assign <- numeric(ncol(X))
    cm <- numeric(ncol(X))
    ### guess intercept
    intercept <- which(colSums(abs(scale(X, center = TRUE, scale = FALSE)), na.rm=TRUE)
                                   < .Machine$double.eps)
    if (length(intercept) > 0)
        intercept <- intercept[colSums(abs(X[, intercept, drop = FALSE]), na.rm=TRUE)
                               > .Machine$double.eps]
    INTERCEPT <- length(intercept) == 1
    if (INTERCEPT) {
        assign[-intercept] <- 1:(ncol(X) - 1)
    } else {
        assign <- 1:ncol(X)
    }
    if (center) {
        cm <- colMeans(X, na.rm = TRUE)
        if (!INTERCEPT)
            warning("model with centered covariates does not contain intercept")
        cm[assign == 0] <- 0
        X <- scale(X, center = cm, scale = FALSE)
    }
    newX <- function(newdata) {
        if (isMATRIX(newdata)) {
            if (all(colnames(X) == colnames(newdata)))
                return(scale(newdata, center=cm, scale=FALSE))
        }
        stop(sQuote("newdata"), " is not a matrix with the same variables as ",
             sQuote("x"))
        return(NULL)
    }
    bl <- list(bolscw(X))
    ret <- mboost_fit(bl, response = y, weights = weights, 
                      offset = offset, family = family, 
                      control = control, oobweights = oobweights, ...)
    ret$newX <- newX
    ret$assign <- assign
    ret$center <- cm
    ret$call <- match.call()
    ### need specialized method (hatvalues etc. anyway)
    ret$hatvalues <- function() {
        H <- vector(mode = "list", length = ncol(X))
        MPinv <- ret$basemodel[[1]]$MPinv()
        for (j in unique(ret$xselect()))
            H[[j]] <- (X[,j] %*% MPinv[j, ,drop = FALSE]) * control$nu
        H
    }
    ret$rownames <- rownames(X)
    ### specialized method for model.frame
    ret$model.frame <- function(which = NULL) {
        if (!is.null(which))
            warning("Argument ", sQuote("which"), " is ignored")
        X
    }
    ### save standard update function for re-use
    update <- ret$update
    ### needs a specialized update function as well
    ret$update <- function(weights = NULL, oobweights = NULL, risk = "oobag",
                           trace = NULL) {
        ## call standard update function
        res <- update(weights = weights, oobweights = oobweights, risk = risk,
                      trace = trace)
        ## now re-set all special arguments
        ret$newX <- newX
        res$assign <- assign
        res$center <- cm
        res$call <- match.call()
        ### need specialized method (hatvalues etc. anyway)
        res$hatvalues <- function() {
            H <- vector(mode = "list", length = ncol(X))
            MPinv <- res$basemodel[[1]]$MPinv()
            for (j in unique(res$xselect()))
                H[[j]] <- (X[,j] %*% MPinv[j, ,drop = FALSE]) * control$nu
            H
        }
        res$rownames <- rownames(X)
        ### specialized method for model.frame
        res$model.frame <- function(which = NULL) {
            if (!is.null(which))
                warning("Argument ", sQuote("which"), " is ignored")
            X
        }
        class(res) <- c("glmboost", "mboost")
        res
    }
    class(ret) <- c("glmboost", "mboost")
    return(ret)
}

glmboost.default <- function(x, ...) {
    if (extends(class(x), "Matrix"))
        return(glmboost.matrix(x = x, ...))
    stop("no method for objects of class ", sQuote(class(x)),
         " implemented")
}

options(digits = 3)

.all.equal <- function(...) isTRUE(all.equal(..., check.environment = FALSE))
library("mboost")

set.seed(290875)

dgp <- function(n = 100, beta = rep(0, 10), sd = 1) {
    p <- length(beta) - 1
    x <- cbind(1, matrix(runif(n * p), ncol = p))
    lp <- x %*% beta
    y <- lp + rnorm(n, sd = sd)
    ls <- data.frame(y = y, x[,-1])
    attr(ls, "lp") <- lp
    ls
}

### well-defined problem
mydf <- dgp(beta = c(1, 2.5, rep(0, 2)))

### for easy comparison with lm
fm <- Gaussian()
fm@offset <- function(y, w) 0

mydf.gb <- glmboost(y ~ ., data = mydf, family = fm,
                    control = boost_control(mstop = 1000, nu = 1))
mydf.lm <- lm(y ~ ., data = mydf)

### compare coefficients
print(max(abs(coef(mydf.gb) - coef(mydf.lm))) < 1e-10)

### a little bit more difficult
mydf <- dgp(beta = c(1, 2.5, rep(0, 38)))

mydf.gb <- glmboost(y ~ ., data = mydf, family = fm,
                    control = boost_control(mstop = 1000, nu = 1))
aic <- AIC(mydf.gb, method = "corrected")
ht <- hatvalues(mydf.gb)
print(mstop(aic) == 12)
plot(aic, ylim = c(0,3))
mydf.lm <- lm(y ~ ., data = mydf)

## use AIC with active set as df
aic2 <- AIC(mydf.gb, method = "corrected", df = "actset")
print(mstop(aic2)== 9)
print(attr(aic2, "df")[9] == 9)

## use gMDL
aic3 <- AIC(mydf.gb, method = "gMDL")
print(mstop(aic3) == 4)

### compare coefficients
which(abs(coef(mydf.lm, which = "")) < abs(coef(mydf.gb[mstop(aic)], which = "")))

#### check boosting hat matrix and subsetting / predict
print(isTRUE(.all.equal(drop(attr(ht, "hatmatrix") %*% mydf$y),
                           as.vector(predict(mydf.gb[1000])))))
ht25 <- hatvalues(mydf.gb[25])
print(isTRUE(.all.equal(drop(attr(ht25, "hatmatrix") %*% mydf$y),
                           as.vector(predict(mydf.gb[25])))))
print(isTRUE(.all.equal(drop(attr(ht25, "hatmatrix") %*% mydf$y),
                           as.vector(fitted(mydf.gb[25])))))

### a simple two-dimensional example from `glmboost.Rd'
data("cars")
cars.gb <- glmboost(dist ~ speed, data = cars, family = fm, center = FALSE,
                    control = boost_control(mstop = 1000, nu = 1))
cars.gb
plot(cars.gb)

### coefficients should coincide
cf <- coef(cars.gb)
attr(cf, "offset") <- NULL
print(.all.equal(cf, coef(lm(dist ~ speed, data = cars))))

### check aggregate = "none"
print(.all.equal(cumsum(c(coef(cars.gb, aggregate = "none", off2int = TRUE)$speed)),
        c(coef(cars.gb, aggregate = "cumsum", off2int = TRUE)$speed)))

### extract model terms
print(.all.equal(extract(cars.gb, what = "design"),
                    matrix(c(rep(1, 50), cars$speed)),
                    check.attributes = FALSE))
print(.all.equal(extract(cars.gb, what = "design", which = "(Intercept)"), 
                    rep(1, 50), check.attributes = FALSE))
print(.all.equal(extract(cars.gb, what = "coefficients"), coef(cars.gb)))
print(.all.equal(extract(cars.gb, what = "residuals"), resid(cars.gb)))
print(.all.equal(extract(cars.gb, what = "variable.names"), variable.names(cars.gb)))
print(.all.equal(extract(cars.gb, what = "offset"), 0))
print(.all.equal(extract(cars.gb, what = "nuisance"), nuisance(cars.gb)))
print(is.na(extract(cars.gb, what = "nuisance")))
print(.all.equal(extract(cars.gb, what = "weights"), model.weights(cars.gb)))
print(.all.equal(extract(cars.gb, what = "control"), boost_control(mstop = 1000, nu = 1)))

### logistic regression
mydf <- data.frame(x = runif(100), z = rnorm(100),
                   y = factor(c(rep(0, 30), rep(1, 70))))
bmod <- glmboost(y ~ x + z, data = mydf, family = Binomial(), center = FALSE,
                 control = boost_control(mstop = 1000, nu = 1))
gmod <- glm(y ~ x + z, data = mydf, family = binomial())
llg <- logLik(gmod)
attributes(llg) <- NULL
print(.all.equal(logLik(bmod), llg))
print(max(abs(predict(gmod, type = "link")/2 - fitted(bmod))) <
                  sqrt(.Machine$double.eps))
cfb <- coef(bmod, off2int = TRUE) * 2
print(.all.equal(cfb, coef(gmod)))
aic <- AIC(bmod, "classical")
print(abs(AIC(gmod) - attr(aic, "AIC")[mstop(bmod)]) < 1e-5)

### weighted least squares problem

x <- runif(100)
df <- data.frame(y = 2 + 3 * x + rnorm(length(x)),
                 x = x, z = runif(length(x)),
                 w = runif(length(x)) * 10)

### linear model, classical fit
lmmod <- lm(y ~ x + z, data = df, weights = w)

### linear model, boosting fit
lmb <- glmboost(y ~ x + z, data = df, weights = df$w, center = FALSE,
                control = boost_control(mstop = 5000, nu = 1))

### compare fitted values
print(max(abs(fitted(lmmod) -fitted(lmb))) < sqrt(.Machine$double.eps))

### compare hat matrices
print(max(abs(hatvalues(lmmod) - hatvalues(lmb))) < sqrt(.Machine$double.eps))

### compare boosting hat matrix with fitted values
print(max(abs(attr(hatvalues(lmb), "hatmatrix") %*% (df$y - lmb$offset) + lmb$offset -
        fitted(lmb))) < sqrt(.Machine$double.eps))

### Cox model (check for CoxPH family)
if (require("survival")) {

    test <- data.frame(time = c(1, 2, 5, 2, 1, 7, 3, 4, 8, 8),
                       event = c(1, 1, 1, 1, 1, 1, 1, 0, 0, 0),
                       x     = c(1, 0, 0, 1, 0, 1, 1, 1, 0, 0))

    print(.all.equal(coef(cx <- coxph(Surv(time, event) ~ x, data = test, method = "breslow")),
                       coef(gl <- glmboost(Surv(time, event) ~ x, data = test,
                       family = CoxPH(), center = FALSE,
                       control = boost_control(mstop = 2000, nu = 1)), which = 1:2)[2]))

    print(.all.equal(cx$loglik[2], logLik(gl)))

    indx <- c(1, 1, 1, 2:10)
    w <- tabulate(indx)

    print(.all.equal(coef(cx <- coxph(Surv(time, event) ~ x, data = test, weights = w,
                                   method = "breslow")),
                       coef(gl <- glmboost(Surv(time, event) ~ x, data = test, weights = w,
                       family = CoxPH(), center = FALSE,
                       control = boost_control(mstop = 200, nu = 1)), which = 1:2)[2]))

    print(.all.equal(cx$loglik[2], logLik(gl)))

    indx <- c(1, 1, 1, 3:10)
    w <- tabulate(indx)

    print(.all.equal(coef(cx <- coxph(Surv(time, event) ~ x, data = test[indx,],
                                   method = "breslow")),
                       coef(gl <- glmboost(Surv(time, event) ~ x, data = test, weights = w,
                       family = CoxPH(), center = FALSE,
                       control = boost_control(mstop = 1000)), which = 1:2)[2], tolerance = .Machine$double.eps ^ 0.125))

    print(.all.equal(cx$loglik[2], logLik(gl)))
}


## Cox model with predictions obtained from survFit function

fm <- Surv(futime,fustat) ~ age + resid.ds + rx + ecog.ps - 1
## no need to drop the intercept here:
fmSurv <- Surv(futime,fustat) ~ age + resid.ds + rx + ecog.ps
fit <- coxph(fmSurv, data = ovarian)
fit2 <- glmboost(fm, data = ovarian, family = CoxPH(),
    control=boost_control(mstop = 1000), center = TRUE)
fit3 <- glmboost(fm, data = ovarian, family = CoxPH(),
    control=boost_control(mstop = 1000), center = FALSE)

A1 <- survfit(fit, censor = FALSE)
A2 <- survFit(fit2)
A3 <- survFit(fit3)

max(A1$surv-A2$surv)
max(A1$surv-A3$surv)

newdata <- ovarian[c(1,3,12),]
A1 <- survfit(fit, newdata = newdata, censor = FALSE)
A2 <- survFit(fit2, newdata = newdata)
A3 <- survFit(fit3, newdata = newdata)

max(A1$surv-A2$surv)
max(A1$surv-A3$surv)

### Poisson models

df <- data.frame(x1 = runif(100), x2 = runif(100))
f <- -1 + 3 * df$x2
df$y <- round(exp(f) )
ctrl <- boost_control(mstop = 2000, nu = 0.1)

gmod <- glm(y ~ x1 + x2, data = df, family = poisson())
gbmod <- glmboost(y ~ x1 + x2, data = df, family = Poisson(), control = ctrl,
                  center = FALSE)

llg <- logLik(gmod)
attributes(llg) <- NULL
print(.all.equal(logLik(gbmod), llg))

### hat matrix is only approximate!
print(abs(AIC(gmod) - attr(AIC(gbmod, "classical"), "AIC")[mstop(gbmod)]) < 1)

print(max(abs(predict(gmod) -  predict(gbmod))) < 1e-4)

################################################################################
### Check predictions
################################################################################

set.seed(1907)
x1 <- rnorm(100)
x2 <- rnorm(100)
x3 <- rnorm(100)
y <- rnorm(100, mean = 3 * x1, sd = 2)
DF <- data.frame(y = y, x1 = x1, x2 = x2, x3 = x3)

amod <- glmboost(y ~ -1 + x1 + x2, data = DF, center = FALSE)
agg <- c("none", "sum", "cumsum")
whi <- list(NULL, 1, 2, c(1,2))
pred <- vector("list", length = 4)
for (i in 1:4){
    pred[[i]][[i]] <- vector("list", length=3)
    for (j in 1:3){
        pred[[i]][[j]] <- predict(amod, aggregate=agg[j], which = whi[[i]])
    }
    if (i == 1){
        print(max(abs(pred[[i]][[2]] - pred[[i]][[3]][,ncol(pred[[i]][[3]])]))  < sqrt(.Machine$double.eps))
        if ((pred[[i]][[2]] - rowSums(pred[[i]][[1]]))[1] - attr(coef(amod), "offset") < sqrt(.Machine$double.eps))
            warning(sQuote("aggregate = sum"), " adds the offset, ", sQuote("aggregate = none"), " doesn't.")
        print(max(abs(pred[[i]][[2]] - rowSums(pred[[i]][[1]]) - attr(coef(amod), "offset")))   < sqrt(.Machine$double.eps))
    } else {
        print(max(abs(pred[[i]][[2]] - sapply(pred[[i]][[3]], function(obj) obj[,ncol(obj)])))  < sqrt(.Machine$double.eps))
        print(max(abs(pred[[i]][[2]] - sapply(pred[[i]][[1]], function(obj) rowSums(obj))))  < sqrt(.Machine$double.eps))
    }
}
## compare which = NULL and which == c(1, 2)
# type = "none"
print(.all.equal(pred[[1]][[1]], pred[[4]][[1]]$x1 + pred[[4]][[1]]$x2, check.attributes = FALSE))
# type = "sum"
predictions <- as.matrix(DF[, c("x1", "x2")]) %*% matrix(coef(amod), ncol = 1) +
    attr(coef(amod), "offset")
print(.all.equal(pred[[1]][[2]], predictions, check.attributes = FALSE))
print(.all.equal(c(pred[[1]][[2]]),
                    rowSums(pred[[4]][[2]]) + attr(coef(amod), "offset"),
                    check.attributes = FALSE))
# type = "cumsum"
print(.all.equal(pred[[1]][[3]], pred[[4]][[3]]$x1 + pred[[4]][[3]]$x2 + attr(coef(amod), "offset")))

## same with names
agg <- c("none", "sum", "cumsum")
whi <- list(NULL, "x1", "x2", c("x1","x2"))
for (i in 1:4){
    pred <- vector("list", length=3)
    for (j in 1:3){
        pred[[j]] <- predict(amod, aggregate=agg[j], which = whi[[i]])
    }
    if (i == 1){
        print(max(abs(pred[[2]] - pred[[3]][,ncol(pred[[3]])]))  < sqrt(.Machine$double.eps))
        if ((pred[[2]] - rowSums(pred[[1]]))[1] - attr(coef(amod), "offset") < sqrt(.Machine$double.eps))
            warning(sQuote("aggregate = sum"), " adds the offset, ", sQuote("aggregate = none"), " doesn't.")
        print(max(abs(pred[[2]] - rowSums(pred[[1]]) - attr(coef(amod), "offset")))   < sqrt(.Machine$double.eps))
    } else {
        print(max(abs(pred[[2]] - sapply(pred[[3]], function(obj) obj[,ncol(obj)])))  < sqrt(.Machine$double.eps))
        print(max(abs(pred[[2]] - sapply(pred[[1]], function(obj) rowSums(obj))))  < sqrt(.Machine$double.eps))
    }
}

y <- rnorm(100, mean = 3 * x1^2, sd = 2)
DF2 <- data.frame(y = y, x1 = x1, x2 = x2, x3 = x3)
amod <- glmboost(y ~ -1 + x1 + I(x1^2), data = DF2, center = FALSE)
print(ncol(predict(amod, which="x1")) == 2 && all(rowSums(predict(amod, which="x1")) + attr(coef(amod), "offset") - predict(amod) < sqrt(.Machine$double.eps)))


amod <- glmboost(y ~ 1+ x1 + x2, data = DF, center = FALSE)
pr1 <- predict(amod, aggre = "sum", which= 1:2)
foo <- DF
foo$x2 <- 0
pr2 <- predict(amod, aggre = "sum", newdata=foo)
print(rowSums(pr1) + attr(coef(amod),"offset") - pr2 < sqrt(.Machine$double.eps))
newData <- as.data.frame(rbind(colMeans(DF)[-1], colMeans(DF)[-2]+1*sapply(DF, sd)[-1]))
if (!is.list(pr <- predict(amod, newdata=newData, which=1:2)))
    warning("predict(amod, newdata=newData, which=1:2) does not return a list") # no list but a matrix is returned!
print(is.list(pr <- predict(amod, newdata=newData, aggregate="cumsum", which=1:2)))
amod[10]
pr <- predict(amod, which=1:3)
print(ncol(pr) == 3 || all(pr[,c(1,ncol)] == 0))
amod[100]

## check predictions with offset
amod <- glmboost(y ~ -1 + x1 + x2, offset = 10, data = DF, center=FALSE)
bmod <- glmboost(y ~ -1 + x1 + x2, offset = fitted(amod), data = DF, center=FALSE)
## specify newdata
newdata <- data.frame(x1 = rnorm(10), x2 = rnorm(10), x3 = rnorm(10))
agg <- c("none", "sum", "cumsum")
whi <- list(NULL, "x1", "x2", c("x1","x2"))

preda <- predb <- vector("list", length = 4)
for (i in 1:4){
    preda[[i]][[i]] <- vector("list", length=3)
    for (j in 1:3){
        preda[[i]][[j]] <- predict(amod, aggregate=agg[j], which = whi[[i]],
                                   newdata = newdata)
        ## a warning is issued for "sum" and "cumsum"
        predb[[i]][[j]] <- predict(bmod, aggregate=agg[j], which = whi[[i]],
                                   newdata = newdata)
    }
    if (i == 1){
        ## checks for amod
        print(max(abs(preda[[i]][[2]] - preda[[i]][[3]][,ncol(preda[[i]][[3]])]))  < sqrt(.Machine$double.eps))
        if ((preda[[i]][[2]] - rowSums(preda[[i]][[1]]))[1] - attr(coef(amod), "offset") < sqrt(.Machine$double.eps))
            warning(sQuote("aggregate = sum"), " adds the offset, ", sQuote("aggregate = none"), " doesn't.")
        print(max(abs(preda[[i]][[2]] - rowSums(preda[[i]][[1]]) - attr(coef(amod), "offset")))   < sqrt(.Machine$double.eps))
        ## same for bmod
        print(max(abs(predb[[i]][[2]] - predb[[i]][[3]][,ncol(predb[[i]][[3]])]))  < sqrt(.Machine$double.eps))
        if ((predb[[i]][[2]] - rowSums(predb[[i]][[1]]))[1] < sqrt(.Machine$double.eps))
            warning(sQuote("aggregate = sum"), " adds the offset, ", sQuote("aggregate = none"), " doesn't.")
        ## here, the offset is not used in both cases!
        print(max(abs(predb[[i]][[2]] - rowSums(predb[[i]][[1]])))   < sqrt(.Machine$double.eps))
    } else {
        ## checks for amod
        print(max(abs(preda[[i]][[2]] - sapply(preda[[i]][[3]], function(obj) obj[,ncol(obj)])))  < sqrt(.Machine$double.eps))
        print(max(abs(preda[[i]][[2]] - sapply(preda[[i]][[1]], function(obj) rowSums(obj))))  < sqrt(.Machine$double.eps))
        ## same for bmod
        print(max(abs(predb[[i]][[2]] - sapply(predb[[i]][[3]], function(obj) obj[,ncol(obj)])))  < sqrt(.Machine$double.eps))
        print(max(abs(predb[[i]][[2]] - sapply(predb[[i]][[1]], function(obj) rowSums(obj))))  < sqrt(.Machine$double.eps))
    }
}
## compare which = NULL and which == c(1, 2);
## The offset should be always dropped for bmod, but kept for amod
# type = "none"
print(.all.equal(preda[[1]][[1]], preda[[4]][[1]]$x1 + preda[[4]][[1]]$x2, check.attributes = FALSE))
print(.all.equal(predb[[1]][[1]], predb[[4]][[1]]$x1 + predb[[4]][[1]]$x2, check.attributes = FALSE))
# type = "sum"
predictionsA <- as.matrix(newdata[, c("x1", "x2")]) %*% matrix(coef(amod), ncol = 1) +
    attr(coef(amod), "offset")
predictionsB <- as.matrix(newdata[, c("x1", "x2")]) %*% matrix(coef(bmod), ncol = 1)
print(.all.equal(preda[[1]][[2]], predictionsA, check.attributes = FALSE))
print(.all.equal(predb[[1]][[2]], predictionsB, check.attributes = FALSE))
print(.all.equal(c(preda[[1]][[2]]), rowSums(preda[[4]][[2]]) + attr(coef(amod), "offset"),
                    check.attributes = FALSE))
print(.all.equal(c(predb[[1]][[2]]), rowSums(predb[[4]][[2]]),
                    check.attributes = FALSE))
# type = "cumsum"
print(.all.equal(predb[[1]][[3]], predb[[4]][[3]]$x1 + predb[[4]][[3]]$x2))
print(.all.equal(predb[[1]][[3]], predb[[4]][[3]]$x1 + predb[[4]][[3]]$x2))

### compare predictions with gamboost
mod1 <- glmboost(y ~ -1 + x1 + x2 + x3, data = DF, center=FALSE)
mod2 <- gamboost(y ~ x1 + x2 + x3, data = DF, baselearner= function(x) bols(x, intercept=FALSE))
pr1_2 <- predict(mod2, aggre = "cumsum")
pr2_2 <- predict(mod2, aggre = "none")
pr3_2 <- predict(mod2, aggre = "sum")

print(max(abs(predict(mod1) - predict(mod2))) < sqrt(.Machine$double.eps))
print(max(abs(predict(mod1, aggre = "none") - predict(mod2, aggre = "none"))) < sqrt(.Machine$double.eps))
print(max(abs(predict(mod1, aggre = "cumsum") - predict(mod2, aggre = "cumsum"))) < sqrt(.Machine$double.eps))

# check type argument
set.seed(1907)
x1 <- rnorm(100)
p <- 1/(1 + exp(- 3 * x1))
y <- as.factor(runif(100) < p)
DF <- data.frame(y = y, x1 = x1)

logitBoost <- glmboost(y ~ x1, family = Binomial(), center = FALSE,
                 data = DF,  control=boost_control(mstop=5000))
logit <- glm(y ~ x1, data=DF, family=binomial)
print(coef(logitBoost)[2]*2 - coef(logit)[2] < 1e-5) # * 2 as we use y = {-1, 1}

pr <- predict(logitBoost)
pr2 <- predict(logit)
print(pr * 2 - pr2 < 1e-5)  # * 2 as we use y = {-1, 1}

pr <- predict(logitBoost, type="class")
pr2 <- predict(logit, type="response") > 0.5
foo <- table(pr, pr2)
print(foo[1,2] + foo[2,1] == 0)

pr <- predict(logitBoost, type="response")
pr2 <- predict(logit, type="response")
print(pr - pr2 < sqrt(.Machine$double.eps))

################################################################################
### Check coefficients
################################################################################

set.seed(1907)
x1 <- rnorm(100)
x2 <- rnorm(100)
x3 <- rnorm(100)
y <- rnorm(100, mean = 3 * x1, sd = 2)
DF <- data.frame(y = y, x1 = x1, x2 = x2, x3 = x3)
amod <- glmboost(y ~ x1 + x2 + x3, data = DF, center = FALSE)

print(length(coef(amod)) == 4)
amod[10]
print(length(coef(amod)) == 1)
print(length(coef(amod, which=1:3)) == 3)

### check with logical design matrices
n <- 1000
p <- 10
X <- matrix(as.logical(rbinom(n * p, size = 1, prob = 0.5)), nrow = n)
y <- rnorm(n)

mod <- glmboost(x = X, y = y)

Xd <- X
storage.mode(Xd) <- "double"
modd <- glmboost(x = Xd, y = y)
print(.all.equal(coef(modd), coef(mod)))

### probit models
set.seed(29)
n <- 1000
x <- runif(n, min = -2, max = 2)
y <- factor(rbinom(n, size = 1, prob = pnorm(3 * x)))
table(y)
mod <- glm(y ~ x, family = binomial(link = "probit"))
coef(mod)

mod2 <- glmboost(y ~ x, family = Binomial(link = "probit"),
                 control = boost_control(nu = 0.5, mstop = 1000))
coef(mod2, off2int = TRUE)
print(.all.equal(round(coef(mod), 2), round(coef(mod2, off2int = TRUE), 2)))

data("GlaucomaM", package = "TH.data")
coef(mod3 <- glm(Class ~ varg, data = GlaucomaM, family = binomial(link = "probit")))
coef(mod4 <- glmboost(Class ~ varg, data = GlaucomaM, family = Binomial(link = "probit"))[1000])
print(.all.equal(round(coef(mod3), 3), round(coef(mod4, off2int = TRUE), 3)))

lwd <- 2
cex <- 1
sink("tmpfile")
library("mboost")
sink()
file.remove("tmpfile")
cat("\n\n\t%%%% DON'T EDIT THIS FILE\n\n")
options(prompt = "R> ", width = 60, continue = "     ", digits = 5)
if (!file.exists("figures"))
    dir.create("figures")
cat("\\setkeys{Gin}{width = 0.95\\textwidth}")
set.seed(290875)

perfplot <- function(x, grid, alpha = NULL, border = 1,
                     xlab = "", ylab = "Performance") {
  x <- as.matrix(x)
  nc <- NCOL(x)
  nr <- NROW(x)
  nam <- grid
  if(is.null(alpha)) alpha <- 0.17 - 0.0002 * nr

  plot(rep(1:nc, 2), rep(range(x), nc), type = "n", axes = FALSE,
       ylab = ylab, xlab = xlab, xlim = c(0.6, nc + 0.4))
  axis(1, at = 1:nc, labels = nam)
  axis(2)
  box()
  matlines(t(x), col = rgb(0,0,0, alpha), lty = 1)
  lines(colMeans(x))
}

data("bodyfat", package = "TH.data")
data("wpbc", package = "TH.data")

options(digits = 3)

library("mboost")

set.seed(290875)
x <- rnorm(100)
y <- rnorm(100)
w <- drop(rmultinom(1, 100, rep(1 / 100, 100)))

G <- Gaussian()
fm <- Family(ngradient = G@ngradient, risk = G@risk)

glmboost(y ~ x, family = G)

glmboost(y ~ x, family = fm)

mG <- glmboost(y ~ x, family = Gaussian())
mH <- glmboost(y ~ x, family = Huber(10))
all.equal(coef(mG), coef(mH))


x <- rnorm(100)
y <- rnbinom(length(x), size = 2, mu = exp(x * 2))
mod <- glmboost(y ~ x, family = NBinomial(), center = FALSE)
mod[1000]
coef(mod)
nuisance(mod)

### QuantReg and ExpectReg
gamboost(y ~ x, family = QuantReg())
gamboost(y ~ x, family = ExpectReg())

if (require("MASS")) {

summary(glm.nb(y ~ x))

y <- cut(x, breaks = c(-Inf, quantile(x, prob = c(0.25, 0.5, 0.75)), Inf), ordered = TRUE)
x <- rnorm(100)
polr(y ~ x)
mod <- glmboost(y ~ x, family = PropOdds(), center = FALSE)
nuisance(mod) - attr(coef(mod), "offset")
coef(mod)

}

## Count models as before but with highly skewed data 
## (used to lead to an error as offset was not properly computed)
x <- rnorm(100)
y <- rnbinom(length(x), size = 2, mu = exp(x * 5))
mod <- glmboost(y ~ x, family = NBinomial())
## check same for Hurdle model
x <- x[y > 0]
y <- y[y > 0]
mod <- glmboost(y ~ x, family = Hurdle())

### Weibull model

if (require("survival")) {

rextrval <- function(x) log( -log(1-x) )
sigma <- 0.5
u <- runif(100)
u.1 <- runif(100)
w <- rextrval(u)
w.1 <- rextrval(u.1)

x1 <- rnorm(100,sd=1)
x2 <- x1 + rnorm(100,sd=1)
x1.1 <- rnorm(100,sd=1)
x2.1 <- x1.1 + rnorm(100,sd=1)
X <- cbind(x1,x2)
X.1 <- cbind(x1.1,x2.1)
beta <- c(1,0.5)
survtime <- exp(X%*%beta + sigma*w)
censtime <- exp(X.1%*%beta + sigma*w.1)
event <- survtime < censtime
stime <- pmin(survtime,censtime)

model1 <- glmboost(Surv(stime,event)~x1+x2, family=Weibull(),
    control = boost_control(mstop=100), center = FALSE)
coef(model1)
nuisance(model1)
model2 <- survreg(Surv(stime,event)~x1+x2)
coef(model2)
model2$scale

}


### Log logistic model

if (require("survival")) {

sigma <- 0.5
w <- rlogis(100)
w.1 <- rlogis(100)

x1 <- rnorm(100,sd=1)
x2 <- x1 + rnorm(100,sd=1)
x1.1 <- rnorm(100,sd=1)
x2.1 <- x1.1 + rnorm(100,sd=1)
X <- cbind(x1,x2)
X.1 <- cbind(x1.1,x2.1)
beta <- c(1,0.5)
survtime <- exp(X%*%beta + sigma*w)
censtime <- exp(X.1%*%beta + sigma*w.1)
event <- survtime < censtime
stime <- pmin(survtime,censtime)

model1 <- glmboost(Surv(stime,event)~x1+x2, family=Loglog(),
    control = boost_control(mstop=200), center = FALSE)
coef(model1)
nuisance(model1)
model2 <- survreg(Surv(stime,event)~x1+x2, dist="loglogistic")
coef(model2)
model2$scale

}


### Log normal model

if (require("survival")) {

sigma <- 0.5
w <- rnorm(100)
w.1 <- rnorm(100)

x1 <- rnorm(100,sd=1)
x2 <- x1 + rnorm(100,sd=1)
x1.1 <- rnorm(100,sd=1)
x2.1 <- x1.1 + rnorm(100,sd=1)
X <- cbind(x1,x2)
X.1 <- cbind(x1.1,x2.1)
beta <- c(1,0.5)
survtime <- exp(X%*%beta + sigma*w)
censtime <- exp(X.1%*%beta + sigma*w.1)
event <- survtime < censtime
stime <- pmin(survtime,censtime)

model1 <- glmboost(Surv(stime,event)~x1+x2, family=Lognormal(),
    control = boost_control(mstop=200), center = FALSE)
coef(model1)
nuisance(model1)
model2 <- survreg(Surv(stime,event)~x1+x2, dist="lognormal")
coef(model2)
model2$scale

}


### AUC
data("wpbc", package = "TH.data")
wpbc[,colnames(wpbc) != "status"] <- scale(wpbc[,colnames(wpbc) != "status"])
wpbc <- wpbc[complete.cases(wpbc), colnames(wpbc) != "time"]
mAUC <- gamboost(status ~ ., data = wpbc, family = AUC())
1 - mAUC$risk()


# rank-based boosting

if (require("survival")) {

set.seed(1907)
n <- 100
beta <- c(3, 1.5, 0, 0, 2, 0, 0, 0)
p <- length(beta)
x <- matrix(rnorm(n*p), n, p)
yt <- x %*% beta + rnorm(n)
x <- cbind(1, x)
colnames(x) <- c("intercept", paste("x", 1:8, sep=""))
cens <- runif(n, 0, 6)
y <- exp(pmin(yt, cens))
del <- yt <= cens
## check fitting and cvrisk
mod <- glmboost(x = x, y = Surv(y, del),
                control = boost_control(mstop = 500, nu = 0.1),
                center = TRUE,
                family = Gehan())
coef(mod)
plot(mod$risk())
cvr <- cvrisk(mod, folds = cv(model.weights(mod), "kfold"), papply=lapply)
plot(cvr)
## check weighting:
wMat <- cv(rep(1, n), type = "kfold",B = 2)
modWeighted <- glmboost(x = x, y = Surv(y, del), weights = wMat[, 1],
                        control = boost_control(mstop = 300, nu = 0.20),
                        family = Gehan())
# same model with data set subseted:
modSubset <- glmboost(x = x[as.logical(wMat[, 1]),],
                      y = Surv(y, del)[as.logical(wMat[, 1]),],
                      control = boost_control(mstop = 300, nu = 0.20),
                      family = Gehan())
## <FIXME> there are still some minor discrepancies. Perhaps this is due to
## different pre-processing? </FIXME>
round(coef(modWeighted, which = "") - coef(modSubset, which = ""), 3)
}


## Binomial
y <- as.factor(sample(0:1, 100, replace = TRUE))
x1 <- rnorm(100)
x2 <- rnorm(100)

mod <- glmboost(y ~ x1 + x2, family = Binomial())
mod[500]
2 * coef(mod, off2int = TRUE)

glmMod <- glm(y ~ x1 + x2, family = 'binomial')
coef(glmMod)
print(all(abs((coef(glmMod) - coef(mod, off2int = TRUE) * 2)) < sqrt(.Machine$double.eps)))

## C-index boosting
if (require("survival")) {
  
  sigma <- 0.5
  w <- rnorm(100)
  w.1 <- rnorm(100)
  
  x1 <- rnorm(100,sd=1)
  x2 <- x1 + rnorm(100,sd=1)
  x1.1 <- rnorm(100,sd=1)
  x2.1 <- x1.1 + rnorm(100,sd=1)
  X <- cbind(x1,x2)
  X.1 <- cbind(x1.1,x2.1)
  beta <- c(1,0.5)
  survtime <- exp(X%*%beta + sigma*w)
  censtime <- exp(X.1%*%beta + sigma*w.1)
  event <- survtime < censtime
  stime <- pmin(survtime,censtime)
  dat <- data.frame(time = stime, event = event, x1 = x1, x2 = x2)
  
  # compute ipcweights outside the family
  ipcw <- IPCweights(x = Surv(dat$time, dat$event))
  model1 <- glmboost(Surv(time,event)~x1+x2, family=Cindex(ipcw = ipcw),
                     control = boost_control(mstop=50), data = dat)
  
  
  # compute ipcweights inside
  model2 <- glmboost(Surv(time,event)~x1+x2, family=Cindex(ipcw = 1),
                     control = boost_control(mstop=50), data = dat)
  rbind(coef(model1, off2int = TRUE, which = ""), 
        coef(model2, off2int = TRUE, which = ""))
  
  print(identical(coef(model1), coef(model2)))
  
  # change sigma
  model1 <- glmboost(Surv(time,event)~x1+x2, family=Cindex(sigma = 0.01),
                     control = boost_control(mstop=20), data = dat)
  model2 <- glmboost(Surv(time,event)~x1+x2, family=Cindex(sigma = 0.2),
                     control = boost_control(mstop=20), data = dat)
  rbind(coef(model1, off2int = TRUE, which = ""), 
        coef(model2, off2int = TRUE, which = ""))
  print(!identical(coef(model1), coef(model2)))
  
}

## check Binomial(type = "glm")
glmModboost <- glmboost(y ~ x1 + x2, family = Binomial(type = "glm"))
glmModboost[1000] 
round(rbind(coef(glmMod), coef(glmModboost, off2int =TRUE), 2*coef(mod, off2int = TRUE)),3)
## use different link
glmMod <- glm(y ~ x1 + x2, family = binomial(link = "probit"))
coef(glmMod)
glmModboost <- glmboost(y ~ x1 + x2, family = Binomial(type = "glm", link = "probit"))
glmModboost[500]
round(rbind(coef(glmMod), coef(glmModboost, off2int =TRUE)),3)
## use matrix of successes and failures
y <- matrix(ncol = 2, nrow = length(x1), data = rpois(lambda = 30, n =2*length(x1)) )
glmMod <- glm(y ~ x1 + x2, family = binomial())
coef(glmMod)
glmModboost <- glmboost(y ~ x1 + x2, family = Binomial(type = "glm"))
glmModboost[500]
round(rbind(coef(glmMod), coef(glmModboost, off2int =TRUE)),3)
## use binary vector
y <- rbinom(prob = plogis(x1 + x2), size = 1, n = length(x1))
glmMod <- glm(y ~ x1 + x2, family = binomial())
coef(glmMod)
glmModboost <- glmboost(y ~ x1 + x2, family = Binomial(type = "glm"),
                        control = boost_control(nu = 0.2))
glmModboost[1000]
round(rbind(coef(glmMod), coef(glmModboost, off2int =TRUE)),2)


## 
## Binomial with other links
## and interface type = "adaboost" or "glm"
set.seed(123)
y <- as.factor(sample(0:1, 100, replace = TRUE))
x1 <- rnorm(100)
x2 <- rnorm(100)

mod <- glmboost(y ~ x1 + x2, family = Binomial(type = "adaboost", link = "cauchit"))
mod[500]
mod2 <- glmboost(y ~ x1 + x2, family = Binomial(type = "glm", link = "cauchit"))
mod2[500]
glmMod <- glm(y ~ x1 + x2 , family = binomial(link = "cauchit"))
print(all(round(coef(glmMod),2) == round(coef(mod, off2int =TRUE),2)))
rbind(coef(glmMod), coef(mod, off2int = TRUE), coef(mod2, off2int = TRUE))

mod <- glmboost(y ~ x1 + x2, family = Binomial(type = "adaboost", link = "probit"))
mod[500]
glmMod <- glm(y ~ x1 + x2 , family = binomial(link = "probit"))
print(all(round(coef(glmMod),2) == round(coef(mod, off2int =TRUE),2)))
mod2 <- glmboost(y ~ x1 + x2, family = Binomial(type = "glm", link = "probit"))
mod2[500]
rbind(coef(glmMod), coef(mod, off2int = TRUE), coef(mod2, off2int = TRUE))


mod <- glmboost(y ~ x1 + x2, family = Binomial(type = "adaboost", link = "log"))
mod[500]
glmMod <- glm(y ~ x1 + x2 , family = binomial(link = "log"))
print(all(round(coef(glmMod),2) == round(coef(mod, off2int =TRUE),2)))
mod2 <- glmboost(y ~ x1 + x2, family = Binomial(type = "glm", link = "log"))
mod2[500]
rbind(coef(glmMod), coef(mod, off2int = TRUE), coef(mod2, off2int = TRUE))

#.onAttach <- function(libname, pkgname) {
#
#    ## get package version
#    vers <- packageDescription("mboost")[["Version"]]
#
#    packageStartupMessage("This is mboost ", vers, ". ", "See ",
#                          sQuote("package?mboost"), " and ",
#                          sQuote('news(package  = "mboost")'), "\n",
#                          "for a complete list of changes.\n",
#                          appendLF = TRUE)
#    return(TRUE)
#}

.onLoad <- function(libname, pkgname) {
    if (options("expressions")[[1]] <= 5000)
        options(expressions = 10000)  ### increase maximum number of expressions
    options(mboost_useMatrix = TRUE, ### allow for Matrix package?
            mboost_indexmin = 10000, ### handle ties for n > 10000
            mboost_dftraceS = FALSE,  ### df = trace(S) or df = trace(2 S - StS)
            mboost_lambdaMax = 1e+15,### maximum value for lambda as used in df2lambda
            mboost_Xmonotone = FALSE,### don't force monotonicity in %X%
            mboost_eps = 10e-10) ### factor for dmat in df2lambda
}

.onUnload <- function(libpath) {
    if (options("expressions")[[1]] == 10000)
        options(expression = 5000)  ### decrease maximum number of expressions again
}

setClass("boost_family", representation = representation(
    ngradient  = "function",
    risk       = "function",
    offset     = "function",
    check_y    = "function",
    weights    = "function",
    nuisance   = "function",
    response   = "function",
    rclass     = "function",
    name       = "character",
    charloss   = "character"
))

setClass("boost_family_glm", contains = "boost_family",
    representation = representation(
        fW      = "function"
))

setMethod("show", "boost_family", function(object) {
    cat("\n\t", object@name, "\n\n")
    cat("Loss function:", object@charloss, "\n")
})

Family <- function(ngradient, loss = NULL, risk = NULL,
                   offset = function(y, w)
                       optimize(risk, interval = range(y), y = y, w = w)$minimum,
                   check_y = function(y) y,
                   weights = c("any", "none", "zeroone", "case"),
                   nuisance = function() return(NA),
                   name = "user-specified", fW = NULL,
                   response = function(f) NA,
                   rclass = function(f) NA)
{

    if (is.null(loss)) {
        charloss <- ""
        stopifnot(!is.null(risk))
    } else {
        charloss <- paste(deparse(body(loss)), "\n")
    }
    if (is.null(risk)) {
        stopifnot(!is.null(loss))
        risk <- function(y, f, w = 1) sum(w * loss(y, f), na.rm = TRUE)
    }
    weights <- match.arg(weights)
    check_w <- function(w) {
        switch(weights,
            "any" = TRUE,
            "none" = isTRUE(all.equal(unique(w), 1)),
            "zeroone" = isTRUE(all.equal(unique(w + abs(w - 1)), 1)),
            "case" = isTRUE(all.equal(unique(w - floor(w)), 0)))
    }
    RET <- new("boost_family", ngradient = ngradient, nuisance = nuisance,
               risk = risk, offset = offset, check_y = check_y,
               response = response, rclass = rclass, weights = check_w,
               name = name, charloss = charloss)
    if (!is.null(fW))
        RET <- new("boost_family_glm", ngradient = ngradient, nuisance = nuisance,
                   risk = risk, offset = offset, fW = fW, check_y = check_y,
                   weights = check_w, name = name, response = response,
                   rclass = rclass, charloss= charloss)
    RET
}

### Gaussian (Regression)
Gaussian <- function()
    Family(ngradient = function(y, f, w = 1) y - f,
           loss = function(y, f) (y - f)^2,
           offset = weighted.mean,
           check_y = function(y) {
               if (!is.numeric(y) || !is.null(dim(y)))
                   stop("response is not a numeric vector but ",
                        sQuote("family = Gaussian()"))
               y
           },
           name = "Squared Error (Regression)",
           fW = function(f) return(rep(1, length = length(f))),
           response = function(f) f)

GaussReg <- Gaussian

### Gaussian (-1 / 1 Binary Classification)
GaussClass <- function()
    stop("Family", sQuote("GaussClass"), "has been removed")

### Laplace
Laplace <- function()
    Family(ngradient = function(y, f, w = 1) sign(y - f),
           loss = function(y, f) abs(y - f),
           check_y = function(y) {
               if (!is.numeric(y) || !is.null(dim(y)))
                   stop("response is not a numeric vector but ",
                        sQuote("family = Laplace()"))
               y
           },
           name = "Absolute Error",
           response = function(f) f)

link2dist <- function(link,  ...) {
    p <- get(paste("p", link, sep = ""))
    d <- get(paste("d", link, sep = ""))
    q <- get(paste("q", link, sep = ""))
    ret <- list(p = function(x) p(x, ...),
                d = function(x) d(x, ...),
                q = function(x) q(x, ...))
    attr(ret, "link") <- link
    ret
}

Binomial <- function(type = c("adaboost", "glm"),
                     link = c("logit", "probit", "cloglog", "cauchit", "log"), ...) {
    
    type <- match.arg(type)
    if (type == "adaboost") {
        return(Binomial_adaboost(link = link, ...))
    } else {
        return(Binomial_glm(link = link))
    }
}

### Binomial
# lfinv <- binomial()$linkinv
Binomial_adaboost <- function(link = c("logit", "probit", "cloglog", "cauchit", "log"), ...) {
    
    tmp <- try(match.arg(link), silent = TRUE)
    if (inherits(tmp, "try-error") ) {
        ## use old interface if link is not one of the above (i.e. if link is a distribution)
        link <- link2dist(link, ...)    
    } else {
        ## use new interface otherwise
        link <- make.link(tmp)   
    }
    
    biny <- function(y) {
        if (!is.factor(y))
            stop("response is not a factor but ",
                 sQuote("family = Binomial()"))
        if (nlevels(y) != 2)
            stop("response is not a factor at two levels but ",
                 sQuote("family = Binomial()"))
        return(c(-1, 1)[as.integer(y)])
    }
    
    if (inherits(link, "link-glm") && isTRUE(all.equal(link$name, "logit"))) {
        return(  
          # This is the classic logit family
          # ngradient refers to loss
          # log_2(1 + exp(-2yf))
          # with f = log(p/(1-p))/2 and y in (-1,1)
          # leading to coefs half the size as usually 
            Family(ngradient = function(y, f, w = 1) {
                exp2yf <- exp(-2 * y * f)
                -(-2 * y * exp2yf) / (log(2) * (1 + exp2yf))
            },
            loss = function(y, f) {
                f <- pmin(abs(f), 36) * sign(f)
                p <- exp(f) / (exp(f) + exp(-f))
                y <- (y + 1) / 2
                -y * log(p) - (1 - y) * log(1 - p)
            },
            offset = function(y, w) {
                p <- weighted.mean(y > 0, w)
                1/2 * log(p / (1 - p))
            },
            fW = function(f) {
                f <- pmin(abs(f), 36) * sign(f)
                p <- exp(f) / (exp(f) + exp(-f))
                4 * p * (1 - p)
            },
            response = function(f) {
                f <- pmin(abs(f), 36) * sign(f)
                p <- exp(f) / (exp(f) + exp(-f))
                return(p)
            },
            rclass = function(f) (f > 0) + 1 ,
            check_y = biny,
            name = "Negative Binomial Likelihood (logit link)")
        )
    }
    
    if (inherits(link, "link-glm")) {
      # now for glm type links that are not logit
      # loss is now 
      # -y * log(p) - (1 - y) * log(1 - p)
      # coefficients hace the usual size! 
      return(Family(ngradient = function(y, f, w = 1) {
        y <- (y + 1) / 2
        p <- link$linkinv(f)
        link$mu.eta(f) * (y / p - (1 - y) / (1 - p))
      },
      loss = function(y, f) {
        p <- link$linkinv(f)
        y <- (y + 1) / 2
        -y * log(p) - (1 - y) * log(1 - p)
      },
      offset = function(y, w) {
        p <- weighted.mean(y > 0, w)
        link$linkfun(p)
      },
      response = function(f) {
        p <- link$linkinv(f)
        return(p)
      },
      rclass = function(f) (f > 0) + 1 ,
      check_y = biny,
      name = paste("Negative Binomial Likelihood --",
                   link$name, "link")))
      
      
    }
    
    trf <- function(f) {
      thresh <- -link$q(.Machine$double.eps)
      pmin(pmax(f, -thresh), thresh)
    }
    
    # now with link2dist (distributions)
    return(Family(ngradient = function(y, f, w = 1) {
        y <- (y + 1) / 2
        p <- link$p(trf(f))
        d <- link$d(trf(f))
        d * (y / p - (1 - y) / (1 - p))
    },
    loss = function(y, f) {
        p <- link$p(trf(f))
        y <- (y + 1) / 2
        -y * log(p) - (1 - y) * log(1 - p)
    },
    offset = function(y, w) {
        p <- weighted.mean(y > 0, w)
        link$q(p)
    },
    response = function(f) {
        p <- link$p(trf(f))
        return(p)
    },
    rclass = function(f) (f > 0) + 1 ,
    check_y = biny,
    name = paste("Negative Binomial Likelihood --",
                 attr(link, "link"), "Link")))
}

### Additional Binomial family 
### Works not only with two-level factors but also 
### with a two-column matrix of number of successes and
### number of failures (see github issue #34 by fabian-s). 
### In case of factors or binary vectors it uses the standard
### 0 and 1 coding; the coefficients hence have the same level
### as the ones resulting from glm() with family = "binomial".
Binomial_glm <- function(link = c("logit", "probit", "cloglog", "cauchit", "log")) {
    
    link <- match.arg(link)
    link <- make.link(link)
    
    y_check <- function(y) {
        if ((is.matrix(y) && NCOL(y)!=2)){
            stop("response should be either a two-column matrix (no. successes ",
                 "and no. failures) or a two level factor or a vector of 0 and 1's ",
                 "for this family")
        }
        if(is.factor(y)){
            if (nlevels(y) != 2) 
                stop("response should be either a two-column matrix ",
                     "(no. successes  and  no. failures) or a two level ", 
                     "factor or a vector of 0 and 1's for this family")
            y <- c(0, 1)[as.integer(y)]
        }
        if(!is.matrix(y)){
            if(!all(y %in% c(0,1))) 
                stop("response should be either a two-column matrix ",
                     "(no. successes  and  no. failures) or a two level ",
                     "factor or a vector of 0 and 1's for this family")
            y <- cbind(y, 1-y)
        }
        return(y)  
    }
    
    loss <- function(y, f, w = 1) {
        ntrials <- rowSums(y)
        y <- y[,1]
        p <- link$linkinv(f)
        -dbinom(x = y, size = ntrials, prob = p, log = log)
    }
    risk <- function(y, f, w = 1) {
        sum(w * loss(y = y, f = f))
    }
    
    ngradient <- function(y, f, w = 1) {
        ntrials <- rowSums(y)
        y <- y[,1]
        p <- link$linkinv(f)
        ngr <-  link$mu.eta(f)*(y - ntrials * p)/(p * (1 - p))
    }
    
    offset <-function(y, w = 1) {
        ntrials <- rowSums(y)
        y <- y[,1]
        p <- mean((y + 0.5)/(ntrials + 1))
        return(link$linkfun(p))
    }
    
    Family(ngradient = ngradient, risk = risk, loss = loss, check_y = y_check, 
           response = function(f) link$linkinv(f), offset = offset,
           rclass = function(f) (f > 0) + 1 ,
           name = "Binomial Distribution (similar to glm)")
}

### Poisson
Poisson <- function()
    Family(ngradient = function(y, f, w = 1) y - exp(f),
           loss = function(y, f) -dpois(y, exp(f), log = TRUE),
           offset = function(y, w) log(weighted.mean(y, w)),
           fW = function(f) exp(f),
           response = function(f) exp(f),
           check_y = function(y) {
               if (any(y < 0) || any((y - round(y)) > 0))
                   stop("response is not an integer variable but ",
                        sQuote("family = Poisson()"))
               y
           },
           name = "Poisson Likelihood")

### L1Huber
Huber <- function(d = NULL) {
    mc <- match.call()
    if (length(mc) == 2)
        dtxt <- deparse(mc[[2]])
    else
        dtxt <- NULL
    fit <- 0
    Family(ngradient = function(y, f, w = 1) {
               if (is.null(d)) d <- median(abs(y - fit))
               fit <<- f
               ifelse(abs(y - f) < d, y - f, d * sign(y - f))
           },
           loss = function(y, f) {
               if (is.null(d)) d <- median(abs(y - fit))
               ifelse((a <- abs(y - f)) < d, a^2/2, d*(a - d/2))
           },
           check_y = function(y) {
               if (!is.numeric(y) || !is.null(dim(y)))
                   stop("response is not a numeric vector but ",
                        sQuote("family = Huber()"))
               y
           },
           name = paste("Huber Error",
               ifelse(is.null(d), "(with adaptive d)",
                                  paste("(with d = ", dtxt, ")", sep = ""))),
           response = function(f) f)
}

### Adaboost
AdaExp <- function()
    Family(ngradient = function(y, f, w = 1) y * exp(-y * f),
           loss = function(y, f) exp(-y * f),
           offset = function(y, w) {
               p <- weighted.mean(y > 0, w)
               1/2 * log(p / (1 - p))
           },
           check_y = function(y) {
               if (!is.factor(y))
                   stop("response is not a factor but ",
                           sQuote("family = AdaExp()"))
               if (nlevels(y) != 2)
                   stop("response is not a factor at two levels but ",
                           sQuote("family = AdaExp()"))
               c(-1, 1)[as.integer(y)]
           },
           rclass = function(f) (f > 0) + 1 ,
           name = "Adaboost Exponential Error")

### Cox proportional hazards model (partial likelihood)
CoxPH <- function() {
    plloss <- function(y, f, w) {
        time <- y[,1]
        event <- y[,2]
        n <- length(time)
        if (length(f) == 1) 
            f <- rep(f, n)
        if (length(w) == 1) 
            w <- rep(w, n)
        indx <- rep(1:n, w)
        time <- time[indx]
        event <- event[indx]
        ef <- exp(f)[indx]
        f <- f[indx]
        n <- length(time)
        risk <- rep(0, n)
        for (i in 1:n)
               risk[i] <- sum((time >= time[i])*ef)
        event * (f - log(risk))
    }
    Family(ngradient = function(y, f, w) {
               time <- y[,1]
               storage.mode(time) <- "double"
               event <- y[,2]
               storage.mode(event) <- "integer"
               if (length(w) == 1) w <- rep(w, length(time))
               storage.mode(w) <- "double"
               if (length(f) == 1)
                   f <- rep(f, length(time))
               storage.mode(f) <- "double"
               w[is.na(f)] <- 0.0
               f[is.na(f)] <- 0.0
               .Call("ngradientCoxPLik", time, event, f, w, PACKAGE = "mboost")
           },
           risk = risk <- function(y, f, w = 1) -sum(plloss(y, f, w), na.rm = TRUE),
           offset = function(y, w = 1) 0, ## perhaps use something different
                       ## Note: offset cannot be computed from Cox Partial LH as
                       ## PLH doesn't depend on constant
           check_y = function(y) {
               if (!inherits(y, "Surv"))
                   stop("response is not an object of class ", sQuote("Surv"),
                        " but ", sQuote("family = CoxPH()"))
               y
           },
           name = "Cox Partial Likelihood")
}

QuantReg <- function(tau = 0.5, qoffset = 0.5) {
    stopifnot(tau > 0 && tau < 1)
    stopifnot(qoffset > 0 && qoffset < 1)
    Family(
        ngradient = function(y, f, w = 1)
            tau*((y - f) >= 0) - (1 - tau)*((y - f)<0) ,
        loss = function(y, f) tau*(y-f)*((y-f)>=0) - (1-tau)*(y-f)*((y-f)<0) ,
        offset = function(y, w = rep(1, length(y)))
            quantile(y[rep(1:length(y), w)], qoffset),
        weights = "case",
        check_y = function(y) {
            if (!is.numeric(y) || !is.null(dim(y)))
                stop("response is not a numeric vector but ",
                     sQuote("family = QuantReg()"))
            y
        },
        name = "Quantile Regression")
}

NBinomial <- function(nuirange = c(0, 100)) {
    sigma <- 1

    plloss <- function(sigma, y, f)
        - (lgamma(y + sigma) - lgamma(sigma) - lgamma(y + 1) +
           sigma * log(sigma) - sigma*log(exp(f) + sigma) + y * f -
           y * log(exp(f) + sigma))

    riskS <- function(sigma, y, fit, w = 1)
        sum(w * plloss(y = y, f = fit, sigma = sigma))
    risk <- function(y, f, w = 1)
       sum(w * plloss(y = y, f = f, sigma = sigma))
    ngradient <- function(y, f, w = 1) {
        sigma <<- optimize(riskS, interval = nuirange,
                           y = y, fit = f, w = w)$minimum
        y - (y + sigma)/(exp(f) + sigma) * exp(f)
    }

    Family(ngradient = ngradient, 
           risk = risk,
           offset = function(y, w) log(weighted.mean(y, w)),
           check_y = function(y) {
               stopifnot(all.equal(unique(y - floor(y)), 0))
               y
           },
           nuisance = function() return(sigma),
           name = "Negative Negative-Binomial Likelihood",
           response = function(f) exp(f))
}

PropOdds <- function(nuirange = c(-0.5, -1), offrange = c(-5, 5)) {

    sigma <- 0
    delta <- 0

    d2s <- function(delta)
        delta[1] + cumsum(c(0, exp(delta[-1])))

    plloss <- function(sigma, y, f, w = 1) {
        if (length(f) == 1) 
            f <- rep(f, length(y))
        tmp <- lapply(1:(length(sigma) + 1), function(i) {
            if (i == 1) return(1 + exp(f - sigma[i]))
            if (i == (length(sigma) + 1))
                return(1 - 1/(1 + exp(f - sigma[i - 1])))
            return(1 / (1 + exp(f - sigma[i])) -
                   1 / (1 + exp(f - sigma[i - 1])))
        })
        loss <- log(tmp[[1]]) * (y == levels(y)[1])
        for (i in 2:nlevels(y))
            loss <- loss - log(tmp[[i]]) * (y == levels(y)[i])
        return(loss)
    }

    riskS <- function(delta, y, fit, w = 1)
        sum(w * plloss(y = y, f = fit, sigma = d2s(delta)))
    risk <- function(y, f, w = 1)
        sum(w * plloss(y = y, f = f, sigma = sigma))

    ngradient <- function(y, f, w = 1) {
        delta <<- optim(par = delta, fn = riskS, y = y,
                        fit = f, w = w, method = "BFGS")$par
        sigma <<- d2s(delta)
        if (length(f) == 1) 
            f <- rep(f, length(y))
        ng <- sapply(1:(length(sigma) + 1), function(i) {
            if (i > 1 & i < (length(sigma) + 1)) {
                ret <- (1 - exp(2 * f - sigma[i - 1] - sigma[i])) /
                   (1 + exp(f - sigma[i - 1]) +
                        exp(f - sigma[i]) +
                        exp(2 * f - sigma[i - 1] - sigma[i]))
            } else {
                if (i == 1) {
                    ret <- -1/(1 + exp(sigma[i] - f))
                } else {
                    ret <- 1 / (1 + exp(f - sigma[i - 1]))
                }
            }
            return(ret * (y == levels(y)[i]))
        })
        rowSums(ng)
    }

    offset <- function(y, w = 1) {
        optimize(risk, interval = offrange, y = y, w = w)$minimum
    }

    response <- function(f) {
        ret <- sapply(1:(length(sigma) + 1), function(i) {
            if (i == 1) return(1 / (1 + exp(f - sigma[i])))
            if (i == (length(sigma) + 1))
            return(1 - 1/(1 + exp(f - sigma[i - 1])))
            return(1 / (1 + exp(f - sigma[i])) -
               1 / (1 + exp(f - sigma[i - 1])))
            })
            ret
    }

    check_y <- function(y) {
        if (!is.ordered(y))
            stop("response must be an ordered factor")
        ## initialize thresholds:
        delta <<- seq(from = nuirange[1], to = nuirange[2],
                      length = nlevels(y) - 1)
        sigma <<- d2s(delta)
        y
    }

    Family(ngradient = ngradient,
           risk = risk, offset = offset,
           check_y = check_y,
           nuisance = function() return(sigma),
           response = response,
           rclass = function(f) apply(response(f), 1, which.max))
}

Weibull <- function(nuirange = c(0, 100)) {
    sigma <- 1

    plloss <- function(sigma, y, f){
        fw <- function(pred){
            exp(pred - exp(pred))
            }
        Sw <- function(pred){
            exp(-exp(pred))
            }
        time <- y[,1]
        ### log(0) won't fly
        time[time == 0] <- sort(unique(time))[2] / 10
        lnt <- log(time)
        Sevent <- y[,2]
        eta <- (lnt - f) / sigma
        - Sevent * log(fw(eta) / sigma) - (1 - Sevent) * log(Sw(eta))
        }

    riskS <- function(sigma, y, fit, w = 1)
        sum(w * plloss(y = y, f = fit, sigma = sigma))
    risk <- function(y, f, w = 1)
       sum(w * plloss(y = y, f = f, sigma = sigma))

    ngradient <- function(y, f, w = 1) {
        sigma <<- optimize(riskS, interval = nuirange,
                           y = y, fit = f, w = w)$minimum

        time <- y[,1]
        ### log(0) won't fly
        time[time == 0] <- sort(unique(time))[2] / 10
        lnt <- log(time)
        event <- y[,2]
        eta <- (lnt - f) / sigma
        (event * (exp(eta) - 1) + (1 - event) * exp(eta)) / sigma
        }

    Family(ngradient = ngradient, risk = risk,
           offset = function(y, w)
               optimize(risk, interval = c(0, max(log(y[,1]), na.rm = TRUE)),
                        y = y, w = w)$minimum,
           check_y = function(y) {
               if (!inherits(y, "Surv"))
                   stop("response is not an object of class ", sQuote("Surv"),
                        " but ", sQuote("family = Weibull()"))
               y
           },
           nuisance = function() return(sigma),
           name = "Negative Weibull Likelihood",
           response = function(f) exp(f))
}

Loglog <- function(nuirange = c(0, 100)) {
    sigma <- 1

    plloss <- function(sigma, y, f){
        fw <- function(pred){
            exp(pred) / (1 + exp(pred))^2
            }
        Sw <- function(pred){
            1 / (1 + exp(pred))
            }

        time <- y[,1]
        ### log(0) won't fly
        time[time == 0] <- sort(time)[2] / 10
        lnt <- log(time)
        Sevent <- y[,2]
        eta <- (lnt - f) / sigma
        - Sevent * log(fw(eta) / sigma) - (1 - Sevent) * log(Sw(eta))
        }

    riskS <- function(sigma, y, fit, w = 1)
        sum(w * plloss(y = y, f = fit, sigma = sigma))
    risk <- function(y, f, w = 1)
       sum(w * plloss(y = y, f = f, sigma = sigma))

    ngradient <- function(y, f, w = 1) {
        sigma <<- optimize(riskS, interval = nuirange,
                           y = y, fit = f, w = w)$minimum
        time <- y[,1]
        ### log(0) won't fly
        time[time == 0] <- sort(unique(time))[2] / 10
        lnt <- log(time)
        event <- y[,2]
        eta <- (lnt - f) / sigma
        nom <- (exp(-eta) + 1)
        (event * (2/nom - 1) + (1 - event) / nom) / sigma
        }

    Family(ngradient = ngradient, risk = risk,
           offset = function(y, w)
               optimize(risk, interval = c(0, max(log(y[,1]), na.rm = TRUE)),
                        y = y, w = w)$minimum,
           check_y = function(y) {
               if (!inherits(y, "Surv"))
                   stop("response is not an object of class ", sQuote("Surv"),
                        " but ", sQuote("family = Loglog()"))
               y
           },
           nuisance = function() return(sigma),
           name = "Negative Log Logistic Likelihood",
           response = function(f) exp(f))
}

Lognormal <- function(nuirange = c(0, 100)) {
    sigma <- 1

    plloss <- function(sigma, y, f){
        fw <- function(pred){
            dnorm(pred)
            }
        Sw <- function(pred){
            1 - pnorm(pred)
            }
        time <- y[,1]
        ### log(0) won't fly
        time[time == 0] <- sort(time)[2] / 10
        lnt <- log(time)
        Sevent <- y[,2]
        eta <- (lnt - f) / sigma
        - Sevent * log(fw(eta) / sigma) - (1 - Sevent) * log(Sw(eta))
        }

    riskS <- function(sigma, y, fit, w = 1)
        sum(w * plloss(y = y, f = fit, sigma = sigma))
    risk <- function(y, f, w = 1)
       sum(w * plloss(y = y, f = f, sigma = sigma))

    ngradient <- function(y, f, w = 1) {
        sigma <<- optimize(riskS, interval = nuirange,
                           y = y, fit = f, w = w)$minimum
        time <- y[,1]
        ### log(0) won't fly
        time[time == 0] <- sort(unique(time))[2] / 10
        lnt <- log(time)
        event <- y[,2]
        eta <- (lnt - f) / sigma
        (event * eta + (1 - event) * dnorm(eta) / (1 - pnorm(eta))) / sigma
        }

    Family(ngradient = ngradient, risk = risk,
           offset = function(y, w)
               optimize(risk, interval = c(0, max(log(y[,1]), na.rm = TRUE)),
                        y = y, w = w)$minimum,
           check_y = function(y) {
               if (!inherits(y, "Surv"))
                   stop("response is not an object of class ", sQuote("Surv"),
                        " but ", sQuote("family = Lognormal()"))
               y
           },
           nuisance = function() return(sigma),
           name = "Negative Lognormal Likelihood",
           response = function(f) exp(f))
}

ExpectReg <- function (tau = 0.5) {
    stopifnot(tau > 0 && tau < 1)
    Family(
        ngradient = function(y, f, w = 1)
            2 * tau * (y - f) * ((y - f) > 0) - 2 * (1 - tau) *
            (f - y) * ((y - f) < 0) + 0 * ((y - f) == 0),
        loss = function(y, f) tau * (y - f)^2 *
            ((y - f) >= 0) + (1 - tau) * (y - f)^2 * ((y - f) < 0),
        offset = function(y, w = rep(1, length(y)))
            mean(y[w == 1]),
        check_y = function(y) {
            if (!is.numeric(y) || !is.null(dim(y)))
                stop("response is not a numeric vector but ",
                  sQuote("family = ExpectReg()"))
            y
        },
        name = "Expectile Regression")
}

## (1 - AUC)-Loss
AUC <- function() {

	approxGrad <- function(x) {
		ifelse(abs(x) >= 1, 0, ifelse(x >= 0, (1 - x), (x + 1)))
	}
	approxLoss <- function(x) {
		ret <- ifelse(x >= 0, 1 - (1 - x)^2/2, (x + 1)^2/2)
		ifelse(x < -1, 0, ifelse(x > 1, 1, ret))
	}

	ind1 <- 0
	ind0 <- 0
	n0 <- 0
	n1 <- 0
	M0 <- matrix(0,0,0)


	Family(ngradient = function(y, f, w = 1) {
				# if there are weights!=1, observations have to be
				# repeated/omitted accordingly
				if (any(w != 1)) {
					ind1 <- which(rep(y, w) == 1)
					ind0 <- which(rep(y, w) == -1)
					n1 <- length(ind1)
					n0 <- length(ind0)
				}
				
	            if (length(f) == 1)
	                f <- rep(f, n1 + n0)
	            # skip this in the first iteration
	            if (length(unique(f)) != 1) 
	                # scale scores s.t. a gradient of zero makes sense for
	                # differences in f that are bigger than +/-1
	                f <- f/sd(f)
				
				M0 <<- (matrix(f[ind1], nrow = n0, ncol = n1, byrow = TRUE) -
							f[ind0])
				M1 <- approxGrad(M0)
				ng <- vector(n1 + n0, mode = "numeric")
				ng[ind1] <- colSums(M1)
				ng[ind0] <- rowSums(-M1)
				return(ng)
			}, loss = function(y, f, w = 1) {
				# if there are weights!=1, observations have to be
				# repeated/omitted accordingly and M0 must be recomputed
				if (any(w != 1)) {
					ind1 <- which(rep(y, w) == 1)
					ind0 <- which(rep(y, w) == -1)
					n1 <- length(ind1)
					n0 <- length(ind0)
					if (length(f) == 1)
					    f <- rep(f, n1 + n0)
					# skip this in the first iteration
					if (length(unique(f)) != 1) 
					    f <- f/sd(f)
					M0 <- (matrix(f[ind1], nrow = n0, ncol = n1, byrow = TRUE) -
								f[ind0])
				}
				# 1 - approxAUC
				1 - (sum(approxLoss(M0)))/(n1 * n0)
			}, risk = function(y, f, w = 1) {
				# if there are weights!=1, observations have to be
				# repeated/omitted accordingly and M0 must be recomputed
				if (any(w != 1)) {
					ind1 <- which(rep(y, w) == 1)
					ind0 <- which(rep(y, w) == -1)
					n1 <- length(ind1)
					n0 <- length(ind0)
					if (any(c(n0, n1) == 0)) {
						warning("response is constant - AUC is 1.")
						return(0)
					}
					M0 <- (matrix(f[ind1], nrow = n0, ncol = n1, byrow = TRUE) -
								f[ind0])
				}
				# 1 - AUC
				return(1 - (sum(as.numeric(M0 > 0)))/(n1 * n0))
			}, weights = "case",
			offset = function(y, w) {
				0
			}, check_y = function(y) {
				if (!is.factor(y))
					stop("response is not a factor but ", sQuote("family = AUC()"))
				if (nlevels(y) != 2)
					stop("response is not a factor at two levels but ",
							sQuote("family = AUC()"))
				if (length(unique(y)) != 2)
					stop("only one class is present in response.")
				# precompute to speed up ngradient (and warn about conflicting assignments)
				ind1 <<- which(y == levels(y)[2])
				ind0 <<- which(y == levels(y)[1])
				n1 <<- length(ind1)
				n0 <<- length(ind0)
				c(-1, 1)[as.integer(y)]
			},
      rclass = function(f) (f > 0) + 1,
      name = paste("(1 - AUC)-Loss") )
}



GammaReg <- function(nuirange = c(0, 100)) {
    sigma <- 1
    plloss <- function(sigma, y, f){
        lgamma(sigma) + sigma * y * exp(-f) - sigma * log(y) -
              sigma * log(sigma) + sigma * f
    }
    riskS <- function(sigma, y, fit, w = 1)
        sum(w * plloss(y = y, f = fit, sigma = sigma))
    risk <- function(y, f, w = 1)
       sum(w * plloss(y = y, f = f, sigma = sigma))
    ngradient <- function(y, f, w = 1) {
        sigma <<- optimize(riskS, interval = nuirange,
                           y = y, fit = f, w = w)$minimum
        sigma * y * exp(-f) - sigma
    }
    Family(ngradient = ngradient,
           risk = risk,
           offset = function(y, w){
               optimize(risk, interval = c(0, max(y^2, na.rm = TRUE)),
                        y = y, w = w)$minimum
           },
           check_y = function(y){
               if (!is.numeric(y) || !is.null(dim(y)))
                   stop("response is not a numeric vector but ",
                        sQuote("family = GammaReg()"))
               if (any(y < 0))
                   stop("response is not positive but ",
                        sQuote("family = GammaReg()"))
               y
           },
           nuisance = function() return(sigma),
           name = "Negative Gamma Likelihood",
           response = function(f) exp(f))
}

### Hinge Loss
HingeLoss <- function(alphaCost = 0.5) {
    if (alphaCost <= 0 | alphaCost >= 1 | !is.numeric(alphaCost) |
        length(alphaCost) != 1)
        stop("cost parameter must be a number within the range (0, 1)")
    Family(ngradient = function(y, f, w = 1){
               ngr <- ifelse(y > 0, 1 - alphaCost, alphaCost) * y
               ngr[(1 - y * f) < 0] <- 0
               return(ngr)
               },
           loss = function(y, f) ifelse(y > 0, 1 - alphaCost, alphaCost) *
               pmax(0, 1 - y * f),
           offset = function(y, w) {
               p <- weighted.mean(y > 0, w)
               1/2 * log(p / (1 - p))
           },
           check_y = function(y) {
               if (!is.factor(y))
                   stop("response is not a factor but ",
                           sQuote("family = HingeLoss()"))
               if (nlevels(y) != 2)
                   stop("response is not a factor at two levels but ",
                           sQuote("family = HingeLoss()"))
               c(-1, 1)[as.integer(y)]
           },
           rclass = function(f) (f > (2 * alphaCost - 1)) + 1,
           name = "Hinge Loss")
}

# Gehan family for 'mboost' package
#   Brent Johnson, with comments from Benjamin Hofner
#   Reference : Johnson BA and Long Q (2011) Survival ensembles by the sum of pairwise differences with application to
#		lung cancer microarray studies, Annals of Applied Statistics, 5:1081-1101.
Gehan <- function() {
    Family(ngradient = function(y, f, w) {
               Z <- as.vector(log(y[,1]))
               Del <- as.vector(y[,2])
               n1 <- length(Z)
               if(length(f)==1) f <- rep(f,n1)
               if(length(w)==1) w <- rep(w,n1)
               idx <- rep(1:n1,w)
               deltaSubset <- Del[idx]
               resSubset <- Z[idx] - f[idx]  		# e_j
               n2 <- length(deltaSubset)		# n2=sum(w)
               tmpg.fun <- function(I, res, del) {
                   ediff <- res[I] - res
                   arg2 <- sum(del * (ediff >= 0) )
                   arg1 <- ifelse(del[I] > 0.5, sum(ediff <= 0), 0)
                   arg1 - arg2
               }

               g <- rep(0,n1)		## here, the gradient is n1-by-1 with 0s for obs out of the Subset
               g[idx] <- unlist(lapply(1:n2, tmpg.fun, resSubset, deltaSubset))
               return(-1 * g / n2)      ## here, divide by n2=sum(w)
           },
           loss = gehan.loss <- function(y, f, w) {
               Z <- as.vector(log(y[,1]))
               Del <- as.vector(y[,2])
               n1 <- length(Z)
               if(length(f)==1) rep(f, n1)
               if(length(w)==1) rep(w, n1)
               idx <- rep(1:n1,w)
               deltaSubset <- Del[idx]
               resSubset <- Z[idx] - f[idx]
               n2 <- length(deltaSubset)
               tmpl.fun <- function(I, res, del) {
                   if(del[I] < 0.5) out1 <- 0
                   else {
                       ediff <- res[I] - res
                       out1 <- -1 * sum(ediff[ediff <= 0])
                   }
                   out1
               }
               out2 <- rep(0,n1)
               out2[idx] <- unlist(lapply(1:n2, tmpl.fun, resSubset, deltaSubset))/n2
               out2
           },
           risk = risk <- function(y, f, w = 1){
               sum(gehan.loss(y, f, w), na.rm=TRUE)
           },
           weights = "case",
           offset = function(y,w){
               optimize(risk,
                        interval = c(0, max(log(y[,1]), na.rm=TRUE)), y = y, w = w)$minimum
           },
           check_y = function(y) {
               if (!inherits(y,"Surv"))
                   stop("response is not an object of class ",
                        sQuote("Surv"), " but ",
                        sQuote("family = Gehan()"))
               y
           },
           name = "Gehan loss")
}


### Hurdle model, negative binomial non-zero component with log link
Hurdle <- function(nuirange = c(0, 100)){
    sigma <- 1
    plloss <- function(sigma, y, f)
                  - (y * (f - log(1 / sigma + exp(f))) -
                  (log(1 / sigma + exp(f)) + log(sigma)) / sigma +
                  lgamma(y + 1 / sigma) - lgamma(y + 1) - lgamma(1 / sigma) -
                  log(1 - (1 + exp(f) * sigma)^{-1 / sigma}))
    riskS <- function(sigma, y, fit, w = 1)
                 sum(w * plloss(y = y, f = fit, sigma = sigma))
    risk <- function(y, f, w = 1) sum(w * plloss(y = y, f = f, sigma = sigma))
    ngradient = function(y, f, w = 1){
                    sigma <<- optimize(riskS, interval = nuirange, y = y,
                                       fit = f, w = w)$minimum
                    y / sigma / (exp(f) + 1 / sigma) - 1 / sigma /
                    (exp(-f) / sigma + 1) - exp(f) * (1 + exp(f) *
                    sigma)^{-1 / sigma - 1} / (1 - (1 + exp(f) *
                    sigma)^{-1 / sigma})
                    }
    Family(ngradient = ngradient, risk = risk, 
           offset = function(y, w) log(weighted.mean(y, w)),
           check_y = function(y) {
               stopifnot(all.equal(unique(y - floor(y)), 0))
               y
           }, 
           nuisance = function() return(sigma),
           name = "Hurdle model, negative binomial non-zero part",
           response = function(f) exp(f))
}

### multinomial logit model
### NOTE: this family can't be applied out-of-the box
### the rhs for formula needs to read
### ~ bl1 %O% bl0 + bl2 %O% bl1 + ...
### where bl1 is some linear baselearner
### and bl0 is a baselearner with design and penalty term
### equal to the unit matrix for number of levels - 1
### See ?Multinom
Multinomial <- function() {
    lev <- NULL
    biny <- function(y) {
        if (!is.factor(y))
            stop("response is not a factor but ",
                  sQuote("family = Multinomial()"))
        lev <<- levels(y)
        as.vector(model.matrix(~ y - 1)[,-length(lev)])
    }
    return(Family(ngradient = function(y, f, w = 1) {
               if (length(f) != length(y))
                   stop("predictor doesn't correspond to multinomial logit model; see ?Multinomial")
               f <- pmin(abs(f), 36) * sign(f)
               p <- matrix(exp(f), ncol = length(lev) - 1)
               p <- as.vector(p / (1 + rowSums(p)))
               y - p
           },
           loss = function(y, f) {
               f <- pmin(abs(f), 36) * sign(f)
               p <- matrix(exp(f), ncol = length(lev) - 1)
               p <- as.vector(p / (1 + rowSums(p)))
               -y * log(p)
           },
           offset = function(y, w) {
               return(rep(0, length(y)))
           },
           response = function(f) {
               f <- pmin(abs(f), 36) * sign(f)
               p <- matrix(exp(f), ncol = length(lev) - 1)
               p <- cbind(p, 1) / (1 + rowSums(p))
               colnames(p) <- lev
               return(p)
           },
           rclass = function(f) {
               f <- pmin(abs(f), 36) * sign(f)
               p <- matrix(exp(f), ncol = length(lev) - 1)
               p <- cbind(p, 1) / (1 + rowSums(p))
               apply(p, 1, which.max)
           },
           check_y = biny,
           name = "Negative Multinomial Likelihood"))
}

### Cindex family to fit the concordance index for survival data
### The family is based on an estimator proposed by Uno et al. which
### incorporates "inverse probability of censoring weights" which can be 
### either computed inside the family (based on all observations with 
### non-zero weights) or computed before fitting and passed as an argument 
### ("ipcw"), which could be better in cases of cross-validation or resampling.


Cindex <- function (sigma = 0.1, ipcw = 1) {
  
  approxGrad <- function(x) {    ## sigmoid function for gradient
    exp(x/sigma) / (sigma * (1 + exp(x/sigma))^2) 
  }
  
  approxLoss <- function(x) { ## sigmoid function for loss
    1 / (1 + exp(x / sigma))
  }
  
  compute_weights <- function(y, w = 1, ipcw = rep(1,nrow(y))){  ## compute weights
    
    if(all(ipcw == 1))
    {  
      ipcw_wow <- IPCweights(y[w != 0,])
      ipcw <- numeric(nrow(y))
      ipcw[w!=0] <- ipcw_wow
    }
    
    survtime <- y[,1]
    n <- nrow(y)
    wweights <- matrix( (ipcw)^2, nrow = n, ncol = n)
    weightsj <- matrix(survtime, nrow = n, ncol = n) 
    weightsk <- matrix(survtime, nrow = n, ncol = n, byrow = TRUE) 
    weightsI <- ifelse(weightsj == weightsk, .5, (weightsj < weightsk) + 0) - diag(.5, n,n)
    wweights <- wweights * weightsI
    Wmat <- w %o% w
    wweights <- wweights * Wmat
    wweights <- wweights / sum(wweights)
    rm(weightsI); rm(weightsk); rm(weightsj)
    return(wweights)
  }
  
  ngradient = function(y, f, w = 1) {   ## negative gradient
    if (!all(w %in% c(0,1)))
      stop(sQuote("weights"), " must be either 0 or 1 for family ",
           sQuote("UnoC"))
    
    survtime <- y[,1]
    event <- y[,2]
    n <- length(survtime)
    
    if (length(w) == 1) 
        w <- rep(1, n)
    if (length(f) == 1)
        f <- rep(f, n)
    
    etaj <- matrix(f, nrow = n, ncol = n, byrow = TRUE)
    etak <- matrix(f, nrow = n, ncol = n)
    etaMat <- etak - etaj
    rm(etaj); rm(etak);
    
    weights_out <- compute_weights(y, w, ipcw = ipcw)
    M1 <- approxGrad(etaMat) * weights_out
    ng <- colSums(M1) - rowSums(M1)
    
    return(ng)
  }
  
  risk = function(y, f, w = 1) { ## empirical risk
    survtime <- y[,1]
    event <- y[,2]
    n <- length(survtime)
    
    if (length(f) == 1)
      f <- rep(f, n)
    
    etaj <- matrix(f, nrow = n, ncol = n, byrow = TRUE)
    etak <- matrix(f, nrow = n, ncol = n)
    etaMat <- (etak - etaj)
    rm(etaj); rm(etak);
    weights_out <- compute_weights(y, w, ipcw = ipcw )
    M1 <- approxLoss(etaMat) * weights_out
    return(- sum(M1))
  }
  
  Family(   ## build the family object
    ngradient = ngradient,
    risk = risk,
    weights = "zeroone",
    offset = function(y, w = 1) {0},
    check_y = function(y) {
      if (!inherits(y, "Surv"))
        stop("response is not an object of class ", sQuote("Surv"),
             " but ", sQuote("family = UnoC()"))
      y},
    rclass = function(f){},
    name = paste("Concordance Probability by Uno"))
}


# RCG model for bounded response variables, log link
RCG <- function (nuirange = c(0, 1), offrange = c(-5, 5)) {
    sigma <- c(1, 0.2)
    plloss <- function(sigma, y, f, w = 1) {
        alpha <- sigma[1]
        rho <- sigma[2]
        if (length(f) == 1)
            f <- rep(f, length(y))
        my <- 1 + (exp(f) - 1) * y
        dmy <- my ^ 2 - 4 * rho * exp(f) * y * (1 - y)
        fy <- lgamma(2 * alpha) - 2 * lgamma(alpha) + alpha * f + 
            alpha * log(1 - rho) + log(my) + (alpha - 1) * log(y * (1 - y)) - 
            (alpha + 0.5) * log(dmy)
        return(-fy)
    }
    riskS <- function(sigma, y, fit, w = 1)
        sum(w * plloss(y = y, f = fit, sigma = sigma))
    risk <- function(y, f, w = 1)
        sum(w * plloss(y = y, f = f, sigma = sigma))
    ngradient <- function(y, f, w = 1) {
        sigma <<- optim(par = sigma, fn = riskS, y = y, fit = f,
            lower = c(0.001, 0.001), upper = c(100, 0.99),
            w = w, method = "L-BFGS-B")$par
        if (length(f) == 1)
            f <- rep(f, length(y))
        alpha <- sigma[1]
        rho <- sigma[2]
        my <- 1 + (exp(f) - 1) * y
        dmy <- my ^ 2 - 4 * rho * exp(f) * y * (1 - y)
        return(alpha + (y * exp(f)) / my - (2 * alpha + 1) * exp(f) *
                   y * (my - 2 * rho * (1 - y)) / dmy)
    }
    offset <- function(y, w = 1)
        0
    dRCG <- function(f, sigma) {
        alpha <- sigma[1]
        rho <- sigma[2]
        densw <- function(w, alpha, rho, f)
            w * gamma(2 * alpha) / (gamma(alpha)) ^ 2 * (1 - rho) ^ alpha *
            (exp(f))^alpha * ((exp(f) - 1) * w + 1) * (w * (1 - w))^(alpha - 1) /
            ((((exp(f) - 1) * w + 1) ^ 2 - 4 * rho * exp(f) * w * (1 - w)) ^ (alpha + 0.5))
        res <- rep(0, length(f))
        for (k in 1:length(f))
            res[k] <- integrate(densw, lower = 0, upper = 1, alpha, rho, f[k])$value
        return(res)
    }
    response <- function(f, sigma)
        dRCG(f, sigma)
    Fisher <- function(y, alpha, rho, X, coefs) {
        X <- data.frame(X)
        f.i <- matrix(0, ncol = length(coefs), nrow = length(coefs))
        for (i in 1:length(y)) {
            wobs <- as.numeric(y[i])
            x <- as.matrix(X[i,])
            xt <- t(x)
            xtx <- xt %*% x
            exb <- as.numeric(exp(x %*% coefs))
            D1 <- (exb - 1) * wobs + 1
            D2 <- D1 ^ 2 - 4 * rho * exb * wobs * (1 - wobs)
            f.i <- f.i + (D1 * wobs * exb - (wobs * exb) ^ 2) * xtx / (D1 ^ 2) -
                (alpha + 0.5) / (D2 ^ 2) * 
                (2 * wobs * exb * xtx * (2 * wobs * exb - wobs + 1 - 2 * rho * (1 - wobs)) 
                 * D2 - xtx * (2 * wobs * exb * (D1 - 2 * rho * (1 - wobs)))^2)
        }
        Fisher.information <- -f.i
        return(Fisher.information)
    }
    
    Family(
        ngradient = ngradient,
        risk = risk,
        offset = offset,
        # Fisher = Fisher,
        check_y = function(y) {
            if(!all(y < 1 & y > 0))
                stop("y must be < 1 and > 0")
            y
        },
        nuisance = function()
            return(sigma),
        response = function(f)
            response(f, sigma = sigma),
        rclass = function(f)
            f,
        name = "Ratio of Correlated Gammas (RCG)"
    )
}

brad <- function(..., by = NULL, index = NULL, knots = 100, df = 4, lambda = NULL,
                 covFun = fields::stationary.cov,
                 args = list(Covariance = "Matern", smoothness = 1.5, theta = NULL)) {

    cll <- match.call()
    cll[[1]] <- as.name("brad")

    mf <- list(...)

    if (length(mf) == 1 && (is.matrix(mf[[1]]) || is.data.frame(mf[[1]]))) {
        mf <- as.data.frame(mf[[1]])
    } else {
        mf <- as.data.frame(mf)
        cl <- as.list(match.call(expand.dots = FALSE))[2][[1]]
        colnames(mf) <- sapply(cl, function(x) deparse(x))
    }
    stopifnot(is.data.frame(mf))
    if(!(all(sapply(mf, is.numeric)))) {
        if (ncol(mf) == 1) return(bols(..., by = by, index = index))
        stop("cannot compute brad for non-numeric variables")
    }
    ### use bols when appropriate
    ## <FIXME>
    #if (!is.null(df)) {
    #    if (df <= ncol(mf))
    #        return(bols(..., by = by, index = index))
    #}
    ## </FIXME>
    vary <- ""
    if (!is.null(by)){
        stopifnot(is.numeric(by) || (is.factor(by) && nlevels(by) == 2))
        mf <- cbind(mf, by)
        colnames(mf)[ncol(mf)] <- vary <- deparse(substitute(by))
    }

    CC <- all(Complete.cases(mf))
    if (!CC)
        warning("base-learner contains missing values;\n",
                "missing values are excluded per base-learner, ",
                "i.e., base-learners may depend on different",
                " numbers of observations.")
    ### option
    DOINDEX <- (nrow(mf) > 10000)
    if (is.null(index)) {
        if (!CC || DOINDEX) {
            index <- get_index(mf)
            mf <- mf[index[[1]],,drop = FALSE]
            index <- index[[2]]
        }
    }

    ret <- list(model.frame = function()
                if (is.null(index)) return(mf) else return(mf[index,,drop = FALSE]),
                get_call = function(){
                    cll <- deparse(cll, width.cutoff=500L)
                    if (length(cll) > 1)
                        cll <- paste(cll, collapse="")
                    cll
                },
                get_data = function() mf,
                get_index = function() index,
                get_vary = function() vary,
                get_names = function() colnames(mf),
                set_names = function(value) {
                    if(length(value) != length(colnames(mf)))
                        stop(sQuote("value"), " must have same length as ",
                             sQuote("colnames(mf)"))
                    for (i in 1:length(value)){
                        cll[[i+1]] <<- as.name(value[i])
                    }
                    attr(mf, "names") <<- value
                })
    class(ret) <- "blg"

    ret$dpp <- bl_lin(ret, Xfun = X_brad,
                      args = hyper_brad(mf, vary, knots = knots,
                      df = df, lambda = lambda, covFun = covFun, args = args))
    return(ret)
}


### model.matrix for kriging base-learners
X_brad <- function(mf, vary, args) {
    stopifnot(is.data.frame(mf))

    INDX <- which(colnames(mf) != vary)
    #if (length(INDX) != 2)
    #    stop("only bivariate kriging implemented atm.")
    args$args$x1 <- mf[INDX]
    args$args$x2 <- args$knots
    X <- do.call(args$covFun, args$args)
    args$args$x1 <- args$knots
    PEN <- do.call(args$covFun, args$args)
    e <- eigen(PEN)
    PEN_sqrt_INV <- solve(e$vectors %*% diag(sqrt(e$values)) %*% t(e$vectors))
    X <- X %*% PEN_sqrt_INV

    K <- diag(ncol(X))
    ### <FIXME>
    if (vary != "") {
        by <- model.matrix(as.formula(paste("~", vary, collapse = "")), data = mf)[,2]
        X <- X * by
    }
    ### </FIXME>
    attr(X, "knots") <- args$knots
    return(list(X = X, K = K))
}

### hyper parameters for kriging base-learners
hyper_brad <- function(mf, vary, knots = 100, df = 4, lambda = NULL,
                       covFun = covFun, args = args) {

    ## first we need to build a correct matrix of mf
    x <- as.matrix(mf[which(colnames(mf) != vary)])
    if (length(knots) == 1) {
        if (!requireNamespace("fields"))
            stop("Cannot load package", sQuote("fields"),
                 ", which is needed for the automatic knot placement")
        knots <- fields::cover.design(R = unique(x), nd = knots)$design
    }
    if ("theta" %in% names(args) && is.null(args$theta)){
        ## (try to) compute effective range
        args$theta <- effective_range(x, eps = 0.001, interval = c(0.1, 100),
                                      covFun = covFun, args = args)
    }
    if (is.list(knots))
        stop(sQuote("knots"), " must be an integer defining the number of knots",
             " or a matrix specifiying the location of the knots")
    list(knots = knots, pen = TRUE, df = df, lambda = lambda, covFun = covFun,
         args = args)
}

######
# theta: effective range (per default theta such that
#       rho( max(x_(i) - x_(j)), smoothness, theta = max(x_(i) - x_(j))/c ) = 0.001
#  <==> rho(c, smoothness, theta = 1) = 0.001
effective_range <- function(x, eps = 0.001, interval = c(0.1, 100),
                            covFun = fields::stationary.cov, args = list()){

    if ( !( length(deparse(covFun)) == length(deparse(fields::stationary.cov))
           && all(deparse(covFun) == deparse(fields::stationary.cov)) ) &&
         !( length(deparse(covFun)) == length(deparse(fields::Exp.cov))
           && all(deparse(covFun) == deparse(fields::Exp.cov)) ) ){
        ## if cov.funcion is not one of stationary.cov and Exp.cov
        warning(sQuote("effective_range()"), " is only implemented for ",
                sQuote("stationary.cov"), " and ", sQuote("Exp.cov"),
                " from package ", sQuote("fields"))
        return(NULL)
    }

    args$theta <- 1
    args$x1 <- 0
    maxX <- max(dist(x))
    rho <- function(cval, eps){
        args$x2 <- cval
        # make a call to covFun with the corresponding arguments
        return(c(do.call(covFun, args) - eps))
    }
    cval <- uniroot(rho, interval = interval, eps = eps)$root
    RET <- maxX/cval
    attr(RET, "c_value") <- cval
    return(RET)
}

options(digits = 3)

require("mboost")
if (require("partykit")) {

set.seed(290875)

tst <- try(data("BostonHousing", package = "mlbench"))
if (!inherits(tst, "try-error")) {

    system.time(a <- blackboost(medv ~ ., data = BostonHousing,
        tree_controls = ctree_control(teststat = "max",
                                testtype = "Teststatistic",
                                mincriterion = 0,
                                maxdepth = 2),
        control = boost_control(mstop = 500)))

    print(ae <- mean((predict(a) - BostonHousing$medv)^2))

    pdiffs <- max(abs(predict(update(a, model.weights(a))) - predict(a)))
    print(pdiffs < sqrt(.Machine$double.eps))


    ### attach `gbm', quietly
    sink("tmpfile")
    if (require("gbm")) cat()
    sink()
    file.remove("tmpfile")

    if (require("gbm")) {
        system.time(b <- gbm(medv ~ ., data = BostonHousing,
            n.trees = 500, interaction = 2, distribution = "gaussian",
            shrinkage = 0.1, bag = 1))
    print(be <- mean((predict(b, newdata = BostonHousing, n.trees = 500) -
                BostonHousing$medv)^2))
    plot(BostonHousing$medv, predict(a), col = "red", pch = "+")
    points(BostonHousing$medv,
           predict(b, newdata = BostonHousing, n.trees = 500),
           col = "blue", pch = "+")
    print(ae < be)
    }
}

### with by-argument, a certain type of interaction
tctrl <- ctree_control(teststat = "max",
                       testtype = "Teststatistic",
                       mincriterion = 0,
                       maxdepth = 2)
bb <- mboost(medv ~ btree(crim, zn, indus, nox, age) + 
                    btree(crim, zn, indus, nox, age, by = chas), 
             data = BostonHousing)
print(isTRUE(all.equal(fitted(bb)[1:10], 
                    c(predict(bb, newdata = BostonHousing[1:10,])), 
                    check.attributes = FALSE)))
nd <- BostonHousing[1:10,]
nd$chas[] <- "0"
p0 <- predict(bb, newdata = nd, which = 1)
print(isTRUE(all.equal(c(unique(predict(bb, newdata = nd, which = 2))), 0, 
                           check.attributes = FALSE)))
nd$chas[] <- "1"
p1 <- predict(bb, newdata = nd, which = 1)
print(isTRUE(all.equal(p0, p1)))
print(predict(bb, newdata = nd, which = 2))
print(table(selected(bb)))
print(table(selected(bb[50])))

### check different interfaces
x <- as.matrix(BostonHousing[,colnames(BostonHousing) != "medv"])
y <- BostonHousing$medv
p2 <- predict(blackboost(medv ~ ., data = BostonHousing, family = Laplace()),
              newdata = BostonHousing)

## Cox model
library("survival")

fit2 <- blackboost(Surv(futime,fustat) ~ age + resid.ds + rx + ecog.ps,
    data = ovarian, family = CoxPH(), control = boost_control(mstop = 1000))

A2 <- survFit(fit2)
print(A2)

newdata <- ovarian[c(1,3,12),]
A2 <- survFit(fit2, newdata = newdata)
print(A2)

### predictions:
set.seed(1907)
x1 <- rnorm(100)
x2 <- rnorm(100)
x3 <- rnorm(100)
y <- rnorm(100, mean = 3 * x1, sd = 2)
DF <- data.frame(y = y, x1 = x1, x2 = x2, x3 = x3)

amod <- blackboost(y ~ -1 + x1 + x2, data = DF)
agg <- c("none", "sum", "cumsum")
whi <- list(NULL, 1)
for (i in 1:2){
    pred <- vector("list", length=3)
    for (j in 1:3){
        pred[[j]] <- predict(amod, aggregate=agg[j], which = whi[[i]])
    }
    if (i == 1){
        print(max(abs(pred[[2]] - pred[[3]][,ncol(pred[[3]])]))  < sqrt(.Machine$double.eps))
        if ((pred[[2]] - rowSums(pred[[1]]))[1] - amod$offset < sqrt(.Machine$double.eps))
            warning(sQuote("aggregate = sum"), " adds the offset, ", sQuote("aggregate = none"), " doesn't.")
        print(max(abs(pred[[2]] - rowSums(pred[[1]]) - amod$offset))   < sqrt(.Machine$double.eps))
    } else {
        print(max(abs(pred[[2]] - sapply(pred[[3]], function(obj) obj[,ncol(obj)])))  < sqrt(.Machine$double.eps))
        print(max(abs(pred[[2]] - sapply(pred[[1]], function(obj) rowSums(obj))))  < sqrt(.Machine$double.eps))
    }
}

print(all(predict(amod, which=1) + amod$offset  - predict(amod) < sqrt(.Machine$double.eps)))


# check type argument
set.seed(1907)
x1 <- rnorm(100)
p <- 1/(1 + exp(- 3 * x1))
y <- as.factor(runif(100) < p)
DF <- data.frame(y = y, x1 = x1)

mod <- blackboost(y ~ x1, family = Binomial(),
                  data = DF,  control=boost_control(mstop=5000))

pr <- predict(mod)
pr <- predict(mod, type="class")
foo <- table(pr, y)
print(foo[1,2] + foo[2,1] == 0)
pr <- predict(mod, type="response")
# <FIXME> How do we check "correctness" of results?</FIXME>

}

options(digits = 3)

.all.equal <- function(...) isTRUE(all.equal(..., check.environment = FALSE))

require("mboost")
require("survival")

set.seed(290875)

### for boosting hat matrix checks
fm <- Gaussian()
fm@offset <- function(y, w) 0

### a simple two-dimensional example from `gamboost.Rd'
data("cars")
cars.gb <- gamboost(dist ~ speed, data = cars, df = 4, family = fm,
                    control = boost_control(mstop = 50))
cars.gb
aic <- AIC(cars.gb, method = "corrected")
aic

aic2 <- AIC(cars.gb, method = "corrected", df = "actset")
print(.all.equal(aic, aic2))

ht <- hatvalues(cars.gb)

## extract residual
print(.all.equal(resid(cars.gb), cars.gb$resid()))
par(mfrow = c(1, 2))
plot(cars$speed, resid(cars.gb[0]))
cars$resid <- resid(cars.gb)
lines(cars$speed, predict(loess(resid ~ speed, cars)), col = "red")
plot(cars$speed, resid(cars.gb[50]))
cars$resid <- resid(cars.gb)
lines(cars$speed, predict(loess(resid ~ speed, cars)), col = "red")

## check return = FALSE
print(is.null(cars.gb[10, return = FALSE]))
print(mstop(cars.gb) == 10)

### plot fit
plot(dist ~ speed, data = cars)
lines(cars$speed, predict(cars.gb[mstop(AIC(cars.gb))]), col = "red")
lines(cars$speed, predict(smooth.spline(cars$speed, cars$dist), cars$speed)$y,
      col = "green")

## print model summary
summary(cars.gb)

#### check boosting hat matrix and subsetting / predict
print(isTRUE(.all.equal(drop(attr(ht, "hatmatrix") %*% cars$dist),
                           as.vector(predict(cars.gb[50])))))
ht25 <- hatvalues(cars.gb[25])
print(isTRUE(.all.equal(drop(attr(ht25, "hatmatrix") %*% cars$dist),
                           as.vector(predict(cars.gb[25])))))
print(isTRUE(.all.equal(drop(attr(ht25, "hatmatrix") %*% cars$dist),
                           as.vector(fitted(cars.gb[25])))))

### check boosting hat matrix with multiple independent variables
### and weights
data("bodyfat", package = "TH.data")
bffm <- DEXfat ~ age + waistcirc + hipcirc + elbowbreadth + kneebreadth +
      anthro3a + anthro3b + anthro3c + anthro4
indep <- names(bodyfat)[names(bodyfat) != "DEXfat"]
bodyfat[indep] <- lapply(bodyfat[indep], function(x) x - mean(x))
bf_gam <- gamboost(bffm, data = bodyfat, control = boost_control(mstop = 10),
                   weights = runif(nrow(bodyfat)) * 10)
### aic <- AIC(bf_gam)
ht <- hatvalues(bf_gam)

off <- bf_gam$offset
u <- bf_gam$ustart

print(isTRUE(.all.equal(drop(attr(ht, "hatmatrix") %*% u + off),
                           as.vector(predict(bf_gam)))))
print(isTRUE(.all.equal(drop(attr(ht, "hatmatrix") %*% u + off),
                           as.vector(fitted(bf_gam)))))


### compare `gamboost' with `lm' in cases where this is actually possible
set.seed(290875)
x <- matrix(runif(1000) * 10, ncol = 10)
xf <- gl(4, nrow(x)/4)

### OK, we need to allow for some small differences (larger mstop values
### would fix this)
stopin <- function(x, y) print(max(abs(x - y)) < 0.1)

### univariate linear model
df <- data.frame(y = 3*x[,2], x = x)
ga <- gamboost(y ~ x.2, data = df,
               control = boost_control(mstop = 500, nu = 1))
stopin(fitted(lm(y ~ x.2 - 1, data = df)), fitted(ga))

### univariate model involving sin transformation
df <- data.frame(y = sin(x[,1]), x = x)
ga <- gamboost(y ~ x.1, data = df,
               control = boost_control(mstop = 500, nu = 1))
stopin(fitted(lm(y ~ sin(x.1) - 1, data = df)), fitted(ga))

### bivariate model: linear and sin
df <- data.frame(y = sin(x[,1]) + 3*x[,2], x = x)
ga <- gamboost(y ~ x.1 + x.2, data = df,
               control = boost_control(mstop = 500, nu = 1))
stopin(fitted(lm(y ~ sin(x.1) + x.2 - 1, data = df)), fitted(ga))
ga <- gamboost(y ~ x.1 + bols(x.2), data = df,
               control = boost_control(mstop = 500, nu = 1))
stopin(fitted(lm(y ~ sin(x.1) + x.2 - 1, data = df)), fitted(ga))

### ANCOVA model
df <- data.frame(y = 3 * x[,2] + (1:4)[xf], x = x)
ga <- gamboost(y ~ xf + x.2, data = df,
               control = boost_control(mstop = 500, nu = 1))
stopin(fitted(lm(y ~ xf + x.2 - 1, data = df)), fitted(ga))
ga <- gamboost(y ~ xf + sin(x.1) + x.2, data = df,
               control = boost_control(mstop = 500, nu = 1))
stopin(fitted(lm(y ~ xf + sin(x.1) + x.2, data = df)), fitted(ga))

### check centering
y <- rnorm(200)
xn <- rnorm(200)
xnm <- xn - mean(xn)
xf <- gl(2, 100)
gc <- gamboost(y ~ xn + xf)
g <- gamboost(y ~ xnm + xf)
print(max(abs(fitted(gc) - fitted(g))) < 1 / 10000)

pc1 <- predict(gc)
pc2 <- predict(gc, newdata = data.frame(xn = xn, xf = xf))
pc3 <- predict(g)
print(.all.equal(pc1, pc2))
print(max(abs(pc2 - pc3)) < 1 / 10000)

### formula interfaces
tmp <- data.frame(x1 = runif(100), x2 = runif(100), y = rnorm(100))
fm1 <- y ~ bbs(x1, df = 3) + bbs(x2, df = 3)
fm2 <- y ~ x1 + x2
mod1 <- gamboost(fm1, data = tmp)
mod2 <- gamboost(fm2, data = tmp, base = "bss", dfbase = 3)
print(max(abs(fitted(mod1) - fitted(mod2))) < sqrt(.Machine$double.eps))
print(max(abs(predict(mod1, newdata = tmp) - predict(mod2, newdata = tmp))) < sqrt(.Machine$double.eps))

fm1 <- y ~ bbs(x1, df = 3) + bbs(x2, df = 3)
fm2 <- y ~ x1 + x2
mod1 <- gamboost(fm1, data = tmp)
mod2 <- gamboost(fm2, data = tmp, base = "bbs", dfbase = 3)
print(max(abs(fitted(mod1) - fitted(mod2)))  < sqrt(.Machine$double.eps))
print(max(abs(predict(mod1, newdata = tmp) - predict(mod2, newdata = tmp)))  < sqrt(.Machine$double.eps))

fm1 <- y ~ bols(x1) + bols(x2)
fm2 <- y ~ x1 + x2
mod1 <- gamboost(fm1, data = tmp)
mod2 <- gamboost(fm2, data = tmp, base = "bols")
print(max(abs(fitted(mod1) - fitted(mod2)))  < sqrt(.Machine$double.eps))
print(max(abs(predict(mod1, newdata = tmp) - predict(mod2, newdata = tmp)))  < sqrt(.Machine$double.eps))

fm1 <- y ~ btree(x1) + btree(x2)
fm2 <- y ~ x1 + x2
mod1 <- gamboost(fm1, data = tmp)
mod2 <- gamboost(fm2, data = tmp, base = "btree")
print(max(abs(fitted(mod1) - fitted(mod2)))  < sqrt(.Machine$double.eps))
print(max(abs(predict(mod1, newdata = tmp) - predict(mod2, newdata = tmp)))  < sqrt(.Machine$double.eps))

## Cox model

fit2 <- gamboost(Surv(futime, fustat) ~ bbs(age) +
    bols(resid.ds) + bols(rx) + bols(ecog.ps), data = ovarian,
    family = CoxPH(), control = boost_control(mstop = 1000))

A2 <- survFit(fit2)
A2

newdata <- ovarian[c(1,3,12),]
A2 <- survFit(fit2, newdata = newdata)
A2

### gamboost with explicit intercept
df <- data.frame(x = 1:100, y = rnorm(1:100), int = rep(1, 100))
mod <- gamboost(y ~ bols(int, intercept = FALSE) + bols(x, intercept = FALSE), data = df,
                control = boost_control(mstop = 2500))
cf <- unlist(coef(mod))
cf[1] <- cf[1] + mod$offset
tmp <- max(abs(cf - coef(lm(y ~ x, data = df))))
print(tmp < 1e-5)
tmp <- max(abs(fitted(mod) - fitted(lm(y ~ x, data = df))))
print(tmp < 1e-5)

### predictions:
data("bodyfat", package = "TH.data")
amod <- gamboost(DEXfat ~ hipcirc + anthro3a, data = bodyfat, baselearner = "bbs")

agg <- c("none", "sum", "cumsum")
whi <- list(NULL, 1, 2, c(1,2))
for (i in 1:4){
    pred <- vector("list", length=3)
    for (j in 1:3){
        pred[[j]] <- predict(amod, aggregate=agg[j], which = whi[[i]])
    }
    if (i == 1){
        print(max(abs(pred[[2]] - pred[[3]][,ncol(pred[[3]])]))  < sqrt(.Machine$double.eps))
        if ((pred[[2]] - rowSums(pred[[1]]))[1] - attr(coef(amod), "offset") < sqrt(.Machine$double.eps))
            warning(sQuote("aggregate = sum"), " adds the offset, ", sQuote("aggregate = none"), " doesn't.")
        print(max(abs(pred[[2]] - rowSums(pred[[1]]) - attr(coef(amod), "offset")))   < sqrt(.Machine$double.eps))
    } else {
        print(max(abs(pred[[2]] - sapply(pred[[3]], function(obj) obj[,ncol(obj)])))  < sqrt(.Machine$double.eps))
        print(max(abs(pred[[2]] - sapply(pred[[1]], function(obj) rowSums(obj))))  < sqrt(.Machine$double.eps))
    }
}

# use names in which instead of numbers
agg <- c("none", "sum", "cumsum")
whi <- list(NULL, "hipcirc", "anthro3a", c("hipcirc", "anthro3a"))
for (i in 1:4){
    pred <- vector("list", length=3)
    for (j in 1:3){
        pred[[j]] <- predict(amod, aggregate=agg[j], which = whi[[i]])
    }
    if (i == 1){
        print(max(abs(pred[[2]] - pred[[3]][,ncol(pred[[3]])]))  < sqrt(.Machine$double.eps))
        if ((pred[[2]] - rowSums(pred[[1]]))[1] - attr(coef(amod), "offset") < sqrt(.Machine$double.eps))
            warning(sQuote("aggregate = sum"), " adds the offset, ", sQuote("aggregate = none"), " doesn't.")
        print(max(abs(pred[[2]] - rowSums(pred[[1]]) - attr(coef(amod), "offset")))   < sqrt(.Machine$double.eps))
    } else {
        print(max(abs(pred[[2]] - sapply(pred[[3]], function(obj) obj[,ncol(obj)])))  < sqrt(.Machine$double.eps))
        print(max(abs(pred[[2]] - sapply(pred[[1]], function(obj) rowSums(obj))))  < sqrt(.Machine$double.eps))
    }
}

# use which to extract all effects for one covariate e.g. for plotting purposes
amod <- gamboost(DEXfat ~ bols(hipcirc, intercept=FALSE) + bbs(hipcirc, df = 1, center=TRUE), data = bodyfat)
print(ncol(predict(amod, which="hip")) == 2 && all(rowSums(predict(amod, which="hip")) + attr(coef(amod), "offset") - predict(amod) < sqrt(.Machine$double.eps)))

amod <- gamboost(DEXfat ~ hipcirc + anthro3a + kneebreadth,
                 data = bodyfat, baselearner = "bbs")
pr1 <- predict(amod, aggre = "sum", which= 1:2)
foo <- bodyfat[,names(bodyfat) %in% c("hipcirc", "anthro3a", "kneebreadth")]
foo$kneebreadth <- mean(bodyfat$kneebreadth)
pr2 <- predict(amod, aggre = "sum", newdata=foo)
print(all(diff(rowSums(pr1) - pr2) < sqrt(.Machine$double.eps))) # changes in level are ok
newData <- as.data.frame(rbind(colMeans(bodyfat)[-2], colMeans(bodyfat)[-2]+1*sapply(bodyfat, sd)[-2]))
if (!is.list(pr <- predict(amod, newdata=newData, which=1:2)))
    warning("predict(amod, newdata=newData, which=1:2) does not return a list") # no list but a matrix is returned!
print(is.list(pr <- predict(amod, newdata=newData, aggregate="cumsum", which=1:2)))
amod[10]
pr <- predict(amod, which=1:3)
print(ncol(pr) == 3 || all(pr[,ncol] == 0))
amod[100]

# check type argument
set.seed(1907)
x1 <- rnorm(100)
p <- 1/(1 + exp(- 3 * x1))
y <- as.factor(runif(100) < p)
DF <- data.frame(y = y, x1 = x1)

logitBoost <- gamboost(y ~ x1, family = Binomial(),
                 data = DF, baselearner = "bols", control=boost_control(mstop=5000))
logit <- glm(y ~ x1, data=DF, family=binomial)
print(coef(logitBoost)[[1]][2]*2 - coef(logit)[2]  < sqrt(.Machine$double.eps)) # * 2 as we use y = {-1, 1}

pr <- predict(logitBoost)
pr2 <- predict(logit)
print(pr * 2 - pr2 < 1e-5)  # * 2 as we use y = {-1, 1}

pr <- predict(logitBoost, type="class")
pr2 <- predict(logit, type="response") > 0.5
foo <- table(pr, pr2)
print(foo[1,2] + foo[2,1] == 0)

pr <- predict(logitBoost, type="response")
pr2 <- predict(logit, type="response")
print(pr - pr2  < sqrt(.Machine$double.eps))

### coefficients:
data("bodyfat", package = "TH.data")
amod <- gamboost(DEXfat ~ hipcirc + anthro3a + kneebreadth,
                 data = bodyfat, baselearner = "bbs")
print(length(coef(amod)) == 3)
amod[10]
print(length(coef(amod)) == 2)
print(length(coef(amod, which=1:3)) == 3)

### cyclic covariates
x <- seq(from = 0, to = 2*pi, length = 100)
y <- sin(x) + rnorm(length(x), sd = 0.5)
mod <- gamboost(y ~ bbs(x, cyclic = TRUE))
print(diff(fitted(mod)[c(1, 100)]) == 0)

### buser
set.seed(1907)
x <- rnorm(100)
y <- rnorm(100, mean = x^2, sd = 0.1)
mod1 <- gamboost(y ~ bbs(x))
X <- extract(bbs(x))
K <- extract(bbs(x), "penalty")
mod2 <- gamboost(y ~ buser(X, K))
print(max(abs(predict(mod1) - predict(mod2))) < sqrt(.Machine$double.eps))

z <- sample(1:2, 100, replace=TRUE)
y[z == 2] <- rnorm(100, mean = - x^2, sd = 0.1)[z == 2]
z <- as.factor(z)
mod3 <- gamboost(y ~ bbs(x) + bbs(x, by = z),
                 control = boost_control(mstop = 1000))
X <- extract(bbs(x))
K <- extract(bbs(x), "penalty")
mod4 <- gamboost(y ~  buser(X, K) + buser(X, K, by = z),
                 control = boost_control(mstop = 1000))
print(max(abs(predict(mod3) - predict(mod4))) < sqrt(.Machine$double.eps))

y <- rnorm(100, mean = as.numeric(z), sd = 0.1)
mod5 <- gamboost(y ~ bols(z))
X <- extract(bols(z))
K <- extract(bols(z), "penalty")
index <- extract(bols(z), "index")
mod6 <- gamboost(y ~  buser(X, K, lambda = 0, index = index))
mod6a <- gamboost(y ~  buser(X, index = index))
print(max(abs(predict(mod5) - predict(mod6))) < sqrt(.Machine$double.eps))
print(max(abs(predict(mod5) - predict(mod6a))) < sqrt(.Machine$double.eps))

z <- sample(1:3, 100, replace = TRUE)
y <- rnorm(100, mean = z, sd = 0.1)
z <- as.ordered(z)
mod7 <- gamboost(y ~ bols(z))
X <- extract(bols(z))
K <- extract(bols(z), "penalty")
index <- extract(bols(z), "index")
mod8 <- gamboost(y ~  buser(X, K, lambda = 0, index = index))
print(max(abs(predict(mod7) - predict(mod8))) < sqrt(.Machine$double.eps))

y[z == 1] <- rnorm(100, mean = x^2, sd = 0.1)[z == 1]
y[z == 2] <- rnorm(100, mean = - x^2, sd = 0.1)[z == 2]
y[z == 3] <- rnorm(100, mean = x, sd = 0.1)[z == 3]
mod9 <- gamboost(y ~ bbs(x) + bbs(x, by = z),
                 control = boost_control(mstop = 1000))
X <- extract(bbs(x))
K <- extract(bbs(x), "penalty")
mod10 <- gamboost(y ~  buser(X, K) + buser(X, K, by = z),
                 control = boost_control(mstop = 1000))
print(max(abs(predict(mod9) - predict(mod10))) < sqrt(.Machine$double.eps))


## test that mstop = 0 is possible
compare_models <- function (m1, m2) {
    print(.all.equal(coef(m1), coef(m2)))
    print(.all.equal(predict(m1), predict(m2)))
    print(.all.equal(fitted(m1), fitted(m2)))
    print(.all.equal(as.vector(residuals(m1)), as.vector(residuals(m2))))
    print(.all.equal(selected(m1), selected(m2)))
    print(.all.equal(risk(m1), risk(m2)))
    ## remove obvious differences from objects
    m1$control <- m2$control <- NULL
    m1$call <- m2$call <- NULL
    if (!.all.equal(m1, m2))
        stop("Objects of offset model + 1 step and model with 1 step not identical")
    invisible(NULL)
}
# set up models
mod <- mboost(DEXfat ~ bbs(age) + bols(waistcirc) + bbs(hipcirc),
              data = bodyfat, control = boost_control(mstop = 0))
mod2 <- mboost(DEXfat ~ bbs(age) + bols(waistcirc) + bbs(hipcirc),
               data = bodyfat, control = boost_control(mstop = 1))
mod3 <- mboost(DEXfat ~ bbs(age) + bols(waistcirc) + bbs(hipcirc),
               data = bodyfat, control = boost_control(mstop = 1))
print(is.null(coef(mod)))
print(predict(mod) == rep(mod$offset, nrow(bodyfat)))
print(fitted(mod) == rep(mod$offset, nrow(bodyfat)))
print(.all.equal(residuals(mod), bodyfat$DEXfat - mean(bodyfat$DEXfat)))
print(is.null(selected(mod)))
print(.all.equal(risk(mod), risk(mod2)[1]))

mstop(mod3) <- 0
compare_models(mod, mod3)
mstop(mod) <- 1
compare_models(mod, mod2)
mstop(mod3) <- 1
compare_models(mod, mod3)

## same with glmboost
mod <- glmboost(DEXfat ~ age + waistcirc + hipcirc,
                data = bodyfat, control = boost_control(mstop = 0))
mod2 <- glmboost(DEXfat ~ age + waistcirc + hipcirc,
                 data = bodyfat, control = boost_control(mstop = 1))
mod3 <- glmboost(DEXfat ~ age + waistcirc + hipcirc,
                 data = bodyfat, control = boost_control(mstop = 1))
print(is.null(coef(mod)))
print(predict(mod) == rep(mod$offset, nrow(bodyfat)))
print(fitted(mod) == rep(mod$offset, nrow(bodyfat)))
print(.all.equal(residuals(mod), bodyfat$DEXfat - mean(bodyfat$DEXfat), check.attributes = FALSE))
print(is.null(selected(mod)))
print(.all.equal(risk(mod), risk(mod2)[1]))

mstop(mod3) <- 0
compare_models(mod, mod3)
mstop(mod) <- 1
compare_models(mod, mod2)
mstop(mod3) <- 1
compare_models(mod, mod3)

### just a try
plot.mboost <- function(x, which = NULL, newdata = NULL,
                        type = "b", rug = TRUE, rugcol = "black", ylim = NULL,
                        xlab = NULL, ylab = expression(f[partial]), add = FALSE,
                        ...) {

    if (inherits(x, "blackboost"))
        stop("partial dependency plots for ", sQuote("blackboost"), " not yet implemented.",
             "See ?blackboost.")
    
    which <- x$which(which, usedonly = is.null(which))
    

    if (is.null(xlab)){
        userspec.xlab <- FALSE
        xlab <- variable.names(x)
    } else {
        userspec.xlab <- TRUE
        if (length(which) == length(xlab) & length(which) != length(variable.names(x))){
            ## if labels are user specified but not all base-learners are selected
            foo <- rep(NA, length(variable.names(x)))
            foo[which] <- xlab ## xlab is supposed to be in the same ordering as which
            xlab <- foo
        }
    }
    
    if (missing(ylab)) {
        userspec.ylab <- FALSE
    } else {
        userspec.ylab <- TRUE
        if (length(which) == length(ylab) & length(which) != length(variable.names(x))){
            ## if labels are user specified but not all base-learners are selected
            foo <- rep(NA, length(variable.names(x)))
            foo[which] <- ylab ## ylab is supposed to be in the same ordering as which
            ylab <- foo
        }
    }
    
    ## labs must have either length 1 or length(variable.names(x))
    stopifnot(length(xlab) %in% c(1, length(variable.names(x))))
    stopifnot(length(ylab) %in% c(1, length(variable.names(x))))
    ## FIXED?

    RET <- NULL
    n_levelplot <- 0

    for (w in which) {

        data <- model.frame(x, which = w)[[1]]
        get_vary <- x$baselearner[[w]]$get_vary
        vary <- ""
        if (!is.null(get_vary))
            vary <- get_vary()
        if (!is.null(newdata)) {
            data <- newdata[colnames(data)]
            if (is.list(data))
                data <- as.data.frame(data)
        }
        if (vary != "") {
            v <- data[[vary]]
            if (is.factor(v)) v <- factor(levels(v)[-1], levels = levels(v))
            if (is.numeric(v)) v <- 1
        }

        plot_helper <- function(xl, yl){
            pr <- predict(x, newdata = data, which = w)
            if (is.null(ylim)) ylim <- range(pr, na.rm = TRUE)

            if (vary != "") {
                datavary <- data[, colnames(data) == vary, drop = FALSE]
                data <- data[, colnames(data) != vary, drop = FALSE]
            }
            if (ncol(data) == 1) {
                if (!add) {
                    if (is.factor(data[[1]])) {
                        xVals <- unique(sort(data[[1]]))
                        xValsN <- as.numeric(xVals)
                        ## make sure that we get the same number of values as in
                        ## x; this is only a problem if pr is equal for
                        ## different x values.
                        yVals <- unique(cbind(pr[order(data[[1]], na.last = NA)],
                                              sort(data[[1]])))[, 1]

                        if (length(pr) == 1 && pr == 0) {
                            yVals <- rep(0, length(xVals))
                        }
                        plot(xValsN, yVals,
                             type = "n", xaxt = "n",
                             xlim = range(as.numeric(xVals)) + c(-0.5, 0.5),
                             xlab = xl, ylab = yl, ylim = ylim)
                        ## make sure that xaxt / axes is respected
                        print.xaxis <- TRUE
                        if (length(list(...)) >= 1 && any(xaxis <- c("xaxt", "axes") %in% names(list(...)))) {
                            if (xaxis[1] && list(...)$xaxt == "n")
                                print.xaxis <- FALSE
                            if (xaxis[2] && list(...)$axes == FALSE)
                                print.xaxis <- FALSE
                        }
                        if (print.xaxis)
                            axis(1, at = xValsN, labels = levels(xVals))
                        for (i in 1:length(xVals)) {
                            lines(x = rep(xValsN[i], 2) + c(-0.35, 0.35),
                                  y = rep(yVals[i], 2), ...)
                        }
                    } else {
                        plot(sort(data[[1]]), pr[order(data[[1]], na.last = NA)], type = type,
                             xlab = xl, ylab = yl, ylim = ylim, ...)
                    }
                    if (rug) rug(data[[1]], col = rugcol)
                } else {
                    if (is.factor(data[[1]])){
                        xVals <- unique(sort(data[[1]]))
                        xValsN <- as.numeric(xVals)
                        yVals <- unique(pr[order(data[[1]], na.last = NA)])
                        if (length(pr) == 1 && pr == 0) {
                            yVals <- rep(0, length(xVals))
                        }
                        for (i in 1:length(xVals)) {
                            lines(x = rep(xValsN[i], 2) + c(-0.35, 0.35),
                                  y = rep(yVals[i], 2), ...)
                        }
                    } else {
                        lines(sort(data[[1]]), pr[order(data[[1]], na.last = NA)], type =
                              type, ...)
                        if (rug){
                            rug(data[[1]], col = rugcol)
                            warning(sQuote("rug = TRUE"),
                                    " should be used with care if ",
                                    sQuote("add = TRUE"))
                        }
                    }
                }
            }
            n_levelplot <<- n_levelplot + (ncol(data) == 2)
            if (ncol(data) == 2 && n_levelplot == 1) {
                if (is.null(newdata)){
                    tmp <- expand.grid(unique(data[,1]), unique(data[,2]))
                    colnames(tmp) <- colnames(data)
                    data <- tmp
                    if (vary != "") {
                        ## datavary contains only 1 value, thus use only first
                        ## entry and use recycling to get appropriate vector
                        tmp <- cbind(datavary[1,], tmp)
                        colnames(tmp)[1] <- vary
                    }
                    pr <- predict(x, newdata = tmp, which = w)
                }
                fm <- as.formula(paste("pr ~ ", paste(colnames(data), collapse = "*"), sep = ""))
                if (!userspec.xlab)
                    xl <- colnames(data)[1]
                if (!userspec.ylab)
                    yl <- colnames(data)[2]
                RET <<- levelplot(fm, data = data, xlab = xl, ylab = yl, ...)
            }
            if (ncol(data) > 2) {
                for (v in colnames(data)) {
                    tmp <- data
                    pardep <- sapply(data[[v]], function(vv) {
                        tmp[[v]] <- vv
                        mean(predict(x, newdata = tmp, which = w))
                    })
                    if (!userspec.xlab)
                        xl <- v
                    plot(sort(data[[v]]), pardep[order(data[[v]], na.last = NA)], type = type,
                         xlab = xl, ylab = yl, ylim = ylim, ...)
                }
            }
        } # END plot_helper()

        xl <- ifelse(length(xlab) > 1, xlab[w], xlab[1])
        yl <- ifelse(length(ylab) > 1, ylab[w], ylab[1])

        if (add && length(which) > 1)
            warning(sQuote("add = TRUE"),
                    " should be used for single plots only")

        if (!isMATRIX(data)){
            if (vary == "") plot_helper(xl, yl)
            if (vary != ""){
                for (i in 1:length(v)){
                    data[[vary]] <- v[rep(i, nrow(data))]
                    if (!userspec.xlab){
                        ## xlab not user specified
                        plot_helper(paste(xl, "=", v[i]), yl)
                    } else {
                        plot_helper(xl, yl)
                    }
                }
            }
        } else {
            warning(paste(extract(x, what = "bnames", which = w),
                          ": automated plot not reasonable for base-learners",
                          " of matrices", sep=""))
        }
    }
    if (n_levelplot > 0){
        if (n_levelplot == 1)
            return(RET)
        stop(paste("Cannot plot multiple surfaces via levelplot;",
             "Plot base-learners seperately via plot(model, which = ...)!"))
    }
}


lines.mboost <- function(x, which = NULL, type = "l", rug = FALSE, ...){
    plot(x, which = which, type = type, add = TRUE, rug = rug, ...)
}


plot.glmboost <- function(x, main = deparse(x$call), col = NULL,
                          off2int = FALSE, ...) {

    cp <- coef(x, aggregate = "cumsum", off2int = off2int)
    ncp <- names(cp)
    cp <- matrix(unlist(cp), nrow = length(cp), byrow = TRUE)
    cf <- cp[, ncol(cp)]
    if (is.null(col))
        col <- hcl(h = 40, l = 50, c= abs(cf) / max(abs(cf)) * 490)
    matplot(t(cp), type = "l", lty = 1, xlab = "Number of boosting iterations",
            ylab = "Coefficients", main = main, col = col, ...)
    abline(h = 0, lty = 1, col = "lightgray")
    axis(4, at = cf, labels = ncp, las = 1)
}

## stabsel method for mboost; requires stabs
stabsel.mboost <- function(x, cutoff, q, PFER, grid = 0:mstop(x),
                    folds = subsample(model.weights(x), B = B),
                    B = ifelse(sampling.type == "MB", 100, 50),
                    assumption = c("unimodal", "r-concave", "none"),
                    sampling.type = c("SS", "MB"),
                    papply = mclapply, verbose = TRUE, FWER, eval = TRUE, ...) {

    cll <- match.call()
    p <- length(variable.names(x))
    ibase <- 1:p

    sampling.type <- match.arg(sampling.type)
    if (sampling.type == "MB")
        assumption <- "none"
    else
        assumption <- match.arg(assumption)

    if (!is.null(folds)) {
        if (!is.null(B)) {
            if (ncol(folds) != B)
                warning("B should be equal to number of folds, i.e., ncol(folds). B was set to ncol(folds)")
        }
        B <- ncol(folds)
    } else {
        folds <- subsample(model.weights(x), B = B)
    }

    pars <- stabsel_parameters(p = p, cutoff = cutoff, q = q,
                               PFER = PFER, B = B,
                               verbose = verbose, sampling.type = sampling.type,
                               assumption = assumption, FWER = FWER)
    ## return parameter combination only if eval == FALSE
    if (!eval)
        return(pars)

    cutoff <- pars$cutoff
    q <- pars$q
    PFER <- pars$PFER

    fun <- function(model) {
        xs <- selected(model)
        qq <- sapply(1:length(xs), function(x) length(unique(xs[1:x])))
        xs[qq > q] <- xs[1]
        xs
    }
    if (sampling.type == "SS") {
        ## use complementary pairs
        folds <- cbind(folds, model.weights(x) - folds)
    }
    
    dots <- list(...)
    if (length(dots) >= 1 && "mstop" %in% names(dots)) {
        mstop <- dots$mstop
        if (!missing(grid)) {
            ## if grid and mstop were specified check if equivalent (and be tolerant), issue an error otherwise
            if (!isTRUE(all.equal(grid, 0:mstop, check.attributes = FALSE)))
                stop("Please specify only one of grid (preferred) or mstop")
        } 
        grid <- 0:mstop
        dots$mstop <- NULL
    } else {                              
        if (!isTRUE(all.equal(grid, 0:max(grid), check.attributes = FALSE)))
            stop("grid must be of the form 0:m, i.e., starting at 0 with increments of 1")
    }
    
    ss <- do.call(cvrisk, c(list(object = x, fun = fun, folds = folds, papply = papply,
                                 grid = grid), dots))
    
    if (verbose){
        qq <- sapply(ss, function(x) length(unique(x)))
        sum_of_violations <- sum(qq < q)
        if (sum_of_violations > 0)
            warning(sQuote("mstop"), " too small in ",
                    sum_of_violations, " of the ", ncol(folds),
                    " subsampling replicates to select ", sQuote("q"),
                    " base-learners; Increase ", sQuote("mstop"),
                    " bevor applying ", sQuote("stabsel"))
    }


    ## extract mstop from grid
    m <- max(grid)
    ret <- matrix(0, nrow = length(ibase), ncol = m)
    for (i in 1:length(ss)) {
        tmp <- sapply(ibase, function(x)
            ifelse(x %in% ss[[i]], which(ss[[i]] == x)[1], m + 1))
        ret <- ret + t(sapply(tmp, function(x) c(rep(0, x - 1), rep(1, m - x + 1))))
    }

    phat <- ret / length(ss)
    rownames(phat) <- names(variable.names(x))
    if (extends(class(x), "glmboost"))
        rownames(phat) <- variable.names(x)
    ret <- list(phat = phat, selected = which((mm <- apply(phat, 1, max)) >= cutoff),
                max = mm, cutoff = cutoff, q = q, PFER = PFER, p = p, B = B, 
                sampling.type = sampling.type, assumption = assumption,
                call = cll)
    ret$call[[1]] <- as.name("stabsel")
    class(ret) <- c("stabsel_mboost", "stabsel")
    ret
}

stabsel_parameters.mboost <- function(p, ...) {
    stabsel(p, ..., eval = FALSE)
}

confint.mboost <- function(object, parm = NULL, level = 0.95,
                           B = 1000, B.mstop = 25, newdata = NULL,
                           which = parm,
                           papply = ifelse(B.mstop == 0, mclapply, lapply),
                           cvrisk_options = list(), ...) {

    which <- object$which(which, usedonly = FALSE)
    if (!all(which %in% object$which(NULL, usedonly = FALSE)))
        stop(sQuote("which"), " is wrongly specified")

    if (!is.list(cvrisk_options))
        stop(sQuote("cvrisk_options"), " must be a named list")
    if (length(cvrisk_options) > 0 && is.null(names(cvrisk_options)))
        stop(sQuote("cvrisk_options"), " must be a named list")
    if ("folds" %in% names(cvrisk_options))
        stop("One cannot modify the folds of the inner bootstrap")
    if ("object" %in% names(cvrisk_options))
        stop("One cannot specify the model (object) of the inner bootstrap")

    ## create new data and/or restructure data
    newdata <- .create_newdata(object, newdata, which)

    outer.folds <- cv(model.weights(object), B = B)

    cat("Start computing bootstrap confidence intervals... \n")

    do_update <- function(i) {
        #for (i in 1:B) {
        cat("\rB =", i)
        ## update model
        mod <- update(object, weights = outer.folds[, i],
                      risk = "inbag", trace = FALSE)
        if (B.mstop > 0) {
            ## <FIXME> are the weights handled correctly?
            cvr <- do.call("cvrisk",
                           args = c(list(object = mod,
                                       folds = cv(model.weights(mod), B = B.mstop)),
                                       cvrisk_options))
            mod[mstop(cvr)]
        }
        .predict_confint(mod, newdata = newdata, which = which)
    }
    predictions <- papply(1:B, do_update, ...)
    cat("\n")

    ## prepare returned object
    res <- list(level = level, boot_pred = predictions, data = newdata,
                model = object)
    attr(res, "which") <- which
    class(res) <- "mboost.ci"
    return(res)
}

confint.glmboost <- function(object, parm = NULL, level = 0.95,
                             B = 1000, B.mstop = 25,
                             which = parm, ...) {

    outer.folds <- cv(model.weights(object), B = B)
    which <- object$which(which, usedonly = FALSE)
    if (!all(which %in% object$which(NULL, usedonly = FALSE)))
        stop(sQuote("which"), " is wrongly specified")

    coefficients <- matrix(NA, ncol = length(which), nrow = B)
    colnames(coefficients) <- names(coef(object, which = which))

    for (i in 1:B) {
        ## update model
        mod <- update(object, weights = outer.folds[, i],
                      risk = "inbag")
        if (B.mstop > 0) {
            ## <FIXME> are the weights handled correctly?
            cvr <- cvrisk(mod, folds = cv(model.weights(mod), B = B.mstop), ...)
            mod[mstop(cvr)]
        }
        coefficients[i, ] <- unlist(coef(mod, which = which, off2int = TRUE))
    }

    ## prepare returned object
    res <- list(confint = .ci_glmboost(coefficients, level = level, which = which),
                level = level, boot_coefs = coefficients, model = object)
    attr(res, "which") <- which
    class(res) <- "glmboost.ci"
    return(res)
}

.ci_glmboost <- function(coefficients, level, which = NULL) {
    quantiles <- c((1 - level)/2, 1 - (1 - level)/2)

    tmp <- apply(coefficients, 2, FUN = quantile, probs = quantiles)
    CI <- as.data.frame(t(tmp))[which, ]
    return(CI)
}

## pe = add point estimte
print.glmboost.ci <- function(x, which = NULL, level = x$level, pe = FALSE, ...) {

    if (is.null(which)) {
        which <- attr(x, "which")
    } else {
        which <- x$model$which(which, usedonly = FALSE)
        if (!all(which %in% attr(x, "which")))
            stop(sQuote("which"), " is wrongly specified")
    }

    if (!is.null(level) && level != x$level) {
        CI <- .ci_glmboost(x$boot_coefs,  level = level, which = which)
    } else {
        CI <- x$confint[which, ]
    }

    if (pe) {
        tmp <- data.frame(beta = coef(x$model, which))
        CI <- cbind(tmp, CI)
    }
    if (length(which) > 1) {
        cat("\tBootstrap Confidence Intervals\n")
    } else {
        cat("\tBootstrap Confidence Interval\n")
    }
    print(CI, ...)
    return(invisible(x))
}


## FIXME: Aditionally needed: Does multivariate bols base-learners work correctly?
.create_newdata <- function(object, newdata = NULL, which, ...) {
    if (is.null(newdata)) {
        data <- newdata <- model.frame(object, which = which)
        nms <- names(object$baselearner)[which]

        for (w in which) {
            ## get data from w-th base-learner
            tmp <- data[[w]][rep(1, 100), , drop = FALSE]

            ## are there varying coefficients (i.e. argument by)
            get_vary <- object$baselearner[[w]]$get_vary
            vary <- ""
            if (!is.null(get_vary)) vary <- get_vary()
            if (vary != "") {
                if (is.factor(tmp[[vary]])) {
                    if (nlevels(tmp[[vary]]) > 2)
                        stop("Atomatic data creation for ", sQuote("by"),
                             " variables with more than two levels is",
                             " currently not supported;",
                             " Specify ", sQuote("newdata"), " instead.")
                    data[[w]][[vary]] <- factor(levels(data[[w]][[vary]])[-1],
                                                levels = levels(data[[w]][[vary]]))
                }
                if (is.numeric(tmp[[vary]]))
                    data[[w]][[vary]] <- 1
            }

            ## now make grid
            grid <- function(x) {
                if (is.numeric(x)) {
                    return(seq(min(x), max(x), length = 100))
                } else {
                    return(rep(unique(x), length.out = 100))
                }
            }

            for (j in 1:ncol(data[[w]]))
                tmp[, colnames(data[[w]])[j]] <- grid(data[[w]][,j])

            ## FIXME: what about btree and bmrf?

            ## check if any base-learner is a bivariate smooth effect, i.e. if
            ## base-learner is multivariate and bbs, bspatial or brad
            which.vary <- colnames(tmp) == vary
            multivar <- grepl("bbs|bspatial|brad", nms[w]) &
                ncol(tmp[!which.vary]) >= 2
            if (multivar) {
                ## make grid
                egrid <- expand.grid(tmp[!which.vary])
                if (vary != "") {
                    x.vary <- tmp[which.vary]
                    rownames(x.vary) <- NULL
                    tmp <- cbind(egrid, x.vary)
                } else {
                    tmp <- egrid
                }
            }
            ## reset rownames
            rownames(tmp) <- NULL
            newdata[[w]] <- tmp
        }
    } else {
        ## restructure new data
        data <- model.frame(object, which = which)
        nms <- lapply(data, colnames)
        tmp <- lapply(nms, function(x) newdata[, x, drop = FALSE])
        newdata <- tmp
    }
    return(newdata)
}

## special prediction function for the construction of confidence intervals:
.predict_confint <- function(object, newdata = NULL, which, ...) {
    nrows <- sapply(newdata, nrow)
    predictions <- sapply(which, function(w)
                          matrix(predict(object, newdata[[w]], which = w),
                                               ncol = 1, nrow = nrows[w]))
    if (is.matrix(predictions))
        predictions <- as.data.frame(predictions)
    names(predictions) <- names(newdata[which])
    return(predictions)
}


### plot functions
plot.mboost.ci <- function(x, which, level = x$level,
                           ylim = NULL, type = "l", col = "black",
                           ci.col = rgb(170, 170, 170, alpha = 85,
                                        maxColorValue = 255),
                           raw = FALSE, print_levelplot = TRUE, ...) {

    if (missing(which)) {
        which <- attr(x, "which")
    } else {
        which <- x$model$which(which, usedonly = FALSE)
        if (!all(which %in% attr(x, "which")))
            stop(sQuote("which"), " is wrongly specified")
    }
    if (length(which) > 1)
        stop("Specify a single base-learner using ", sQuote("which"))

    CI <- .ci_mboost(x$boot_pred, level = level, which = which, raw = raw)

    ## check if data (without by variable, which is not varying in the plot
    ## data) has more than one column
    varying <- which(sapply(x$data[[which]], function(x) length(unique(x))) > 1)
    if (ncol(x$data[[which]]) > 1 && length(varying) > 1) {

        if (length(varying) > 2)
            stop("Plots are only implemented for up to 2 variables.")

        p1 <- plot(x$model, which = which, newdata = x$data[[which]],
                   main = "Mean surface", ...)
        ## make level plots for upper and lower CI
        fm <- as.formula(paste("pr ~ ", paste(names(varying), collapse = "*"), sep = ""))
        pr <- CI[1, ]  ## lower CI
        p2 <- levelplot(fm, data = x$data[[which]], main = paste(rownames(CI)[1], "CI surface"), ...)
        pr <- CI[2, ]  ## upper CI
        p3 <- levelplot(fm, data = x$data[[which]], main = paste(rownames(CI)[2], "CI surface"), ...)
        if (print_levelplot) {
                   ## position = left, bottom, right, top
            print(p1, position=c(0, 0, 0.33, 1), more=TRUE)
            print(p2, position=c(0.33, 0, 0.66, 1), more=TRUE)
            print(p3, position=c(0.66, 0, 1, 1))
            warning("The scale is not the same")
        } else {
            return(list(mean = p1, lowerCI = p2, upperCI = p3))
        }
    } else {
        if (is.null(ylim)) {
            ylim <- range(CI)
        }
        plot(x$model, which = which, newdata = x$data[[which]], rug = FALSE,
             type = "n", ylim = ylim, col = col, ...)
        lines(x, which, level, col = ci.col, raw = raw, ...)
        lines(x$model, which = which, newdata = x$data[[which]], rug = FALSE,
              type = "l", col = col, ...)
    }
}

lines.mboost.ci <- function(x, which, level = x$level,
                            col = rgb(170, 170, 170, alpha = 85,
                                      maxColorValue = 255),
                            raw = FALSE, ...) {

    if (missing(which)) {
        which <- attr(x, "which")
    } else {
        which <- x$model$which(which, usedonly = FALSE)
        if (!all(which %in% attr(x, "which")))
            stop(sQuote("which"), " is wrongly specified")
    }
    if (length(which) > 1)
        stop("Specify a single base-learner using ", sQuote("which"))


    CI <- .ci_mboost(x$boot_pred, level = level, which = which, raw = raw)

    x.data <- x$data[[which]]
    ## check if data (without by variable, which is not varying in the plot
    ## data) has more than one column
    if (ncol(x.data) > 1 &&
        sum(sapply(x.data, function(x) length(unique(x))) > 1) > 1) {
        stop("Cannot plot lines for more than 1 dimension")
    } else {
        x.data <- x.data[, 1]
    }

    if (is.factor(x.data)) {
        if (raw)
            warning("plotting raw values is currently not implemented",
                    " for factors")
        pData <- cbind(x.data, t(CI))
        pData <- unique(pData)
        for (i in 1:nrow(pData)) {
            polygon(x = pData[i, 1] + c(-0.35, 0.35, 0.35, -0.35),
                    y = rep(pData[i, 2:3], each = 2),
                    col = col, border = col, ...)
        }
    } else {
        if (!raw) {
            polygon(c(x.data, rev(x.data)),
                    c(CI[1, ], rev(CI[2,])),
                    col = col, border = col)
        } else {
            matlines(x$data[[which]], CI, col = col, lty = "solid", ...)
        }
    }
}

.ci_mboost <- function(predictions, level, which = NULL, raw = FALSE) {

    preds <- sapply(predictions, function(p) p[[which]])
    if (!raw) {
        quantiles <- c((1 - level)/2, 1 - (1 - level)/2)
        preds <- apply(preds, 1, FUN = quantile, probs = quantiles)
    }

    return(preds)
}

bkernel <- function(..., df = 4, lambda = NULL,
                    kernel = c("lin", "sia", "net"),
                    pathway = NULL, knots = NULL, args = list()) {

    if (!requireNamespace("kangar00", quietly = TRUE))
        stop("Please install package 'kangar00' for using bkernel base-learners.")

    cll <- match.call()
    cll[[1]] <- as.name("bkernel")

    mf <- list(...)
    if (length(mf) > 1 || !inherits(mf[[1]], "GWASdata"))
        stop(sQuote("..."), " must be a single object of class ",
             sQuote("GWASdata"), " as defined in package 'kangar00'")
    mf <- mf[[1]]

    kernel <- match.arg(kernel)

    ret <- list(model.frame = function() mf,
                get_call = function(){
                    cll <- deparse(cll, width.cutoff=500L)
                    if (length(cll) > 1)
                        cll <- paste(cll, collapse="")
                    cll
                },
                get_data = function() mf,
                get_index = function() NULL,
                get_vary = function() "",
### <FIXME> This now returns the name of the GWASdata object and not the column
###         names of mf@geno.
                get_names = function() as.character(cll[2]),
                set_names = function(value) {
                    if(length(value) != 1)
                        stop(sQuote("value"), " must have same length 1.")
                    cll[[2]] <<- as.name(value)
                })
## </FIXME>
    class(ret) <- "blg"

    ret$dpp <- bl_lin(ret, Xfun = X_kernel,
                      args = hyper_kernel(df = df, lambda = lambda,
                                          kernel = kernel, pathway = pathway,
                                          knots = if(!is.null(knots)) knots else mf,
                                          args = args))
    return(ret)
}

## - Think about storing all but X (e.g. (ANA')X') in args?
##   As we have different kernels, we would need to return this from
##   calc_kernel(), then we can use this.

## <FIXME> Slot @anno should be an hyper argument

### model.matrix for kernel base-learners
X_kernel <- function(mf, vary, args) {

    if (length(args$args) > 0)
        stop("further arguments are curently not implemented")

    ## compute kernel with arguments args$args
    X <- kangar00::calc_kernel(object = mf,
                               type = args$kernel,
                               pathway = args$pathway,
                               knots = args$knots)@kernel
    
    ## compute penalty matrix for prediction or if X matrix is not quadratic
    if (args$prediction || nrow(X) != ncol(X)) {
        K <- kangar00::calc_kernel(object = args$knots,
                                   type = args$kernel,
                                   pathway = args$pathway,
                                   knots = args$knots)@kernel
    } else {
        K <- X
    }
    ## make sure K is positive semidefinit
    K <- make_psd(K)

    ## compute inverse of the square root of the penalty matrix (always use knots)
    e <- eigen(K)
    if (any(e$values < 0)) {
        warning("Kernel should be positiv semidefinit, yet some eigenvalues are < 0.\n",
                "These eigenvalues were set to 0.")
        e$values[e$values < 0] <- 0
    }

    ## t(e$vectors) can be used instead of solve(e$vectors) as K is symmetric
    sqrtK <- e$vectors %*% diag(sqrt(e$values)) %*% t(e$vectors)
    sqrtK <- forceSymmetric(sqrtK)
    ## compute inverse of sqrt(K):
    ## chol2inv works for symmetric, posdef matrices and is much quicker than
    ## solve. Note that its argument is chol(...).
    chol_sqrtK <- try(chol(sqrtK), silent = TRUE)
    if (inherits(chol_sqrtK, "try-error")) {
        PEN_sqrt_INV <- solve(sqrtK)
    } else {
        PEN_sqrt_INV <- chol2inv(chol_sqrtK)
    }

    X <- X %*% PEN_sqrt_INV
    K <- diag(ncol(X))
    return(list(X = X, K = K))
}

### hyper parameters for kernel base-learners
# knots are used for prediction
hyper_kernel <- function(df = 4, lambda = NULL, kernel = "lin",
                         knots, pathway = NULL, args = list()) {
    ## knots takes the observations which where used at the time of fitting.
    ## needed for predictions.
    list(df = df, lambda = lambda, kernel = kernel, knots = knots,
         pathway = pathway, args = args)
}

### <FIXME> Add possibility to directly specify ANA'

\documentclass{article}
\usepackage{amstext}
\usepackage{amsfonts}
\usepackage{hyperref}
\usepackage[round]{natbib}
\usepackage{hyperref}
\usepackage{graphicx}
\usepackage{rotating}
%%\usepackage[nolists]{endfloat}
%%\usepackage{Sweave}

%%\VignetteIndexEntry{Survival Ensembles}
%%\VignetteDepends{mboost, survival, rpart, TH.data, partykit}

\newcommand{\Rpackage}[1]{{\normalfont\fontseries{b}\selectfont #1}}
\newcommand{\Robject}[1]{\texttt{#1}}
\newcommand{\Rclass}[1]{\textit{#1}}
\newcommand{\Rcmd}[1]{\texttt{#1}}
\newcommand{\Roperator}[1]{\texttt{#1}}
\newcommand{\Rarg}[1]{\texttt{#1}}
\newcommand{\Rlevel}[1]{\texttt{#1}}

\newcommand{\RR}{\textsf{R}}
\renewcommand{\S}{\textsf{S}}

\RequirePackage[T1]{fontenc}
\RequirePackage{graphicx,ae,fancyvrb}
\IfFileExists{upquote.sty}{\RequirePackage{upquote}}{}
\usepackage{relsize}

\DefineVerbatimEnvironment{Sinput}{Verbatim}{baselinestretch=1}
\DefineVerbatimEnvironment{Soutput}{Verbatim}{fontfamily=courier,
                                              baselinestretch=1,
                                              fontshape=it,
                                              fontsize=\relsize{-1}}
\DefineVerbatimEnvironment{Scode}{Verbatim}{}
\newenvironment{Schunk}{}{}

\renewcommand{\baselinestretch}{1}

\hypersetup{%
  pdftitle = {mboost Illustrations},
  pdfsubject = {package vignette},
  pdfauthor = {Torsten Hothorn and Peter Buhlmann},
%% change colorlinks to false for pretty printing
  colorlinks = {true},
  linkcolor = {blue},
  citecolor = {blue},
  urlcolor = {red},
  hyperindex = {true},
  linktocpage = {true},
}

\begin{document}

\setkeys{Gin}{width=\textwidth}

\title{Survival Ensembles}

\author{Torsten Hothorn$^{1,\star}$, Peter B\"uhlmann$^2$, Sandrine Dudoit$^3$, \\
        Annette Molinaro$^4$ and Mark J. van der Laan$^3$}
\date{}
\maketitle

\noindent$^1$Institut f\"ur Statistik \\
           Ludwig-Maximilians-Universit\"at M\"unchen \\
           Ludwigstra{\ss}e 33, D-80539 M\"unchen, Germany \\
     Tel: ++49--9131--8522707 \\
     Fax: ++49--9131--8525740 \\
     \texttt{Torsten.Hothorn@R-project.org}
\newline

\noindent$^2$Seminar f\"ur Statistik, ETH Z\"urich,
             CH-8032 Z\"urich, Switzerland \\
            \texttt{buhlmann@stat.math.ethz.ch}
\newline

\noindent$^3$Division of Biostatistics, University of California, Berkeley \\
     140 Earl Warren Hall, \#7360, Berkeley, CA 94720-7360, USA \\
    \texttt{sandrine@stat.Berkeley.EDU} \\
    \texttt{laan@stat.Berkeley.EDU}
\newline

\noindent$^4$Division of Biostatistics, Epidemiology and Public Health\\
    Yale University School of Medicine, 206 LEPH \\
    60 College Street PO Box 208034, New Haven CT 06520-8034 \\
    \texttt{annette.molinaro@yale.edu}
\newline

\section{Illustrations and Applications}

This document reproduces the data analyses presented in
\cite{hothetal06}. For a description of the theory behind
applications shown here we refer to the original manuscript.
The results differ slightly due to technical changes or bugfixes in \textbf{mboost}
that have been implemented after the paper was printed.

\subsection{Acute myeloid leukemia}

<<setup, echo = FALSE, results = hide>>=
source("setup.R")
if (!require("TH.data"))
    stop("cannot attach package ", sQuote("TH.data"))
if (!require("rpart"))
    stop("cannot attach package ", sQuote("rpart"))
if (!require("survival"))
    stop("cannot attach package ", sQuote("survival"))
if (!require("partykit"))
    stop("cannot attach package ", sQuote("partykit"))

set.seed(290875)
CEX <- 0.85
options(digits = 3)

### mean difference plots
mdplot <- function(obs, pred, main = "", ...) {

    m <- (obs + pred)/2
    d <- obs - pred
    plot(m, d, xlab = "(Observed + Predicted) / 2",
         ylab = "Observed - Predicted", main =
         main, cex.axis = CEX, cex.main = CEX, cex.lab = CEX, ...)
    abline(h = 0, lty = 3)
}
@

<<loaddata, echo = FALSE, results = tex>>=
### load data. See `mboost/inst/readAML_Bullinger.R' for
### how the data were generated from the raw data.
load(file.path(path.package(package = "TH.data"), "rda", "AML_Bullinger.rda"))
@

\paragraph{Data preprocessing}

Compute IPC weights, define risk score and set up learning sample:
<<AML-dpp, echo = TRUE>>=
### compute IPC weights
AMLw <- IPCweights(Surv(clinical$time, clinical$event))

### risk score
risk <- rep(0, nrow(clinical))
rlev <- levels(clinical[, "Cytogenetic.group"])
risk[clinical[, "Cytogenetic.group"] %in% rlev[c(7,8,4)]] <- "low"
risk[clinical[, "Cytogenetic.group"] %in% rlev[c(5, 9)]] <- "intermediate"
risk[clinical[, "Cytogenetic.group"] %in% rlev[-c(4,5, 7,8,9)]] <- "high"
risk <- as.factor(risk)

### set-up learning sample
AMLlearn <- cbind(clinical[, c("time", "Sex", "Age", "LDH", "WBC",
                        "FLT3.aberration.", "MLL.PTD", "Tx.Group.")],
               risk = risk,
               iexpressions[, colnames(iexpressions) %in% selgenes[["Clone.ID"]]])
cc <- complete.cases(AMLlearn)
AMLlearn <- AMLlearn[AMLw > 0 & cc,]
AMLw <- AMLw[AMLw > 0 & cc]
@

\paragraph{Model fitting}

Fit random forest for censored data
<<AML-RF, echo = TRUE>>=
### controls for tree growing
ctrl <- ctree_control(testtype = "Teststatistic", 
                      teststat = "maximum", mincriterion = .1, minsplit = 5)
### was: cforest_control(mincriterion = 0.1, mtry = 5, minsplit = 5, ntree = 250)

### fit random forest for censored data (warnings are OK here)
AMLrf <- cforest(log(time) ~ ., data = AMLlearn, control = ctrl,
                 weights = AMLw, mtry = 5, ntree = 250, 
                 perturb = list(replace = TRUE, fraction = 0.632))
@
and $L_2$Boosting for censored data
<<AML-boost, echo = TRUE>>=
AMLl2b <- glmboost(I(log(time)) ~ ., data = AMLlearn, weights = AMLw,
                    control = boost_control(mstop = 5000))
@

\begin{figure}
\begin{center}
<<AML-AIC, echo = TRUE, fig = TRUE>>=
### AIC criterion
plot(aic <- AIC(AMLl2b))
@
\caption{AIC criterion for AML data.}
\end{center}
\end{figure}

Compute fitted values
<<AML-fitted, echo = TRUE>>=
### restrict number of boosting iterations and inspect selected variables
AMLl2b <- AMLl2b[mstop(aic)]
cAML <- coef(AMLl2b)
cAML[abs(cAML) > 0]

### fitted values
AMLprf <- predict(AMLrf, newdata = AMLlearn)
AMLpb <- predict(AMLl2b, newdata = AMLlearn)
@

\begin{figure}
\begin{center}
<<Figure1, echo = FALSE, fig = TRUE>>=
Mmod <- sum(AMLw * log(AMLlearn$time))/sum(AMLw )
par(mai = par("mai") * c(0.7, 0.8, 0.7, 0.6))
layout(matrix(1:4, ncol = 2))

mdplot(log(AMLlearn$time), AMLprf, main = "Random Forest",
       cex = AMLw / 4, ylim = c(-4, 4), xlim = c(0, 7))

plot(log(AMLlearn$time), AMLprf, cex = AMLw / 4,
       ylim = range(log(AMLlearn$time)), ylab = "Predicted", xlab = "Observed",
       main = "Random Forest",  cex.axis = CEX, cex.main = CEX, cex.lab = CEX)
abline(h = Mmod, lty = 2)

mdplot(log(AMLlearn$time), AMLpb,
        cex = AMLw / 4, main = "Boosting", ylim = c(-4, 4), xlim = c(0, 7))

plot(log(AMLlearn$time), AMLpb, cex = AMLw / 4,
       ylim = range(log(AMLlearn$time)), ylab = "Predicted", xlab = "Observed",
       main = "Boosting",  cex.axis = CEX, cex.main = CEX, cex.lab = CEX)
abline(h = Mmod, lty = 2)
@
\caption{AML data: Reproduction of Figure 1.}
\end{center}
\end{figure}

\subsection{Node-positive breast cancer}

\paragraph{Data preprocessing}

Compute IPC weights and set up learning sample:
<<GBSG2-dpp, echo = TRUE>>=
### attach data
data("GBSG2", package = "TH.data")

### IPC weights
GBSG2w <- IPCweights(Surv(GBSG2$time, GBSG2$cens))

### set-up learning sample
GBSG2learn <- cbind(GBSG2[,-which(names(GBSG2) %in% c("time", "cens"))],
               ltime = log(GBSG2$time))
n <- nrow(GBSG2learn)
@

\paragraph{Model fitting}

<<GBSG2-models, echo = TRUE>>=
### linear model
LMmod <- lm(ltime ~ . , data = GBSG2learn, weights = GBSG2w)
LMerisk <- sum((GBSG2learn$ltime - predict(LMmod))^2*GBSG2w) / n

### regression tree
pos <- GBSG2w > 0
TRmod <- rpart(ltime ~ . , data = GBSG2learn, weights = GBSG2w, 
               subset = pos)
TRerisk <- sum((GBSG2learn$ltime[pos] - predict(TRmod))^2*GBSG2w[pos]) / n

### tree controls
ctrl <- ctree_control(testtype = "Teststatistic", 
                      teststat = "maximum", mincriterion = qnorm(.95), 
                      minsplit = 5)
### was: cforest_control(mincriterion = qnorm(0.95), mtry = 5,
###                      minsplit = 5, ntree = 100)


### fit random forest for censored data (warnings are OK here)
RFmod <- cforest(ltime ~ . , data = GBSG2learn, weights = GBSG2w,
                 control = ctrl, mtry = 5, ntree = 100, 
                 perturb = list(replace = TRUE, 
                     fraction = 0.632 * sum(GBSG2w > 0)))

### fit L2 boosting for censored data
L2Bmod <- glmboost(ltime ~ ., data = GBSG2learn, weights = GBSG2w,
                   control = boost_control(mstop = 250))

### with Huber loss function
L2BHubermod <- glmboost(ltime ~ ., data = GBSG2learn, weights = GBSG2w,
                        family = Huber(d = log(2)))
@

\begin{figure}
\begin{center}
<<GBSG2-AIC, echo = TRUE, fig = TRUE>>=
plot(aic <- AIC(L2Bmod))
@
\caption{AIC criterion for GBSG2 data.}
\end{center}
\end{figure}

Compute fitted values:
<<GBSG2-fitted, echo = TRUE>>=
GBSG2Hp <- predict(L2BHubermod, newdata = GBSG2learn)
L2Berisk <- sum((GBSG2learn$ltime - predict(L2Bmod, newdata = GBSG2learn))^2*GBSG2w) / n
RFerisk <- sum((GBSG2learn$ltime - predict(RFmod, newdata = GBSG2learn))^2*GBSG2w) / n
@

\begin{figure}
\begin{center}
<<Figure3, echo = FALSE, fig = TRUE>>=
lim <- c(4,9)
mylwd <- 0.5
par(mai = par("mai") * c(0.7, 0.8, 0.7, 0.6))

layout(matrix(1:4, ncol = 2))
Mmod <- sum(GBSG2w * GBSG2learn$ltime)/sum(GBSG2w)

mdplot(GBSG2learn$ltime, predict(LMmod), cex = GBSG2w / 4, main = "Linear Model",
       ylim = c(-3, 3), xlim = c(5, 8))
mdplot(GBSG2learn$ltime[pos], predict(TRmod), cex = GBSG2w / 4, main = "Tree",
       ylim = c(-3, 3), xlim = c(5, 8))
mdplot(GBSG2learn$ltime, predict(RFmod, newdata = GBSG2learn), cex = GBSG2w / 4,
       main = "Random Forest", ylim = c(-3, 3), xlim = c(5, 8))
mdplot(GBSG2learn$ltime, predict(L2Bmod, newdata = GBSG2learn), cex = GBSG2w / 4,
       main = "Boosting", ylim = c(-3, 3), xlim = c(5, 8))
@
\caption{GBSG-2 data: Reproduction of Figure 3.}
\end{center}
\end{figure}

\begin{figure}
\begin{center}
<<Figure5, echo = FALSE, fig = TRUE>>=
RFpr <- predict(RFmod, newdata = GBSG2learn)
L2Bpr <- predict(L2Bmod, newdata = GBSG2learn)
ylim <- range(c(RFpr[GBSG2w > 0], L2Bpr[GBSG2w > 0]))
mydf <- 4
par(mai = par("mai") * c(0.7, 0.8, 0.4, 0.6))
layout(matrix(1:4, ncol = 2))
plot(GBSG2learn$pnodes, RFpr, cex = GBSG2w/4, xlim = c(0,40), lwd = mylwd,
     xlab = "Nr. positive lymph nodes", ylim = ylim, ylab =
     expression(hat(Y)), cex.axis = CEX, cex.main = CEX, cex.lab = CEX,)
lines(smooth.spline(GBSG2learn$pnodes, RFpr, GBSG2w/4, df = mydf))
plot(GBSG2learn$age, RFpr, cex = GBSG2w/4, xlab = "Age",
     ylab = expression(hat(Y)), ylim = ylim, lwd = mylwd, cex.axis = CEX,
     cex.main = CEX, cex.lab = CEX)
lines(smooth.spline(GBSG2learn$age, RFpr, GBSG2w/4, df = mydf))
plot(GBSG2learn$estrec, RFpr, cex = GBSG2w/4, xlab = "Estrogen receptor",
     ylab = expression(hat(Y)), ylim = ylim, lwd = mylwd, cex.axis = CEX,
     cex.main = CEX, cex.lab = CEX)
lines(smooth.spline(GBSG2learn$estrec, RFpr, GBSG2w/4, df = mydf))
indx <- which(GBSG2learn$progrec < 100)
plot(GBSG2learn$progrec[indx], RFpr[indx], cex = GBSG2w[indx]/4,
     xlab = "Progesterone receptor (< 100 fmol / l)",
     ylab = expression(hat(Y)), ylim = ylim, lwd = mylwd, cex.axis = CEX,
     cex.main = CEX, cex.lab = CEX)
lines(smooth.spline(GBSG2learn$progrec[indx], RFpr[indx], GBSG2w[indx]/4, df = mydf))
@
\caption{GBSG-2 data: Reproduction of Figure 5.}
\end{center}
\end{figure}


\begin{figure}
\begin{center}
<<Figure6, echo = FALSE, fig = TRUE>>=
par(mai = par("mai") * c(0.7, 0.8, 0.4, 0.6))
layout(matrix(1:4, ncol = 2))
plot(GBSG2learn$pnodes, L2Bpr, cex = GBSG2w/4, xlim = c(0,40),
     ylab = expression(hat(Y)),
     xlab = "Nr. positive lymph nodes", ylim = ylim, lwd = mylwd, cex.axis =
     CEX, cex.main = CEX, cex.lab = CEX)
lines(smooth.spline(GBSG2learn$pnodes, L2Bpr, GBSG2w/4, df = mydf))
plot(GBSG2learn$age, L2Bpr, cex = GBSG2w/4, xlab = "Age",
     ylab = expression(hat(Y)),
     ylim = ylim, lwd = mylwd, cex.axis = CEX, cex.main = CEX, cex.lab =
     CEX)
lines(smooth.spline(GBSG2learn$age, L2Bpr, GBSG2w/4, df = mydf))
plot(GBSG2learn$estrec, L2Bpr, cex = GBSG2w/4, xlab = "Estrogen receptor",
     ylab = expression(hat(Y)), ylim = ylim, lwd = mylwd, cex.axis = CEX,
     cex.main = CEX, cex.lab = CEX)
lines(smooth.spline(GBSG2learn$estrec, L2Bpr, GBSG2w/4, df = mydf))
indx <- which(GBSG2learn$progrec < 100)
plot(GBSG2learn$progrec[indx], L2Bpr[indx], cex = GBSG2w[indx]/4,
     xlab = "Progesterone receptor (< 100 fmol / l)",
     ylab = expression(hat(Y)), ylim = ylim, lwd = mylwd, cex.axis = CEX,
     cex.main = CEX, cex.lab = CEX)
lines(smooth.spline(GBSG2learn$progrec[indx], L2Bpr[indx], GBSG2w[indx]/4, df = mydf))
@
\caption{GBSG-2 data: Reproduction of Figure 6.}
\end{center}
\end{figure}

\begin{figure}
\begin{center}
<<Figure7, echo = FALSE, fig = TRUE>>=
Mmod <- sum(GBSG2w * GBSG2learn$ltime)/sum(GBSG2w)
par(mai = par("mai") * c(0.7, 0.8, 0.7, 0.6))
layout(matrix(1:4, ncol = 2))

yl <- range(c(GBSG2Hp[GBSG2w > 0], L2Bpr[GBSG2w > 0]))
mdplot(GBSG2learn$ltime, GBSG2Hp, main = "Huber Loss",
       cex = GBSG2w / 4, ylim = c(-3, 3), xlim = c(5, 8))

plot(GBSG2learn$ltime, GBSG2Hp, cex = GBSG2w / 4,
       xlim = range(GBSG2learn$ltime[GBSG2w > 0]), ylim = yl, ylab = "Predicted", xlab = "Observed",
       main = "Huber Loss",  cex.axis = CEX, cex.main = CEX, cex.lab = CEX)

mdplot(GBSG2learn$ltime, L2Bpr,
        cex = GBSG2w / 4, main = "Quadratic Loss", ylim = c(-3, 3), xlim = c(5, 8))

plot(GBSG2learn$ltime, L2Bpr, cex = GBSG2w / 4,
       xlim = range(GBSG2learn$ltime[GBSG2w > 0]), ylim = yl, ylab = "Predicted", xlab = "Observed",
       main = "Quadratic Loss",  cex.axis = CEX, cex.main = CEX, cex.lab = CEX)
@
\caption{GBSG-2 data: Reproduction of Figure 7.}
\end{center}
\end{figure}

\clearpage

\bibliographystyle{plainnat}
\bibliography{boost}


\end{document}

\RequirePackage{fix-cm}

\documentclass[10pt]{scrartcl}

%%\VignetteIndexEntry{mboost Tutorial}
%%\VignetteDepends{mboost,lattice,RColorBrewer,TH.data}

\usepackage{threeparttable}  % needed for \tnote (footnotes in table)
\usepackage[utf8]{inputenc}
\usepackage[T1]{fontenc}
\usepackage{color}
\usepackage{xcolor}
\usepackage{amsmath, amssymb, amstext}
\usepackage{graphicx}
\usepackage{dsfont}
\usepackage{subfigure}
\usepackage{url}
\usepackage{bm}                % bold Greek letters. Usage $\bm{\beta}$
\usepackage{booktabs}
\usepackage[authoryear]{natbib}
\usepackage{hyperref}
\definecolor{Blue}{rgb}{0,0,0.5}
\hypersetup{%
  pdftitle = {Model-based Boosting in R: A Hands-on Tutorial Using the R Package mboost.},
  pdfsubject = {},
  pdfkeywords = {},
  pdfauthor = {Benjamin Hofner, Andreas Mayr, Nikolay Robinzonov and Mattthias Schmid},
  %% change colorlinks to false for pretty printing
  colorlinks = {true},
  linkcolor = {Blue},
  citecolor = {Blue},
  urlcolor = {Blue},
  hyperindex = {true},
  linktocpage = {true},
}
\bibpunct{(}{)}{;}{a}{}{,}
\def\citeapos#1{\citeauthor{#1}'s (\citeyear{#1})}  % possesive citation
\def\citepos#1{\citeauthor{#1}' (\citeyear{#1})}  % possesive citation
\usepackage{enumerate}         % needed for "[i]" as enumeration style etc.
\usepackage{ifpdf}

%%%%%%%%%%%%%%%%%%
%  Newcommands   %
%%%%%%%%%%%%%%%%%%

\newcommand{\mbf}[1]{\bm{#1}}
\newcommand{\eps}{\ensuremath{\varepsilon}}
\newcommand{\R}[1]{\texttt{#1}}
\newcommand{\x}{\mathbf{x}}
\newcommand{\X}{\mathbf{X}}
\newcommand{\K}{\mathbf{K}}
\newcommand{\y}{\mathbf{y}}
\newcommand{\uv}{\mathbf{u}}
\newcommand{\bv}{\mbf{\beta}}
\DeclareMathOperator{\df}{df}
\DeclareMathOperator{\argmin}{argmin}

%%%%%%%%%%%%%
%  Layout   %
%%%%%%%%%%%%%

\usepackage{geometry}
\geometry{verbose,a4paper,tmargin=2.2cm,bmargin=2.2cm,lmargin=2.2cm,rmargin=2.2cm}

%%%%%%%%%%%%%
%  Sweave   %
%%%%%%%%%%%%%

%% \usepackage{Sweave} is essentially
\RequirePackage[T1]{fontenc}
\RequirePackage{ae,fancyvrb}
\DefineVerbatimEnvironment{Sinput}{Verbatim}{fontshape=sl,fontsize=\normalsize}
\DefineVerbatimEnvironment{Soutput}{Verbatim}{fontsize=\normalsize}
\DefineVerbatimEnvironment{Scode}{Verbatim}{fontshape=sl,fontsize=\normalsize}
\newenvironment{Schunk}{}{}

\SweaveOpts{echo=true, results=verbatim, prefix.string=graphics/fig, keep.source=true}

<<init, echo=false, results = hide>>=
## set up R session
options(prompt = "R> ", continue = "+  ", width = 80)
pd <- packageDescription("mboost")
if (any(is.na(pd))){
    install.packages("mboost", repos = "http://cran.at.r-project.org")
    pd <- packageDescription("mboost")
}
if (compareVersion(pd$Version, "2.1-0") < 0){ # must be mboost 2.1-X or newer
    warning("Current version of mboost is installed!")
    install.packages("mboost", repos = "http://cran.at.r-project.org")
}
options(digits = 3)
require("mboost")
set.seed(190781)
### make graphics directory if not existing
if (!file.exists("graphics"))
    dir.create("graphics")
@


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%  Begin Document                                                              %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


\begin{document}

\title{Model-based Boosting in \textsf{R}}
\subtitle{A Hands-on Tutorial Using the \textsf{R} Package mboost}

\author{Benjamin Hofner\thanks{E-mail: {benjamin.hofner@imbe.med.uni-erlangen.de}}
  \thanks{Department of Medical Informatics, Biometry and Epidemiology,
    Friedrich-Alexander-Universit\"at Erlangen-N\"urnberg} \and
  Andreas Mayr\footnotemark[2] \and Nikolay Robinzonov\thanks{Department of
    Statistics, Ludwig-Maximilians-Universit\"at M\"unchen} \and Matthias
  Schmid\footnotemark[2]\\[1em]
}

\date{}

\maketitle

\begin{center}
  \small
  \textbf{This is an extended and (slightly) modified version of the manuscript}\\
  Benjamin Hofner, Andreas Mayr, Nikolay Robinzonov and Mattthias Schmid (2014),
  Model-based Boosting in \textsf{R} -- A Hands-on Tutorial Using the \textsf{R}
  Package mboost. \emph{Computational Statistics}, 29:3--35.\\
  DOI \href{http://dx.doi.org/10.1007/s00180-012-0382-5}{10.1007/s00180-012-0382-5}.\\
  The final publication is available at \url{http://link.springer.com}.\\[2em]
  Changes to the results in the original manuscript are due to changes in some
  defaults of \textbf{mboost} (most notably the definition of degrees of freedom
  has changed in \textbf{mboost} 2.2-0). See NEWS for details.
\end{center}
\bigskip

\begin{abstract}
  We provide a detailed hands-on tutorial for the \textsf{R} add-on package
  \textbf{mboost}. The package implements boosting for optimizing general risk
  functions utilizing component-wise (penalized) least squares estimates as
  base-learners for fitting various kinds of generalized linear and generalized
  additive models to potentially high-dimensional data. We give a theoretical
  background and demonstrate how \textbf{mboost} can be used to fit
  interpretable models of different complexity. As an example we use
  \textbf{mboost} to predict the body fat based on anthropometric measurements
  throughout the tutorial.
\end{abstract}

\section{Introduction}
\label{sec:intro}


A key issue in statistical research is the development of algorithms for model
building and variable selection \citep{elements2}. Due to the recent advances in
computational research and biotechnology, this issue has become even more
relevant \citep{Fan2010}. For example, microarray and DNA sequencing experiments
typically result in high-dimensional data sets with large numbers of predictor
variables but relatively small sample sizes. In these experiments, it is usually
of interest to build a good prognostic model using a small subset of marker
genes. Hence, there is a need for statistical techniques to select the most
informative features out of a large set of predictor variables. A related
problem is to select the appropriate modeling alternative for each of the
covariates \citep[``model choice'', ][]{Kneib:Hothorn:Tutz:modelchoice:2009}.
This problem even arises when data sets are not high-dimensional. For example, a
continuous covariate could be included into a statistical model using linear,
non-linear or interaction effects with other predictor variables.

In order to address these issues, a variety of regression techniques has been
developed during the past years (see, e.g., \citealt{elements2}). This progress
in statistical methodology is primarily due to the fact that classical
techniques for model building and variable selection (such as generalized linear
modeling with stepwise selection) are known to be unreliable or might even be
biased. In this tutorial, we consider \emph{component-wise gradient boosting}
(\citealt{Breiman98,Breiman99}, \citealt{friedmanetal2000},
\citealt{friedman2001}), which is a machine learning method for optimizing
prediction accuracy and for obtaining statistical model estimates via gradient
descent techniques. A key feature of the method is that it carries out variable
selection during the fitting process (\citealt{BuehlmannYu2003},
\citealt{Buehlmann2006}) without relying on heuristic techniques such as
stepwise variable selection. Moreover, gradient boosting algorithms result in
prediction rules that have the same interpretation as common statistical model
fits. This is a major advantage over machine learning methods such as random
forests (\citealt{rf}) that result in non-interpretable ``black-box''
predictions.

In this tutorial, we describe the \textsf{R} \citep{r-core:2012} add-on package
\textbf{mboost} \citep{Hothorn+Buehlmann+Kneib+Schmid+Hofner:mboost:2010,
  pkg:mboost:CRAN:2.1}, which implements methods to fit generalized linear
models (GLMs), generalized additive models (GAMs, \citealt{hastietib}), and
generalizations thereof using component-wise gradient boosting techniques. The
\textbf{mboost} package can thus be used for regression, classification,
time-to-event analysis, and a variety of other statistical modeling problems
based on potentially high-dimensional data. Because of its user-friendly formula
interface, \textbf{mboost} can be used in a similar way as classical functions
for statistical modeling in \textsf{R}. In addition, the package implements
several convenience functions for hyper-parameter selection, parallelization of
computations, and visualization of results.

The rest of the paper is organized as follows: In Section \ref{sec:theory}, we
provide a brief theoretical overview of component-wise gradient boosting and its
properties. In Section \ref{sec:mboost}, the \textbf{mboost} package is
presented in detail. Here we present the infrastructure of the package and show
how to use \textbf{mboost} to obtain interpretable statistical model fits. All
steps are illustrated using a clinical data set that was collected by
\citet{garcia}. The authors conducted a study to predict the body fat content of
study participants by means of common anthropometric measurements. The response
variable was obtained by Dual X-Ray Absorptiometry (DXA). DXA is an accurate but
expensive method for measuring the body fat content. In order to reduce costs,
it is therefore desirable to develop a regression equation for predicting DXA
measurements by means of anthropometric measurements which are easy to obtain.
Using the \textbf{mboost} package, we provide a step-by-step illustration on how
to use gradient boosting to fit a prediction model for body fat. A summary of
the paper is given in Section~\ref{sec:summary}.


\section{A Brief Theoretical Overview of Component-Wise
  Gradient Boosting}
\label{sec:theory}

Throughout the paper, we consider data sets containing the values of an outcome
variable $y$ and some predictor variables $x_1,\dots,x_p$. For example, in case
of the body fat data, $y$ is the body fat content of the study participants
(measured using DXA) and $x_1,\dots,x_p$ represent the following anthropometric
measurements: age in years, waist circumference, hip circumference, breadth of
the elbow, breadth of the knee, and four aggregated predictor variables that
were obtained from other anthropometric measurements.

The aim is to model the relationship between $y$ and
$\x:=(x_1,\dots,x_p)^{\top}$, and to obtain an ``optimal''
prediction of $y$ given $\x$. This is accomplished by minimizing a loss
function $\rho (y,f) \in \mathbb{R}$ over a prediction function~$f$
(depending on $\x$). Usually, for GLMs and GAMs, the loss function is simply
the negative log-likelihood function of the outcome distribution. Linear
regression with a continuous outcome variable $y \in\mathbb{R}$ is a
well-known example of this approach: Here, $\rho$ corresponds to the least
squares objective function (which is equivalent to the negative log-likelihood
of a Gaussian model), and $f$ is a parametric (linear) function of $\x$.

In the gradient boosting framework, the aim is to estimate the optimal
prediction function~$f^*$ that is defined by
\begin{eqnarray}\label{thmean}
&& f^{*} := \argmin_{f} \mathbb{E}_{Y,\X} \big[\rho (y, f(\x^\top)) \big] \ ,
\end{eqnarray}
where the loss function $\rho$ is assumed to be differentiable with respect to
$f$. In practice, we usually deal with realizations $(y_i, \x_i^\top)$,
$i=1,\dots,n$, of $(y, \x^{\top})$, and the expectation in (\ref{thmean}) is
therefore not known. For this reason, instead of minimizing the expected value
given in (\ref{thmean}), boosting algorithms minimize the observed mean
$\mathcal{R} := \sum_{i=1}^n \rho(y_i,f(\x_i^\top))$ (also called the
``empirical risk''). The following algorithm (``component-wise gradient
boosting'') is used
to minimize $\mathcal{R}$ over $f$:\\
\begin{enumerate}
\item Initialize the function estimate $\hat{f}^{[0]}$ with offset values. Note
  that $\hat{f}^{[0]}$ is a vector of length $n$. In the following paragraphs,
  we will generally denote the vector of function estimates at iteration $m$ by
  $\hat{f}^{[m]}$.
\item Specify a set of \emph{base-learners}. Base-learners are simple regression
  estimators with a fixed set of input variables and a univariate response. The
  sets of input variables are allowed to differ among the base-learners.
  Usually, the input variables of the base-learners are small subsets of the set
  of predictor variables $x_1,\dots,x_p$. For example, in the simplest case,
  there is exactly one base-learner for each predictor variable, and the
  base-learners are just simple linear models using the predictor variables as
  input variables. Generally, the base-learners considered in this paper are
  either penalized or unpenalized least squares estimators using small subsets
  of the predictor variables as input variables (see
  Section~\ref{sec:baselearners} for details and examples). Each base-learner
  represents a modeling alternative for the statistical model. Denote the number
  of base-learners by $P$ and set $m=0$.
\item Increase $m$ by 1, where $m$ is the number of iterations.
\item
\begin{enumerate}
\item Compute the negative gradient $-\frac{\partial \rho}{\partial f}$ of the
  loss function and evaluate it at $\hat{f}^{[m-1]} (\x_i^\top)$, $i=1,\dots,n$ (i.e.,
  at the estimate of the previous iteration). This yields the negative gradient
  vector
\begin{eqnarray*}
  && \uv^{[m]} = \left(u_{i}^{[m]}\right)_{i=1,\dots,n} := \left( -
    \frac{\partial}{\partial f} \rho \left(y_i,\hat{f}^{[m-1]} (\x_i^\top)\right)
  \right)_{i=1,\dots,n} \ .
\end{eqnarray*}
\item Fit each of the $P$ base-learners to the negative gradient vector, i.e.,
  use each of the regression estimators specified in step 2 separately to fit
  the negative gradient. The resulting $P$ regression fits yield $P$ vectors of
  predicted values, where each vector is an estimate of the negative gradient
  vector $\uv^{[m]}$.
\item Select the base-learner that fits $\uv^{[m]}$ best according to the
  residual sum of squares (RSS) criterion and set $\hat{\uv}^{[m]}$ equal to the
  fitted values of the best-fitting base-learner.
\item Update the current estimate by setting $\hat{f}^{[m]} = \hat{f}^{[m-1]} +
  \nu \, \hat{\uv}^{[m]}$, where $0 < \nu \le 1$ is a real-valued step length
  factor.
\end{enumerate}
\item Iterate Steps 3 and 4 until the stopping iteration $m_{\mathrm{stop}}$ is
  reached (the choice of $m_{\mathrm{stop}}$ is discussed below).
\end{enumerate}

From step 4 it is seen that an estimate of the true negative gradient of
$\mathcal{R}$ is added to the current estimate of $f^*$ in each iteration.
Consequently, the component-wise boosting algorithm descends along the gradient
of the empirical risk $\mathcal{R}$. The empirical risk $\mathcal{R}$ is
thus minimized in a stage-wise fashion, and a structural (regression)
relationship between $y$ and $\x$ is established. This strategy corresponds to
replacing classical Fisher scoring algorithms for maximum likelihood estimation
of $f^*$ (\citealt{mccullagh}) by a gradient descent algorithm in function
space. As seen from steps 4(c) and 4(d), the algorithm additionally carries out
variable selection and model choice, as only one base-learner is selected for
updating~$\hat{f}^{[m]}$ in each iteration. For example, if each base-learner
corresponds to exactly one predictor variable (that is used as the input
variable of the respective base-learner), only {\em one} predictor variable is
selected in each iteration (hence the term ``component-wise''). Note that the
variable selection property of the above-described algorithm is not a general
feature of all boosting methods (where, in principle, any type of regression
estimator could be used to fit the negative gradient) but is caused by the
specific definition of the base-learning mechanism in steps 2 and 4.

Due to the additive update, the final boosting estimate in iteration $m_{\mathrm{stop}}$
can be interpreted as an additive prediction function. It is easily seen that
the structure of this function is equivalent to the structure of the additive
predictor of a GAM (see \citealt{hastietib}). This means that
\begin{equation}
\label{gamfun}
\hat{f} = \hat{f}_1 + \dots + \hat{f}_P \ ,
\end{equation}
where $\hat{f}_1,\dots,\hat{f}_P$ correspond to the functions specified by the
base-learners. Consequently, $\hat{f}_1,\dots,\hat{f}_P$ depend on the predictor
variables that were used as input variables of the respective base-learners.
Note that a base-learner can be selected multiple times in the course of the
boosting algorithm. In this case, its function estimate $\hat{f}_j$, $j \in
1,\dots,P$, is the sum of the individual estimates $\nu \cdot \hat{\uv}^{[m-1]}$
obtained in the iterations in which the base-learner was selected. Note also
that some of the $\hat{f}_j$, $j=1,\dots,P$, might be equal to zero, as the
corresponding base-learners might not have been selected in step 4(c). This can
then be considered as variable selection or model choice (depending on the
specification of the base-learners).

From step 4 of the component-wise gradient boosting algorithm it is clear that
the specification of the base-learners is crucial for interpreting the model
fit. As a general rule, due to the additive update in step 4(d), the estimate of
a function~$f_j$ at iteration $m_{\mathrm{stop}}$ has the same structure as the
corresponding base-learner. For example, $f_j$ is a linear function if the
base-learner used to model $f_j$ in step~4(b) is a simple linear model (see
\citealt{BuhlmannHothorn06}, p.~484, also see Section \ref{sec:mboost} for
details). Similarly, $f_j$ is a smooth function of the $j$th covariate $\x_j$ if
the corresponding base-learner is smooth as well. Note that it is generally
possible to incorporate observation weights into component-wise gradient
boosting. This is accomplished by using weighted regression techniques for each
of the base-learners in step 4.

A crucial issue is the choice of the stopping iteration
$m_{\mathrm{stop}}$. Several authors have argued that boosting algorithms should
generally not be run until convergence. Otherwise, overfits resulting in a
suboptimal prediction accuracy would be likely to occur (see
\citealt{BuhlmannHothorn06}). We therefore use a finite stopping iteration for
component-wise gradient boosting that optimizes the prediction accuracy (``early
stopping strategy''). As a consequence, the stopping iteration becomes a tuning
parameter of the algorithm, and cross-validation techniques or AIC-based
techniques can be used to estimate the optimal $m_{\mathrm{stop}}$. In
contrast to the choice of the stopping iteration, the choice of the step length
factor $\nu$ has been shown to be of minor importance for the predictive
performance of a boosting algorithm. The only requirement is that the value of
$\nu$ is small (e.g., $\nu=0.1$, see
\citealt{Schmid:Hothorn:boosting-p-Splines}). Small values of $\nu$ are
necessary to guarantee that the algorithm does not overshoot the minimum of the
empirical risk $\mathcal{R}$.

A major consequence of early stopping is that boosting optimizes prediction
accuracy by regularizing the estimates of $f^*$ via shrinkage techniques. This
is due to the fact that using a small step length $\nu$ ensures that  effect estimates increase ``slowly'' in the course of the
boosting algorithm, and that the estimates stop increasing as soon as the
optimal stopping iteration $m_{\mathrm{stop}}$ is reached. Stopping a
component-wise gradient boosting algorithm at the optimal iteration therefore
implies that effect estimates are shrunken such that the predictive power of the
GAM is maximal. Shrinking estimates is a widely used strategy for
building prognostic models: Estimates tend to have a slightly increased bias but
a decreased variance, thereby improving prediction accuracy. In addition,
shrinkage generally stabilizes effect estimates and avoids multicollinearity
problems. Despite the bias induced by shrinking effect estimates, however, the
structure of function (\ref{gamfun}) ensures that results are
interpretable and that black-box estimates are avoided. The interpretation of
boosting estimates is essentially the same as those of classical maximum
likelihood estimates.

\section{The Package mboost}
\label{sec:mboost}

As pointed out above, the \textsf{R} add-on package \textbf{mboost} implements
model-based boosting methods as introduced above that result in interpretable
models. This is in contrast to other boosting packages such as \textbf{gbm}
\citep{Ridgeway:gbm:2010}, which implements tree-based boosting methods that
lead to black-box predictions. The \textbf{mboost} package offers a modular
nature that allows to specify a wide range of statistical models.

A generalized additive model is specified as the combination of a distributional
assumption and a structural assumption. The distributional assumption specifies
the conditional distribution of the outcome. The structural assumption specifies
the types of effects that are to be used in the model, i.e., it represents the
deterministic structure of the model. Usually, it specifies how the predictors
are related to the conditional mean of the outcome. To handle a broad class of
models within one framework, \textbf{mboost} also allows to specify effects for
conditional quantiles, conditional expectiles or hazard rates. The
distributional assumption, i.e., the loss function that we want to minimize, is
specified as a \R{family}. The structural assumption is given as a \R{formula}
using base-learners.

The loss function, as specified by the \R{family} is independent of the
estimation of the base-learners. As one can see in the component-wise boosting
algorithm, the loss function is only used to compute the negative gradient in
each boosting step. The predictors are then related to these values by penalized
ordinary least squares estimation, irrespective of the loss function. Hence, the
user can freely combine structural and distributional assumptions to tackle
new estimation problems.

In Section~\ref{sec:glmboost} we will derive a special case of component-wise
boosting to fit generalized linear models. In Section~\ref{sec:gamboost} we
introduce the methods to fit generalized additive models and give an
introduction to the available base-learners. How different loss functions can be
specified is shown in Section~\ref{sec:family}.

\subsection{Fitting Generalized Linear Models: \R{glmboost}}
\label{sec:glmboost}

The function \R{glmboost()} provides an interface to fit (generalized) linear
models. A (generalized) linear model of the covariates $\x = (x_1,
  \ldots, x_p)^\top$ has the form
  \begin{equation*}
    g(\vec{\mu}) = \beta_0 + \beta_1 x_1 + \ldots +  \beta_px_p
  \end{equation*}
  with the (conditional) expectation of the response $\vec{\mu} = \mathds{E}(y |
  \vec{x})$, the link function $g$ and parameters $\vec{\beta}$. The resulting
  models from \R{glmboost()} can be essentially interpreted the same way as
  models that are derived from \R{glm()}. The only difference is that the
  boosted generalized linear model additionally performs variable selection as
  described in Section~\ref{sec:theory} and the effects are shrunken toward zero
  if early stopping is applied. In each boosting iteration, \R{glmboost()} fits
  simple linear models without intercept \emph{separately for each column of the
    design matrix} to the negative gradient vector. Only the best-fitting linear
  model (i.e., the best fitting base-learner) is used in the update step. If
  factors are used in the model, this results in separate updates for the
  effects of each factor level as these are (after dummy coding) represented by
  separate columns in the design matrix. If one wants to treat factors or groups
  of variables as one base-learner with a common update, one needs to use the
  \R{gamboost()} function with a \R{bols()} base-learner for each factor or
  group (see Section~\ref{sec:gamboost}, especially Table~\ref{tab:bols}, for
  details).

The interface of \R{glmboost()} is in essence the same as for \R{glm()}. Before
we show how one can use the function to fit a linear model to predict the body
fat content, we give a short overview on the function\footnote{Note that here
  and in the following we sometimes restrict the focus to the most important or
  most interesting arguments of a function. Further arguments might exist. Thus,
  for a complete list of arguments and their description refer to the respective
  manual.}:

\begin{Schunk}
\begin{Sinput}
  glmboost(formula, data = list(), weights = NULL,
           center = TRUE, control = boost_control(), ...)
\end{Sinput}
\end{Schunk}

\noindent The model is specified using a \R{formula} as in \R{glm()} of the form
\R{response \~{} predictor1 + predictor2} and the data set is provided as a
\R{data.frame} via the \R{data} argument. Optionally, \R{weights} can be given
for weighted regression estimation. The argument \R{center} is specific for
\R{glmboost()}. It controls whether the data is internally centered. Centering
is of great importance, as this allows much faster ``convergence'' of the
algorithm or even ensures that the algorithm converges in the direction of the
true value at all. We will discuss this in detail at the end of
Section~\ref{sec:glmboost}. The second boosting-specific argument, \R{control},
allows to define the hyper-parameters of the boosting algorithm. This is done
using the function \R{boost\_control()}. For example one could specify:

\begin{Schunk}
\begin{Sinput}
R> boost_control(mstop = 200,  ## initial number of boosting iterations. Default: 100
+                nu = 0.05,    ## step length. Default: 0.1
+                trace = TRUE) ## print status information? Default: FALSE
\end{Sinput}
\end{Schunk}

Finally, the user can specify the distributional assumption via a \R{family},
which is ``hidden'' in the \,`\R{\ldots}'\, argument (see
\R{?mboost\_fit}\footnote{\R{glmboost()} merely handles the preprocessing of the
  data. The actual fitting takes place in a unified framework in the function
  \R{mboost\_fit()}.} for details and further possible parameters). The default
\R{family} is \R{Gaussian()}. Details on families are given in
Section~\ref{sec:family}. Ways to specify new families are described in the
Appendix.

\paragraph{Case Study: Prediction of Body Fat}\hfill\\

\noindent The aim of this case study is to compute accurate predictions for the
body fat of women based on available anthropometric measurements. Observations
of 71 German women are available with the data set \R{bodyfat} \citep{garcia}
included in \textbf{mboost}. We first load the package and the data set%
\footnote{The data set \R{bodyfat} has been moved to the package \textbf{TH.data}.}.

<<setup, echo = true, results = hide>>=
library("mboost")                         ## load package
data("bodyfat", package = "TH.data")      ## load data
@

The response variable is the body fat measured by DXA (\R{DEXfat}), which can be
seen as the gold standard to measure body fat. However, DXA measurements are too
expensive and complicated for a broad use. Anthropometric measurements as waist
or hip circumferences are in comparison very easy to measure in a standard
screening. A prediction formula only based on these measures could therefore be
a valuable alternative with high clinical relevance for daily usage. The
available variables and anthropometric measurements in the data set are
presented in Table~\ref{tab:casepred}.

\begin{table}[h!]
  \centering
  \caption{Available variables in the \R{bodyfat} data, for details see \cite{garcia}.}
   \label{tab:casepred}
  \begin{tabular}{lp{0.5\linewidth}}
    \toprule
    Name & Description \\
    \midrule
    \R{DEXfat} &body fat measured by DXA (response variable) \\
    \R{age} & age of the women in years \\
    \R{waistcirc} & waist circumference \\
    \R{hipcirc} & hip circumference \\
    \R{elbowbreadth} & breadth of the elbow \\
    \R{kneebreadth} & breadth of the knee \\
    \R{anthro3a} & sum of logarithm of three anthropometric measurements \\
    \R{anthro3b} & sum of logarithm of three anthropometric measurements \\
    \R{anthro3c} & sum of logarithm of three anthropometric measurements \\
    \R{anthro4} & sum of logarithm of four anthropometric measurements \\
   \bottomrule

  \end{tabular}
\end{table}

In the original publication \citep{garcia}, the presented prediction formula was
based on a linear model with backward-elimination for variable selection. The
resulting final model utilized hip circumference (\R{hipcirc}), knee breadth
(\R{kneebreadth}) and a compound covariate (\R{anthro3a}), which is defined as
the sum of the logarithmic measurements of chin skinfold, triceps skinfold and
subscapular skinfold:

<<mod_garcia>>=
## Reproduce formula of Garcia et al., 2005
lm1 <- lm(DEXfat ~ hipcirc  + kneebreadth + anthro3a, data = bodyfat)
coef(lm1)
@

\noindent A very similar model can be easily fitted by boosting, applying
\R{glmboost()} with default settings:

<<mod_garcia_boosted>>=
## Estimate same model by glmboost
glm1 <- glmboost(DEXfat ~ hipcirc  + kneebreadth + anthro3a, data = bodyfat)
coef(glm1, off2int=TRUE)    ## off2int adds the offset to the intercept
@

\noindent Note that in this case we used the default settings in \R{control} and
the default family \R{Gaussian()} leading to boosting with the $L_2$ loss.

We now want to consider all available variables as potential predictors. One way
is to simply specify \R{"."} on the right side of the formula:
<<mod_glmboost>>=
glm2 <- glmboost(DEXfat ~ ., data = bodyfat)
@
As an alternative one can explicitly provide the whole formula by using the
\R{paste()} function\footnote{Another alternative is given by the matrix
  interface for \R{glmboost()} where one can directly use the design matrix as
  an argument. For details see \R{?glmboost}.}. Therefore, one could essentially
call:
<<mod_glmboost2>>=
preds <- names(bodyfat[, names(bodyfat) != "DEXfat"])        ## names of predictors
fm <- as.formula(paste("DEXfat ~", paste(preds, collapse = "+")))  ## build formula
fm
@
and provide \R{fm} to the \R{formula} argument in \R{glmboost()}. Note that a
solution using the \R{paste()} function is somewhat unavoidable when we intend
to combine different base-learners for plenty of predictors in \R{gamboost()}.

Note that at this iteration (\R{mstop} is still 100 as it is the default value)
\R{anthro4} is not included in the resulting model as the corresponding
base-learner was never selected in the update step. The function \R{coef()} by
default only displays the selected variables but can be forced to show all
effects by specifying \R{which = ""}:
<<coef_glmboost>>=
coef(glm2,        ## usually the argument 'which' is used to specify single base-
     which = "")  ## learners via partial matching; With which = "" we select all.
@
A plot of the coefficient paths, similar to the ones commonly known from the
LARS algorithm \citep{Efron2002}, can be easily produced by using \R{plot()} on
the \R{glmboost} object (see Figure~\ref{fig:coefpaths}):

<<plot_glmboost>>=
plot(glm2, off2int = TRUE)    ## default plot, offset added to intercept
## now change ylim to the range of the coefficients without intercept (zoom-in)
plot(glm2, ylim = range(coef(glm2, which = preds)))
@

\setkeys{Gin}{width = 0.45\textwidth}
\begin{figure}[ht]
  \centering
<<coefsplot1, width = 5, height = 5, fig = true, echo = false>>=
## same as first plot from chunk "plot_glmboost" but with better margins
par(mar = c(4, 4, 0, 6) + 0.1)
plot(glm2, main="", off2int=TRUE)
@
\hfill
<<coefsplot2, width = 5,height = 5, fig = true, echo = false>>=
## same as second plot from chunk "plot_glmboost" but with better margins
par(mar = c(4, 4, 0, 6) + 0.1)
plot(glm2, ylim = range(coef(glm2, which = preds)), main = "")
@
\caption{Coefficients paths for the body fat data: Here, the default plot on the
  left leads to hardly readable variable names due to the inclusion of the
  intercept. For the plot on the right we therefore adjusted the y-scale to
  avoid this problem. \label{fig:coefpaths}}
\end{figure}

\paragraph{Centering of Linear Base-learners Without Intercept}\hfill\\

\noindent For linear base-learners that are specified without
intercept,\footnote{If the fitting function \texttt{glmboost()} is used the
  base-learners never contain an intercept. Furthermore, linear base-learners
  without intercept can be obtained by specifying a base-learner \texttt{bols(x,
    intercept = FALSE)} (see below).} it is of great importance to center the
covariates before fitting the model. Without centering of the covariates, linear
effects that result from base-learners without intercept are forced through the
origin (with no data lying there). Hence, the convergence will be very slow or
the algorithm will not converge to the ``correct'' solution even in very simple
cases. As an example, consider one normally distributed predictor $\x = (x_1,
\ldots, x_n)^\top$, and a model
\begin{equation*}
  \y = \beta \x + \mbf{\eps},
\end{equation*}
with $\beta = 1$ and $\mbf{\eps} \sim \mathcal{N}(\mbf{0}, 0.3^2)$. Usually, a
model without intercept could be fitted to estimate $\beta$. However, if we
apply boosting with the L$_2$ loss the negative gradient in the first boosting
step is, by default, the centered response, i.e., $\uv^{[1]} = \y - 1/n \sum_{i
  = 1}^n y_i$. For other loss functions the negative gradient in the first
boosting iteration is not exactly the mean-centered response. Yet, the negative
gradient in the first step is always ``centered'' around zero. In this
situation, the application of a base-learner without intercept (i.e., a simple
linear model without intercept) is not sufficient anymore to recover the effect
$\beta$ (see Figure~\ref{fig:without_centering}). The true effect is completely
missed. To solve this problem, it is sufficient to use a centered predictor
$\x$. Then, the center of the data is shifted to the origin (see
Figure~\ref{fig:with_centering}) and the model without intercept goes through
the origin as well.

%% figure is build at the end of the document
\setkeys{Gin}{width = 0.45\textwidth}
\begin{figure}[ht]
  \centering
  \subfigure[Without Centering \label{fig:without_centering}]{
    \includegraphics{graphics/fig-center_false}
  }
  \subfigure[With Centering \label{fig:with_centering}]{
    \includegraphics{graphics/fig-center_true}
  }
  \caption{L$_2$Boosting in the first boosting step, i.e., with centered response
    variable as outcome. A base-learner without intercept misses the true effect
    completely if $\x$ is not centered (left) and is able to capture the true
    structure if $\x$ is centered (right).\label{fig:centering}}
\end{figure}

Centering the predictors does not change the estimated effects of the
predictors. Yet, the intercept needs to be corrected as can be seen from the
following example. Consider two predictors and estimate a model with centered
predictors, i.e,
\begin{align*}
  \hat{\y} &= \hat{\beta}_0 + \hat{\beta}_1 (\x_1 - \bar{x}_1) + \hat{\beta}_2
  (\x_2 -
  \bar{x}_2) \quad\quad \Leftrightarrow\\
  \hat{\y} &= \underbrace{(\hat{\beta}_0 - \hat{\beta}_1 \bar{x}_1 -
    \hat{\beta}_2 \bar{x}_2)}_{= \hat{\beta}_0^*} + \hat{\beta}_1 \x_1 +
  \hat{\beta}_2 \x_2.
\end{align*}
Hence, the intercept from a model without centering of the covariates equals
$\hat{\beta}_0^* = \hat{\beta}_0 - \sum_j \hat{\beta}_j \bar{x}_j$, where
$\hat{\beta}_0$ is the intercept estimated from a model with centered
predictors.

\subsection{Fitting Generalized Additive Models: \R{gamboost}}
\label{sec:gamboost}

Besides an interface to fit linear models, \textbf{mboost} offers a very
flexible and powerful function to fit additive models. An additive model of the
covariates $\x = (x_1, \ldots, x_p)^\top$ has, in general, the form
\begin{equation*}
  g(\vec{\mu}) = \beta_0 + f_1 + \ldots +  f_p
\end{equation*}
with the (conditional) expectation of the response $\vec{\mu} = \mathds{E}(y |
\vec{x})$, the link function $g$ and arbitrary functions $f_1, \ldots, f_p$ of
the covariates. These functions include simple, linear functions as well as
smooth, non-linear functions. The functions are specified by the base-learners
that are introduced in the following paragraphs.

The function \R{gamboost()} can be used to fit linear models or (non-linear)
additive models via component-wise boosting. The user additionally needs to
state which variable should enter the model in which fashion, e.g. as a linear
effect or as a smooth effect. In general, however, the interface of
\R{gamboost()} is very similar to \R{glmboost()}.
\begin{Schunk}
\begin{Sinput}
  gamboost(formula, data = list(), ...)
\end{Sinput}
\end{Schunk}
Again, the function requires a formula to specify the model. Furthermore, a data
set needs to be specified as for linear models. Additional arguments that are
passed to \R{mboost\_fit()}\footnote{\R{gamboost()} also calls \R{mboost\_fit()}
  for the actual boosting algorithm.} include \R{weights}, \R{control} and
\R{family}. These arguments can be used in the same way as described for
\R{glmboost()} above.

\subsubsection{Base-learners}\label{sec:baselearners}

The structural assumption of the model, i.e., the types of effects that are to
be used, can be specified in terms of base-learners. Each base-learner results
in a related type of effect. An overview of available base-learners is given in
the following paragraphs. An example of how these base-learners can then be
combined to formulate the model is given afterwards.

The base-learners should be defined such that the degrees of freedom of the
single base-learner are small enough to prevent overshooting. Typically one
uses, for example, 4 degrees of freedom (the default for many base-learners) or
less. Despite the small initial degrees of freedom, the final estimate that
results from this base-learner can adopt higher order degrees of freedom due to
the iterative nature of the algorithm. If the base-learner is chosen multiple
times, it overall adapts to an appropriate degree of flexibility and smoothness.

All base-learners considered in this tutorial are simple penalized least squares
models of the form
\begin{equation*}
  \hat{\uv} = \X (\X^\top\X + \lambda
  \K)^{-1}\X^\top \uv = \mathcal{S} \uv,
\end{equation*}
where the hat-matrix is defined as $\mathcal{S} = \X (\X^\top\X + \lambda
\K)^{-1}\X^\top$, with design matrix $\X$, penalty parameter $\lambda$ and
penalty matrix $\K$. The design and penalty matrices depend on the type of
base-learner. A penalty parameter $\lambda = 0$ results in unpenalized
estimation.

\paragraph{Linear and Categorical Effects}\hfill\\

\noindent The \R{bols()} function\footnote{the name refers to \textbf{o}rdinary
  \textbf{l}east \textbf{s}quares base-learner} allows the definition of
(penalized) ordinary least squares base-learners. Examples of base-learners of
this type include (a) linear effects, (b) categorical effects, (c) linear
effects for groups of variables $\x = (x_1, x_2, \ldots, x_p)^\top$, (d)
ridge-penalized effects for (b) and (c), (e) varying coefficient terms (i.e.,
interactions). If a penalized base-learner is specified, a special penalty based
on the differences of the effects of adjacent categories is automatically used
for ordinal variables (\R{x <- as.ordered(x)}) instead of the ridge penalty
\citep{Hofner:unbiased:2011}. Figure~\ref{fig:expl_bols} shows two effect
estimates that result from \R{bols()} base-learners, a simple linear effect and
an effect estimate for a factor variable. The call to an ordinary penalized
least squares base-learner is as follows:
\begin{Schunk}
\begin{Sinput}
  bols(..., by = NULL, intercept = TRUE, df = NULL, lambda = 0)
\end{Sinput}
\end{Schunk}
The variables that correspond to the base-learner are specified in the
\,`\R{\ldots}'\, argument, separated by commas. If multiple variables are
specified, they are treated as \emph{one group}. A linear model is fitted for
all given variables together and they are either all updated or not at all. An
additional variable that defines varying coefficients can optionally
be given in the \R{by} argument. The logical variable \R{intercept} determines
whether an intercept is added to the design matrix (\R{intercept = TRUE}, the
default). If \R{intercept = FALSE}, continuous covariates should be (mean-)
centered as discussed above. This must be done `by hand' in advance of fitting
the model. The impact of the penalty in case of penalized OLS (ordinary least squares) base-learners can
be determined either via the degrees of freedom \R{df} or the penalty parameter
\R{lambda}. If degrees of freedom are specified, the penalty parameter
\R{lambda} is computed from \R{df}\footnote{If \R{df} is specified in
  \R{bols()}, \R{lambda} is always ignored.}. Note that per default unpenalized
linear models are used. Two definitions of degrees of freedom are implemented in
\textbf{mboost}: The first uses the trace of the hat-matrix
($\text{trace}(\mathcal{S})$) as degrees of freedom, while the second definition
uses $\text{trace}(2 \mathcal{S} - \mathcal{S}^\top\mathcal{S})$. The latter
definition is tailored to the comparison of models based on residual sums of
squares and hence is better suitable in the boosting context \citep[see][for
details]{Hofner:unbiased:2011}.

%% figure is build at the end of the document
\setkeys{Gin}{width = 0.45\textwidth}
\begin{figure}[ht]
  \centering
  \includegraphics{graphics/fig-bolsx1}
  \hfill
  \includegraphics{graphics/fig-bolsx2}
  \caption{Examples for linear and categorical effects estimated using \R{bols()}:
    Linear effect (left; $f(x_1) = 0.5 x_1$) and categorical effect (right;
    $f(x_2) = 0 x_2^{(1)} - 1 x_2^{(2)} + 0.5 x_2^{(3)} + 3 x_2^{(4)}$). The
  effects can be estimated with base-learners \R{bols(x1)} and \R{bols(x2)},
  where \R{x1} is a continuous covariate and \R{x2} is a categorical covariate
  coded as \R{factor} in \textsf{R}. \label{fig:expl_bols}}
\end{figure}

Table~\ref{tab:bols} shows some calls to \R{bols()} and gives the resulting type
of effect. To gain more information on a specific base-learner it is possible to
inspect the base-learners in \textbf{mboost} using the function \R{extract()} on
a base-learner or a boosting model. With the function one can extract, for
example, the design matrices, the penalty matrices and many more characteristics
of the base-learners. To extract, for example, the design matrix that results
from a linear base-learner of a factor \R{z}, say \R{z <- factor(1:3)}, we can
use
\begin{Schunk}
\begin{Sinput}
R> extract(bols(z))
\end{Sinput}
\begin{Soutput}
  (Intercept) z2 z3
1           1  0  0
2           1  1  0
3           1  0  1
attr(,"assign")
[1] 0 1 1
attr(,"contrasts")
attr(,"contrasts")$z
[1] "contr.treatment"
\end{Soutput}
\end{Schunk}
Thus, we see that base-learners of factors use treatment contrasts per default.
For a detailed instruction on how to use the function see
the manual for \R{extract()} and especially the examples therein.

\begin{table}[h!]
  \centering
  \caption{Some examples of effects that result from \R{bols()}}
  \label{tab:bols}
  \begin{tabular}{lp{0.5\linewidth}}
    \toprule
    Call & Type of Effect\\
    \midrule
    \R{bols(x)} & linear effect: $\x^\top\bv$ with $\x^\top = (1, x)$\\
    \R{bols(x, intercept = FALSE)} & linear effect without intercept:
    $\beta \cdot x$\\
    \R{bols(z)} & OLS fit with factor $z$ (i.e., linear effect
    after dummy coding)\\
    \R{bols(z, df = 1)} & ridge-penalized OLS fit with one degree of freedom
    and factor $z$; If $z$ is an ordered factor a difference penalty is used
    instead of the ridge penalty.\\
    \R{bols(x1, x2, x3)} & one base-learner for three variables
    (group-wise selection):\newline $\x^\top\bv$ with $\x^\top = (1, x_1, x_2, x_3)$\\
    \R{bols(x, by = z)} & interaction: $\x^\top\bv \cdot z$ (with continuous
    variable $z$). If $z$ is a factor, a separate effect is estimated for each
    factor level; Note that in this case, the main effect needs to be specified
    additionally via \R{bols(x)}.\\
    \bottomrule
  \end{tabular}
\end{table}

\paragraph{Smooth Effects}\hfill\\

\noindent The \R{bbs()} base-learner\footnote{the name refers to B-splines with
  penalty, hence the second b} allows the definition of smooth effects (i.e.,
functions of the covariate that are required to be sufficiently smooth, see
Figure~\ref{fig:expl_bbs}) based on B-splines \citep{deBoor:splines:1978} with
difference penalty \citep[i.e., P-splines, cf.][]{eilers1996}. P-splines can be
seen as a versatile modeling tool for smooth effects. A wide class of effects
can be specified using P-splines. Examples include (a) smooth effects, (b)
bivariate smooth effects (e.g., spatial effects), (c) varying coefficient terms,
(d) cyclic effects (= periodic effects) and many more. Two examples of smooth
function estimates fitted using a \R{bbs()} base-learner are given in
Figure~\ref{fig:expl_bbs}. The call to a P-spline base-learner is as follows:
\begin{Schunk}
\begin{Sinput}
  bbs(..., by = NULL, knots = 20, boundary.knots = NULL,
      degree = 3, differences = 2, df = 4, lambda = NULL,
      center = FALSE, cyclic = FALSE)
\end{Sinput}
\end{Schunk}
As for all base-learners, the variables that correspond to the base-learner can
be specified in the \,`\R{\ldots}'\, argument. Usually, only one variable is
specified here for smooth effects, and at maximum two variables are allowed for
\R{bbs()}. Varying coefficient terms $f(x) \cdot z$ can be specified using
\R{bbs(x, by = z)}. In this case, $x$ is called the effect modifier of the
effect of $z$. The \R{knots} argument can be used to specify the number of
equidistant knots, or the positions of the interior knots. In case of two
variables, one can also use a named list to specify the number or the position
of the interior knots for each of the variables separately. For an example of
the usage of a named list see Section~\ref{sec:model_building}. The location of
boundary knots (default: range of the data) can be specified using
\R{boundary.knots}. Usually, no user-input is required here. The only exception
is given for cyclic splines (see below). The degree of the B-spline bases
(\R{degree}) and the order of the difference penalty (\R{differences}; $\in \{0,
1, 2, 3\}$) can be used to specify further characteristics of the spline
estimate. The latter parameter specifies the type of boundary effect. For
\R{differences = 2}, for example, deviations from a linear function are
penalized. In the case of first order differences, deviations from a constant
are subject to penalization. The smoothness of the base-learner can be specified
using degrees of freedom (\R{df}) or the smoothing parameter (\R{lambda})\footnote{If
  \R{lambda} is specified in \R{bbs()}, \R{df} is always ignored.}.

%% figure is build at the end of the document
\setkeys{Gin}{width = 0.45\textwidth}
\begin{figure}[ht]
  \centering
  \includegraphics{graphics/fig-bbsx1}
  \hfill
  \includegraphics{graphics/fig-bbsx2}
  \caption{Examples for smooth effects estimated using \R{bbs(x1)} (left) and
  \R{bbs(x2, knots = quantile(x2, c(0.25, 0.5, 0.75)), df = 5)} (right). Hence,
  the left P-spline uses the default values of 20 equidistant inner knots and 4
  degrees of freedom, while the right P-spline estimate is derived with 3 inner
  knots placed at the quartiles and 5 degrees of freedom. True effects:
  $f_1(x_1) = 3 \cdot \sin(x_1)$ (left) and $f_2(x_2) = x_2^2$
  (right). \label{fig:expl_bbs}}
\end{figure}

An issue in the context of P-splines is that one cannot make the degrees of
freedom arbitrary small. A polynomial of order \R{differences - 1} always
remains unpenalized (i.e., the so-called null space). As we usually apply second
order differences, a linear effect (with intercept) remains unpenalized and thus
$\df \geq 2$ for all smoothing parameters. A solution is given by a P-spline
decomposition
\citep[see][]{Kneib:Hothorn:Tutz:modelchoice:2009,Hofner:unbiased:2011}. The
smooth effect $f(x)$ is decomposed in the unpenalized polynomial and a smooth
deviation from this polynomial. For differences of order 2 (= default), we thus
get:
\begin{equation}\label{eq:decomp}
  f(x) = \underbrace{\beta_0 + \beta_1 x}_{\text{unpenalized polynomial}} +
  \underbrace{f_{\text{centered}}(x)}_{\text{smooth deviation}}
\end{equation}
The unpenalized polynomial can then be specified using \R{bols(x, intercept =
  FALSE)} and the smooth deviation is obtained by \R{bbs(x, center = TRUE)}.
Additionally, it is usually advised to specify an explicit base-learner for the
intercept (see Section~\ref{sec:model_building}).

A special P-spline base-learner is obtained if \R{cyclic = TRUE}. In this case,
the fitted values are forced to coincide at the boundary knots and the function
estimate is smoothly joined (see Figure~\ref{fig:expl_cyclic}). This is
especially interesting for time-series or longitudinal data were smooth,
periodic functions should be modeled. In this case it is of great importance
that the boundary knots are properly specified to match the points were the
function should be joined (due to subject matter knowledge).

An non-exhaustive overview of some usage scenarios for \R{bbs()} base-learners
is given in Table~\ref{tab:bbs}.

\begin{table}[h!]
  \centering
  \caption{Some examples of effects that result from \R{bbs()}}
  \label{tab:bbs}
  \begin{tabular}{p{0.5\linewidth}p{0.45\linewidth}}
    \toprule
    Call & Type of Effect\\
    \midrule
    \R{bbs(x, by = z)} & varying coefficient: $f(x) \cdot z = \beta(x)z$ (with
    continuous variable $z$). If $z$ is a factor, a separate smooth effect is
    estimated for each factor level; Note that in this case, the main effect
    needs to be specified additionally via \R{bbs(x)}.\\
    \R{bbs(x, knots = 10)} & smooth effect with 10 inner knots  \\
    \R{bbs(x, boundary.knots = c(0, 2 * pi),}\newline\hphantom{\R{bbs(}}\R{cyclic = TRUE)} &
    periodic function with period $2\pi$\\
    \R{bbs(x, center = TRUE, df = 1)} & P-spline decomposition (\R{center =
      TRUE}), which is needed to specify arbitrary small \R{df} for P-splines\\
    \bottomrule
  \end{tabular}
\end{table}

%% figure is build at the end of the document
\setkeys{Gin}{width = 0.45\textwidth}
\begin{figure}[ht]
  \centering
  \includegraphics{graphics/fig-cyclic1}
  \hfill
  \includegraphics{graphics/fig-cyclic2}
  \caption{Example for cyclic splines. The unconstrained effect in the left figure
    is fitted using \R{bbs(x, knots = 12)}, the cyclic effect in the right is
    estimated using \R{bbs(x, cyclic = TRUE, knots = 12, boundary.knots = c(0,
      2*pi))}. True effect: $f(x) = \sin(x)$.}.\label{fig:expl_cyclic}
\end{figure}


\paragraph{Smooth Surface Estimation}\hfill\\

\noindent An extension of P-splines to two dimensions is given by bivariate
P-splines. They allow to fit spatial effects and smooth interaction surfaces. An
example of a two dimensional function estimate is given in
Figure~\ref{fig:expl_bspatial}. For further details on bivariate P-splines in
the boosting context see \citet{Kneib:Hothorn:Tutz:modelchoice:2009}. The
effects can be obtained from a call to
\begin{Schunk}
\begin{Sinput}
  bspatial(..., df = 6)
\end{Sinput}
\end{Schunk}
To specify two dimensional smooth effects, the \,`\R{\ldots}'\, argument
requires two variables, i.e., a call such as \R{bspatial(x, y)}. Note that
\R{bspatial()} is just a wrapper to \R{bbs()} with redefined degrees of
freedom\footnote{Note that \R{df = 4} was changed to \R{df = 6} in
  \textbf{mboost} 2.1-0.}. Thus, all arguments from \R{bbs()} exist and can be
used for \R{bspatial()}. An example is given in
Section~\ref{sec:model_building}.

%% figure is build at the end of the document
\setkeys{Gin}{width=0.45\textwidth}
\begin{figure}[ht]
  \centering
  \includegraphics{graphics/fig-bspatial1} \hfill
  \includegraphics{graphics/fig-bspatial2}
  \caption{Example for interaction surface fitted using \R{bspatial()}. Two
    displays of the same function estimate are presented (levelplot: left;
    perspective plot: right). True function: $f(x_1, x_2) = \sin(x_1) \cdot
    \sin(x_2)$. \label{fig:expl_bspatial}}
\end{figure}

\paragraph{Random Effects}\hfill\\
\\
\noindent To specify random intercept or random slope terms in \textbf{mboost},
one can use a random effects base-learner. Random effects are used to model
subject specific effects (in contrast to the usual -- fixed -- effects which can
be seen as population specific effects). Usually, one is only interested in
modeling the increased variability but not in the effect estimate of a specific
individual as this effect is not transferable. For a comprehensive overview of
random effects models we refer to \citet{PinheiroBates:2000}. Random effects are
modeled as ridge-penalized group-specific effects. One can show that these
coincide with standard random effects. The ridge penalty parameter is then
simply the ratio of the error variance $\sigma^2$ and the random effects
variance $\tau^2$. Hence, the penalty parameter is inversely linked to $\tau^2$
(and the degrees of freedom are directly linked to $\tau^2$). As for all
base-learners, we initially specify a small value for the degrees of freedom,
i.e., we set a small random effects variance (relative to the residual
variance). By repeatedly selecting the base-learner, the boosting algorithm can
then adapt to the ``true'' random effects variance. For more details see
\citet[Web Appendix]{Kneib:Hothorn:Tutz:modelchoice:2009}. We can specify random
effects base-learners with a call to
\begin{Schunk}
\begin{Sinput}
  brandom(..., df = 4)
\end{Sinput}
\end{Schunk}
As \R{brandom()} is just a wrapper of \R{bols()} with redefined degrees of
freedom, one can use all arguments that are available for \R{bols()}. To specify
a random intercept for a group variable, say \R{id}, we simply call
\R{brandom(id)} (and could additionally specify the \emph{initial} error
variance via \R{df} or \R{lambda}). To specify a random slope for another
variable, say \R{time}, within the groups of \R{id}, one can use \R{brandom(id,
  by = time)}, i.e., the first argument determines the grouping (as for the
random intercept) and the \R{by} argument then determines the variable for which
the effect is allowed to vary between the groups. In the notation of
\textbf{nlme} \citep{nlme2012} and \textbf{lme4} \citep{lme4} the random
intercept would be specified as \R{(1 | id)} and the random slope would be
specified as \R{(time - 1 | id)}, i.e., a random slope without random intercept
term.

\paragraph{Additional Base-learners in a Nutshell}\hfill\\

\noindent Tree-based base-learner (per default stumps) can be included in the
model using the \R{btree()} base-learner
\citep{Hothorn:2006:JCGS,Hothorn+Buehlmann+Kneib+Schmid+Hofner:mboost:2010}.
Note that this is the only base-learner which is not fitted using penalized
least squares. Radial basis functions (e.g., for spatial effects) can be
specified using the base-learner \R{brad()}. Details on this base-learner are
given in \citep{Hofner:PhDThesis:2011}. Monotonic effects of continuous or
ordered categorical variables can be estimated using the \R{bmono()}
base-learner which can also estimate convex or concave effects. Details on
monotonic effect estimation and examples of the estimation of boosting models
with monotonic effects are given in \citet{Hofner:monotonic:2011} in the context
of species distribution models. The base-learner \R{bmrf()} implements Markov
random fields, which can be used for the estimation of spatial effects of
regions \citep{Sobotka:2010,mayr:gamboostlss:2011}. With the base-learner
\R{buser()}, one can specify arbitrary base-learners with quadratic penalty.
This base-learner is dedicated to advanced users only. Additionally, special
concatenation operators for expert users exist in \textbf{mboost} that all
combine two or more base-learners to a single base-learner: \verb_%+%_
additively joins two or more arbitrary base-learners to be treated as one group,
\verb_%X%_ defines a tensor product of two base-learners and
\verb_%O%_ implements the Kronecker product of two base-learners. In all
cases the degrees of freedom increase compared to the degrees of freedom of the
single base-learners (additively in the first, and multiplicatively in the
second and third case). For more details on any of these base-learners and for
some usage examples see the manual.

\subsubsection{Building a Model -- or: How to Combine Different Base-learners}
\label{sec:model_building}

In general, the choice of effect for a given covariate has to be based on the
subject matter knowledge of the involved researchers. There is no general rule
when to use a certain effect. In order to allow researchers to make a decision
on which effect to use we exemplified the different modeling types in graphical
displays (see Figures~\ref{fig:expl_bols} to \ref{fig:expl_bspatial}).

Once this decision is made, it is relatively easy to estimate the model using
the described boosting methodology. Base-learners can be combined in a simple
manner to form the desired model. They are combined in the formula as a sum of
base-learners to specify the desired model. We will show this using a simple toy
example:
\begin{Schunk}
\begin{Sinput}
R> m1 <- gamboost(y ~ bols(x1) + bbs(x2) + bspatial(s1, s2) + brandom(id),
+                 data = example)
\end{Sinput}
\end{Schunk}
In this case, a linear effect for \R{x1}, a smooth (P-spline) effect for \R{x2},
a spatial effect for \R{s1} and \R{s2} are specified together with a random
intercept for \R{id}. This model could be further refined and expanded as shown
in the following example:
\begin{Schunk}
\begin{Sinput}
R> m2 <- gamboost(y ~ bols(int, intercept = FALSE) +
+                     bols(x1, intercept = FALSE) +
+                     bols(x2, intercept = FALSE) +
+                     bbs(x2, center = TRUE, df = 1) +
+                     bspatial(s1, s2, knots = list(s1 = 10, s2 = 20)) +
+                     brandom(id) + brandom(id, by = x1),
+                 data = example)
\end{Sinput}
\end{Schunk}
Now, with \R{example\$int = rep(1, length(y))}, we specify a separate intercept
in the model. In the first formula (\R{m1}), the intercept was implicitly
included in the base-learners. Now we allow the boosting algorithm to explicitly
choose and update solely the intercept. Note that therefore we need to remove
the implicit intercept from the base-learner for \R{int} (\R{intercept = FALSE})
as otherwise we would get a linear base-learner with \emph{two} intercepts which
has no unique solution. We can now also remove the intercept from the
base-learner of \R{x1}. This leads to a base-learner of the form $x_1\beta_1$.
For the smooth effect of \R{x2} we now use the decomposition~\eqref{eq:decomp}.
Hence, we specify the unpenalized part as a linear effect without intercept (the
intercept is already specified in the formula) and the smooth deviation from the
linear effect (with one degree of freedom). Now the boosting algorithm can
choose how \R{x2} should enter the model: (i) not at all, (ii) as a linear
effect, (iii) as a smooth effect centered around zero, or (iv) as the
combination of the linear effect and the smooth deviation. In the latter case we
essentially get the same result as from \R{bbs(x2)}. The spatial base-learner in
the second formula (\R{m2}) explicitly specifies the numbers of knots in the
direction of \R{s1} (10 inner knots) and \R{s2} (20 inner knots). This is
achieved by a named list where the names correspond to the names of the
variables. In total we get $10 \cdot 20 = 200$ inner knots. Usually, one
specifies equal numbers of knots in both directions, which requires no named
lists. Instead one can simply specify, e.g., \R{knots = 10}, which results in 10
inner knots per direction (i.e., 100 inner knots in total). Note that, as for
the smooth, univariate base-learner of \R{x2} one could also specify a
decomposition for the spatial base-learner:
\begin{Schunk}
\begin{Sinput}
    bols(int, intercept = FALSE) +
    bols(s1, intercept = FALSE) + bols(s2, intercept = FALSE) +
    bols(s1, by = s2, intercept = FALSE) +
    bspatial(s1, s2, center = TRUE, df = 1)
\end{Sinput}
\end{Schunk}
where \R{bols(s1, by = s2, intercept = FALSE)} specifies the interaction of
\R{s1} and \R{s2} (without intercept). Finally, we added a random slope for \R{x1} (in the groups of \R{id}) to \R{m2}.
Note that the groups are the argument of the base-learner and the slope variable
is specified via the \R{by} argument.

\paragraph{Case Study (ctd.): Prediction of Body Fat}\hfill\\

% case2: Case study non-linear base-learner

\noindent Until now, we only included linear effects in the prediction formula
for body fat of women. It might be of interest to evaluate if there exists also
a non-linear relationship between some of the predictors and the DXA measurement
of body fat. To investigate this issue, we fit a model with the same predictors
as in \cite{garcia} but without assuming linearity of the effects. We apply
\R{gamboost()} with the P-spline base-learner \R{bbs()} to incorporate smooth
effects.

<<mod_gamboost>>=
## now an additive model with the same variables as lm1
gam1 <- gamboost(DEXfat ~ bbs(hipcirc) + bbs(kneebreadth) + bbs(anthro3a),
                 data = bodyfat)
@
Using \R{plot()} on a \R{gamboost} object delivers automatically the partial
effects of the different base-learners:
<<plot_gamboost>>=
par(mfrow = c(1,3))    ## 3 plots in one device
plot(gam1)             ## get the partial effects
@

\setkeys{Gin}{width = \textwidth}
\begin{figure}[ht]
  \centering
<<partialplots, width = 10, height = 5, echo = false, fig = true>>=
## same as plot from chunk "plot_gamboost" but with better margins
par(mfrow=c(1,3))
par(mar = c(4, 4, 0, 0) + 0.1)
plot(gam1)
@
\caption{Partial effects of three anthropometric measurements on the body fat of
  women. \label{fig:partialplot}}
\end{figure}
\setkeys{Gin}{width = 0.45\textwidth}

\noindent From the resulting Figure~\ref{fig:partialplot}, it seems as if in the
center of the predictor-grid (where most of the observations lie), the
relationship between these three anthropometric measurements and the body fat is
quite close to a linear function. However, at least for \R{hipcirc}, it seems
that there are indications of the existence of a non-linear relationship for
higher values of the predictor.

Alternatively, one could apply decomposition~\eqref{eq:decomp} for each of the 3
base-learners, as described in Section~\ref{sec:model_building}, to distinguish
between modeling alternatives. In this case, we would have a more rigorous
treatment of the decision between (purely) linear and non-linear effects if we
stop the algorithm at an appropriate iteration $m_\text{stop}$.

\subsection{Early Stopping to Prevent Overfitting}
\label{sec:cvrisk}

As already discussed in Section~\ref{sec:theory}, the major tuning parameter of
boosting is the number of iterations $m_{\text{stop}}$. To prevent overfitting
it is important that the optimal stopping iteration is carefully chosen. Various
possibilities to determine the stopping iteration exist. One can use information
criteria such as AIC\footnote{see \R{?AIC.boost} for further details} to find
the optimal model. However, this is usually not recommended as AIC-based
stopping tends to overshoot the optimal $m_{\text{stop}}$ dramatically
\citep[see][]{Hast:comment:2007,Mayr:mstop:2012}. Instead, it is advised to use
cross-validated estimates of the empirical risk to choose an appropriate number
of boosting iterations. This approach aims at optimizing the prognosis on new
data. In \textbf{mboost} infrastructure exists to compute bootstrap estimates,
k-fold cross-validation estimates and sub-sampling estimates. The main function
to determine the cross-validated risk is
\begin{Sinput}
  cvrisk(object, folds = cv(model.weights(object)),
         papply = mclapply)
\end{Sinput}
In the simplest case, the user only needs to specify the fitted boosting model
as \R{object}. If one wants to change the default cross-validation method
(25-fold bootstrap) the user can specify a weight matrix that determines the
cross-validation samples via the \R{folds} argument. This can be done either by
hand or using the convenience function \R{cv()} (see below). Finally, the user
can specify a function of the \R{lapply} ``type'' to \R{papply}. Per default
this is either \R{mclapply} for parallel computing if package \textbf{multicore}
\citep{Urbanek:2011} is available, or the code is run sequentially using
\R{lapply}. Alternatively, the user can apply other parallelization methods such
as \R{clusterApplyLB} \citep[package \textbf{snow}][]{Tierny:Snow:2011} with
some further effort for the setup (see \R{?cvrisk}).

The easiest way to set up a variety of weight matrices for cross-validation is
the function
\begin{Sinput}
  cv(weights, type = c("bootstrap", "kfold", "subsampling"),
     B = ifelse(type == "kfold", 10, 25))
\end{Sinput}
One simply specifies the weights of the originally fitted model (e.g. using the
function \R{model.weights()} on the model) and the \R{type} of cross-validation.
This can be either \R{"bootstrap"} (default), \R{"kfold"} (k-fold
cross-validation) or \R{"subsampling"}\footnote{the percentage of observations
  to be included in the learning samples for subsampling can be specified using
  a further argument in \R{cv()} called \R{prob}. Per default this is 0.5.}. The
number of cross-validation replicates, per default, is chosen to be 10 for
k-fold cross-validation and 25 otherwise. However, one can simply specify
another number of replicates using the argument \R{B}.

To extract the appropriate number of boosting iterations from an object returned
by \R{cvrisk()} (or \R{AIC()}) one can use the extractor function \R{mstop()}.
Once an appropriate number of iterations is found, we finally need to stop the
model at this number of iterations. To increase or reduce the number of boosting
steps for the model \R{mod}, one can use the indexing / subsetting operator
directly on the model:
\begin{Schunk}
\begin{Sinput}
  mod[i]
\end{Sinput}
\end{Schunk}
where \R{i} is the number of boosting iterations. Attention, the subset operator
differs in this context from the standard \textsf{R} behavior as it
\emph{directly changes the model} \R{mod}. Hence, there is no need to save
\R{mod} under a new name. This helps to reduce the memory footprint. Be aware
that even if you call something like
\begin{Schunk}
\begin{Sinput}
  newmod <- mod[10]
\end{Sinput}
\end{Schunk}
you will change the boosting iteration of \R{mod}! Even more, if you now change
\R{mstop} for \R{newmod}, the model \R{mod} is also changed (and vice versa)!
This said, the good news is that nothing gets lost. If we reduce the model to a
lower value of $m_{\text{stop}}$, the additional boosting steps are kept
internally in the model object. Consider as an example the following scenario:
\begin{itemize}
\item We fit an initial model \R{mod} with \R{mstop = 100}.
\item We call \R{mod[10]}, which sets \R{mstop = 10}.
\item We now increase \R{mstop} to 40 with a call to \R{mod[40]}.
\end{itemize}
This now requires \emph{no re-computation} of the model as internally everything
was kept in storage. Again the warning, if we now extract anything from the
model, such as \R{coef(mod)}, we get the characteristics of the model with 40
iterations, i.e., here the coefficient estimates from the 40th boosting
iteration.

\paragraph{Case Study (ctd.): Prediction of Body Fat}\hfill\\

\noindent Until now, we used the default settings of \R{boost\_control()} with
\R{mstop = 100} for all boosted models. Now we want to optimize this tuning
parameter with respect to predictive accuracy, in order get the best prediction
for body fat. Note that tuning \R{mstop} also leads to models including only the
most informative predictors as variable selection is carried out simultaneously.
We therefore first fit a model with all available predictors and then tune
\R{mstop} by 25-fold bootstrapping. Using the \R{baselearner} argument in
\R{gamboost()}, we specify the default base-learner which is used for each
variable in the formula for which no base-learner is explicitly specified.
<<mod_gamboost2>>=
## every predictor enters the model via a bbs() base-learner (i.e., as smooth effect)
gam2 <- gamboost(DEXfat ~ ., baselearner = "bbs", data = bodyfat,
                 control = boost_control(trace = TRUE))
@
<<mod_gamboost2_refit, echo = false, results = hide>>=
## refit model to supress trace in cvrisk
## this is only required for the output in the PDF
gam2 <- gamboost(DEXfat ~ ., baselearner = "bbs", data = bodyfat)
@
<<mod_gamboost2_ctd>>=
set.seed(123)          ## set seed to make results reproducible
cvm <- cvrisk(gam2)    ## default method is 25-fold bootstrap cross-validation
## if package 'multicore' is not available this will trigger a warning
cvm
plot(cvm)    ## get the paths
@

\setkeys{Gin}{width = .45\textwidth}
\begin{figure}[ht]
  \centering
<<cvpaths, width = 5, height=5, echo = false, results = hide, fig = true>>=
## same as plot from chunk "mod_gamboost2_ctd"with better margins
par(mar = c(4, 4, 0, 0) + 0.1)
plot(cvm, main = "")
@
\caption{Cross-validated predictive risk with 25-fold bootstrapping.\label{fig:cvpaths}}
\end{figure}

\noindent The plot displays the predictive risk on the 25 bootstrap samples for
$m_{\text{stop}} = 1,...,100$ (see Figure~\ref{fig:cvpaths}). The optimal
stopping iteration is the one minimizing the average risk over all 25 samples.
We can extract this iteration via
<<mstop>>=
mstop(cvm)    ## extract the optimal mstop
@
<<mstop_ctd, results = hide>>=
gam2[ mstop(cvm) ]    ## set the model automatically to the optimal mstop
@
<<mstop_save, echo = false, results = hide>>=
m_cvm <- mstop(cvm)
@
We have now reduced the model of the object \R{gam2} to the one with only \Sexpr{m_cvm}
boosting iterations, without further assignment. However, as pointed out above,
the other iterations are not lost. To check which variables are now included in
the additive predictor we again use the function \R{coef()}:
<<mod_gamboost2_refit2, echo = false, results = hide>>=
## refit model to get trace again
## this is only required for the output in the PDF
gam2 <- gamboost(DEXfat ~ ., baselearner = "bbs", data = bodyfat,
                 control = boost_control(trace = TRUE))
gam2[ mstop(cvm) ]
@
<<coef_gamboost_save, echo = false, results = hide>>=
n_coef <- length(coef(gam2))
coef_string <-
    paste(c("zero", "one", "two", "three", "four",
            "five", "six", "seven", "eight", "nine")[n_coef + 1],
          if (n_coef == 1) "predictor" else "predictors")
@
<<coef_gamboost>>=
names(coef(gam2)) ## displays the selected base-learners at iteration "mstop(cvm)"
## To see that nothing got lost we now increase mstop to 1000:
gam2[1000, return = FALSE]  # return = FALSE just supresses "print(gam2)"
@
Although we earlier had reduced to iteration \Sexpr{m_cvm}, the fitting algorithm started
at iteration 101. The iterations \Sexpr{m_cvm+1}--100 are not re-computed.
<<coef_gamboost2>>=
names(coef(gam2))  ## displays the selected base-learners, now at iteration 1000
@
The stopping iteration \R{mstop} is the main tuning parameter for boosting and
controls the complexity of the model. Larger values for \R{mstop} lead to larger
and more complex models, while for smaller values the complexity of the model is
generally reduced. In our example the final model at iteration 1000 includes all
available variables as predictors for body fat, while the model at the optimal
iteration \Sexpr{m_cvm} included only \Sexpr{coef_string}. Optimizing the stopping iteration usually
leads to selecting the most influential predictors.

\subsection{Specifying the Fitting Problem: The \R{family}}
\label{sec:family}

The list of implemented families in \textbf{mboost} is diverse and wide-ranging.
At the time of writing this paper, the user has access to sixteen different
families. An overview is given in Table~\ref{tab:families}. A family (most
importantly) implements the loss function $\rho$ and the corresponding negative
gradient. A careful specification of the loss function leads to the estimation
of any desired characteristic of the conditional distribution of the response.
This coupled with the large number of base learners guarantees a rich set of
models that can be addressed by boosting. We can specify the connection between
the response and the covariates in a fairly modular nature such as
\begin{equation*}
    \xi(y|\x) = \hat{f}_1 + \cdots + \hat{f}_P
\end{equation*}
having on the right hand side any desired combination of base learners. On the
left hand side, $\xi(.)$ describes \emph{some} characteristic of the conditional
distribution specified by the \R{family} argument. In the following subsections
we discuss major aspects related to the choice of the family.

\begin{table}[h!]
  \begin{threeparttable}[b]
    \centering
  \caption{An overview on the currently implemented families in \textbf{mboost}.
  \label{tab:families}}
  \begin{tabular}{llllll}
    \toprule
        & continuous response & binary response & count data & ordered response & censored data \\
    \cmidrule{2-6}
     Standard regression & \R{Gaussian} & & & & \\
     Median regression & \R{Laplace} & & & & \\
     Quantile regression& \R{QuantReg}   & & & & \\
     Expectile regression & \R{ExprectReg}  & & & & \\
     Robust regression &  \R{Huber} & & & & \\
     Gamma regression\tnote{a} &  \R{GammaReg} & & & & \\

     Logistic regression& &  \R{Binomial} & & & \\
     AdaBoost classification & &\R{AdaExp}  & & & \\
     AUC regression & &\R{AUC} & & & \\

     Poisson regression & &&\R{Poisson}  & & \\
     Negative binomial model & &&\R{NBinomial}  & & \\

     Proportional odds model & &&&\R{ProppOdds}   & \\

     Proportional hazards model   & &&&  &\R{CoxPH} \\

     Weibull AFT\tnote{b}\;\, model & &&&  &\R{Weibull} \\
     Log-logistic AFT\tnote{b}\;\, model & &&&  &\R{Loglog} \\
     Log-normal AFT\tnote{b}\;\, model & &&&  &\R{Lognormal} \\
    \bottomrule
  \end{tabular}
  \begin{tablenotes}
  \item[a] for non-negative continuous response
  \item[b] accelerated failure time
  \end{tablenotes}
\end{threeparttable}
 \end{table}

\subsubsection{Families for Continuous Response}\label{sec:famcont}

\begin{figure}[t]
    \subfigure[\R{family = Gaussian()}
    \label{fig:fam_gauss}]{
      \includegraphics[page=1]{graphics/fig-family}
    }
    \subfigure[\R{family = Laplace()}
    \label{fig:fam_laplace}]{
      \includegraphics[page=2]{graphics/fig-family}
    }
    \caption{The loss function allows flexible specification of the link between
      the response and the covariates. The figure on the left hand side
      illustrates the L$_2$ loss (the default in \textbf{mboost}), the figure on
      the right hand side shows the L$_1$ loss function.}
    \label{fig:family1}
\end{figure}

Until now the focus was on the conditional mean of a continuous response which
is the default setting: \R{family = Gaussian()}. In this case, our assumption is
that $Y|\x$ is normally distributed and the loss function is the negative
Gaussian log-likelihood which is equivalent to the $\text{L}_2$ loss
\begin{equation*}
    \rho(y,f) =\frac{1}{2}(y-f)^2
\end{equation*}
(see Figure~\ref{fig:fam_gauss}). A plain \R{Gaussian()} call in the console
returns its definition
\begin{Sinput}
R> Gaussian()
\end{Sinput}
\begin{Soutput}
	 Squared Error (Regression)

Loss function: (y - f)^2
\end{Soutput}
The corresponding negative gradient is simply $(y-f)$ whose definition
can be displayed on the screen via
\begin{Sinput}
R> slot(Gaussian(),"ngradient")
\end{Sinput}
\begin{Soutput}
function (y, f, w = 1)
y - f
\end{Soutput}

If we are interested in the median of the conditional distribution, the
\R{Laplace()} family is the right choice. It implements a distribution free,
median regression approach especially useful for long-tailed error
distributions. In this case, we use the L$_1$ loss defined as
\begin{equation*}
    \rho(y,f) = |y-f|
\end{equation*}
and shown in Figure~\ref{fig:fam_laplace}. Note that the L$_1$ loss is not
differentiable at $y = f$ and the value of the negative gradient at such points
is fixed at zero.

\begin{figure}[t]
    \subfigure[\R{family = Huber()}
    \label{fig:fam_huber}]{
      \includegraphics[page=3]{graphics/fig-family}
    }
    \subfigure[\R{family = QuantReg()}
    \label{fig:fam_quantreg}]{
      \includegraphics[page=4]{graphics/fig-family}
    }
    \caption{The Huber loss function on the left hand side is useful when
      robustness is a concern. In \textbf{mboost} it adaptively changes the
      limit for L$_1$ penalization of outliers when \R{d = NULL} (the default).
      The figure on right hand side illustrates several examples of the check
      function loss with different quantiles (\R{tau = 0.5} is the default).}
    \label{fig:family2}
\end{figure}

A compromise between the L$_1$ and the L$_2$ loss is the Huber loss function
shown in Figure~\ref{fig:fam_huber}. It is defined as
\begin{equation*}
    \rho(y,f; \delta) =
    \begin{cases}
        (y-f)^2/2 & \text{if $|y - f| \le \delta$,}\\
        \delta(|y-f|-\delta /2)   &\text{if $|y - f| > \delta$}
    \end{cases}
\end{equation*}
where the parameter $\delta$ limits the outliers which are subject to absolute
error loss. The Huber loss can be seen as a robust alternative to the L$_2$
loss. The user can either specify $\delta$ subjectively, e.g. \R{Huber(d = 2)},
or leave it adaptively chosen by the boosting algorithm (the default behaviour).
An adaptive specification of $\delta$, proposed by \citet{friedman2001}, means
that each boosting step produces a new $\delta^{[m]}$ matching the actual median
of the absolute values of the residuals, i.e.
\begin{equation*}
    \delta^{[m]} = \text{median}\left(\Bigl|y_i - \hat{f}^{[m-1]}(x_i)\Bigr|, i=1,\ldots,n
    \right).
\end{equation*}

Another alternative for settings with continuous response is modeling
conditional quantiles through quantile regression \citep{Koen2005} --
implemented in \textbf{mboost} with the \R{QuantReg()} family
\citep{Fenske:2011}. The main advantage of quantile regression is (beyond its
robustness towards outliers) that it does not rely on any distributional
assumptions on the response or the error terms. The appropriate loss function
here is the check-function shown in Figure~\ref{fig:fam_quantreg}. For the
special case of the 0.5 quantile both \R{QuantReg(0.5)} and \R{Laplace()} will
lead to median regression. A detailed description of the loss function of
quantile regression is given in the Appendix.

\paragraph{Case Study (ctd.): Prediction of Body Fat}\hfill\\

We reproduce the model of the original publication \citep{garcia}, but instead
of modeling the mean we focus on the median:
<<quantreg_gamboost>>=
## Same model as glm1 but now with QuantReg() family
glm3 <- glmboost(DEXfat ~ hipcirc + kneebreadth + anthro3a, data = bodyfat,
                 family = QuantReg(tau = 0.5), control = boost_control(mstop = 500))
coef(glm3, off2int = TRUE)
@
Comparing the results to those of model \R{glm1} shows that \R{hipcirc} and
\R{anthro3a} have almost the same influence on the mean as on the median, yet,
the magnitude of the effect of \R{kneebreadth} is considerably smaller in
\R{glm3}. One should note that \R{mstop} generally needs to be larger for
quantile regression, as the single updates are smaller than in the mean
regression case. For a discussion see the Appendix.

\subsubsection{Families for Binary Response}
Analogously to Gaussian regression, the probability parameter of a binary
response can be estimated by minimizing the negative binomial log-likelihood
\begin{align}
    \label{eq:binary}
    \rho(y, f) & = -\left[y \:\log\bigl( \pi(f) \bigr) + (1-y)\:\log\bigl(
      1-\pi(f) \bigr)\right] \notag \\
    & = \log(1 + \exp(-2\,\tilde{y}f))
\end{align}
where $\tilde{y} = 2 y -1$ and $\pi(f) = \mathbb{P}(Y = 1| \x )$. For reasons of
computational efficiency the binary response $y \in\{0,1\}$ is converted into
$\tilde{y} = 2y -1$ where $\tilde{y} \in \{-1,1\}$. In
equation~\eqref{eq:binary}, $\tilde{y}f$ are the so-called margin values
(depicted in Figure~\ref{fig:fam_binary}) which are, roughly speaking, the
equivalent of the continuous residuals $y-f$ for the binomial case. This
internal re-coding\footnote{Note that in \textbf{mboost} the response must be
  specified as a binary \R{factor}.} means that the negative binomial
log-likelihood loss (\R{family = Binomial()}) and the exponential loss
(\R{family = AdaExp()}) coincide in their population minimizer
\citep[see][Section 3]{BuhlmannHothorn06}.

Note that the transformation $\tilde{y} = 2y -1$ changes the interpretation of
the effect estimates because now we get the half of the log-odds ratio. One
implication is that the \R{coef()} output is half the estimates that result
from \R{glm()}. This means that the user has to double the coefficients
\emph{manually} in order to get the final standard estimates of a logistic
regression model.

\begin{figure}[t]\centering
    \includegraphics[page=5]{graphics/fig-family}
    \caption{The \R{Binomial} and the \R{AdaExp} families as functions of the
      marginal values $\tilde{y}f$. Since $\tilde{y} \in \{-1,1\}$, a positive
      product between $\tilde{y}$ and half the estimated log-odds ratio $f$
      means correct classification.}
    \label{fig:fam_binary}
\end{figure}

However, \textbf{mboost} automatically doubles the logits prior to the
reverse probability transformation. This means that calling \mbox{\R{predict(fit, newdata,
    type = "response")}} produces the \emph{final} probability estimations. In
addition to the logit model, the user can also estimate probit models using
\R{family = Binomial(link = "probit")}.

Alternatively one can also use the exponential loss function $\rho(y,f) =
\exp(-\tilde{y}f)$ (\R{family = AdaExp()}). This essentially leads to the famous
AdaBoost algorithm by \citet{nr:freund.schapire:1996}. As can be seen in
Figure~\ref{fig:fam_binary}, this loss function is similar to \R{Binomial()}.

\subsubsection{Families for Count Data and Censored Response}

In \textbf{mboost}, currently two families handle data with count response. The
\R{Poisson()} family uses the negative Poisson log-likelihood with the natural
link function $\log(\mu) = \eta$. Alternatively, the negative binomial
distribution can be used to model overdispersed data. The
negative log-likelihood density of this distribution is implemented in
\R{NBinomial(nuirange = c(0, 100))} where the parameter
\R{nuirange} (accounting for overdispersion) is optimized additionally within each boosting iteration $m$. One
simply minimizes the empirical risk $\mathcal{R}$ over the overdispersion
parameter given the current boosting estimate $\hat{f}^{[m]}$ after step~4 of
the component-wise gradient boosting algorithm. A thorough introduction and the
detailed algorithm is given by \citet{Schmid:Multidim:2010}.

Survival models can also be considered in \textbf{mboost}: \R{CoxPH()},
\R{Weibull()}, \R{Loglog()} and \R{Lognormal()} all implement families for
censored data. \R{CoxPH()} implements the Cox proportional hazards model while
the other three families specify accelerated failure time (AFT) models
\citep[see][for further details]{Schmid:Hothorn:AFT-boost}.

\subsubsection{Further Families}

Additionally to the families discussed above, \textbf{mboost} implements some
further families: \R{AUC()} can be used to optimize the area under the ROC
curve, \R{GammaReg()} implements the negative Gamma log-likelihood with
logarithmic link function, \R{ExpectReg()} implements expectile regression
\citep{Sobotka:2010} and \R{PropOdds()} leads to proportional odds models for
ordinal outcome variables \citep{Schmid:2010:PropOdds}.

Despite the wide range of available families, users might wish to implement new
loss functions. One only needs a differentiable loss function and the
corresponding negative gradient $\delta \rho(y, f)\, / \, \delta f$, which both
are independent of the base-learners. Using the constructor function
\R{Family()}, one can then easily define new families and thus new estimation
problems as we show in the Appendix.

\section{Summary}
\label{sec:summary}

The \textsf{R} package \textbf{mboost} offers an easy entry into the world of
boosting. It implements a model-based boosting approach that results in
interpretable structured additive models of the same form most researchers will
feel familiar with. The interfaces of fitting functions are quite similar to
standard implementations like \R{lm()} or \R{glm()} and are hence relatively
easy to use. However, the fitting algorithms of \textbf{mboost} additionally
offer a high flexibility when it comes to the effect type of potential
predictors and the type of risk function to be optimized. There exist a large
number of pre-defined families for various risk functions as well as a large
number of pre-defined base-learners to specify various types of effects. As
\textbf{mboost} has a modular nature, both can be combined in any form as
desired by the user: For many model classes, \textbf{mboost} therefore offers
much more modeling alternatives than the classical fitting algorithms.
Additionally, the user can also easily extend \textbf{mboost} by implementing
new families or base-learners.

As seen in the case study, many functions for the manipulation and extraction of
the results are available. These allow the user to fit, tune and finally
interpret the model. We note that the present tutorial has been designed as an
introduction to the basic functionalities of the \textbf{mboost} package,
highlighting its usage from a practical perspective. Readers who are interested
in further illustrations, as well as in a more technical description of the
\textbf{mboost} package, are referred to the package manual and the vignettes
that are found at \url{http://cran.r-project.org/package=mboost} or in
\textsf{R} via \R{vignette(package = "mboost")}.

\bibliographystyle{abbrvnat}
\bibliography{mboost_tutorial}   % name your BibTeX data base


\clearpage
\section*{Appendix: Building Your Own Family}


Via the constructor function \R{Family()}, in \textbf{mboost} there exists an
easy way for the user to set up new families. The main required arguments are
the \R{loss} to be minimized and the negative gradient (\R{ngradient}) of the
loss. The risk is then commonly defined as the sum of the loss over all
observations.
\begin{Schunk}
\begin{Sinput}
  Family(ngradient, loss = NULL, risk = NULL, offset = function(y, w)
         optimize(risk, interval = range(y), y = y, w = w)$minimum, ...)
\end{Sinput}
\end{Schunk}
% just to end the highlighting of math mode in latex $

We will demonstrate the usage of this function by (re-) implementing the family
to fit quantile regression (the pre-defined family is \R{QuantReg()}). In
contrast to standard regression analysis, quantile regression \citep{Koen2005}
does not estimate the conditional expectation of the conditional distribution
but the conditional quantiles. Estimation is carried out by minimizing the check
function $\rho_{\tau}(\cdot)$:
$$
\rho_{\tau}(y_i - f_{\tau i} ) = \left\{\begin{array}{ll}
            (y_i - f_{\tau i} ) \cdot\tau & \quad   (y_i - f_{\tau i} ) \geq 0 \\
            (y_i - f_{\tau i} ) \cdot(\tau-1) & \quad  (y_i - f_{\tau i} ) <0,
            \end{array} \right.
$$
which is depicted in Figure~\ref{fig:fam_quantreg}. The \R{loss} for our new
family is therefore given as:
<<loss>>=
loss =  function(y, f) tau * (y - f) * ((y - f) >= 0) +
                       (tau - 1) * (y - f) * ((y - f) < 0)
@
The check-function is not differentiable at the point 0. However in practice, as
the response is continuous, we can ignore this by defining:
 $$ - \frac{\partial \rho_{\tau}(y_i, f_{\tau i})}{\partial f}  =
            \left\{\begin{array}{ll}
            \tau &   (y_i - f_{\tau i}) \geq 0 \\
            \tau-1 &  (y_i - f_{\tau i}) <0.
          \end{array} \right.  $$
        The negative gradient of our loss is therefore\footnote{The unused
          weights argument \R{w} is required to exist by \textbf{mboost} when the
          function is (internally) called. It is hence 'specified' as \R{NULL}.}:
<<ngradient>>=
ngradient = function(y, f, w = NULL)  tau * ((y - f) >= 0) +
                    (tau- 1) * ((y - f) < 0)
@
Of further interest is also the starting value for the algorithm, which is
specified via the \R{offset} argument. For quantile regression it was
demonstrated that the offset may be set to the median of the response
\citep{Fenske:2011}. With this information, we can already specify our new family
for quantile regression:
<<OurQR>>=
OurQuantReg <- function(tau = 0.5){    ## function to include dependency on tau
  Family(                              ## applying the Family function
         loss = function(y, f)                               ## loss as above
                    tau       * (y - f) * ((y - f) >= 0) +
                    (tau - 1) * (y - f) * ((y - f) < 0) ,
         ngradient = function(y, f, w = NULL)                ## ngradient as above
                    tau * ((y - f) >= 0) + (tau - 1) * ((y - f) < 0),
         offset = function(y, w = NULL)                      ## median as offset
                    quantile(y, p = 0.5),
         name = "Our new family for quantile regression" )}
OurQuantReg()
@

\paragraph{Case Study (ctd.): Prediction of Body Fat}\hfill\\

To try our new family we go back to the case study regarding the prediction of
body fat. First, we reproduce the model for the median, computed with the
pre-defined \R{QuantReg()} family (see Section~\ref{sec:famcont}), to show that
our new family delivers the same results:
<<ourQR_gamboost>>=
## Same model as glm3 but now with our new family
glm3b <- glmboost(DEXfat ~ hipcirc + kneebreadth + anthro3a, data = bodyfat,
                  family = OurQuantReg(tau = 0.5),
                  control = boost_control(mstop = 500))
identical(coef(glm3b), coef(glm3))
@

To get a better idea of the shape of the conditional distribution we model the
median, and the 0.05 and 0.95 quantiles in a small, illustrative example
containing only the predictor \R{hipcirc}:
<<QR_hipcirc>>=
glm4a <- glmboost(DEXfat ~ hipcirc, family = OurQuantReg(tau = 0.05), data = bodyfat,
                  control = boost_control(mstop = 2000))
glm4b <- glmboost(DEXfat ~ hipcirc, family = OurQuantReg(tau = 0.5),  data = bodyfat,
                  control = boost_control(mstop = 2000))
glm4c <- glmboost(DEXfat ~ hipcirc, family = OurQuantReg(tau = 0.95), data = bodyfat,
                  control = boost_control(mstop = 2000))
@
Note that for different quantiles, fitting has to be carried out separately, as
$\tau$ enters directly in the loss. It is also important that fitting quantile
regression generally requires higher stopping iterations than standard
regression with the $L_2$ loss, as the negative gradients which are fitted to
the base-learners are vectors containing only small values, i.e., $\tau$ and
$1-\tau$.
<<plot_QR_hipcirc>>=
ord <- order(bodyfat$hipcirc)    ## order the data to avoid problems when plotting
plot(bodyfat$hipcirc[ord], bodyfat$DEXfat[ord])                   ## observed data
lines(bodyfat$hipcirc[ord], fitted(glm4a)[ord], lty = 2, lwd = 2) ## 0.05 quantile
lines(bodyfat$hipcirc[ord], fitted(glm4b)[ord], lty = 1, lwd = 2) ## median
lines(bodyfat$hipcirc[ord], fitted(glm4c)[ord], lty = 2, lwd = 2) ## 0.95 quantile
@

\setkeys{Gin}{width = .45\textwidth}
\begin{figure}[ht]
  \centering
<<quantregbodyfat, width = 5, height = 5, echo = false, fig = true>>=
## same plot but with better margings and parameters
par(mar = c(4, 4, 0, 0) + 0.1)
plot(bodyfat$hipcirc[ord], bodyfat$DEXfat[ord])
lines(bodyfat$hipcirc[ord], fitted(glm4a)[ord], lty=2,lwd=2)
lines(bodyfat$hipcirc[ord], fitted(glm4b)[ord], lty=1, lwd=2)
lines(bodyfat$hipcirc[ord], fitted(glm4c)[ord], lty=2, lwd=2)
@
\caption{Resulting quantile regression lines, for the median (solid line) and the 0.95 and 0.05 quantiles (upper and lower dashed lines).  \label{fig:quantreg}}
\end{figure}

\noindent The resulting plot (see Figure~\ref{fig:quantreg}) shows how quantile regression
can be used to get a better impression of the whole conditional distribution
function in a regression setting. In this case, the upper and lower quantiles
are not just parallel lines to the median regression line but adapt nicely to
the slight heteroscedasticity found in this data example: For smaller values of
\R{hipcirc} the range between the quantiles is smaller than for higher values.
Note that the outer quantile-lines can be interpreted as prediction intervals for new
observations \citep{Meins2006, MayrPI}. For more on quantile regression in the context of
boosting we refer to \cite{Fenske:2011}.

\end{document}

%% not part of the document; only used to produce some of the figures;

<<init_figs, fig = false, echo = false>>=
################################################################################
## the following chunks produce the graphics that are NOT part of the bodyfat ##
## example                                                                    ##
################################################################################
@

\SweaveOpts{echo=false, results=hide, fig=true, prefix.string=graphics/fig}

<<init, fig=false>>=
red <- rgb(103,0,31, max = 255) ## define red color
@

<<center_false,width=2.94,height=2.5>>=
## load library mboost
library("mboost")
library("RColorBrewer")
## define colors
cols <- paste(brewer.pal(11,"RdBu")[c(10,2)], "E6", sep="")

## create graphics to show importance of centering
set.seed(1907)
x <- rnorm(50, mean = 5)
y <- rnorm(50, mean = x, sd = 0.3)
## subtract offset as usual in boosting
y <- y - mean(y)

par(ps = 8, cex=1, cex.lab=1, mar=c(3.1,3,0.5,0.1), mgp=c(2,1,0))
xrange <- range(0,x)
plot(y ~ x, xlim = xrange, cex = 1,
     xlab = "x", ylab = "negative gradient in step m = 1",
     pch = 20,
     col = rgb(0.5, 0.5, 0.5, alpha = 0.8))
abline(h = 0, col = "gray", lwd = 0.5)
abline(v = 0, col = "gray", lwd = 0.5)
abline(lm(y ~ x -1), col = cols[1], lwd = 1.5)
points(0, 0, col = cols[2], lwd = 1.5)
points(mean(x), mean(y), col = cols[2], pch = 3, lwd = 1.5)
legend(0.1, 2.35,  legend = c("origin", "center of data", "base-learner"),
       pch = c(21, 3, -1), lty = c(-1, -1, 1),
       col = c(cols[2], cols[2], cols[1]), lwd = 1.5, bg = "white", bty = "n")
@

<<center_true, width = 2.94,height = 2.5>>=
## create graphics to show importance of centering
set.seed(1907)
x <- rnorm(50, mean = 5)
y <- rnorm(50, mean = x, sd = 0.3)
## subtract offset as usual in boosting
y <- y - mean(y)

## figure with centering of x
mx <- mean(x)
xrange <- range(0,x)
x <- x - mean(x)

par(ps = 8, cex=1, cex.lab=1, mar=c(3.1,3,0.5,0.1), mgp=c(2,1,0))
plot(y ~ x, xlim = xrange - mx, cex = 1,
     xlab = "x (centered)", ylab = "negative gradient in step m = 1",
     pch = 20,
     col = rgb(0.5, 0.5, 0.5, alpha = 0.8))
abline(h = 0, col = "gray", lwd = 0.5)
abline(v = 0, col = "gray", lwd = 0.5)
abline(lm(y ~ x -1), col = cols[1], lwd = 1.5)
points(0, 0, col = cols[2], lwd = 1.5)
points(mean(x), mean(y), col = cols[2], pch = 3, lwd = 1.5)
@


<<bolsx1,width=5,height=5>>=
### Simulate some data
set.seed(1907)
n <- 100
x1 <- rnorm(n)
x2 <- as.factor(sample(0:3, n, replace = TRUE))
y <- 0.5 * x1 +  rnorm(n)
mod <- gamboost(y ~ bols(x1), control = boost_control(mstop = 25))
par(mar = c(4, 4, 0, 0) + 0.1)
plot(sort(x1), (0.5 * x1)[order(x1)], type = "l", lwd = 2,
     xlab = expression(x[1]), ylab = expression(f(x[1])))
lines(sort(x1), fitted(mod, which = 1)[order(x1)], col = red,
      lwd = 2, type = "b", pch = 20)
legend("topleft", c("true effect", "model"),
       lty = c(1, 1), pch = c(-1, 20), merge = TRUE,
       lwd = 2,
       col = c("black", red), bty = "n")
@

<<bolsx2, width = 5,height = 5>>=
beta <- c(0, -1, 0.5, 3)
y <- drop(model.matrix(~ x2) %*% beta + rnorm(n, sd = 0.3))
mod <- gamboost(y ~ bols(x2), control = boost_control(mstop = 50))
par(mar = c(4, 4, 0, 0) + 0.1)
betaPred <- coef(mod)[[1]]
betaPred[1] <- betaPred[1] + attr(coef(mod), "offset")
betaTrue <- c(0, -1, 0.5, 3)
plot(1:4, betaPred, type = "n", xaxt = "n",
     xlab = expression(x[2]), ylab = expression(f(x[2])),
     xlim = c(0.5, 4.5), ylim = c(-1.5, 3.5))
axis(1, at = 1:4, labels = expression(x[2]^(1), x[2]^(2), x[2]^(3), x[2]^(4)))
for (i in 1:4)
    lines(i + c(-0.38, 0, 0.38), rep(betaPred[i], each = 3),
          lwd = 2, col = red, type = "b", pch = 20)
for (i in 1:4)
    lines(i + c(-0.4, 0.4), rep(betaTrue[i], each = 2),
          lwd = 2, col = "black")
legend("topleft", c("true effect", "model"),
       pch = c(-1,20), lty = c(1, 1), lwd = 2,
       col = c("black", red), bty = "n")
@



<<bbsx1,width=5,height=5>>=
set.seed(1907)
n <- 100
x1 <- rnorm(n)
x2 <- rnorm(n) + 0.25 * x1
x3 <- as.factor(sample(0:1, n, replace = TRUE))
x4 <- gl(4, 25)
y <- 3 * sin(x1) + x2^2 + rnorm(n)

## specify knot locations for bbs(x2, ...)
knots.x2 <- quantile(x2, c(0.25, 0.5, 0.75))

## fit the model
mod <- gamboost(y ~ bbs(x1, knots = 20, df = 4) +
                bbs(x2, knots = knots.x2, df = 5) +
                bols(x3) + bols(x4))

## plot partial effects
par(mar = c(4, 4, 0, 0) + 0.1)
plot(sort(x1), (3 * sin(x1))[order(x1)], type = "l", lwd = 2,
     xlab = expression(x[1]), ylab =  expression(f[1](x[1])))
lines(sort(x1), fitted(mod, which = 1)[order(x1)], col = red, lwd = 2,
      type = "b", pch = 20)
legend("topleft", c("true effect", "model"),
       lty = c(1, 1), pch = c(-1,20), merge = TRUE, lwd = 2,
       col = c("black", red), bty = "n")
@

<<bbsx2, width = 5,height = 5>>=
par(mar = c(4, 4, 0, 0) + 0.1)
plot(sort(x2), (x2^2)[order(x2)], type = "l", lwd = 2,
     xlab = expression(x[2]), ylab =  expression(f[2](x[2])))
## offset needed to conserve correct level (otherwise center both effects around
## zero to make them comparable)
lines(sort(x2), fitted(mod, which = 2)[order(x2)] + mod$offset, col = red, lwd = 2,
      type = "b", pch = 20)
@



<<cyclic1, width = 5,height = 5>>=
### example for cyclic spline
set.seed(1907)
true_f <- function(x)
    cos(x) + 0.25 * sin(4 * x)
x <- runif(150, 0, 2 * pi)
y <- rnorm(150, mean = true_f(x), sd=0.1)
newX <- seq(0, 2*pi, length = 200)
mod <- gamboost(y ~ bbs(x, knots = 20, degree = 4))
mod[3000]
mod_cyclic <- gamboost(y ~ bbs(x, cyclic=TRUE, knots = 20,
                               degree = 4, boundary.knots=c(0, 2*pi)))
mod_cyclic[3000]

par(mar = c(4, 4, 0, 0) + 0.1)
plot(x,y, ylab = "f(x)",
     pch = 20, xlim = c(0, 7),
     col = rgb(0.5, 0.5, 0.5, alpha = 0.8))
lines(newX, predict(mod, data.frame(x = newX)),
      col = "black", lwd = 2)
lines(newX + 2 * pi, predict(mod, data.frame(x = newX)),
      col = "black", lwd = 2)
legend("bottomleft", c("cyclic = FALSE"),
       lty = c(1), col = c("black"), lwd = 2, bty = "n")
@

<<cyclic2, width = 5,height = 5>>=
par(mar = c(4, 4, 0, 0) + 0.1)
plot(x, y, ylab = "f(x)",
     pch = 20, xlim = c(0, 7),
     col = rgb(0.5, 0.5, 0.5, alpha = 0.8))
lines(newX, predict(mod_cyclic, data.frame(x = newX)),
      col = red, lwd = 2)
lines(newX + 2 * pi, predict(mod_cyclic, data.frame(x = newX)),
      col = red, lwd = 2)
abline(v = 2 * pi, col = "gray", lty = "dashed", lwd = 2)
legend("bottomleft", c("cyclic = TRUE"),
       lty = c(1), col = c(red), lwd = 2, bty = "n")
@



<<bspatial1,width=5,height=5>>=
set.seed(1907)
x1 <- runif(250,-pi,pi)
x2 <- runif(250,-pi,pi)
y <- sin(x1) * sin(x2) + rnorm(250, sd = 0.4)
modspa <- gamboost(y ~ bspatial(x1, x2, knots = list(x1 = 12, x2 = 12)))
# possible to specify different knot mesh for x1 and x2:
# modspa <- gamboost(y ~ bspatial(x1, x2, knots = list(x1 = 12, x2 = 20)))

library(lattice)
library(RColorBrewer)
## set up colorscheme
nCols <- 50
cols <- colorRampPalette(rev(brewer.pal(11, "RdBu")), space = "Lab",
                         interpolate = "spline")(nCols)
## make new data on a grid:
nd <- expand.grid(x1 = seq(-pi, pi, length = 100),
                  x2 = seq(-pi, pi, length = 100))
preds <- predict(modspa, newdata = nd)
breaks <- seq(min(preds), max(preds), length = nCols + 1)
print(levelplot(preds ~ nd$x1 + nd$x2,
          xlab = expression(x[1]),
          ylab = expression(x[2]),
          at = breaks,
          col.regions = cols, cex = 0.7,
          colorkey = list(space = "left", at = breaks, width = 1)))
@

<<bspatial2, width = 5,height = 5>>=
x1 <- x2 <- seq(-pi, pi, length = 50)
nd <- expand.grid(x1 = x1,
                  x2 = x2)
preds <- predict(modspa, newdata = nd)
z <- matrix(preds, ncol = length(x1))
nrz <- nrow(z)
ncz <- ncol(z)
jet.colors <- colorRampPalette(paste(brewer.pal(11,"RdBu")[c(10,2)],
                                     "E6", sep=""))
nbcol <- 10
color <- jet.colors(nbcol)
zfacet <- z[-1, -1] + z[-1, -ncz] + z[-nrz, -1] + z[-nrz, -ncz]
facetcol <- cut(zfacet, nbcol)
par(mar=c(1, 0.1, 0.1, 0.1), mgp = c(1.8,1,0))
persp(x1, x2, z,
      theta = 45, phi = 30, expand = 0.5,
      col = color[facetcol], ticktype = "detailed",
      nticks = 3,
      xlab = "x1", ylab = "x2")
@

<<families, echo=FALSE, results=hide, fig=FALSE>>=
pdf("./graphics/fig-family.pdf", width = 5, height = 4)
par(mar = c(4, 4, 0, 0) + 0.1)
## Gaussian
x <- seq(-2, 2, length = 200)
plot(x,x^2, type = "l", ylab = expression(rho), xlab = expression(y-f))

## Laplace
x <- seq(-2, 2, length = 200)
lossL1 <- function(x, param) abs(x)
mp <- 0.5
dat <- data.frame(x = rep(x, length(mp)),
                  y = as.numeric(sapply(mp, function(z) lossL1(x, param = z))),
                  param = rep(mp, each = length(x)))
dat$tau <- factor(dat$param)
plot(x, dat$y, type = "l", ylab = expression(rho), xlab = expression(y-f))

## Huber
x <- seq(-2, 2, length = 200)
lossH <- function(x, param) ifelse(abs(x) <= param, (1/2) * x^2, param * (abs(x) - param / 2))
mp <- c(0.2, 0.5, 1, 2, 10)
dat <- data.frame(x = x, sapply(mp, function(z) lossH(x, param = z)))
plot(x, dat$X1, type = "l", ylim = c(0,2), ylab = expression(rho),
     xlab = expression(y-f))
legend(0, 2, xjust = 0.5, legend = paste(mp), title = expression(delta),
       lty = 1, col = c(1,3,4,5,6), cex = 0.6, box.col = "gray")
for(i in 1:(length(mp)-1)){
    lines(x, dat[,i+2], col = i+2)
}

## Quantile
x <- seq(-2, 2, length = 200)
lossL1 <- function(x, param) ifelse(x >= 0, param  * x, (param - 1) * x)
mp <- c(5:9 / 10)
dat <- data.frame(x = x, sapply(mp, function(z) lossL1(x, param = z)))
plot(x, dat$X1, type = "l", ylab = expression(rho), ylim = c(0,2),
     xlab = expression(y-f))
legend(0,2, xjust = 0.5, legend = paste(mp), title = expression(tau),
       lty = 1, col = c(1,3,4,5,6), cex = 0.6, box.col = "gray")
for(i in 1:(length(mp)-1)){
    lines(x, dat[,i+2], col = i+2)
}

## Binary
x <- seq(-2, 2, length = 200)
dat <- data.frame(x = x, Binomial = log(1+exp(-2*x), base=2), AdaExp = exp(-x))
plot(x, dat$Binomial, type = "l", lty = 1, col = 1, xlab = expression(tilde(y) * f),
     ylab = expression(rho), ylim = c(0,7.5))
lines(x, dat$AdaExp, lty = 2, col = 2)
legend("topright", legend = c("Binomial","AdaExp"), xjust = 0.5, title = "loss",
       lty = 1:2, col = 1:2, cex = 0.8, bty = "n")
dev.off()
@

\documentclass{article}
\usepackage{amstext}
\usepackage{amsfonts}
\usepackage{hyperref}
\usepackage[round]{natbib}
\usepackage{hyperref}
\usepackage{graphicx}
\usepackage{rotating}
%%\usepackage[nolists]{endfloat}
%%\usepackage{Sweave}

%%\VignetteIndexEntry{mboost Illustrations}
%%\VignetteDepends{mboost, survival, TH.data}

\newcommand{\Rpackage}[1]{{\normalfont\fontseries{b}\selectfont #1}}
\newcommand{\Robject}[1]{\texttt{#1}}
\newcommand{\Rclass}[1]{\textit{#1}}
\newcommand{\Rcmd}[1]{\texttt{#1}}
\newcommand{\Roperator}[1]{\texttt{#1}}
\newcommand{\Rarg}[1]{\texttt{#1}}
\newcommand{\Rlevel}[1]{\texttt{#1}}

\newcommand{\RR}{\textsf{R}}
\renewcommand{\S}{\textsf{S}}
\newcommand{\df}{\mbox{df}}

\RequirePackage[T1]{fontenc}
\RequirePackage{graphicx,ae,fancyvrb}
\IfFileExists{upquote.sty}{\RequirePackage{upquote}}{}
\usepackage{relsize}

\DefineVerbatimEnvironment{Sinput}{Verbatim}{baselinestretch=1}
\DefineVerbatimEnvironment{Soutput}{Verbatim}{fontfamily=courier,
                                              baselinestretch=1,
                                              fontshape=it,
                                              fontsize=\relsize{-1}}
\DefineVerbatimEnvironment{Scode}{Verbatim}{}
\newenvironment{Schunk}{}{}

\renewcommand{\baselinestretch}{1}

\SweaveOpts{engine=R,eps=FALSE}

\hypersetup{%
  pdftitle = {mboost Illustrations},
  pdfsubject = {package vignette},
  pdfauthor = {Torsten Hothorn and Peter Buhlmann},
%% change colorlinks to false for pretty printing
  colorlinks = {true},
  linkcolor = {blue},
  citecolor = {blue},
  urlcolor = {red},
  hyperindex = {true},
  linktocpage = {true},
}

\begin{document}

\setkeys{Gin}{width=\textwidth}

\title{\Rpackage{mboost} Illustrations}

\author{Torsten Hothorn$^1$ and Peter B\"uhlmann$^2$}

\date{}

\maketitle

\noindent$^1$ Institut f\"ur Statistik \\
           Ludwig-Maximilians-Universit\"at M\"unchen \\
           Ludwigstra{\ss}e 33, D-80539 M\"unchen, Germany \\
           \texttt{Torsten.Hothorn@R-project.org}
           \newline

\noindent$^2$ Seminar f{\"u}r Statistik \\
              ETH Z\"urich, CH-8092 Z{\"u}rich, Switzerland \\
              \texttt{buhlmann@stat.math.ethz.ch}
           \newline

\section{Illustrations}

This document reproduces the data analyses presented in
\cite{BuhlmannHothorn06}. For a description of the theory behind
applications shown here we refer to the original manuscript. The results
differ slightly due to technical changes or bugfixes in \textbf{mboost}
that have been implemented after the paper was printed. Most important,
\texttt{gamboost} uses penalized $B$-splines instead of smoothing
splines as baselearners. The computations are much faster
and the results differ only slightly \citep{Schmid+Hothorn:2008b}.


\SweaveOpts{prefix.string=figures/BH}

\paragraph{Illustration: Prediction of total body fat}

<<pkg-attach, echo = FALSE, results = tex>>=
source("setup.R")
@

\citet{garcia2005} report on the development of predictive regression equations
for body fat content by means of $p = 9$ common anthropometric
measurements which were obtained for $n = 71$ healthy German women.
In addition, the women's body composition was measured by
Dual Energy X-Ray Absorptiometry (DXA). This reference method
is very accurate in measuring body fat but finds little applicability
in practical environments, mainly because of high costs and the
methodological efforts needed. Therefore, a simple regression equation
for predicting DXA measurements of body fat is of special interest for the practitioner.
Backward-elimination was applied to select
important variables from the available anthropometrical measurements and
\citet{garcia2005} report a final linear model utilizing
hip circumference, knee breadth and a compound covariate which is defined as
the sum of log chin skinfold, log triceps skinfold and log subscapular skinfold:
<<bodyfat-lm-fit, echo = TRUE>>=
bf_lm <- lm(DEXfat ~ hipcirc + kneebreadth + anthro3a, data = bodyfat)
coef(bf_lm)
@
A simple regression formula which is easy to communicate, such as a
linear combination of only a few covariates, is of special interest in this
application: we employ the \Rcmd{glmboost} function from package
\Rpackage{mboost}
to fit a linear regression model by means of $L_2$Boosting with componentwise
linear least squares. By default, the function \Rcmd{glmboost} fits a linear model (with
initial $m_\text{stop} = 100$ and shrinkage parameter $\nu = 0.1$) by
minimizing squared error (argument \Rcmd{family = Gaussian()} is
the default):
<<bodyfat-glmboost-fit, echo = TRUE>>=
bf_glm <- glmboost(DEXfat ~ ., data = bodyfat, center = TRUE)
@
Note that, by default, the mean of the response variable is used as an
offset in the first step of the boosting algorithm. We center the covariates
prior to model fitting in addition.
As mentioned above, the special form of the base learner,
i.e., componentwise linear least squares,
allows for a reformulation of the boosting fit in terms of a linear combination
of the covariates which can be assessed via
<<bodyfat-glmboost-coef, echo = TRUE>>=
coef(bf_glm)
@

\setkeys{Gin}{height=0.9\textheight, keepaspectratio}
\begin{figure}
\begin{center}
<<bodyfat-oob-plot, echo = FALSE, fig = FALSE, results = hide>>=
load(system.file("cache/bodyfat_benchmarks.rda", package = "mboost"))
aic <- AIC(bf_glm)
pdf("figures/bodyfat_glmboost-bodyfat-oob-plot.pdf", version = "1.4", width = 6, height = 10)
par(mai = par("mai") * c(1, 1, 0.5, 1))
mopt <- grid[which.min(colMeans(boob))]
layout(matrix(1:2, nrow = 2))
perfplot(boob, grid, ylab = "Out-of-bootstrap squared error",
    xlab = "Number of boosting iterations", alpha = 0.05)
abline(h = mean(boobrest), lty = 2)
lines(c(which.min(colMeans(boob)), which.min(colMeans(boob))),
      c(0, min(colMeans(boob))), lty = 2)
points(which.min(colMeans(boob)), min(colMeans(boob)))
plot(aic, ylim = c(3, 5.5))
dev.off()
@
\includegraphics{figures/bodyfat_glmboost-bodyfat-oob-plot.pdf}
\caption{\Robject{bodyfat} data: Out-of-bootstrap squared error for varying
  number of
         boosting iterations $m_\text{stop}$ (top). The dashed horizontal line
         depicts the average out-of-bootstrap error of the linear model
         for the pre-selected variables \Robject{hipcirc},
         \Robject{kneebreadth}
         and \Robject{anthro3a} fitted via ordinary least squares.
         The lower part shows the corrected AIC criterion.
         \label{bodyfat-oob-plot}}
\end{center}
\end{figure}


We notice that most covariates have been used for fitting
and thus no extensive variable selection was performed in the above model.
Thus, we need to investigate how many boosting iterations are appropriate. Resampling
methods such as cross-validation or the bootstrap can be used to estimate
the out-of-sample error for a varying number of boosting iterations. The
out-of-bootstrap mean
squared error for $100$ bootstrap samples is depicted in the upper part of
Figure~\ref{bodyfat-oob-plot}. The plot
leads to the impression that approximately $m_\text{stop} = \Sexpr{mopt}$ would be a sufficient
number of boosting iterations.
In Section~\ref{subsec.stopping}, a corrected version of the Akaike
information criterion (AIC) is proposed for determining the optimal number
of boosting iterations. This criterion attains its
minimum for
<<bodyfat-glmboost-AIC, echo = TRUE>>=
mstop(aic <- AIC(bf_glm))
@
boosting iterations, see the bottom part of
Figure~\ref{bodyfat-oob-plot} in addition.
The coef\-ficients of the linear model with
$m_\text{stop} = \Sexpr{mstop(aic)}$
boosting iterations are
<<bodyfat-glmboost-coef, echo = TRUE>>=
coef(bf_glm[mstop(aic)])
@
<<bodyfat-glmboost-coef-count, echo = FALSE>>=
cf <- coef(bf_glm[mopt])
nsel <- length(cf)
@
and thus \Sexpr{nsel} covariates have been selected for the final
model (intercept equal to zero occurs here for mean centered response and
predictors and hence,
%TORSTEN: PLEASE CHECK
$n^{-1} \sum_{i=1}^n Y_i = \Sexpr{round(bf_glm$offset, 3)}$
%TORSTEN
is the intercept in the uncentered model). Note that
the variables \Robject{hipcirc}, \Robject{kneebreadth} and
\Robject{anthro3a}, which
we have used for fitting a linear model at the beginning of this
paragraph, have been selected by the boosting algorithm as well.


\SweaveOpts{prefix.string=figures/BH}

\paragraph{Illustration: Prediction of total body fat (cont.)}

<<bodyfat-pkg-attach, echo = FALSE, results = tex>>=
source("setup.R")
@

Being more flexible than the linear model which we fitted to the
\Robject{bodyfat} data in Section~\ref{subsec.compLS}, we estimate an additive
model
%for the mean-centered covariates
using the \Rcmd{gamboost} function from \Rpackage{mboost} (first with
pre-specified $m_\text{stop} = 100$
boosting iterations, $\nu = 0.1$ and squared error loss):
<<bodyfat-gamboost-fit, echo = TRUE>>=
bf_gam <- gamboost(DEXfat ~ ., data = bodyfat, baselearner = "bss")
@
The degrees of freedom in the componentwise smoothing spline base procedure
can be defined by the \Rcmd{dfbase} argument, defaulting to $4$.

We can estimate the number of boosting iterations $m_\text{stop}$
using the corrected AIC criterion described in
Section~\ref{subsec.stopping}
%(see Figure~\ref{bodyfat-gamboost-AIC})
via
<<bodyfat-gamboost-prune, echo = TRUE>>=
mstop(aic <- AIC(bf_gam))
@
Similar to the linear regression model, the partial contributions of the covariates
can be extracted from the boosting fit. For the most important variables,
the partial fits are given in Figure~\ref{bodyfat-gamboost-plot} showing some
slight non-linearity, mainly for \Robject{kneebreadth}.

%\begin{figure}
%\begin{center}
%<<bodyfat-gamboost-AIC, echo = FALSE, fig = TRUE, height = 4>>=
%plot(aic, ylim = c(3, 5.5))
%@
%\caption{\Robject{bodyfat} data: Corrected AIC as a function of the number of
%boosting iterations $m_\text{stop}$
%for fitting an additive model via \Rmd{gamboost}. \label{bodyfat-gamboost-AIC}}
%\end{center}
%\end{figure}

\setkeys{Gin}{width=0.95\textwidth, keepaspectratio}
\begin{figure}[t]
\begin{center}
<<bodyfat-gamboost-plot, echo = FALSE, fig = TRUE>>=
bf_gam <- bf_gam[mstop(aic)]
layout(matrix(1:4, ncol = 2))
plot(bf_gam, which = c("hipcirc", "kneebreadth", "waistcirc", "anthro3b"))
@
\caption{\Robject{bodyfat} data: Partial contributions of four covariates
  in an additive model (without centering of estimated functions to mean
  zero).
         \label{bodyfat-gamboost-plot}}
\end{center}
\end{figure}

\SweaveOpts{prefix.string=figures/BH}

\paragraph{Illustration: Prediction of total body fat (cont.)}

<<bodyfat-pkg-attach, echo = FALSE, results = tex>>=
source("setup.R")
library("splines")
indep <- names(bodyfat)[names(bodyfat) != "DEXfat"]
bsfm <- as.formula(paste("DEXfat ~ ", paste("bs(", indep, ")", collapse = " + ", sep = ""), sep = ""))
@

Such transformations and estimation of a corresponding linear model can be
done with the \Rcmd{glmboost} function,
%%\citep[an implementation of the original fractional polynomials approach is
%%available in package \Rpackage{mfp}, see][]{pkg:mfp,Sauerbreietal2006},
where the model formula performs the computations of all transformations by
means of the \Rcmd{bs} (B-spline basis) function from the package
\Rpackage{splines}.
First, we set up a formula transforming each covariate
<<bodyfat-bs, echo = TRUE>>=
bsfm
@
and then fit the complex linear model by using the \Rcmd{glmboost} function with
initial $m_\text{stop} = 5000$ boosting iterations:
<<bodyfat-fpboost-fit, echo = TRUE>>=
ctrl <- boost_control(mstop = 5000)
bf_bs <- glmboost(bsfm, data = bodyfat, control = ctrl)
mstop(aic <- AIC(bf_bs))
@
The corrected AIC criterion (see Section~\ref{subsec.stopping}) suggests to
stop after $m_\text{stop} = \Sexpr{mstop(aic)}$
boosting iterations and the final model selects
$\Sexpr{sum(abs(coef(bf_bs[mstop(aic)])) > 0 )}$ (transformed) predictor
variables. Again, the partial
contributions of each of the $\Sexpr{length(indep)}$ original covariates
can be computed
easily and are shown in Figure~\ref{bodyfat-fpboost-plot} (for the same
variables as in Figure~\ref{bodyfat-gamboost-plot}). Note that the depicted
functional relationship derived from the model fitted above
(Figure~\ref{bodyfat-fpboost-plot}) is qualitatively the same as the one
derived from the additive model (Figure~\ref{bodyfat-gamboost-plot}).
%, which is computationally more demanding.

\setkeys{Gin}{width=0.95\textwidth, keepaspectratio}
\begin{figure}[t]
\begin{center}
<<bodyfat-fpboost-plot, echo = FALSE, fig = TRUE>>=
layout(matrix(1:4, ncol = 2, byrow = TRUE))
par(mai = par("mai") * c(1, 1, 0.5, 1))
cf <- coef(bf_bs[mstop(aic)])
varorder <- c("hipcirc", "waistcirc", "kneebreadth", "anthro3b")
fpartial <- sapply(varorder, function(v) {
    rowSums(predict(bf_bs, which = v))
})
out <- sapply(varorder, function(i) {
    x <- bodyfat[,i]
    plot(sort(x), fpartial[order(x),i],  main = "", type = "b",
         xlab = i, ylab = expression(f[partial]), ylim = max(abs(fpartial)) * c(-1, 1))
    abline(h = 0, lty = 2, lwd = 0.5)
    })
@
\caption{\Robject{bodyfat} data: Partial fits for a linear model fitted to
  transformed covariates using B-splines (without centering of estimated
  functions to mean zero). \label{bodyfat-fpboost-plot}}
\end{center}
\end{figure}


\SweaveOpts{prefix.string=figures/BH}

\paragraph{Illustration: Breast cancer subtypes}

<<pkg-attach, echo = FALSE, results = hide>>=
source("setup.R")

### OK, this once required Biobase and is a very dirty hack...
data("Westbc", package = "TH.data")
westbc <- Westbc
exprs <- function(x) x$assay
pData <- function(x) x$pheno

p <- nrow(exprs(westbc))
n <- ncol(exprs(westbc))
ytab <- table(pData(westbc)$nodal.y)
@

Variable selection is especially important in high-dimensional situations.
As an example, we study a binary classification problem involving $p = \Sexpr{p}$
gene expression levels in $n = \Sexpr{n}$ breast cancer tumor samples
\citep[data taken from][]{west01}. For each sample, a binary response variable
describes the lymph node status (\Sexpr{ytab[1]} negative and
\Sexpr{ytab[2]} positive).
%of the patient is to be explained by the  expression profiles.

The data are stored in form of an \Rclass{exprSet} object \Robject{westbc}
\citep[see][]{BioC2004}
and we first extract the matrix of expression levels and the response variable:
<<west-data-attach, echo = TRUE>>=
### extract matrix of expression levels and binary response
x <- t(exprs(westbc))
y <- pData(westbc)$nodal.y
@
We aim at using $L_2$Boosting for classification, see
Section~\ref{subsubsec.binclass},
with classical AIC based on the binomial log-likelihood for stopping the
boosting iterations. Thus, we first
transform the factor \Robject{y} to a numeric variable with $0/1$ coding:
%(different from the $-1/+1$-encoding in Section~\ref{subsec.binclass}):
<<west-glmboost-ytrans, echo = TRUE>>=
### numeric 0/1 response variable
yfit <- as.numeric(y) - 1
@
The general framework implemented in \Rpackage{mboost} allows us to specify
the negative gradient (the
\Rcmd{ngradient} argument) corresponding to the surrogate loss function,
here the squared error loss implemented as a function \Rcmd{rho}, and a different
evaluating loss function (the \Rcmd{loss} argument), here the negative
binomial log-likelihood, with the \Rcmd{Family} function as follows:
<<west-glmboost-family, echo = TRUE>>=
### L2 boosting for classification with response in 0/1
### and binomial log-likelihood as loss function
### ATTENTION: use offset = 1/2 instead of 0!!!
rho <- function(y, f, w = 1) {
    p <- pmax(pmin(1 - 1e-5, f), 1e-5)
    -y * log(p) - (1 - y) * log(1 - p)
}
ngradient <- function(y, f, w = 1) y - f
offset <- function(y, w) weighted.mean(y, w)
L2fm <- Family(ngradient = ngradient,
               loss = rho, offset = offset)
@
The resulting object (called \Rcmd{L2fm}), bundling the negative gradient,
the loss function and a function for computing an offset term (\Rcmd{offset}),
can now be passed to the \Rcmd{glmboost} function for boosting
with componentwise linear least squares (here initial $m_\text{stop} = 200$
iterations are used):
<<west-glmboost-fit, echo = TRUE, eval = FALSE>>=
### fit a linear model with initial mstop = 200 boosting iterations
ctrl <- boost_control(mstop = 200)
west_glm <- glmboost(x, yfit, family = L2fm, center = TRUE,
                     control = ctrl)
@
<<west-glmboost-fit-lars, echo = FALSE>>=
### fit a linear model with initial mstop = 200 boosting iterations
### and record time
time <- system.time(west_glm <- glmboost(x, yfit, family = L2fm, center = TRUE,
                    control = boost_control(mstop = 200)))[1]
x <- t(exprs(westbc) - rowMeans(exprs(westbc)))
@
Fitting such a linear model to $p = \Sexpr{p}$ covariates for $n = \Sexpr{n}$
observations takes about \Sexpr{formatC(round(time, 1), digits = 2)}
seconds on a medium scale desktop computer
(Intel Pentium 4, 2.8GHz). Thus, this form of estimation and variable
selection is computationally very efficient.

The question how to choose $m_\text{stop}$ can be
addressed by the
classical AIC criterion
%(see Section~\ref{subsec.stopping})
as follows
<<west-glmboost-AIC, echo = TRUE>>=
### evaluate AIC based on binomial log-likelihood for _all_ boosting
### iterations m = 1, ..., mstop = 200
aic <- AIC(west_glm, method = "classical")
### where should one stop? mstop = 108 or 107
mstop(aic)
@
<<west-glmboost-cf, echo = FALSE>>=
cf <- coef(west_glm[mstop(aic)], which = 1:ncol(x))
ps <- length(cf[abs(cf) > 0])
@
where the AIC is computed as -2(log-likelihood) + 2(degrees of freedom) = 2
(evaluating loss) + 2(degrees of freedom), see Formula~(\ref{aicl2}).
The notion of degrees of freedom
is discussed in Section~\ref{subsec.df}.

Figure~\ref{west-glmboost-plot} shows the AIC curve depending on
the number of boosting iterations. When we stop after $m_\text{stop} =
\Sexpr{mstop(aic)}$
boosting iterations, we obtain $\Sexpr{ps}$
genes with non-zero regression coefficients whose standardized values
$\hat{\beta}^{(j)}\sqrt{\widehat{\text{Var}}(X^{(j)})}$
are depicted in the left panel of Figure~\ref{west-glmboost-plot}.
\setkeys{Gin}{width=0.95\textwidth, keepaspectratio}
\begin{figure}
\begin{center}
<<west-glmboost-plot, echo = FALSE, fig = TRUE, width = 7, height = 4>>=
### compute standard deviations of expression levels for each gene
sdx <- sqrt(rowSums((t(x) - colMeans(x))^2)/(nrow(x) - 1))
layout(matrix(1:2, ncol = 2))
cf <- cf * sdx
plot(sort(cf[abs(cf) > 0]), ylab = "Standardized coefficients")
abline(h = 0, lty = 2)
plot(aic, ylim = c(25, 40))
@
\caption{\Robject{westbc} data: Standardized regression coefficients
    $\hat{\beta}^{(j)}\sqrt{\widehat{\text{Var}}(X^{(j)})}$ (left panel) for
    $m_\text{stop} = \Sexpr{mstop(aic)}$ determined from the classical AIC
    criterion shown in the right panel. \label{west-glmboost-plot}}
\end{center}
\end{figure}

Of course, we could also use BinomialBoosting for analyzing the data: the
computational CPU time would be of the same order of magnitude, i.e.,
only a few seconds.


\SweaveOpts{prefix.string=figures/BH}

\paragraph{Illustration: Wisconsin prognostic breast cancer}

<<pkg-attach, echo = FALSE, results = tex>>=
source("setup.R")
n <- sum(complete.cases(wpbc))
p <- ncol(wpbc) - 2
@

Prediction models for recurrence events in breast cancer patients
based on covariates which have been computed from a digitized image of a
fine needle aspirate of breast tissue (those measurements describe
characteristics of the cell nuclei present in the image) have been studied
by \citet{street1995} \citep[the data is part of the UCI repository][]{uci1998}.

We first analyze this data as a binary prediction problem
(recurrence vs. non-recurrence) and later in Section~\ref{sec.survival}
by means of survival models. We are faced with many covariates ($p = \Sexpr{p}$)
for a limited number of observations without missing values
($n = \Sexpr{n}$), and variable selection is an important issue. We can
choose a classical logistic regression model
via AIC in a stepwise algorithm as follows
%(after centering the covariates)
<<wpbc-glm-fit, echo = TRUE>>=
### remove missing values and time variable
cc <- complete.cases(wpbc)
wpbc2 <- wpbc[cc, colnames(wpbc) != "time"]
### fit logistic regression model
wpbc_step <- step(glm(status ~ ., data = wpbc2, family = binomial()), trace = 0)
@
The final model consists of \Sexpr{length(coef(wpbc_step))} parameters with
<<wpbc-glm-aic, echo = TRUE>>=
logLik(wpbc_step)
AIC(wpbc_step)
@
and we want to compare this model to a logistic regression model fitted via gradient boosting.
We simply select the \Robject{Binomial} family (with default offset of $1/2
\log(\hat{p} / (1 - \hat{p}))$,
where $\hat{p}$ is the empirical proportion of recurrences) and we
initially use
$m_\text{stop} = 500$ boosting iterations
<<wpbc-glmboost-fit, echo = TRUE>>=
### fit logistic regression model via gradient boosting
ctrl <- boost_control(mstop = 500)
wpbc_glm <- glmboost(status ~ ., data = wpbc2, family = Binomial(),
                     center = TRUE, control = ctrl)
@
%The negative binomial log-likelihood is
%<<wpbc-glmboost-loglik, echo = TRUE>>=
%logLik(wpbc_glm)
%@
The classical AIC criterion (-2 log-likelihood + 2 $\df$) suggests to stop after
<<wpbc-glmboost-AIC, echo = TRUE>>=
aic <- AIC(wpbc_glm, "classical")
aic
@
boosting iterations.
%%Again, we want to investigate the out-of-bootstrap performance
%%of the boosted logistic regression model. Figure~\ref{wpbc-benchmarks-plot} depicts the
%%out-of-bootstrap (positive) binomial log-likelihood, standardized with
%%the number of out-of-bootstrap observations, for varying number of boosting iterations $m_\text{stop}$.
%%\begin{figure}
%%\begin{center}
%%<<wpbc-benchmarks-plot, echo = FALSE, fig = TRUE>>=
%%load(system.file("cache/wpbc_benchmarks.rda", package = "mboost"))
%%grid <- seq(from = 5, to = 500, by = 5)
%%plot(c(0, grid[-1] - 5), -colMeans(boob), type = "b",
%%     ylab = "Log-Likelihood / n",
%%     xlab = "Number of Boosting Iterations")
%%mopt <- grid[which.max(colMeans(boob))]
%%@
%%\caption{\Robject{wpbc} data: Out-of-bootstrap (positive) binomial
%%    log-likelihood. \label{wpbc-benchmarks-plot}}
%%\end{center}
%%\end{figure}
We now restrict the number of boosting iterations to
$m_\text{stop} = \Sexpr{mstop(aic)}$ and then obtain the estimated
coefficients via
<<wpbc-glmboost-fit2, echo = TRUE>>=
### fit with new mstop
wpbc_glm <- wpbc_glm[mstop(aic)]
coef(wpbc_glm)[abs(coef(wpbc_glm)) > 0]
@
(because of using the offset-value $\hat{f}^{[0]}$, we have to add the value
$\hat{f}^{[0]}$ to the reported intercept estimate above for the logistic
regression model).
%%We then can
%%extract the fitted conditional probabilities
%%<<wpbc-glmboost-prob, echo = TRUE>>=
%%f <- fitted(wpbc_glm)
%%### probabilities
%%p <- exp(f) / (exp(f) + exp(-f))
%%@
%%which are depicted by a conditional density plot in
%%Figure~\ref{wpbc-glmboost-prob-plot}.

%\setkeys{Gin}{width=0.7\textwidth, keepaspectratio}
%\begin{figure}
%\begin{center}
%<<wpbc-glmboost-prob-plot, echo = FALSE, fig = TRUE>>=
%cdplot(status ~ p, data = wpbc2)
%@
%\caption{\Robject{wpbc} data: Conditional density plot of the fitted
%         probabilities of recurrence / non-recurrence. \label{wpbc-glmboost-prob-plot}}
%\end{center}
%\end{figure}

A generalized additive model adds more flexibility to the regression function but is still
interpretable. We fit a logistic additive model to the \Robject{wpbc} data as follows:
<<wpbc-gamboost-fit, echo = TRUE>>=
wpbc_gam <- gamboost(status ~ ., data = wpbc2, family = Binomial(), baselearner = "bss")
mopt <- mstop(aic <- AIC(wpbc_gam, "classical"))
aic
@
This model selected $\Sexpr{length(unique(wpbc_gam$xselect()))}$ out of $\Sexpr{ncol(wpbc2)-1}$
covariates. The partial contributions of the four most important variables
are depicted in
Figure~\ref{wpbc-gamboost-plot} indicating a remarkable degree of non-linearity.
\setkeys{Gin}{width=0.95\textwidth, keepaspectratio}
\begin{figure}[t]
\begin{center}
<<wpbc-gamboost-plot, echo = FALSE, fig = TRUE, width = 7, height = 7>>=
wpbc_gam <- wpbc_gam[mopt]
fpartial <- predict(wpbc_gam, which = 1:length(variable.names(wpbc_gam)))
layout(matrix(1:4, nrow = 2, byrow = TRUE))
par(mai = par("mai") * c(1, 1, 0.5, 1))
plot(wpbc_gam, which = rev(order(colMeans(abs(fpartial))))[1:4])
@
\caption{\Robject{wpbc} data: Partial contributions of four selected
    covariates in an additive logistic model (without centering of
    estimated functions to mean zero).
    \label{wpbc-gamboost-plot}}
\end{center}
\end{figure}

\SweaveOpts{prefix.string=figures/BH}

\paragraph{Illustration: Wisconsin prognostic breast cancer (cont.)}

<<pkg-attach, echo = FALSE, results = tex>>=
source("setup.R")
@
Instead of the binary response variable describing the recurrence status, we
make use of the additionally available time information for modeling
the time to recurrence, i.e., all observations with non-recurrence are censored.
First, we calculate IPC weights
<<wpbc-glmboost-PIC, echo = TRUE>>=
library("survival")
### calculate IPC weights
censored <- wpbc$status == "R"
iw <- IPCweights(Surv(wpbc$time, censored))
wpbc3 <- wpbc[,names(wpbc) != "status"]
@
and fit a weighted linear model by boosting with componentwise linear
weighted least squares as base procedure:
<<wpbc-glmboost-censored-fit, echo = TRUE>>=
ctrl <- boost_control(mstop = 500)
wpbc_surv <- glmboost(log(time) ~ ., data = wpbc3,
                  weights = iw, center = TRUE, control = ctrl)
mstop(aic <- AIC(wpbc_surv))
wpbc_surv <- wpbc_surv[mstop(aic)]
@
The following variables have been selected for fitting
<<wpbc-glmboost-coef, echo = TRUE>>=
names(coef(wpbc_surv)[abs(coef(wpbc_surv)) > 0])
@
and the fitted values are
depicted in Figure~\ref{wpbc-glmboost-censored-fit}, showing a reasonable model fit.

\setkeys{Gin}{width=0.7\textwidth, keepaspectratio}
\begin{figure}[t]
\begin{center}
<<wpbc-glmboost-censored-fit, echo = FALSE, fig = TRUE, width = 5, height = 5>>=
plot(log(wpbc3$time), predict(wpbc_surv, newdata = wpbc3),
     cex = iw, ylim = c(0, 5), xlim = c(0, 5),
     xlab = "Time to recurrence (log-scale)",
     ylab = "Predicted time to recurrence")
abline(a = 0, b = 1, lty = 2, lwd = 0.5)
@
\caption{\Robject{wpbc} data: Fitted values of an IPC-weighted linear
  model, taking both time to recurrence and
    censoring information into account. The radius of the circles is proportional to the
    IPC weight of the corresponding observation, censored observations with IPC weight zero
    are not plotted. \label{wpbc-glmboost-censored-fit}}
\end{center}
\end{figure}

Alternatively, a Cox model with linear predictor can be fitted using
$L_2$Boosting by
implementing the negative gradient
of the partial likelihood (see \cite{ridgew99}) via
%%<<wpbc-CoxPH, echo = TRUE, eval = FALSE>>=
%%glmboost(Surv(wpbc$time, wpbc$status == "N") ~ .,
%%    data = wpbc, family = CoxPH(), center = TRUE)
%%@
\begin{Schunk}
\begin{Sinput}
R> glmboost(Surv(wpbc$time, wpbc$status == "R") ~ .,
         data = wpbc, family = CoxPH(), center = TRUE)
\end{Sinput}
\end{Schunk}
For more examples, such as fitting an additive Cox model using
\Rpackage{mboost}, see \citep{Hothorn:2006:Bioinformatics:16940323}.

\clearpage

\bibliographystyle{plainnat}
\bibliography{boost}

\end{document}

\name{boost_control}
\alias{boost_control}
\title{ Control Hyper-parameters for Boosting Algorithms }
\description{
  Definition of the initial number of boosting iterations, step size
  and other hyper-parameters for boosting algorithms.
}
\usage{
boost_control(mstop = 100, nu = 0.1,
              risk = c("inbag", "oobag", "none"), stopintern = FALSE,
              center = TRUE, trace = FALSE)
}
\arguments{
  \item{mstop}{ an integer giving the number of initial boosting iterations. 
        If \code{mstop = 0}, the offset model is returned.}
  \item{nu}{ a double (between 0 and 1) defining the step size or shrinkage parameter.
             The default is probably too large for many applications
             with \code{family = Poisson()} and a smaller value is better.}
  \item{risk}{ a character indicating how the empirical risk should be
               computed for each boosting iteration. \code{inbag} leads to
               risks computed for the learning sample (i.e., all non-zero weights),
               \code{oobag} to risks based on the out-of-bag (all observations with
               zero weights) and \code{none} to no risk computations at all.}
  \item{stopintern}{ a logical that defines if the boosting algorithm stops internally
                     when the out-of-bag risk in one iteration is larger than the 
                     out-of-bag risk in the iteration before. Can also be a positive
                     number giving the risk difference that needs to be exceeded.}
  \item{center}{ deprecated. A logical indicating if the numerical covariates should be mean
                 centered before fitting. Only implemented for
		 \code{\link{glmboost}}. In \code{\link{blackboost}}
		 centering is not needed. In \code{\link{gamboost}}
		 centering is only needed if \code{\link{bols}}
		 base-learners are specified without intercept. In this
		 case centering of the covariates is essential and should be done
		 manually (at the moment). Will be removed in favour of
                 a corresponding argument in \code{glmboost} in the future
                 (and gives a warning).}
  \item{trace}{ a logical triggering printout of status information during
                the fitting process.}
}
\details{

  Objects returned by this function specify hyper-parameters of the
  boosting algorithms implemented in \code{\link{glmboost}},
  \code{\link{gamboost}} and \code{\link{blackboost}}
  (via the \code{control} argument).

}
\value{
  An object of class \code{boost_control}, a list.
}
\seealso{\code{\link{mboost}}}
\keyword{misc}

\name{mboost_fit}
\alias{mboost_fit}
\title{ Model-based Gradient Boosting }
\description{
  Work-horse for gradient boosting for optimizing arbitrary loss functions, 
  where component-wise models are utilized as base-learners. Usually, this function 
  is not called directly by the user.
}
\usage{
mboost_fit(blg, response, weights = rep(1, NROW(response)), offset = NULL,
           family = Gaussian(), control = boost_control(), oobweights =
           as.numeric(weights == 0))
}
\arguments{
  \item{blg}{ a list of objects of elements of class \code{blg}, as returned by 
    all base-learners.}
  \item{response}{ the response variable.}
  \item{weights}{ (optional) a numeric vector of weights to be used in 
    the fitting process.}
  \item{offset}{ a numeric vector to be used as offset (optional).}
  \item{family}{a \code{\link{Family}} object.}
  \item{control}{ a list of parameters controlling the algorithm. For
    more details see \code{\link{boost_control}}. }
  \item{oobweights}{ an additional vector of out-of-bag weights, which is
    used for the out-of-bag risk (i.e., if \code{boost_control(risk =
      "oobag")}). This argument is also used internally by
    \code{cvrisk}. }
}
\details{
  The function implements component-wise functional gradient boosting in
  a generic way. This function is the main work horse and used as back-end by
  all boosting algorithms in a unified way. Usually, this function is not
  called directly.  Note that the more convenient modelling interfaces 
  \code{\link{gamboost}}, \code{\link{glmboost}} and \code{\link{blackboost}} 
  all call \code{mboost_fit}.
  
  Basically, the algorithm is initialized with a function
  for computing the negative gradient of the loss function (via its
  \code{family} argument) and one or more base-learners (given as
  \code{blg}). Usually \code{blg} and \code{response} are computed in
  the functions \code{\link{gamboost}}, \code{\link{glmboost}},
  \code{\link{blackboost}} or \code{\link{mboost}}. See there for details 
  on the specification of base-learners.

  The algorithm minimized the in-sample empirical risk defined as
  the weighted sum (by \code{weights}) of the loss function (corresponding
  to the negative gradient) evaluated at the data.

  The structure of the model is determined by the structure
  of the base-learners. If more than one base-learner is given,
  the model is additive in these components.

  Base-learners can be specified via a formula interface
  (function \code{mboost}) or as a list of objects of class \code{bl},
  see, e.g., \code{\link{bols}}.

  \code{oobweights} is a vector used internally by \code{cvrisk}. When carrying
  out cross-validation to determine the optimal stopping iteration of a boosting
  model, the default value of \code{oobweights} (out-of-bag weights) assures
  that the cross-validated risk is computed using the same observation weights
  as those used for fitting the boosting model. It is strongly recommended to
  leave this argument unspecified.
}
\value{
  An object of class \code{mboost} with \code{\link{print}},
  \code{\link{AIC}}, \code{\link{plot}} and \code{\link{predict}}
  methods being available.
}
\seealso{\code{\link{glmboost}} for boosted linear models and
  \code{\link{blackboost}} for boosted trees. See e.g. \code{\link{bbs}}
  for possible base-learners. See \code{\link{cvrisk}} for
  cross-validated stopping iteration. Furthermore see
  \code{\link{boost_control}}, \code{\link{Family}} and
  \code{\link[mboost]{methods}}. }
\examples{
  data("bodyfat", package = "TH.data")

  ### formula interface: additive Gaussian model with
  ### a non-linear step-function in `age', a linear function in `waistcirc'
  ### and a smooth non-linear smooth function in `hipcirc'
  mod <- mboost(DEXfat ~ btree(age) + bols(waistcirc) + bbs(hipcirc),
                data = bodyfat)
  layout(matrix(1:6, nc = 3, byrow = TRUE))
  plot(mod, main = "formula")

  ### the same
  with(bodyfat,
       mod <- mboost_fit(list(btree(age), bols(waistcirc), bbs(hipcirc)),
                         response = DEXfat))
  plot(mod, main = "base-learner")
}
\keyword{models}
\keyword{nonlinear}

\name{confint.mboost}

\alias{confint.mboost}
\alias{confint.glmboost}

\alias{plot.mboost.ci}
\alias{lines.mboost.ci}
\alias{print.glmboost.ci}

\title{
  Pointwise Bootstrap Confidence Intervals
}
\description{
  Compute and display pointwise confidence intervals
}
\usage{
\method{confint}{mboost}(object, parm = NULL, level = 0.95, B = 1000,
        B.mstop = 25, newdata = NULL, which = parm,
        papply = ifelse(B.mstop == 0, mclapply, lapply),
        cvrisk_options = list(), ...)
\method{plot}{mboost.ci}(x, which, level = x$level, ylim = NULL, type = "l", col = "black",
     ci.col = rgb(170, 170, 170, alpha = 85, maxColorValue = 255),
     raw = FALSE, print_levelplot = TRUE,...)
\method{lines}{mboost.ci}(x, which, level = x$level,
     col = rgb(170, 170, 170, alpha = 85, maxColorValue = 255),
     raw = FALSE, ...)


\method{confint}{glmboost}(object, parm = NULL, level = 0.95,
        B = 1000, B.mstop = 25, which = parm, ...)
\method{print}{glmboost.ci}(x, which = NULL, level = x$level, pe = FALSE, ...)
}

\arguments{
  \item{object}{
    a fitted model object of class \code{glmboost}, \code{gamboost} or
    \code{mboost} for which the confidence intervals should be computed.
  }
  \item{parm, which}{
    a subset of base-learners to take into account for computing
    confidence intervals. See \code{\link{mboost_methods}} for details.
    \code{parm} is just a synonyme for \code{which} to be in line with
    the generic \code{confint} function. Preferably use \code{which}.
  }
  \item{level}{
    the confidence level required.
  }
  \item{B}{
    number of outer bootstrap replicates used to compute the empirical
    bootstrap confidence intervals.
  }
  \item{B.mstop}{
    number of inner bootstrap replicates used to determine the optimal
    mstop on each of the \code{B} bootstrap samples.
  }
  \item{newdata}{
    optionally, a data frame on which to compute the predictions for the
    confidence intervals.
  }
  \item{papply}{
    (parallel) apply function for the outer bootstrap, defaults to
    \code{\link[parallel]{mclapply}} if no inner bootstrap is used to
    determine the optimal stopping iteration. For details see
    argument \code{papply} in \code{\link{cvrisk}}. Be careful with your
    computing resources if you use parallel computing for both, the
    inner and the outer bootstrap.
  }
  \item{cvrisk_options}{
    (optionally) specify a named list with arguments to the inner
    bootstrap. For example use \code{cvrisk_options = list(mc.cores =
      2)} to specify that the \code{\link{mclapply}} function within
    \code{\link{cvrisk}} uses 2 cores to compute the optimal
    \code{mstop}.
  }
  \item{x}{
    a confidence interval object.
  }
  \item{ylim}{
    limits of the y scale. Per default computed from the data to plot.
  }
  \item{type}{
    type of graphic for the point estimate, i.e., for the predicted
    function. Per default a line is plotted.
  }
  \item{col}{
    color of the point estimate, i.e., for the predicted function.
  }
  \item{ci.col}{
    color of the confidence interval.
  }
  \item{raw}{
    logical, should the raw function estimates or the derived confidence
    estimates be plotted?
  }
  \item{print_levelplot}{
    logical, should the \pkg{lattice} \code{\link[lattice]{levelplot}} be printed
    or simply returned for further modifications. This argument is only
    considered if bivariate effect estimates are plotted. If
    \code{print_levelplot} is set to \code{FALSE}, a list with objects
    \code{mean}, \code{lowerPI} and \code{upperPI} is returned
    containing the three \code{\link[lattice]{levelplot}} objects.
  }
  \item{pe}{
    logical, should the point estimtate (PE) be also returned?
  }
  \item{\dots}{
    additional arguments to the outer bootstrap such as \code{mc.cores}.
  }
}
\details{
  Use a nested boostrap approach to compute pointwise confidence
  intervals for the predicted partial functions or regression
  parameters. The approach is further described in Hofner et al. (2016).
}
\value{
  An object of class \code{glmboost.ci} or \code{mboost.ci} with special
  \code{print} and/or \code{plot} functions.
}
\references{
  Benjamin Hofner, Thomas Kneib and Torsten Hothorn (2016), 
  A Unified Framework of Constrained Regression. 
  \emph{Statistics & Computing}, \bold{26}, 1--14.
}
\author{
  Benjamin Hofner <benjamin.hofner@pei.de>
}
\seealso{
  \code{\link{cvrisk}} for crossvalidation approaches and
  \code{\link{mboost_methods}} for other methods.
}
\examples{
\dontrun{
############################################################
## Do not run these examples automatically as they take
## some time (~ 30 seconds depending on the system)

### a simple linear example
set.seed(1907)
data <- data.frame(x1 = rnorm(100), x2 = rnorm(100),
                   z = factor(sample(1:3, 100, replace = TRUE)))
data$y <- rnorm(100, mean = data$x1 - data$x2 - 1 * (data$z == 2) +
                            1 * (data$z == 3), sd = 0.1)
linmod <- glmboost(y ~ x1 + x2 + z, data = data,
                   control = boost_control(mstop = 200))

## compute confidence interval from 10 samples. Usually one should use
## at least 1000 samples.
CI <- confint(linmod, B = 10, level = 0.9)
CI

## to compute a confidence interval for another level simply change the
## level in the print function:
print(CI, level = 0.8)
## or print a subset (with point estimates):
print(CI, level = 0.8, pe = TRUE, which = "z")

### a simple smooth example
set.seed(1907)
data <- data.frame(x1 = rnorm(100), x2 = rnorm(100))
data$y <- rnorm(100, mean = data$x1^2 - sin(data$x2), sd = 0.1)
gam <- gamboost(y ~ x1 + x2, data = data,
                control = boost_control(mstop = 200))

## compute confidence interval from 10 samples. Usually one should use
## at least 1000 samples.
CI_gam <- confint(gam, B = 10, level = 0.9)

par(mfrow = c(1, 2))
plot(CI_gam, which = 1)
plot(CI_gam, which = 2)
## to compute a confidence interval for another level simply change the
## level in the plot or lines function:
lines(CI_gam, which = 2, level = 0.8)
}
}
\keyword{methods}

\name{glmboost}
\alias{glmboost}
\alias{glmboost.formula}
\alias{glmboost.matrix}
\alias{glmboost.default}

\title{ Gradient Boosting with Component-wise Linear Models }
\description{
  Gradient boosting for optimizing arbitrary loss functions where component-wise
  linear models are utilized as base-learners.
}
\usage{
\method{glmboost}{formula}(formula, data = list(), weights = NULL,
          offset = NULL, family = Gaussian(),
          na.action = na.pass, contrasts.arg = NULL,
          center = TRUE, control = boost_control(), oobweights = NULL, ...)
\method{glmboost}{matrix}(x, y, center = TRUE, weights = NULL,
          offset = NULL, family = Gaussian(),
          na.action = na.pass, control = boost_control(), oobweights = NULL, ...)
\method{glmboost}{default}(x, ...)
}
\arguments{
  \item{formula}{ a symbolic description of the model to be fit. }
  \item{data}{ a data frame containing the variables in the model. }
  \item{weights}{ an optional vector of weights to be used in the fitting
    process. }
  \item{offset}{ a numeric vector to be used as offset (optional).}
  \item{family}{a \code{\link{Family}} object.}
  \item{na.action}{a function which indicates what should happen when the data
    contain \code{NA}s.}
  \item{contrasts.arg}{a list, whose entries are contrasts suitable for input
    to the \code{contrasts} replacement function and whose names are
    the names of columns of \code{data} containing factors.
    See \code{\link{model.matrix.default}}.}
  \item{center}{logical indicating of the predictor variables are centered before fitting.}
  \item{control}{ a list of parameters controlling the algorithm. For
    more details see \code{\link{boost_control}}. }
  \item{oobweights}{ an additional vector of out-of-bag weights, which is
    used for the out-of-bag risk (i.e., if \code{boost_control(risk =
      "oobag")}). This argument is also used internally by
    \code{cvrisk}. }    
  \item{x}{ design matrix. Sparse matrices of class \code{Matrix} can be used as well.}
  \item{y}{ vector of responses. }
  \item{\dots}{ additional arguments passed to \code{\link{mboost_fit}}; currently none.}
}
\details{

  A (generalized) linear model is fitted using a boosting algorithm based on component-wise
  univariate linear models. The fit, i.e., the regression coefficients, can be
  interpreted in the usual way. The methodology is described in
  \bibcitet{mboost::buehlmannyu2003,mboost::buehlmann2006} and
  \bibcitet{mboost::Buehlmann:2008:StatSci}.

}
\value{
  An object of class \code{glmboost} with \code{\link{print}}, \code{\link{coef}},
  \code{\link{AIC}} and \code{\link{predict}} methods being available.
  For inputs with longer variable names, you might want to change
  \code{par("mai")} before calling the \code{plot} method of \code{glmboost}
  objects visualizing the coefficients path.
}
\references{\bibshow{*}}
\seealso{
  See \code{\link{mboost_fit}} for the generic boosting function, 
  \code{\link{gamboost}} for boosted additive models, and
  \code{\link{blackboost}} for boosted trees. 
  
  See \code{\link{baselearners}} for possible base-learners. 
  
  See \code{\link{cvrisk}} for cross-validated stopping iteration. 
  
  Furthermore see \code{\link{boost_control}}, \code{\link{Family}} and
  \code{\link[mboost]{methods}}.
}
\examples{

    ### a simple two-dimensional example: cars data
    cars.gb <- glmboost(dist ~ speed, data = cars,
                        control = boost_control(mstop = 2000),
                        center = FALSE)
    cars.gb

    ### coefficients should coincide
    cf <- coef(cars.gb, off2int = TRUE)     ## add offset to intercept
    coef(cars.gb) + c(cars.gb$offset, 0)    ## add offset to intercept (by hand)
    signif(cf, 3)
    signif(coef(lm(dist ~ speed, data = cars)), 3)
    ## almost converged. With higher mstop the results get even better

    ### now we center the design matrix for
    ### much quicker "convergence"
    cars.gb_centered <- glmboost(dist ~ speed, data = cars,
                                 control = boost_control(mstop = 2000),
                                 center = TRUE)

    ## plot coefficient paths of glmboost
    par(mfrow=c(1,2), mai = par("mai") * c(1, 1, 1, 2.5))
    plot(cars.gb, main = "without centering")
    plot(cars.gb_centered, main = "with centering")

    ### alternative loss function: absolute loss
    cars.gbl <- glmboost(dist ~ speed, data = cars,
                         control = boost_control(mstop = 1000),
                         family = Laplace())
    cars.gbl
    coef(cars.gbl, off2int = TRUE)

    ### plot fit
    par(mfrow = c(1,1))
    plot(dist ~ speed, data = cars)
    lines(cars$speed, predict(cars.gb), col = "red")     ## quadratic loss
    lines(cars$speed, predict(cars.gbl), col = "green")  ## absolute loss

    ### Huber loss with adaptive choice of delta
    cars.gbh <- glmboost(dist ~ speed, data = cars,
                         control = boost_control(mstop = 1000),
                         family = Huber())

    lines(cars$speed, predict(cars.gbh), col = "blue")   ## Huber loss
    legend("topleft", col = c("red", "green", "blue"), lty = 1,
           legend = c("Gaussian", "Laplace", "Huber"), bty = "n")

    ### sparse high-dimensional example that makes use of the matrix
    ### interface of glmboost and uses the matrix representation from
    ### package Matrix
    library("Matrix")
    n <- 100
    p <- 10000
    ptrue <- 10
    X <- Matrix(0, nrow = n, ncol = p)
    X[sample(1:(n * p), floor(n * p / 20))] <- runif(floor(n * p / 20))
    beta <- numeric(p)
    beta[sample(1:p, ptrue)] <- 10
    y <- drop(X \%*\% beta + rnorm(n, sd = 0.1))
    mod <- glmboost(y = y, x = X, center = TRUE) ### mstop needs tuning
    coef(mod, which = which(beta > 0))

}
\keyword{models}
\keyword{regression}

\name{stabsel}
\alias{stabsel}
\alias{stabsel.mboost}
\alias{stabsel_parameters.mboost}
\title{
    Stability Selection
}
\description{
    Selection of influential variables or model components with error control.
}
\usage{
## a method to compute stability selection paths for fitted mboost models
\method{stabsel}{mboost}(x, cutoff, q, PFER, grid = 0:mstop(x),
        folds = subsample(model.weights(x), B = B),
        B = ifelse(sampling.type == "MB", 100, 50),
        assumption = c("unimodal", "r-concave", "none"),
        sampling.type = c("SS", "MB"),
        papply = mclapply, verbose = TRUE, FWER, eval = TRUE, ...)

## just a wrapper to stabsel(p, ..., eval = FALSE)
\method{stabsel_parameters}{mboost}(p, ...)
}
\arguments{
  \item{x, p}{an fitted model of class \code{"mboost"}.}
  \item{cutoff}{cutoff between 0.5 and 1. Preferably a value between 0.6
    and 0.9 should be used.}
  \item{q}{number of (unique) selected variables (or groups of variables
    depending on the model) that are selected on each subsample.}
  \item{PFER}{upper bound for the per-family error rate. This
    specifies the amount of falsely selected base-learners, which is
    tolerated. See details.}
  \item{grid}{ a numeric vector of the form \code{0:m}. See also \code{\link{cvrisk}}. }   
  \item{folds}{ a weight matrix with number of rows equal to the number
    of observations, see \code{\link{cvrisk}} and
    \code{\link[stabs]{subsample}}. Usually one should not
    change the default here as subsampling with a fraction of \eqn{1/2}
    is needed for the error bounds to hold. One usage scenario where
    specifying the folds by hand might be the case when one has
    dependent data (e.g. clusters) and thus wants to draw clusters
    (i.e., multiple rows together) not individuals.}
  \item{assumption}{ Defines the type of assumptions on the
    distributions of the selection probabilities and simultaneous
    selection probabilities. Only applicable for
    \code{sampling.type = "SS"}. For \code{sampling.type = "MB"} we
    always use \code{"none"}.}
  \item{sampling.type}{ use sampling scheme of of
    \bibcitet{mboost::Shah_Samworth_2013}, i.e., with complementarty pairs (\code{sampling.type = "SS"}),
    or the original sampling scheme of \bibcitet{mboost::mebu10}.}
  \item{B}{ number of subsampling replicates. Per default, we use 50
    complementary pairs for the error bounds of \bibcitet{mboost::Shah_Samworth_2013}
    and 100 for the error bound derived in \bibcitet{mboost::mebu10}. 
    As we use \eqn{B} complementray pairs in the former case
    this leads to \eqn{2B} subsamples.}
  \item{papply}{ (parallel) apply function, defaults to
    \code{\link[parallel]{mclapply}}. Alternatively, \code{parLapply}
    can be used. In the latter case, usually more setup is needed (see
    example of \code{\link{cvrisk}} for some details).}
  \item{verbose}{ logical (default: \code{TRUE}) that determines wether
    \code{warnings} should be issued. }
  \item{FWER}{ deprecated. Only for compatibility with older versions,
    use PFER instead.}
  \item{eval}{ logical. Determines whether stability selection is
    evaluated (\code{eval = TRUE}; default) or if only the parameter
    combination is returned.}
  \item{\dots}{ additional arguments to parallel apply methods such as
    \code{\link{mclapply}} and to \code{\link{cvrisk}}.}
}
\details{

  For details see \code{\link[stabs]{stabsel}} in package \pkg{stabs}
  and \bibcitet{mboost::Hofner_Boccuto_2015}.

}
\value{
  An object of class \code{stabsel} with a special \code{print} method.
  The object has the following elements:
  \item{phat}{selection probabilities.}
  \item{selected}{elements with maximal selection probability greater
    \code{cutoff}.}
  \item{max}{maximum of selection probabilities.}
  \item{cutoff}{cutoff used.}
  \item{q}{average number of selected variables used.}
  \item{PFER}{per-family error rate.}
  \item{sampling.type}{the sampling type used for stability selection.}
  \item{assumption}{the assumptions made on the selection
    probabilities.}
  \item{call}{the call.}
}
\references{\bibshow{*}}

\seealso{
  \code{\link[stabs]{stabsel}} and
  \code{\link[stabs]{stabsel_parameters}}
}
\examples{
  ## make data set available
  data("bodyfat", package = "TH.data")
  ## set seed
  set.seed(1234)

  ### low-dimensional example
  mod <- glmboost(DEXfat ~ ., data = bodyfat)

  ## compute cutoff ahead of running stabsel to see if it is a sensible
  ## parameter choice.
  ##   p = ncol(bodyfat) - 1 (= Outcome) + 1 ( = Intercept)
  stabsel_parameters(q = 3, PFER = 1, p = ncol(bodyfat) - 1 + 1,
                     sampling.type = "MB")
  ## the same:
  stabsel(mod, q = 3, PFER = 1, sampling.type = "MB", eval = FALSE)

\dontrun{############################################################
## Do not run and check these examples automatically as
## they take some time (~ 10 seconds depending on the system)

  ## now run stability selection
  (sbody <- stabsel(mod, q = 3, PFER = 1, sampling.type = "MB"))
  opar <- par(mai = par("mai") * c(1, 1, 1, 2.7))
  plot(sbody)
  par(opar)

  plot(sbody, type = "maxsel", ymargin = 6)

## End(Not run and test)
}
}
\keyword{nonparametric}

\name{mboost-package}
\alias{mboost-package}
\alias{mboost_package}
\alias{package_mboost}
\alias{package-mboost}
\docType{package}
\title{
  mboost: Model-Based Boosting
}
\description{
  Functional gradient descent algorithm
  (boosting) for optimizing general risk functions utilizing
  component-wise (penalized) least squares estimates or regression
  trees as base-learners for fitting generalized linear, additive
  and interaction models to potentially high-dimensional data.
}
\details{

  This package is intended for modern regression modeling and stands
  in-between classical generalized linear and additive models, as for example
  implemented by \code{\link{lm}}, \code{\link{glm}}, or \code{\link[mgcv]{gam}},
  and machine-learning approaches for complex interactions models,
  most prominently represented by \code{\link[gbm]{gbm}} and
  \code{\link[randomForest]{randomForest}}.

  All functionality in this package is based on the generic
  implementation of the optimization algorithm  (function
  \code{\link{mboost_fit}}) that allows for fitting linear, additive,
  and interaction models (and mixtures of those) in low and
  high dimensions. The response may be numeric, binary, ordered,
  censored or count data.

  Both theory and applications are discussed by
  \bibcitet{mboost::Buehlmann:2008:StatSci}.
  UseRs without a basic knowledge of boosting methods are asked
  to read this introduction before analyzing data using this package.
  The examples presented in this paper are available as package vignette
  \code{mboost_illustrations}.

  Note that the model fitting procedures in this package DO NOT automatically
  determine an appropriate model complexity. This task is the responsibility
  of the data analyst.

  A description of novel features that were introduced in version 2.0 is
  given in \bibcitet{mboost::Hothorn+Buehlmann+Kneib+Schmid+Hofner_2010}.

  \bibcitet{mboost::Hofner_Mayr_2014} present a comprehensive hands-on tutorial for using the
  package \code{mboost}, which is also available as
  \code{vignette(package = "mboost", "mboost_tutorial")}.

  \bibcitet{mboost::Taieba_Hyndman_2014} used this package for fitting their model in the
  Kaggle Global Energy Forecasting Competition 2012. The corresponding research
  paper is a good starting point when you plan to analyze your data using
  \code{mboost}.

}

\section{NEWS in 2.9-series}{

  Series 2.9 provides a new family (\code{RCG}), uses \code{partykit::ctree} 
  instead of \code{party::ctree} to be more flexible, allows for multivariate 
  negative gradients, and leave-one-out crossvalidation. Further minor changes were 
  introduces and quite some bugs were fixed. 

  \bold{For more details and other changes see}\cr\code{news(Version >= "2.9-0", package  = "mboost")}
  
}

\section{NEWS in 2.8-series}{

  Series 2.8 allows to fit models with zero boosting steps (i.e., models containing 
  only the offset). Furthermore, cross-validation can now also select a model
  without base-learners. In a \code{\link{Binomial}} family one can now specifiy
  links via \code{make.link}. With \code{Binomial(type = "glm")} an alternative 
  implementation of \code{Binomial} models is now existing and defines the model
  along the lines of the \code{glm} implementation. Additionally, it works not only with a 
  two-level factor but also with a two-column matrix containing the number of 
  successes and number of failures. Finally, a new base-learner \code{\link{bkernel}} for
  kernel boosting was added. The references were updated and a lot of bugs fixed. 

  \bold{For more details and other changes see}\cr\code{news(Version >= "2.8-0", package  = "mboost")}
  
}

\section{NEWS in 2.7-series}{

  Series 2.7 provides a new family (\code{Cindex}), variable importance measures
  (\code{varimp}) and improved plotting facilities. The manual was updated in 
  various places, vignettes were improved and a lot of bugs were fixed. 

  \bold{For more details and other changes see}\cr\code{news(Version >= "2.7-0", package  = "mboost")}
  
}

\section{NEWS in 2.6-series}{

  Series 2.6 includes a lot of bug fixes and improvements. Most notably,
  the development of the package is now hosted entirely on github in the
  project \href{https://github.com/boost-R/mboost/}{boost-R/mboost}.
  Furthermore, the package is now maintained by Benjamin Hofner.

  \bold{For more details and other changes see}\cr\code{news(Version >= "2.6-0", package  = "mboost")}
  
}

\section{NEWS in 2.5-series}{

  Crossvaliation does not stop on errors in single folds anymore an was
  sped up by setting \code{mc.preschedule = FALSE} if parallel
  computations via \code{\link{mclapply}} are used. The
  \code{\link{plot.mboost}} function is now documented. Values outside
  the boundary knots are now better handeled (forbidden during fitting,
  while linear extrapolation is used for prediction). Further perfomance
  improvements and a lot of bug fixes have been added.

  \bold{For more details and other changes see}\cr\code{news(Version >= "2.5-0", package  = "mboost")}
}

\section{NEWS in 2.4-series}{

  Bootstrap confidence intervals have been implemented in the novel
  \code{\link[=confint.mboost]{confint}} function. The stability
  selection procedure has now been moved to a stand-alone package called
  \pkg{stabs}, which now also implements an iterface to use stability
  selection with other fitting functions. A generic function for
  \code{"mboost"} models is implemented in \pkg{mboost}.

  \bold{For more details and other changes see}\cr\code{news(Version >= "2.4-0", package  = "mboost")}
}

\section{NEWS in 2.3-series}{

  The stability selection procedure has been completely rewritten and
  improved. The code base is now extensively tested. New options allow
  for a less conservative error control.

  Constrained effects can now be fitted using quadratic programming
  methods using the option \code{type = "quad.prog"} (default) for
  highly improved speed. Additionally, new constraints have been added.

  Other important changes include:
  \itemize{
    \item A new replacement function \code{mstop(mod) <- i} as an alternative to
    \code{mod[i]} was added (as suggested by Achim Zeileis).

    \item We added new families \code{Hurdle} and \code{Multinomial}.

    \item We added a new argument \code{stopintern} for internal stopping
     (based on out-of-bag data) during fitting to \code{boost_control}.
  }

  \bold{For more details and other changes see}\cr\code{news(Version >= "2.3-0", package  = "mboost")}
}

\section{NEWS in 2.2-series}{
  Starting from version 2.2, the default for the degrees of freedom has
  changed. Now the degrees of freedom are (per default) defined as
  \deqn{\mathrm{df}(\lambda) = \mathrm{trace}(2S -
  S^{\top}S),}{df(\lambda) = trace(2S - S'S),} with smoother matrix
  \eqn{S = X(X^{\top}X + \lambda K)^{-1} X}{S = X(X'X + \lambda K)^(-1)
  X} (see Hofner et al., 2011). Earlier versions used the trace of the
  smoother matrix \eqn{\mathrm{df}(\lambda) = \mathrm{trace}(S)} as
  degrees of freedom. One can change the old definition using
  \code{options(mboost_dftraceS = TRUE)} (see also B. Hofner et al.,
  2011 and \code{\link{bols}}).

  Other important changes include:
  \itemize{
    \item We switched from packages \code{multicore} and \code{snow} to
    \code{parallel}

    \item We changed the behavior of \code{bols(x, intercept = FALSE)}
    when \code{x} is a factor: now the intercept is simply dropped from
    the design matrix and the coding can be specified as usually for
    factors. Additionally, a new contrast is introduced:
    \code{"contr.dummy"} (see \code{\link{bols}} for details).

    \item We changed the computation of B-spline basis at the
    boundaries; B-splines now also use equidistant knots in the
    boundaries (per default).
  }

  \bold{For more details and other changes see}\cr\code{news(Version >= "2.2-0" & Version < "2.3-0", package  = "mboost")}
}
\section{NEWS in 2.1-series}{

  In the 2.1 series, we added multiple new base-learners including
  \code{\link{bmono}} (monotonic effects), \code{\link{brad}} (radial
  basis functions) and \code{\link{bmrf}} (Markov random fields), and
  extended \code{\link{bbs}} to incorporate cyclic splines (via argument
  \code{cyclic = TRUE}). We also changed the default \code{df} for
  \code{\link{bspatial}} to \code{6}.

  Starting from this version, we now also automatically center the
  variables in \code{\link{glmboost}} (argument \code{center = TRUE}).

  \bold{For more details and other changes see}\cr\code{news(Version >= "2.1-0" & Version < "2.2-0", package  = "mboost")}
}
\section{NEWS in 2.0-series}{

  Version 2.0 comes with new features, is faster and more accurate
  in some aspects. In addition, some changes to the user interface
  were necessary: Subsetting \code{mboost} objects changes the object.
  At each time, a model is associated with a number of boosting iterations
  which can be changed (increased or decreased) using the subset operator.

  The \code{center} argument in \code{\link{bols}} was renamed
  to \code{intercept}. Argument \code{z} renamed to \code{by}.

  The base-learners \code{bns} and \code{bss} are deprecated
  and replaced by \code{bbs} (which results in qualitatively the
  same models but is computationally much more attractive).

  New features include new families (for example for ordinal regression)
  and the \code{which} argument to the \code{coef} and \code{predict}
  methods for selecting interesting base-learners. Predict
  methods are much faster now.

  The memory consumption could be reduced considerably,
  thanks to sparse matrix technology in package \code{Matrix}.
  Resampling procedures run automatically in parallel
  on OSes where parallelization via package \code{parallel} is available.

  The most important advancement is a generic implementation
  of the optimizer in function \code{\link{mboost_fit}}.

  \bold{For more details and other changes see}\cr\code{news(Version >= "2.0-0" & Version < "2.1-0", package  = "mboost")}

}
\references{\bibshow{*}}

\seealso{
  The main fitting functions include:\cr
  \itemize{
    \item \code{\link{gamboost}} for boosted (generalized) additive models,
    \item \code{\link{glmboost}} for boosted linear models and
    \item \code{\link{blackboost}} for boosted trees.
  }
  Model tuning is done via cross-validation as implemented in \code{\link{cvrisk}}.\cr
  See there for more details and further links.
}

\examples{

  data("bodyfat", package = "TH.data")
  set.seed(290875)

  ### model conditional expectation of DEXfat given
  model <- mboost(DEXfat ~
      bols(age) +                 ### a linear function of age
      btree(hipcirc, waistcirc) + ### a smooth non-linear interaction of
                                  ### hip and waist circumference
      bbs(kneebreadth),           ### a smooth function of kneebreadth
      data = bodyfat, control = boost_control(mstop = 100))

  ### 10-fold cv for assessing `optimal' number of boosting iterations
  cvm <- cvrisk(model, papply = lapply, 
                folds = cv(model.weights(model), type = "kfold"))
  ### probably needs larger initial mstop but the
  ### CRAN team is picky about running times for examples
  plot(cvm)

  ### restrict model to mstop(cvm)
  model[mstop(cvm), return = FALSE]
  mstop(model)

  ### plot age and kneebreadth
  layout(matrix(1:2, nc = 2))
  plot(model, which = c("age", "kneebreadth"))

  ### plot interaction of hip and waist circumference
  attach(bodyfat)
  nd <- expand.grid(hipcirc = h <- seq(from = min(hipcirc),
                                  to = max(hipcirc),
                                  length = 100),
                    waistcirc = w <- seq(from = min(waistcirc),
                                  to = max(waistcirc),
                                  length = 100))
  plot(model, which = 2, newdata = nd)
  detach(bodyfat)

  ### customized plot
  layout(1)
  pr <- predict(model, which = "hip", newdata = nd)
  persp(x = h, y = w, z = matrix(pr, nrow = 100, ncol = 100))

}

\keyword{ package }
\keyword{ smooth }
\keyword{ nonparametric }
\keyword{ models }

\name{survFit}
\alias{survFit}
\alias{survFit.mboost}
\alias{plot.survFit}
\title{ Survival Curves for a Cox Proportional Hazards Model }
\description{
  Computes the predicted survivor function for a Cox proportional hazards model.
}
\usage{
\method{survFit}{mboost}(object, newdata = NULL, ...)
\method{plot}{survFit}(x, xlab = "Time", ylab = "Probability", \dots)
}
\arguments{
  \item{object}{ an object of class \code{mboost} which is assumed to have a \code{\link{CoxPH}}
                 family component. }
  \item{newdata}{ an optional data frame in which to look for variables with
                  which to predict the survivor function. }
  \item{x}{ an object of class \code{survFit} for plotting. }
  \item{xlab}{ the label of the x axis. }
  \item{ylab}{ the label of the y axis. }
  \item{...}{ additional arguments passed to callies.}
}
\details{

  If \code{newdata = NULL}, the survivor function of the Cox proportional
  hazards model is computed for the mean of the covariates used in the
  \code{\link{blackboost}}, \code{\link{gamboost}}, or \code{\link{glmboost}}
  call. The Breslow estimator is used for computing the baseline survivor
  function. If \code{newdata} is a data frame, the \code{\link{predict}} method
  of \code{object}, along with the Breslow estimator, is used for computing the
  predicted survivor function for each row in \code{newdata}.

}
\value{
 An object of class \code{survFit} containing the following components:
  \item{surv}{ the estimated survival probabilities at the time points
               given in \code{time}. }
  \item{time}{ the time points at which the survivor functions are
               evaluated. }
  \item{n.event}{ the number of events observed at each time point given
                  in \code{time}.}
}
\seealso{ \code{\link{gamboost}}, \code{\link{glmboost}} and
  \code{\link{blackboost}} for model fitting.}
\examples{

library("survival")
data("cancer", package = "survival")


fm <- Surv(futime,fustat) ~ age + resid.ds + rx + ecog.ps
fit <- glmboost(fm, data = ovarian, family = CoxPH(),
    control=boost_control(mstop = 500))

S1 <- survFit(fit)
S1
newdata <- ovarian[c(1,3,12),]
S2 <- survFit(fit, newdata = newdata)
S2

plot(S1)
}

\name{plot}
\alias{plot}

\alias{plot.glmboost}
\alias{plot.mboost}
\alias{lines.mboost}

\title{
  Plot effect estimates of boosting models
}
\description{
  Plot coefficient plots for \code{glmboost} models and partial effect
  plots for all other \code{mboost} models.
}
\usage{

\method{plot}{glmboost}(x, main = deparse(x$call), col = NULL,
     off2int = FALSE, ...)

\method{plot}{mboost}(x, which = NULL, newdata = NULL,
     type = "b", rug = TRUE, rugcol = "black",
     ylim = NULL, xlab = NULL, ylab = expression(f[partial]),
     add = FALSE, ...)

\method{lines}{mboost}(x, which = NULL, type = "l", rug = FALSE, ...)

}

\arguments{
  \item{x}{
    object of class \code{glmboost} or an object inheriting from
    \code{mboost} for plotting.
  }
  \item{main}{
    a title for the plot.
  }
  \item{col}{
    (a vector of) colors for plotting the lines representing the
    coefficient paths.
  }
  \item{off2int}{
    logical indicating whether the offset should be added to the
    intercept (if there is any) or if the offset is neglected for
    plotting (default).
  }
  \item{which}{
    a subset of base-learners used for plotting. If \code{which} is
    given (as an integer vector or characters corresponding
    to base-learners) only the corresponding partial effect plots are
    depicted. Per default all selected base-learners are plotted.
  }
  \item{newdata}{
    optionally, a data frame in which to look for variables with
    which to make predictions that are then plotted. This is especially
    useful if the data that was used to fit the model shows some larger
    gaps as effect plots are linearly interpolated between observations.
    For an example using \code{newdata} see below.
  }
  \item{type}{
    character string giving the type of plot desired. Per default,
    points and lines are plotted (\code{"b"}). Other useful options are
    points (\code{"p"}) or lines (\code{"l"}). See
    \code{\link{plot.default}} for details.
  }
  \item{rug}{
    logical. Should a rug be added to the x-axis?
  }
  \item{rugcol}{
    color for the rug.
  }
  \item{ylim}{
    the y limits of the plot.
  }
  \item{xlab}{
    a label for the x axis.
  }
  \item{ylab}{
    a label for the y axis.
  }
  \item{add}{
    logical. Should the plot be added to the previous plot?
  }
  \item{\dots}{
    Additional arguments to the \code{plot} functions. E.g. one can
    specify the x limits \code{xlim} or the color of the plot using
    \code{col}.
  }

}
\details{
  The coefficient paths for \code{glmboost} models show how the
  coefficient estimates evolve with increasing \code{mstop}. Each line
  represents one parameter estimate. Parameter estimates are only
  depicted when they they are selected at any time in the boosting
  model. Parameters that are not selected are droped from the figure
  (see example).

  Models specified with \code{gamboost} or \code{mboost} are plotted as
  partial effects. Only the effect of the current bossting iteration is
  depicted instead of the coefficient paths as for linear models. The
  function \code{lines} is just a wrapper to \code{plot(\ldots , add =
    TRUE)} where per default the effect is plotted as line and the
  \code{rug} is set to \code{FALSE}.

  Spatial effects can be also plotted using the function \code{plot}
  for mboost models (using \code{lattice} graphics). More complex
  effects reuquire manual plotting: One needs to predict the effects on
  a disired grid and plot the effect estimates.
}
\value{
  A plot of the fitted model.
}
\seealso{
  \code{\link{mboost_methods}} for further methods.
}
\examples{

### a simple example: cars data with one random variable
set.seed(1234)
cars$z <- rnorm(50)

########################################
## Plot linear models
########################################

## fit a linear model
cars.lm <- glmboost(dist ~ speed + z, data = cars)

## plot coefficient paths of glmboost
par(mfrow = c(3, 1), mar = c(4, 4, 4, 8))
plot(cars.lm,
     main = "Coefficient paths (offset not included)")
plot(cars.lm, off2int = TRUE,
     main = "Coefficient paths (offset included in intercept)")

## plot coefficient paths only for the first 15 steps,
## i.e., bevore z is selected
mstop(cars.lm) <- 15
plot(cars.lm, off2int = TRUE, main = "z is not yet selected")


########################################
## Plot additive models; basics
########################################

## fit an additive model
cars.gam <- gamboost(dist ~ speed + z, data = cars)

## plot effects
par(mfrow = c(1, 2), mar = c(4, 4, 0.1, 0.1))
plot(cars.gam)

## use same y-lims
plot(cars.gam, ylim = c(-50, 50))

## plot only the effect of speed
plot(cars.gam, which = "speed")
## as partial matching is used we could also use
plot(cars.gam, which = "sp")


########################################
## More complex plots
########################################

## Let us use more boosting iterations and compare the effects.

## We change the plot type and plot both effects in one figure:
par(mfrow = c(1, 1), mar = c(4, 4, 4, 0.1))
mstop(cars.gam) <- 100
plot(cars.gam, which = 1, col = "red", type = "l", rug = FALSE,
     main = "Compare effect for various models")

## Now the same model with 1000 iterations
mstop(cars.gam) <- 1000
lines(cars.gam, which = 1, col = "grey", lty = "dotted")

## There are some gaps in the data. Use newdata to get a smoother curve:
newdata <- data.frame(speed = seq(min(cars$speed), max(cars$speed),
                                  length = 200))
lines(cars.gam, which = 1, col = "grey", lty = "dashed",
      newdata = newdata)

## The model with 1000 steps seems to overfit the data.
## Usually one should use e.g. cross-validation to tune the model.

## Finally we refit the model using linear effects as comparison
cars.glm <- gamboost(dist ~ speed + z, baselearner = bols, data = cars)
lines(cars.glm, which = 1, col = "black")
## We see that all effects are more or less linear.

## Add a legend
legend("topleft", title = "Model",
       legend = c("... with mstop = 100", "... with mstop = 1000",
         "... with linear effects"),
       lty = c("solid", "dashed", "solid"),
       col = c("red", "grey", "black"))

}
\keyword{ methods }

\name{FP}
\alias{FP}
\title{ Fractional Polynomials }
\description{
  Fractional polynomials transformation for continuous covariates.
}
\usage{
FP(x, p = c(-2, -1, -0.5, 0.5, 1, 2, 3), scaling = TRUE)
}
\arguments{
  \item{x}{ a numeric vector. }
  \item{p}{ all powers of \code{x} to be included. }
  \item{scaling}{ a logical indicating if the measurements are scaled prior to model fitting.}
}
\details{

    A fractional polynomial \bibcitep{mboost::sauerbrei1999} refers to a model
    \eqn{\sum_{j = 1}^k (\beta_j x^{p_j} + \gamma_j x^{p_j} \log(x)) +
    \beta_{k + 1} \log(x)  + \gamma_{k + 1} \log(x)^2},
    where the degree of the fractional polynomial is the number of non-zero regression coefficients
    \eqn{\beta} and \eqn{\gamma}.

}
\value{
  A matrix including all powers \code{p} of \code{x},
  all powers \code{p} of \code{log(x)}, and \code{log(x)}.
}
\references{\bibshow{*}}
\seealso{\code{\link{gamboost}} to fit smooth models, \code{\link{bbs}}
  for P-spline base-learners}
\examples{

    data("bodyfat", package = "TH.data")
    tbodyfat <- bodyfat

    ### map covariates into [1, 2]
    indep <- names(tbodyfat)[-2]
    tbodyfat[indep] <- lapply(bodyfat[indep], function(x) {
        x <- x - min(x)
        x / max(x) + 1
    })

    ### generate formula
    fpfm <- as.formula(paste("DEXfat ~ ",
        paste("FP(", indep, ", scaling = FALSE)", collapse = "+")))
    fpfm

    ### fit linear model
    bf_fp <- glmboost(fpfm, data = tbodyfat,
                      control = boost_control(mstop = 3000))

    ### when to stop
    mstop(aic <- AIC(bf_fp))
    plot(aic)

    ### coefficients
    cf <- coef(bf_fp[mstop(aic)])
    length(cf)
    cf[abs(cf) > 0]

}
\keyword{datagen}

\name{methods}
\alias{mboost_methods}
\alias{print.glmboost}
\alias{print.mboost}

\alias{summary.mboost}

\alias{coef.mboost}
\alias{coef.glmboost}

\alias{[.mboost}

\alias{AIC.mboost}

\alias{mstop}
\alias{mstop.gbAIC}
\alias{mstop.mboost}
\alias{mstop.cvrisk}
\alias{mstop<-}

\alias{predict.mboost}
\alias{predict.gamboost}
\alias{predict.blackboost}
\alias{predict.glmboost}
\alias{fitted.mboost}
\alias{residuals.mboost}
\alias{resid.mboost}

\alias{variable.names.glmboost}
\alias{variable.names.mboost}

\alias{risk}
\alias{risk.mboost}

\alias{extract}
\alias{extract.mboost}
\alias{extract.gamboost}
\alias{extract.glmboost}
\alias{extract.blackboost}
\alias{extract.blg}
\alias{extract.bl_lin}
\alias{extract.bl_tree}

\alias{logLik.mboost}
\alias{hatvalues.gamboost}
\alias{hatvalues.glmboost}

\alias{selected}
\alias{selected.mboost}

\alias{nuisance}
\alias{nuisance.mboost}

\alias{downstream.test}

\title{ Methods for Gradient Boosting Objects }
\description{
  Methods for models fitted by boosting algorithms.
}
\usage{
\method{print}{glmboost}(x, ...)
\method{print}{mboost}(x, ...)

\method{summary}{mboost}(object, ...)

\method{coef}{mboost}(object, which = NULL,
    aggregate = c("sum", "cumsum", "none"), ...)
\method{coef}{glmboost}(object, which = NULL,
     aggregate = c("sum", "cumsum", "none"), off2int = FALSE, ...)

\method{[}{mboost}(x, i, return = TRUE, ...)
mstop(x) <- value

\method{AIC}{mboost}(object, method = c("corrected", "classical", "gMDL"),
    df = c("trace", "actset"), ..., k = 2)

\method{mstop}{mboost}(object, ...)
\method{mstop}{gbAIC}(object, ...)
\method{mstop}{cvrisk}(object, ...)

\method{predict}{mboost}(object, newdata = NULL,
        type = c("link", "response", "class"), which = NULL,
        aggregate = c("sum", "cumsum", "none"), ...)
\method{predict}{glmboost}(object, newdata = NULL,
        type = c("link", "response", "class"), which = NULL,
        aggregate = c("sum", "cumsum", "none"), ...)

\method{fitted}{mboost}(object, ...)

\method{residuals}{mboost}(object, ...)
\method{resid}{mboost}(object, ...)

\method{variable.names}{glmboost}(object, which = NULL, usedonly = FALSE, ...)
\method{variable.names}{mboost}(object, which = NULL, usedonly = FALSE, ...)

\method{extract}{mboost}(object, what = c("design", "penalty", "lambda", "df",
                         "coefficients", "residuals",
                         "variable.names", "bnames", "offset",
                         "nuisance", "weights", "index", "control"),
        which = NULL, ...)
\method{extract}{glmboost}(object, what = c("design", "coefficients", "residuals",
                         "variable.names", "offset",
                         "nuisance", "weights", "control"),
        which = NULL, asmatrix = FALSE, ...)
\method{extract}{blg}(object, what = c("design", "penalty", "index"),
        asmatrix = FALSE, expand = FALSE, ...)

\method{logLik}{mboost}(object, ...)
\method{hatvalues}{gamboost}(model, ...)
\method{hatvalues}{glmboost}(model, ...)

\method{selected}{mboost}(object, ...)

\method{risk}{mboost}(object, ...)

\method{nuisance}{mboost}(object)

downstream.test(object, ...)
}
\arguments{
  \item{object}{ objects of class \code{glmboost}, \code{gamboost},
    \code{blackboost} or \code{gbAIC}. }
  \item{x}{ objects of class \code{glmboost} or \code{gamboost}. }
  \item{model}{objects of class mboost}
  \item{newdata}{ optionally, a data frame in which to look for variables with
          which to predict. In case the model was fitted using the \code{matrix}
          interface to \code{\link{glmboost}}, \code{newdata} must be a \code{matrix}
          as well (an error is given otherwise).}
  \item{which}{ a subset of base-learners to take into account for computing
                predictions or coefficients. If \code{which} is given
                (as an integer vector or characters corresponding
                 to base-learners) a list or matrix is returned.}
  \item{usedonly}{ logical. Indicating whether all variable names should
     be returned or only those selected in the boosting algorithm.}
  \item{type}{ the type of prediction required.  The default is on the scale
          of the predictors; the alternative \code{"response"} is on
          the scale of the response variable.  Thus for a
          binomial model the default predictions are of log-odds
          (probabilities on logit scale) and \code{type = "response"} gives
          the predicted probabilities.  The \code{"class"} option returns
          predicted classes.}
  \item{aggregate}{ a character specifying how to aggregate predictions
                    or coefficients of single base-learners. The default
                    returns the prediction or coefficient for the final number of
                    boosting iterations. \code{"cumsum"} returns a
                    list with matrices (one per base-learner) with the
                    cumulative coefficients for all iterations
                    simultaneously (in columns). \code{"none"} returns a
                    list of matrices where the \eqn{j}th columns of the
                    respective matrix contains the predictions
                    of the base-learner of the \eqn{j}th boosting
                    iteration (and zero if the base-learner is not
                    selected in this iteration).}
  \item{off2int}{ logical. Indicating whether the offset should be
                     added to the intercept (if there is any)
                     or if the offset is returned as attribute of
                     the coefficient (default).}
  \item{i}{ integer. Index specifying the model to extract. If \code{i = 0}, 
            the offset model is returned. If \code{i} is smaller than the initial 
            \code{mstop}, a subset is used. If \code{i} is larger than the 
            initial \code{mstop}, additional boosting steps are performed until 
            step \code{i} is reached. See details for more information. }
  \item{value}{ integer. See \code{i}.}
  \item{return}{ a logical indicating whether the changed object is
                 returned. }
  \item{method}{ a character specifying if the corrected AIC criterion or
                 a classical (-2 logLik + k * df) should be computed.}
  \item{df}{ a character specifying how degrees of freedom should be computed:
             \code{trace} defines degrees of freedom by the trace of the
             boosting hat matrix and \code{actset} uses the number of
             non-zero coefficients for each boosting iteration.}
  \item{k}{  numeric, the \emph{penalty} per parameter to be used; the default
             \code{k = 2} is the classical AIC. Only used when
             \code{method = "classical"}.}
  \item{what}{ a character specifying the quantities to \code{extract}.
               Depending on \code{object} this can be a subset of
               \code{"design"} (default; design matrix),
               \code{"penalty"} (penalty matrix),
               \code{"lambda"} (smoothing parameter), \code{"df"}
               (degrees of freedom), \code{"coefficients"},
               \code{"residuals"}, \code{"variable.names"},
               \code{"bnames"} (names of the base-learners),
               \code{"offset"}, \code{"nuisance"}, \code{"weights"},
               \code{"index"} (index of ties used to expand the design
               matrix) and \code{"control"}. In future versions additional
               extractors might be specified.}
  \item{asmatrix}{ a logical indicating whether the the returned
        matrix should be coerced to a matrix (default) or if the
        returned object stays as it is (i.e., potentially a
        \emph{sparse} matrix). This option is only applicable if
        \code{extract} returns matrices, i.e., \code{what = "design"} or
        \code{what = "penalty"}. }
  \item{expand}{ a logical indicating whether the design matrix should
        be expanded (default: \code{FALSE}). This is useful if  ties
        where taken into account either manually (via argument
        \code{index} in a base-learner) or automatically for data sets
        with many observations. \code{expand = TRUE} is equivalent to
        \code{extract(B)[extract(B, what = "index"),]} for a
        base-learner \code{B}.}
  \item{\dots}{ additional arguments passed to callies. }
}
\details{

  These functions can be used to extract details from fitted models.
  \code{print} shows a dense representation of the model fit and
  \code{summary} gives a more detailed representation.

  The function \code{coef} extracts the regression coefficients of a
  linear model fitted using the \code{\link{glmboost}} function or an
  additive model fitted using the \code{\link{gamboost}}. Per default,
  only coefficients of selected base-learners are returned. However, any
  desired coefficient can be extracted using the \code{which} argument
  (see examples for details). Per default, the coefficient of the final
  iteration is returned (\code{aggregate = "sum"}) but it is also
  possible to return the coefficients from all iterations simultaniously
  (\code{aggregate = "cumsum"}). If \code{aggregate = "none"} is
  specified, the coefficients of the \emph{selected} base-learners are
  returned (see examples below).
  For models fitted via \code{\link{glmboost}} with option \code{center
  = TRUE} the intercept is rarely selected. However, it is implicitly
  estimated through the centering of the design matrix. In this case the
  intercept is always returned except \code{which} is specified such
  that the intercept is not selected. See examples below.

  The \code{predict} function can be used to predict the status of the
  response variable for new observations whereas \code{fitted} extracts
  the regression fit for the observations in the learning sample. For
  \code{predict} \code{newdata} can be specified, otherwise the fitted
  values are returned. If \code{which} is specified, marginal effects of
  the corresponding base-learner(s) are returned. The argument
  \code{type} can be used to make predictions on the scale of the
  \code{link} (i.e., the linear predictor \eqn{X\beta}{X * beta}),
  the \code{response} (i.e. \eqn{h(X\beta)}{h(X * beta)}, where h is the
  response function) or the \code{class} (in case of
  classification). Furthermore, the predictions can be aggregated
  analogously to \code{coef} by setting \code{aggregate} to either
  \code{sum} (default; predictions of the final iteration are given),
  \code{cumsum} (predictions of all iterations are returned
  simultaniously) or \code{none} (change of prediction in each
  iteration). If applicable the \code{offset} is added to the predictions.
  If marginal predictions are requested the \code{offset} is attached
  to the object via \code{attr(..., "offset")} as adding the offset to
  one of the marginal predictions doesn't make much sense.

  The \code{[.mboost} function can be used to enhance or restrict a given
  boosting model to the specified boosting iteration \code{i}. Note that
  in both cases the original \code{x} will be changed to reduce the
  memory footprint. If the boosting model is enhanced by specifying an
  index that is larger than the initial \code{mstop}, only the missing
  \code{i - mstop} steps are fitted. If the model is restricted, the
  spare steps are not dropped, i.e., if we increase \code{i} again,
  these boosting steps are immediately available. Alternatively, the
  same operation can be done by \code{mstop(x) <- i}.

  The \code{residuals} function can be used to extract the residuals
  (i.e., the negative gradient of the current iteration). \code{resid}
  is is an alias for \code{residuals}.

  Variable names (including those of interaction effects specified via
  \code{by} in a base-learner) can be extracted using the generic
  function \code{variable.names}, which has special methods for boosting
  objects.

  The generic \code{extract} function can be used to extract various
  characteristics of a fitted model or a base-learner. Note that the
  sometimes a penalty function is returned (e.g. by
  \code{extract(bols(x), what = "penalty")}) even if the estimation is
  unpenalized. However, in this case the penalty paramter \code{lambda}
  is set to zero. If a matrix is returned by \code{extract} one can to
  set \code{asmatrix = TRUE} if the returned matrix should be coerced to
  class \code{matrix}. If \code{asmatrix = FALSE} one might get a sparse
  matrix as implemented in package \code{Matrix}. If one requests the
  design matrix (\code{what = "design"}) \code{expand = TRUE} expands
  the resulting matrix by taking the duplicates handeled via
  \code{index} into account.

  The ids of base-learners selected during the fitting process can be
  extracted using \code{selected()}. The \code{nuisance()} method
  extracts nuisance parameters from the fit that are handled internally
  by the corresponding family object, see
  \code{"\linkS4class{boost_family}"}. The \code{risk()} function can be
  used to extract the computed risk (either the \code{"inbag"} risk or
  the \code{"oobag"} risk, depending on the control argument; see
  \code{\link{boost_control}}).

  For (generalized) linear and additive models, the \code{AIC} function
  can be used to compute both the classical AIC (only available for
  \code{familiy = Binomial()} and \code{familiy = Poisson()}) and
  corrected AIC \bibcitep{mboost::hur98}, only available when \code{family
  = Gaussian()} was used. Details on the used approximations for the
  hat matrix can be found in \bibcitet{mboost::Buehlmann:2008:StatSci}. The AIC is
  useful for the determination of the optimal number of boosting
  iterations to be applied (which can be extracted via \code{mstop}).
  The degrees of freedom are either computed via the trace of the
  boosting hat matrix (which is rather slow even for moderate sample
  sizes) or the number of variables (non-zero coefficients) that entered
  the model so far (faster but only meaningful for linear models fitted
  via \code{\link{gamboost}} \bibcitep{see|mboost::hast:comment:2007|}. For a discussion of
  the use of AIC based stopping see also \bibcitet{mboost::Mayr_2012}.

  In addition, the general Minimum Description Length criterion
  \bibcitep{mboost::JMLR:v7:buehlmann06a} can be computed using function \code{AIC}.

  Note that \code{logLik} and \code{AIC} only make sense when the
  corresponding \code{\link{Family}} implements the appropriate loss
  function.
  
  \code{downstream.test} computes tests for linear models fitted via \code{\link{glmboost}}
  with a likelihood based loss function and only suitable without early stopping, i.e., 
  if likelihood based model converged. In order to work, the Fisher matrix must
  be implemented in the \code{\link{Family}}; currently this is only the case for 
  family \code{\link{RCG}}.

}
\note{
  The \code{[.mboost} function changes the original object, i.e.
  \code{gbmodel[10]} changes \code{gbmodel} directly!
}
\section{Warning}{
  The coefficients resulting from boosting with family
  \code{Binomial(link = "logit")} are \eqn{1/2} of the coefficients of a logit  model obtained via \code{\link{glm}} (see \code{\link{Binomial}}).
}
\references{\bibshow{*}}

\seealso{
  \code{\link{gamboost}}, \code{\link{glmboost}} and
  \code{\link{blackboost}} for model fitting.

  \code{\link{plot.mboost}} for plotting methods.

  \code{\link{cvrisk}} for cross-validated stopping iteration.
}
\examples{

  ### a simple two-dimensional example: cars data
  cars.gb <- glmboost(dist ~ speed, data = cars,
                      control = boost_control(mstop = 2000),
                      center = FALSE)
  cars.gb

  ### initial number of boosting iterations
  mstop(cars.gb)

  ### AIC criterion
  aic <- AIC(cars.gb, method = "corrected")
  aic

  ### extract coefficients for glmboost
  coef(cars.gb)
  coef(cars.gb, off2int = TRUE)        # offset added to intercept
  coef(lm(dist ~ speed, data = cars))  # directly comparable

  cars.gb_centered <- glmboost(dist ~ speed, data = cars,
                               center = TRUE)
  selected(cars.gb_centered)           # intercept never selected
  coef(cars.gb_centered)               # intercept implicitly estimated
                                       # and thus returned
  ## intercept is internally corrected for mean-centering
  - mean(cars$speed) * coef(cars.gb_centered, which="speed") # = intercept
  # not asked for intercept thus not returned
  coef(cars.gb_centered, which="speed")
  # explicitly asked for intercept
  coef(cars.gb_centered, which=c("Intercept", "speed"))

  ### enhance or restrict model
  cars.gb <- gamboost(dist ~ speed, data = cars,
                      control = boost_control(mstop = 100, trace = TRUE))
  cars.gb[10]
  cars.gb[100, return = FALSE] # no refitting required
  cars.gb[150, return = FALSE] # only iterations 101 to 150
                               # are newly fitted

  ### coefficients for optimal number of boosting iterations
  coef(cars.gb[mstop(aic)])
  plot(cars$dist, predict(cars.gb[mstop(aic)]),
       ylim = range(cars$dist))
  abline(a = 0, b = 1)

  ### example for extraction of coefficients
  set.seed(1907)
  n <- 100
  x1 <- rnorm(n)
  x2 <- rnorm(n)
  x3 <- rnorm(n)
  x4 <- rnorm(n)
  int <- rep(1, n)
  y <- 3 * x1^2 - 0.5 * x2 + rnorm(n, sd = 0.1)
  data <- data.frame(y = y, int = int, x1 = x1, x2 = x2, x3 = x3, x4 = x4)

  model <- gamboost(y ~ bols(int, intercept = FALSE) +
                        bbs(x1, center = TRUE, df = 1) +
                        bols(x1, intercept = FALSE) +
                        bols(x2, intercept = FALSE) +
                        bols(x3, intercept = FALSE) +
                        bols(x4, intercept = FALSE),
                    data = data, control = boost_control(mstop = 500))

  coef(model) # standard output (only selected base-learners)
  coef(model,
       which = 1:length(variable.names(model))) # all base-learners
  coef(model, which = "x1") # shows all base-learners for x1

  cf1 <- coef(model, which = c(1,3,4), aggregate = "cumsum")
  tmp <- sapply(cf1, function(x) x)
  matplot(tmp, type = "l", main = "Coefficient Paths")

  cf1_all <- coef(model, aggregate = "cumsum")
  cf1_all <- lapply(cf1_all, function(x) x[, ncol(x)]) # last element
  ## same as coef(model)

  cf2 <- coef(model, aggregate = "none")
  cf2 <- lapply(cf2, rowSums) # same as coef(model)

  ### example continued for extraction of predictions

  yhat <- predict(model) # standard prediction; here same as fitted(model)
  p1 <- predict(model, which = "x1") # marginal effects of x1
  orderX <- order(data$x1)
  ## rowSums needed as p1 is a matrix
  plot(data$x1[orderX], rowSums(p1)[orderX], type = "b")

  ## better: predictions on a equidistant grid
  new_data <- data.frame(x1 = seq(min(data$x1), max(data$x1), length = 100))
  p2 <- predict(model, newdata = new_data, which = "x1")
  lines(new_data$x1, rowSums(p2), col = "red")

  ### extraction of model characteristics
  extract(model, which = "x1")  # design matrices for x1
  extract(model, what = "penalty", which = "x1") # penalty matrices for x1
  extract(model, what = "lambda", which = "x1") # df and corresponding lambda for x1
       ## note that bols(x1, intercept = FALSE) is unpenalized

  extract(model, what = "bnames")  ## name of complete base-learner
  extract(model, what = "variable.names") ## only variable names
  variable.names(model)            ## the same

  ### extract from base-learners
  extract(bbs(x1), what = "design")
  extract(bbs(x1), what = "penalty")
  ## weights and lambda can only be extracted after using dpp
  weights <- rep(1, length(x1))
  extract(bbs(x1)$dpp(weights), what = "lambda")
}
\keyword{methods}

\name{varimp}
\alias{varimp}
\alias{varimp.mboost}
\alias{plot.varimp}
\alias{as.data.frame.varimp}

\title{ Variable Importance }
\description{
In-bag risk reduction per base-learner as variable importance for boosting.
}
\usage{
\method{varimp}{mboost}(object, ...)

\method{plot}{varimp}(x, percent = TRUE, type = c("variable", "blearner"), 
  blorder = c("importance", "alphabetical", "rev_alphabetical", "formula"),
  nbars = 10L, maxchar = 20L, xlab = NULL, ylab = NULL, xlim, auto.key, ...)
\method{as.data.frame}{varimp}(x, row.names = NULL, optional = FALSE, ...) 
}


\arguments{
  
  \item{object}{ an object of class \code{mboost}.}
  
  \item{x}{ an object of class \code{varimp}.}
  
  \item{percent}{ logical, indicating whether variable importance should be 
  specified in percent.}
  
  \item{type}{ a character string specifying whether to draw bars for variables
  (\code{"variable"}, default) or base-learners (\code{"blearner"}) in the 
  model (no effect for a glmboost object).}
  
  \item{blorder}{ a character string specifying the order of the base-learners
  in the plot. The default \code{"importance"} corresponds to the order of the
  base-learner importance, \code{"alphabetical"} and \code{"rev_alphabetical"} 
  to alphabetical order, respectively its reverse, and \code{"formula"} to 
  their order in the model formula.}
  
  \item{nbars}{ integer, maximum number of bars to be plotted. If \code{nbars} 
  is exceeded, least important variables / base-learners are summarized as
  "other".}
  
  \item{maxchar}{ integer, maximum number of characters in bar labels.}
  
  \item{xlab}{text for the x-axis label. If not set (default is \code{NULL}) 
    x-axis label is generated automatically depending on argument 
    \code{percent}.}  
  
  \item{ylab}{text for the y-axis label. If not set (default is \code{NULL}) 
    y-axis label is generated automatically depending on argument \code{type}.}
  
  \item{xlim}{ the x limits of the plot. Defaults are from \code{0} to total 
  reduction, or from \code{0} to \code{1} for \code{percent = TRUE}. (In case 
  of negative risk reductions, default limits are from total negative to total
  positve reduction, or the latter normalized by the total absolute reduction 
  for \code{percent = TRUE}.)}
  
  \item{auto.key}{ logical, or a list passed to \code{lattice::barchart}. By 
  default \code{auto.key=TRUE} provides automatically generated legends showing 
  the underlying base-learners in the stacked barchart 
  (\code{type = "variable"}). If there is an unique base-learner for each 
  variable(-interaction), \code{auto.key = FALSE} is default setting. 
  For \code{type = "blearner"} the argument has no effect at all.} 
  
  \item{...}{ additional arguments passed to \code{lattice::barchart}.}
  
  \item{row.names}{ NULL or a character vector giving the row names for the 
  data frame. Missing values are not allowed.}

  \item{optional}{logical. If TRUE, setting row names and converting column 
  names (to syntactic names: see make.names) is optional.}
  
}
\details{
This function extracts the in-bag risk reductions per boosting step of a 
fitted \code{mboost} model and accumulates it individually for each base-learner
contained in the model. This quantifies the individual contribution to risk 
reduction of each base-learner and can thus be used to compare the importance 
of different base-learners or variables in the model. Starting from offset only, 
in each boosting step risk reduction is computed as the difference between 
in-bag risk of the current and the previous model and is accounted for the 
base-learner selected in the particular step.

The results can be plotted in a bar plot either for the base-learners, or the
variables contained in the model. The bars are ordered according to variable 
importance. If their number  exceeds \code{nbars} the least important are 
summarized as "other". If bars are plotted per  variable, all base-learners 
containing the same variable will be accumulated in a stacked bar. This is of 
use for models including for example seperate base-learners for the linear and 
non-linear part of a covariate effect (see \code{?bbs} option 
\code{center=TRUE}). However, variable interactions are treated as individual 
variables, as their desired handling might depend on context.

As a comparison the selection frequencies are added to the respective 
base-learner labels in the plot (rounded to three digits). For stacked bars 
they are ordered accordingly.
}

\value{ An object of class \code{varimp} with available \code{plot} and
\code{as.data.frame} methods. 

Converting a \code{varimp} object results in a \code{data.frame} containing the 
risk reductions, selection frequencies and the corresponding base-learner and 
variable names as ordered \code{factors} (ordered according to their particular 
importance).}

\author{
Tobias Kuehn (\email{tobi.kuehn@gmx.de}), 
Almond Stoecker (\email{almond.stoecker@gmail.com})
}

\examples{

data(iris)
### glmboost with multiple variables and intercept
iris$setosa <- factor(iris$Species == "setosa")
iris_glm <- glmboost(setosa ~ 1 + Sepal.Width + Sepal.Length + Petal.Width +
                         Petal.Length,
                     data = iris, control = boost_control(mstop = 50), 
                     family = Binomial(link = c("logit")))
varimp(iris_glm)
### importance plot with four bars only
plot(varimp(iris_glm), nbars = 4)

### gamboost with multiple variables
iris_gam <- gamboost(Sepal.Width ~ 
                         bols(Sepal.Length, by = setosa) +
                         bbs(Sepal.Length, by = setosa, center = TRUE) + 
                         bols(Petal.Width) +
                         bbs(Petal.Width, center = TRUE) + 
                         bols(Petal.Length) +
                         bbs(Petal.Length, center = TRUE),
                     data = iris)
varimp(iris_gam)
### stacked importance plot with base-learners in rev. alphabetical order
plot(varimp(iris_gam), blorder = "rev_alphabetical")

### similar ggplot
\dontrun{ 
library(ggplot2)
ggplot(data.frame(varimp(iris_gam)), aes(variable, reduction, fill = blearner)) + 
    geom_bar(stat = "identity") + coord_flip() }
}

\name{baselearners}
\alias{baselearners}
\alias{baselearner}
\alias{base-learner}
\alias{bols}
\alias{bbs}
\alias{bspatial}
\alias{brad}
\alias{bkernel}
\alias{brandom}
\alias{btree}
\alias{bmono}
\alias{bmrf}
\alias{buser}
\alias{bns}
\alias{bss}
\alias{\%+\%}
\alias{\%X\%}
\alias{\%O\%}
\title{ Base-learners for Gradient Boosting }
\description{
  Base-learners for fitting base-models in the generic implementation of
  component-wise gradient boosting in function \code{mboost}.
}
\usage{
## linear base-learner
bols(..., by = NULL, index = NULL, intercept = TRUE, df = NULL,
     lambda = 0, contrasts.arg = "contr.treatment")

## smooth P-spline base-learner
bbs(..., by = NULL, index = NULL, knots = 20, boundary.knots = NULL,
    degree = 3, differences = 2, df = 4, lambda = NULL, center = FALSE,
    cyclic = FALSE, constraint = c("none", "increasing", "decreasing"),
    deriv = 0)

## bivariate P-spline base-learner
bspatial(..., df = 6)

## radial basis functions base-learner
brad(..., by = NULL, index = NULL, knots = 100, df = 4, lambda = NULL,
     covFun = fields::stationary.cov,
     args = list(Covariance="Matern", smoothness = 1.5, theta=NULL))
     
## (genetic) pathway-based kernel base-learner
bkernel(..., df = 4, lambda = NULL, kernel = c("lin", "sia", "net"),
        pathway = NULL, knots = NULL, args = list())

## random effects base-learner
brandom(..., by = NULL, index = NULL, df = 4, lambda = NULL,
        contrasts.arg = "contr.dummy")

## tree based base-learner
btree(..., by = NULL, nmax = Inf, tree_controls = partykit::ctree_control(
    teststat = "quad", testtype = "Teststatistic", 
    mincriterion = 0, minsplit = 10, minbucket = 4,
    maxdepth = 1, saveinfo = FALSE))

## constrained effects base-learner
bmono(...,
      constraint = c("increasing", "decreasing", "convex", "concave",
                     "none", "positive", "negative"),
      type = c("quad.prog", "iterative"),
      by = NULL, index = NULL, knots = 20, boundary.knots = NULL,
      degree = 3, differences = 2, df = 4, lambda = NULL,
      lambda2 = 1e6, niter=10, intercept = TRUE,
      contrasts.arg = "contr.treatment",
      boundary.constraints = FALSE,
      cons.arg = list(lambda = 1e+06, n = NULL, diff_order = NULL))

## Markov random field base-learner
bmrf(..., by = NULL, index = NULL, bnd = NULL, df = 4, lambda = NULL,
    center = FALSE)

## user-specified base-learner
buser(X, K = NULL, by = NULL, index = NULL, df = 4, lambda = NULL)

## combining single base-learners to form new,
## more complex base-learners
bl1 \%+\% bl2
bl1 \%X\% bl2
bl1 \%O\% bl2
}
\arguments{
  \item{...}{ one or more predictor variables or one matrix or data
              frame of predictor variables. For smooth base-learners,
	      the number of predictor variables and the number of
	      columns in the data frame / matrix must be less than or
	      equal to 2. If a matrix (with at least 2 columns) is
	      given to \code{bols} or \code{brandom}, it is directly
	      used as the design matrix. Especially, no intercept term
	      is added regardless of argument \code{intercept}.
	      If the argument has only one column, it is simplified
	      to a vector and an intercept is added or not
	      according to the argmuent \code{intercept}.}
  \item{by}{ an optional variable defining varying coefficients,
             either a factor or numeric variable.
	     If \code{by} is a factor, the coding is determined by
	     the global \code{options("contrasts")} or as specified
	     "locally" for the factor (see \code{\link{contrasts}}). Per
	     default treatment coding is used. Note that the main
	     effect needs to be specified in a separate base-learner.
             \code{btree} currently only allows binary factors.}

  \item{index}{ a vector of integers for expanding the variables in
    \code{...}. For example, \code{bols(x, index = index)} is equal to
    \code{bols(x[index])}, where \code{index} is an integer of length
    greater or equal to \code{length(x)}.}

  \item{df}{ trace of the hat matrix for the base-learner defining the
    base-learner complexity. Low values of \code{df} correspond to a
    large amount of smoothing and thus to "weaker" base-learners.
    Certain restrictions have to be kept for the specification of
    \code{df} since most of the base-learners rely on penalization
    approaches with a non-trivial null space. For example, for P-splines
    fitted with \code{bbs}, \code{df} has to be larger than the order of
    differences employed in the construction of the penalty term.
    However, when option \code{center != FALSE}, the effect is centered
    around its unpenalized part and therefore any positive number is
    admissible for \code{df}. For details on the computation of degrees
    of freedom see section \sQuote{Global Options}.}
  \item{lambda}{ smoothing penalty, computed from \code{df} when
    \code{df} is specified. For details on the computation of degrees
    of freedom see section \sQuote{Global Options}.}
  \item{knots}{ either the number of knots or a vector of the positions
    of the interior knots (for more details see below). For multiple
    predictor variables, \code{knots} may be a named list where the
    names in the list are the variable names.}
  \item{boundary.knots}{ boundary points at which to anchor the B-spline basis
    (default the range of the data). A vector (of length 2)
    for the lower and the upper boundary knot can be specified.This is
    only advised for \code{bbs(..., cyclic = TRUE)}, where the boundary
    knots specify the points at which the cyclic function should be joined. In
    analogy to \code{knots} a names list can be specified. }
  \item{degree}{ degree of the regression spline.}
  \item{differences}{ a non-negative integer, typically 1, 2 or 3. If \code{differences} =
                     \emph{k}, \emph{k}-th-order differences are used as
		     a penalty (\emph{0}-th order differences specify a
		     ridge penalty).}
  \item{intercept}{ if \code{intercept = TRUE} an intercept is added to
                    the design matrix of a linear base-learner. If
                    \code{intercept = FALSE}, continuous covariates
                    should be (mean-) centered.}
  \item{center}{ if \code{center != FALSE} the corresponding effect is
                 re-parameterized such that the unpenalized part of the fit is subtracted and
                 only the deviation effect is fitted. The unpenalized, parametric part has then
                 to be included in separate base-learners using \code{bols} (see the examples below).
                 There are two possible ways to re-parameterization;
                 \code{center = "differenceMatrix"} is based on the difference matrix
                 (the default for \code{bbs} with one covariate only)
                 and \code{center = "spectralDecomp"} uses a spectral decomposition
                 of the penalty matrix \bibcitep{see|mboost::Fahrmeir2004|Section 2.3 for details}.
                 The latter option is the default (and currently only option) for \code{bbs}
                 with multiple covariates or \code{bmrf}.}
  \item{cyclic}{ if \code{cyclic = TRUE} the fitted values coincide at the boundaries
                 (useful for cyclic covariates such as day time etc.). 
                 For details see \bibcitet{mboost::Hofner_Kneib_Hothorn_2015}.
}
  \item{covFun}{ the covariance function (i.e. radial basis)
                 needed to compute the basis functions. Per
                 default \code{\link[fields]{stationary.cov}} function
                 (from package \code{fields}) is used. }
  \item{args}{ a named list of arguments to be passed to
               \code{cov.function}. Thus strongly dependent on the
               specified \code{cov.function}.}
  \item{kernel}{one of \code{"lin"} (linear kernel), \code{"sia"} (size adjusted kernel), 
                or \code{"net"} (network kernel). For details see 
                \code{\link[kangar00]{calc_kernel}}}
  \item{pathway}{name of pathway; Pathway needs to be contained in the GWAS data set.}
  \item{contrasts.arg}{ a named list of characters suitable for input to
    the \code{\link{contrasts}} replacement function, or the contrast
    matrix itself, see \code{\link{model.matrix}}, or a single character
    string (or contrast matrix) which is then used as contrasts for all
    factors in this base-learner (with the exception of factors in
    \code{by}). See also example below for setting contrasts. Note that
    a special \code{contrasts.arg} exists in package \code{mboost},
    namely "contr.dummy". This contrast is used per default in
    \code{brandom} and can also be used in \code{bols}. It leads to a
    dummy coding as returned by \code{model.matrix(~ x - 1)} were the
    intercept is implicitly included but each factor level gets a
    seperate effect estimate (see example below).}
  \item{nmax}{integer, maximal number of
          bins in the predictor variables. Use \code{Inf} to switch-off binning.}
  \item{tree_controls}{ an object of class \code{"TreeControl"}, which can be
                  obtained using \code{\link[partykit]{ctree_control}}.
                  Defines hyper-parameters for the trees which are used as base-learners,
                  stumps are fitted by default. By default, stumps and thus
                  additive models are fitted.}
  \item{constraint}{type of constraint to be used. For \code{bmono}, 
    the constraint can be either monotonic \code{"increasing"} (default), 
    \code{"decreasing"}, or \code{"convex"} or \code{"concave"}. 
    Additionally, \code{"none"} can  be used to specify unconstrained P-splines. 
    This is especially of interest in conjunction with \code{boundary.constraints = TRUE}.
    For \code{bbs}, the constraint can be \code{"none"}, monotonic \code{"increasing"}, or
    \code{"decreasing"}. In general it is advisable to use \code{bmono} to fit 
    monotonic splines.}
  \item{type}{
    determines how the constrained least squares problem should be
    solved. If \code{type = "quad.prog"}, a numeric quadratic
    programming method
    %%% PROBLEM: mboost::Goldfarb:1982
    \bibcitep{mboost::Goldfarb:1983} is used
    (see \code{\link[quadprog]{solve.QP}} in package \pkg{quadprog}). If
    \code{type = "iterative"}, the iterative procedure described in
    \bibcitet{mboost::Hofner+Mueller+Hothorn_2011}. 
    The quadratic programming approach is
    usually much faster than the iterative approach. For details see
    \bibcitet{mboost::Hofner_Kneib_Hothorn_2015}.
}
  \item{lambda2}{ penalty parameter for the (monotonicity) constraint. }
  \item{niter}{ maximum number of iterations used to compute constraint
    estimates. Increase this number if a warning is displayed. }
  \item{boundary.constraints}{ a logical indicating whether additional
    constraints on the boundaries of the spline should be applied
    (default: FALSE). This is still experimental.}
  \item{cons.arg}{ a named list with additional arguments for boundary
    constraints. The element \code{lambda} specifies the
    penalty parameter that is used for the additional boundary
    constraint. The element \code{n} specifies the number of knots to be
    subject to the constraint and  can be either a scalar (use same
    number of constrained knots on each side) or a vector. Per default
    10\% of the knots on each side are used. The element
    \code{diff_order} can be used to specify the order of the boundary
    penalty: 1 (constant; default for monotonically constrained effects)
    or 2 (linear; default for all other effects).}
  \item{bnd}{
    Object of class \code{bnd}, in which the boundaries of a map are
    defined and from which neighborhood relations can be constructed. See
    \code{\link[BayesX]{read.bnd}}. If a boundary object is not
    available, the neighborhood matrix can also be given directly. }
  \item{X}{ design matrix as it should be used in the penalized least
    squares estimation. Effect modifiers do not need to be included here
    (\code{by} can be used for convenience). }
  \item{K}{ penalty matrix as it should be used in the penalized least
    squares estimation. If \code{NULL} (default), unpenalized estimation
    is used. }
  \item{deriv}{an integer; the derivative of the spline of the given order
               at the data is computed, defaults to zero. Note that this
	       argument is only used to set up the design matrix and
	       cannot be used in the fitting process.}
  \item{bl1}{a linear base-learner or a list of linear base-learners.}
  \item{bl2}{a linear base-learner or a list of linear base-learners.}
}
\details{
  \code{bols} refers to linear base-learners (potentially estimated with
  a ridge penalty), while \code{bbs} provide penalized regression
  splines. \code{bspatial} fits bivariate surfaces and \code{brandom}
  defines random effects base-learners. In combination with option
  \code{by}, these base-learners can be turned into varying coefficient
  terms. The linear base-learners are fitted using Ridge Regression
  where the penalty parameter \code{lambda} is either computed from
  \code{df} (default for \code{bbs}, \code{bspatial}, and
  \code{brandom}) or specified directly (\code{lambda = 0} means no
  penalization as default for \code{bols}).

  In \code{bols(x)}, \code{x} may be a numeric vector or factor.
  Alternatively, \code{x} can be a data frame containing numeric or
  factor variables. In this case, or when multiple predictor variables
  are specified, e.g., using \code{bols(x1, x2)}, the model is
  equivalent to \code{lm(y ~ ., data = x)} or \code{lm(y ~ x1 + x2)},
  respectively. By default, an intercept term is added to the
  corresponding design matrix (which can be omitted using
  \code{intercept = FALSE}). It is \emph{strongly} advised to (mean-)
  center continuous covariates, if no intercept is used in \code{bols}
  \bibcitep{see|mboost::Hofner+Hothorn+Kneib_2011|}. If \code{x} is a matrix, it is directly used
  as the design matrix and no further preprocessing (such as addition of
  an intercept) is conducted. When \code{df} (or \code{lambda}) is
  given, a ridge estimator with \code{df} degrees of freedom (see
  section \sQuote{Global Options}) is used as base-learner. Note that
  all variables are treated as a group, i.e., they enter the model
  together if the corresponding base-learner is selected. For ordinal
  variables, a ridge penalty for the differences of the adjacent
  categories \bibcitep{mboost::gertheiss:pen_ordinal:2009,mboost::Hofner+Hothorn+Kneib_2011} 
  is applied.

  With \code{bbs}, the P-spline approach of \bibcitet{mboost::Eilers1996} is
  used. P-splines use a squared \emph{k}-th-order difference penalty
  which can be interpreted as an approximation of the integrated squared
  \emph{k}-th derivative of the spline. In \code{bbs} the argument
  \code{knots} specifies either the number of (equidistant) \emph{interior}
  knots to be used for the regression spline fit or a vector including
  the positions of the \emph{interior} knots. Additionally,
  \code{boundary.knots} can be specified. However, this is only advised
  if one uses cyclic constraints, where the \code{boundary.knots}
  specify the points where the function is joined (e.g.,
  \code{boundary.knots = c(0, 2 * pi)} for angles as in a sine function
  or \code{boundary.knots = c(0, 24)} for hours during the day). For
  details on cylcic splines in the context of boosting see 
  \bibcitet{mboost::Hofner_Kneib_Hothorn_2015}.

  \code{bspatial} implements bivariate tensor product P-splines for the
  estimation of either spatial effects or interaction surfaces. Note
  that \code{bspatial(x, y)} is equivalent to \code{bbs(x, y, df = 6)}.
  For possible arguments and defaults see there. The penalty term is
  constructed based on bivariate extensions of the univariate penalties
  in \code{x} and \code{y} directions, see  \bibcitet{mboost::kneib+hothorn+tutz:2009}. 
  Note that the dimensions of the penalty matrix
  increase (quickly) with the number of knots with strong impact on
  computational time. Thus, both should not be chosen to large.
  Different knots for \code{x} and \code{y} can be specified by a named
  list.

  \code{brandom(x)} specifies a random effects base-learner based on a
  factor variable \code{x} that defines the grouping structure of the
  data set. For each level of \code{x}, a separate random intercept is
  fitted, where the random effects variance is governed by the
  specification of the degrees of freedom \code{df} or \code{lambda}
  (see section \sQuote{Global Options}). Note that \code{brandom(...)}
  is essentially a wrapper to \code{bols(..., df = 4, contrasts.arg =
  "contr.dummy")}, i.e., a wrapper that utilizes ridge-penalized
  categorical effects. For possible arguments and defaults see \code{bols}.

  For all linear base-learners the amount of smoothing is determined by
  the trace of the hat matrix, as indicated by \code{df}.

  If \code{by} is specified as an additional argument, a varying
  coefficients term is estimated, where \code{by} is the interaction
  variable and the effect modifier is given by either \code{x} or
  \code{x} and \code{y} (specified via \code{...}). If \code{bbs} is
  used, this corresponds to the classical situation of varying
  coefficients, where the effect of \code{by} varies over the co-domain
  of \code{x}. In case of \code{bspatial} as base-learner, the effect of
  \code{by} varies with respect to both \code{x} and \code{y}, i.e. an
  interaction surface between \code{x} and \code{y} is specified as
  effect modifier. For \code{brandom} specification of \code{by} leads
  to the estimation of random slopes for covariate \code{by} with
  grouping structure defined by factor \code{x} instead of a simple
  random intercept. In \code{bbs}, \code{bspatial} and \code{brandom}
  the computation of the smoothing parameter \code{lambda} for given
  \code{df}, or vice versa, might become (numerically) instable if the
  values of the interaction variable \code{by} become too large. In this
  case, we recommend to rescale the interaction covariate e.g. by
  dividing by \code{max(abs(by))}. If \code{bbs} or \code{bspatial} is
  specified with an factor variable \code{by} with more than two
  factors, the degrees of freedom are shared for the complete
  base-learner (i.e., spread over all factor levels). Note that the null
  space (see next paragraph) increases, as a separate null space for
  each factor level is present. Thus, the minimum degrees of freedom
  increase with increasing number of levels of \code{by} (if
  \code{center = FALSE}).

  For \code{bbs} and \code{bspatial}, option \code{center != FALSE} requests that
  the fitted effect is centered around its parametric, unpenalized part
  (the so called null space). For example, with second order difference
  penalty, a linear effect of \code{x} remains unpenalized by \code{bbs}
  and therefore the degrees of freedom for the base-learner have to be
  larger than two. To avoid this restriction, option \code{center =
  TRUE} subtracts the unpenalized linear effect from the fit, allowing
  to specify any positive number as \code{df}. Note that in this case
  the linear effect \code{x} should generally be specified as an
  additional base-learner \code{bols(x)}. For \code{bspatial} and, for
  example, second order differences, a linear effect of \code{x}
  (\code{bols(x)}), a linear effect of \code{y} (\code{bols(y)}), and
  their interaction (\code{bols(x*y)}) are subtracted from the effect
  and have to be added separately to the model equation. More details on
  centering can be found in \bibcitet{mboost::kneib+hothorn+tutz:2009} and
  \bibcitet{mboost::Fahrmeir2004}. We strongly recommend to consult the latter reference
  before using this option.

  \code{brad(x)} specifies penalized radial basis functions as used in
  Kriging. If \code{knots} is used to specify the number of knots, the
  function \code{\link[fields]{cover.design}} is used to specify the
  location of the knots such that they minimize a geometric
  space-filling criterion. Furthermore, knots can be specified directly
  via a matrix. The \code{cov.function} allows to specify the
  radial basis functions. Per default, the flexible Matern correlation
  function is used. This is specified using \code{cov.function =
  stationary.cov} with \code{Covariance = "Matern"} specified via
  \code{args}. If an effective range \code{theta} is applicable for the
  correlation function (e.g., the Matern family) the user can specify
  this value. Per default (if \code{theta = NULL}) the effective range is
  chosen as \eqn{\theta = max(||x_i - x_j||)/c}{\theta =
  max(||x_i - x_j||)/c} such that the correlation function
  \deqn{\rho(c; \theta = 1) = \varepsilon,}{\rho(c; theta = 1) = epsilon,}
  where \eqn{\varepsilon = 0.001}{epsilon = 0.001}.

  \code{bmrf} builds a base of a Markov random field consisting of
  several regions with a neighborhood structure. The input variable is
  the observed region. The penalty matrix is either construed from a
  boundary object or must be given directly via the option \code{bnd}.
  In that case the \code{dimnames} of the matrix have to be the region
  names, on the diagonal the number of neighbors have to be given for
  each region, and for each neighborhood relation the value in the
  matrix has to be -1, else 0. With a boundary object at hand, the
  fitted or predicted values can be directly plotted into the map using
  \code{\link[BayesX]{drawmap}}.
  
  \code{bkernel} can be used to fit linear (\code{kernel = "lin"}), 
  size-adjusted (\code{kernel = "sia"}) or network (\code{kernel = "net"})
  kernels based on genetic pathways for genome-wide assosiation studies. 
  For details see \bibcitet{mboost::Friedrichs_2017} and check the associated package
  \CRANpkg{kangar00}.

  \code{buser(X, K)} specifies a base-learner with user-specified design
  matrix \code{X} and penalty matrix \code{K}, where \code{X} and
  \code{K} are used to minimize a (penalized) least squares
  criterion with quadratic penalty. This can be used to easily specify
  base-learners that are not implemented (yet). See examples
  below for details how \code{buser} can be used to mimic existing
  base-learners. Note that for predictions you need to set up the
  design matrix for the new data manually.

  For a categorical covariate with non-observed categories
  \code{bols(x)} and \code{brandom(x)} both assign a zero effect
  to these categories. However, the non-observed categories must be
  listed in \code{levels(x)}. Thus, predictions are possible
  for new observations if they correspond to this category.

  By default, all linear base-learners include an intercept term (which can
  be removed using \code{intercept = FALSE} for \code{bols}). In this case, 
  the respective covariate should be mean centered (if continuous) and an 
  explicit global intercept term should be added to \code{gamboost} 
  via \code{bols} (see example below). With \code{bols(x, intercept = FALSE)} 
  with categorical covariate \code{x} a separate effect for each group 
  (mean effect) is estimated (see examples for resulting design matrices).

  Smooth estimates with constraints can be computed using the
  base-learner \code{bmono()} which specifies P-spline base-learners
  with an additional asymmetric penalty enforcing monotonicity or
  convexity/concavity \bibcitep{see|mboost::Eilers2005|}. For more details in the
  boosting context and monotonic effects of ordinal factors see
  \bibcitet{mboost::Hofner+Mueller+Hothorn_2011}. The quadratic-programming based algorithm
  is described in \bibcitet{mboost::Hofner_Kneib_Hothorn_2015}. Alternative monotonicity
  constraints are implemented via T-splines in \code{bbs()}
  \bibcitep{mboost::Beliakov2000}. 
  In general it is advisable to use \code{bmono} to fit monotonic splines 
  as T-splines show undesirable behaviour if the observed data deviates 
  from monotonicty.

  Two or more linear base-learners can be joined using \code{\%+\%}. A
  tensor product of two or more linear base-learners is returned by
  \code{\%X\%}. When the design matrix can be written as the Kronecker
  product of two matrices \code{X = kronecker(X2, X1)}, then \code{bl1
  \%O\% bl2} with design matrices X1 and X2, respectively, can be used
  to efficiently compute Ridge-estimates following
  \bibcitet{mboost::Currie_Durban_Eilers_2006}. 
  In all cases the overall degrees of freedom of the
  combined base-learner increase (additive or multiplicative,
  respectively). These three features are experimental and for expert
  use only.

  \code{btree} fits a stump to one or more variables. Note that
  \code{\link{blackboost}} is more efficient for boosting stumps. For
  further references see \bibcitet{mboost::Hothorn:2006:JCGS} and 
  \bibcitet{mboost::Hothorn+Buehlmann+Kneib+Schmid+Hofner_2010}.

  Note that the base-learners \code{bns} and \code{bss} are deprecated
  (and no longer available). Please use \code{bbs} instead, which
  results in qualitatively the same models but is computationally much
  more attractive.

}
\section{Global Options}{
  Three global options affect the base-learners:

  \describe{

    \item{\code{options("mboost_useMatrix")}}{ defaulting to \code{TRUE}
      indicates that the base-learner may use sparse matrix techniques
      for its computations. This reduces the memory consumption but
      might (for smaller sample sizes) require more computing time.}

    \item{\code{options("mboost_indexmin")}}{is an integer that
      specifies the minimum sample size needed to optimize model fitting
      by automatically taking ties into account (default = 10000).}

    \item{\code{options("mboost_dftraceS")}}{\code{FALSE} by default,
      indicating how the degrees of freedom should be computed. Per
      default \deqn{\mathrm{df}(\lambda) = \mathrm{trace}(2S -
      S^{\top}S),}{df(\lambda) = trace(2S - S'S),} with smoother matrix
      \eqn{S = X(X^{\top}X + \lambda K)^{-1} X}{S = X(X'X + \lambda
      K)^(-1) X} is used \bibcitep{see|mboost::Hofner+Hothorn+Kneib_2011|}. If \code{TRUE}, the
      trace of the smoother matrix \eqn{\mathrm{df}(\lambda) =
      \mathrm{trace}(S)} is used as degrees of freedom.

      Note that these formulae specify the relation of \code{df} and
      \code{lambda} as the smoother matrix \eqn{S} depends only on
      \eqn{\lambda} (and the (fixed) design matrix \eqn{X}, the (fixed)
      penalty matrix \eqn{K}).}

  }
}

\value{
 An object of class \code{blg} (base-learner generator) with a
 \code{dpp} function.

 The call of \code{dpp} returns an object of class
 \code{bl} (base-learner) with a \code{fit} function. The call to
 \code{fit} finally returns an object of class \code{bm} (base-model).
}
\references{\bibshow{*}}

\seealso{\code{\link{mboost}}}

\examples{

  set.seed(290875)

  n <- 100
  x1 <- rnorm(n)
  x2 <- rnorm(n) + 0.25 * x1
  x3 <- as.factor(sample(0:1, 100, replace = TRUE))
  x4 <- gl(4, 25)
  y <- 3 * sin(x1) + x2^2 + rnorm(n)
  weights <- drop(rmultinom(1, n, rep.int(1, n) / n))

  ### set up base-learners
  spline1 <- bbs(x1, knots = 20, df = 4)
  extract(spline1, "design")[1:10, 1:10]
  extract(spline1, "penalty")
  knots.x2 <- quantile(x2, c(0.25, 0.5, 0.75))
  spline2 <- bbs(x2, knots = knots.x2, df = 5)
  ols3 <- bols(x3)
  extract(ols3)
  ols4 <- bols(x4)

  ### compute base-models
  drop(ols3$dpp(weights)$fit(y)$model) ## same as:
  coef(lm(y ~ x3, weights = weights))

  drop(ols4$dpp(weights)$fit(y)$model) ## same as:
  coef(lm(y ~ x4, weights = weights))

  ### fit model, component-wise
  mod1 <- mboost_fit(list(spline1, spline2, ols3, ols4), y, weights)

  ### more convenient formula interface
  mod2 <- mboost(y ~ bbs(x1, knots = 20, df = 4) +
                     bbs(x2, knots = knots.x2, df = 5) +
                     bols(x3) + bols(x4), weights = weights)
  all.equal(coef(mod1), coef(mod2))


  ### grouped linear effects
  # center x1 and x2 first
  x1 <- scale(x1, center = TRUE, scale = FALSE)
  x2 <- scale(x2, center = TRUE, scale = FALSE)
  model <- gamboost(y ~ bols(x1, x2, intercept = FALSE) +
                        bols(x1, intercept = FALSE) +
                        bols(x2, intercept = FALSE),
                        control = boost_control(mstop = 50))
  coef(model, which = 1)   # one base-learner for x1 and x2
  coef(model, which = 2:3) # two separate base-learners for x1 and x2
                           # zero because they were (not yet) selected.

  ### example for bspatial
  x1 <- runif(100,-pi,pi)
  x2 <- runif(length(x1),-pi,pi)

  y <- sin(x1) * sin(x2) + rnorm(length(x1), sd = 0.4)

  spline3 <- bspatial(x1, x2, knots = 12)
  Xmat <- extract(spline3, "design")
  ## 12 inner knots + 4 boundary knots = 16 knots per direction
  ## THUS: 16 * 16 = 256 columns
  dim(Xmat)
  extract(spline3, "penalty")[1:10, 1:10]

  ## specify number of knots separately
  form1 <- y ~ bspatial(x1, x2, knots = list(x1 = 12, x2 = 14))

  ## decompose spatial effect into parametric part and
  ## deviation with one df
  form2 <- y ~ bols(x1) + bols(x2) + bols(x1, by = x2, intercept = FALSE) +
               bspatial(x1, x2, knots = 12, center = TRUE, df = 1)

  mod1 <- gamboost(form1)
  \dontrun{
  plot(mod1)
  }

  mod2 <- gamboost(form2)
  ## automated plot function:
  \dontrun{
  plot(mod2)
  }
  ## plot sum of linear and smooth effects:
  library("lattice")
  df <- expand.grid(x1 = unique(x1), x2 = unique(x2))
  df$pred <- predict(mod2, newdata = df)
  \dontrun{
  levelplot(pred ~ x1 * x2, data = df)
  }

  ## specify radial basis function base-learner for spatial effect
  ## with non-default settings, e.g. 50 knots and theta fixed to 0.4
  ## (not really a good setting)
  form3 <- y ~ brad(x1, x2, knots = 50, args = list(theta = 0.4))

  mod3 <- gamboost(form3)
  dim(extract(mod3, what = "design", which = "brad")[[1]])
  \dontrun{
  plot(mod3)
  }

  ### random intercept
  id <- factor(rep(1:10, each = 5))
  raneff <- brandom(id)
  extract(raneff, "design")
  extract(raneff, "penalty")

  ## random intercept with non-observed category
  set.seed(1907)
  y <- rnorm(50, mean = rep(rnorm(10), each = 5), sd = 0.1)
  plot(y ~ id)
  # category 10 not observed
  obs <- c(rep(1, 45), rep(0, 5))
  model <- gamboost(y ~ brandom(id), weights = obs)
  coef(model)
  fitted(model)[46:50] # just the grand mean as usual for
                       # random effects models


  ### random slope
  z <- runif(50)
  raneff <- brandom(id, by = z)
  extract(raneff, "design")
  extract(raneff, "penalty")

  ### specify simple interaction model (with main effect)
  n <- 210
  x <- rnorm(n)
  X <- model.matrix(~ x)
  z <- gl(3, n/3)
  Z <- model.matrix(~z)
  beta <- list(c(0,1), c(-3,4), c(2, -4))
  y <- rnorm(length(x), mean = (X * Z[,1]) \%*\% beta[[1]] +
                               (X * Z[,2]) \%*\% beta[[2]] +
                               (X * Z[,3]) \%*\% beta[[3]])
  plot(y ~ x, col = z)
  ## specify main effect and interaction
  mod_glm <- gamboost(y ~ bols(x) + bols(x, by = z),
                  control = boost_control(mstop = 100))
  nd <- data.frame(x, z)
  nd <- nd[order(x),]
  nd$pred_glm <- predict(mod_glm, newdata = nd)
  for (i in seq(along = levels(z)))
      with(nd[nd$z == i,], lines(x, pred_glm, col = z))
  mod_gam <- gamboost(y ~ bbs(x) + bbs(x, by = z, df = 8),
                      control = boost_control(mstop = 100))
  nd$pred_gam <- predict(mod_gam, newdata = nd)
  for (i in seq(along = levels(z)))
      with(nd[nd$z == i,], lines(x, pred_gam, col = z, lty = "dashed"))
  ### convenience function for plotting
  \dontrun{
  par(mfrow = c(1,3))
  plot(mod_gam)
  }


  ### remove intercept from base-learner
  ### and add explicit intercept to the model
  tmpdata <- data.frame(x = 1:100, y = rnorm(1:100), int = rep(1, 100))
  mod <- gamboost(y ~ bols(int, intercept = FALSE) +
                      bols(x, intercept = FALSE),
                  data = tmpdata,
                  control = boost_control(mstop = 1000))
  cf <- unlist(coef(mod))
  ## add offset
  cf[1] <- cf[1] + mod$offset
  signif(cf, 3)
  signif(coef(lm(y ~ x, data = tmpdata)), 3)

  ### much quicker and better with (mean-) centering
  tmpdata$x_center <- tmpdata$x - mean(tmpdata$x)
  mod_center <- gamboost(y ~ bols(int, intercept = FALSE) +
                             bols(x_center, intercept = FALSE),
                         data = tmpdata,
                         control = boost_control(mstop = 100))
  cf_center <- unlist(coef(mod_center, which=1:2))
  ## due to the shift in x direction we need to subtract
  ## beta_1 * mean(x) to get the correct intercept
  cf_center[1] <- cf_center[1] + mod_center$offset -
                  cf_center[2] * mean(tmpdata$x)
  signif(cf_center, 3)
  signif(coef(lm(y ~ x, data = tmpdata)), 3)

\dontrun{############################################################
## Do not run and check these examples automatically as
## they take some time

  ### large data set with ties
  nunique <- 100
  xindex <- sample(1:nunique, 1000000, replace = TRUE)
  x <- runif(nunique)
  y <- rnorm(length(xindex))
  w <- rep.int(1, length(xindex))

  ### brute force computations
  op <- options()
  options(mboost_indexmin = Inf, mboost_useMatrix = FALSE)
  ## data pre-processing
  b1 <- bbs(x[xindex])$dpp(w)
  ## model fitting
  c1 <- b1$fit(y)$model
  options(op)

  ### automatic search for ties, faster
  b2 <- bbs(x[xindex])$dpp(w)
  c2 <- b2$fit(y)$model

  ### manual specification of ties, even faster
  b3 <- bbs(x, index = xindex)$dpp(w)
  c3 <- b3$fit(y)$model

  all.equal(c1, c2)
  all.equal(c1, c3)

## End(Not run and test)
}

  ### cyclic P-splines
  set.seed(781)
  x <- runif(200, 0,(2*pi))
  y <- rnorm(200, mean=sin(x), sd=0.2)
  newX <- seq(0,2*pi, length=100)
  ### model without cyclic constraints
  mod <- gamboost(y ~ bbs(x, knots = 20))
  ### model with cyclic constraints
  mod_cyclic <- gamboost(y ~ bbs(x, cyclic=TRUE, knots = 20,
                                 boundary.knots=c(0, 2*pi)))
  par(mfrow = c(1,2))
  plot(x,y, main="bbs (non-cyclic)", cex=0.5)
  lines(newX, sin(newX), lty="dotted")
  lines(newX + 2 * pi, sin(newX), lty="dashed")
  lines(newX, predict(mod, data.frame(x = newX)),
        col="red", lwd = 1.5)
  lines(newX + 2 * pi, predict(mod, data.frame(x = newX)),
        col="blue", lwd=1.5)
  plot(x,y, main="bbs (cyclic)", cex=0.5)
  lines(newX, sin(newX), lty="dotted")
  lines(newX + 2 * pi, sin(newX), lty="dashed")
  lines(newX, predict(mod_cyclic, data.frame(x = newX)),
        col="red", lwd = 1.5)
  lines(newX + 2 * pi, predict(mod_cyclic, data.frame(x = newX)),
        col="blue", lwd = 1.5)

  ### use buser() to mimic p-spline base-learner:
  set.seed(1907)
  x <- rnorm(100)
  y <- rnorm(100, mean = x^2, sd = 0.1)
  mod1 <- gamboost(y ~ bbs(x))
  ## now extract design and penalty matrix
  X <- extract(bbs(x), "design")
  K <- extract(bbs(x), "penalty")
  ## use X and K in buser()
  mod2 <- gamboost(y ~ buser(X, K))
  max(abs(predict(mod1) - predict(mod2)))  # same results

  ### use buser() to mimic penalized ordinal base-learner:
  z <- as.ordered(sample(1:3, 100, replace=TRUE))
  y <- rnorm(100, mean = as.numeric(z), sd = 0.1)
  X <- extract(bols(z))
  K <- extract(bols(z), "penalty")
  index <- extract(bols(z), "index")
  mod1 <- gamboost(y ~  buser(X, K, df = 1, index = index))
  mod2 <- gamboost(y ~  bols(z, df = 1))
  max(abs(predict(mod1) - predict(mod2)))  # same results

  ### kronecker product for matrix-valued responses
  data("volcano", package = "datasets")
  layout(matrix(1:2, ncol = 2))

  ## estimate mean of image treating image as matrix
  image(volcano, main = "data")
  x1 <- 1:nrow(volcano)
  x2 <- 1:ncol(volcano)

  vol <- as.vector(volcano)
  mod <- mboost(vol ~ bbs(x1, df = 3, knots = 10)\%O\%
                      bbs(x2, df = 3, knots = 10),
                      control = boost_control(nu = 0.25))
  volf <- matrix(fitted(mod), nrow = nrow(volcano))
  image(volf, main = "fitted")

\dontrun{############################################################
## Do not run and check these examples automatically as
## they take some time

  ## the old-fashioned way, a waste of space and time
  x <- expand.grid(x1, x2)
  modx <- mboost(vol ~ bbs(Var2, df = 3, knots = 10) \%X\%
                       bbs(Var1, df = 3, knots = 10), data = x,
                       control = boost_control(nu = 0.25))
  modx[250]

  max(abs(fitted(mod) - fitted(modx)))

## End(Not run and test)
}

  ### setting contrasts via contrasts.arg
  x <- as.factor(sample(1:4, 100, replace = TRUE))

  ## compute base-learners with different reference categories
  BL1 <- bols(x, contrasts.arg = contr.treatment(4, base = 1)) # default
  BL2 <- bols(x, contrasts.arg = contr.treatment(4, base = 2))
  ## compute 'sum to zero contrasts' using character string
  BL3 <- bols(x, contrasts.arg = "contr.sum")

  ## extract model matrices to check if it works
  extract(BL1)
  extract(BL2)
  extract(BL3)

  ### setting contrasts using named lists in contrasts.arg
  x2 <- as.factor(sample(1:4, 100, replace = TRUE))

  BL4 <- bols(x, x2,
              contrasts.arg = list(x = contr.treatment(4, base = 2),
                                   x2 = "contr.helmert"))
  extract(BL4)

  ### using special contrast: "contr.dummy":
  BL5 <- bols(x, contrasts.arg = "contr.dummy")
  extract(BL5)
}
\keyword{models}

\name{cvrisk}
\alias{cvrisk}
\alias{cvrisk.mboost}
\alias{print.cvrisk}
\alias{plot.cvrisk}
\alias{cv}

\title{ Cross-Validation }
\description{
  Cross-validated estimation of the empirical risk for hyper-parameter selection.
}
\usage{
\method{cvrisk}{mboost}(object, folds = cv(model.weights(object)),
       grid = 0:mstop(object),
       papply = mclapply,
       fun = NULL, mc.preschedule = FALSE, ...)
cv(weights, type = c("bootstrap", "kfold", "subsampling"),
   B = ifelse(type == "kfold", 10, 25), prob = 0.5, strata = NULL)

## Plot cross-valiation results   
\method{plot}{cvrisk}(x, 
     xlab = "Number of boosting iterations", ylab = attr(x, "risk"),
     ylim = range(x), main = attr(x, "type"), ...)
}
\arguments{
  \item{object}{ an object of class \code{mboost}.}
  \item{folds}{ a weight matrix with number of rows equal to the number
                of observations. The number of columns corresponds to
                the number of cross-validation runs. Can be computed
                using function \code{cv} and defaults to 25 bootstrap samples.}
  \item{grid}{ a vector of stopping parameters the empirical risk
                is to be evaluated for. }
  \item{papply}{
    (parallel) apply function, defaults to  \code{\link[parallel]{mclapply}}.
    Alternatively, \code{\link[parallel]{parLapply}} can be used. In the
    latter case, usually more setup is needed (see example for some
    details). To run \code{cvrisk} sequentially (i.e. not in parallel),
    one can use \code{\link{lapply}}.
  }
  \item{fun}{ if \code{fun} is NULL, the out-of-sample risk is returned. \code{fun},
              as a function of \code{object}, may extract any other characteristic
              of the cross-validated models. These are returned as is.}
  \item{mc.preschedule}{
    preschedule tasks if are parallelized using \code{\link{mclapply}}
    (default: \code{FALSE})? For details see \code{\link{mclapply}}.
  }
  \item{weights}{ a numeric vector of weights for the model to be cross-validated.}
  \item{type}{ character argument for specifying the cross-validation
               method. Currently (stratified) bootstrap, k-fold cross-validation
               and subsampling are implemented.}
  \item{B}{ number of folds, per default 25 for \code{bootstrap} and
            \code{subsampling} and 10 for \code{kfold}.}
  \item{prob}{ percentage of observations to be included in the learning samples
               for subsampling.}
  \item{strata}{ a factor of the same length as \code{weights} for stratification.}
  \item{x}{ an object of class \code{cvrisk}.}
  \item{xlab, ylab}{axis labels.}
  \item{ylim}{limits of y-axis.}
  \item{main}{main title of graphic.}
  \item{...}{additional arguments passed to \code{\link[parallel]{mclapply}} or 
     \code{\link{plot}}.}
}
\details{

  The number of boosting iterations is a hyper-parameter of the
  boosting algorithms implemented in this package \bibcitep{mboost::Mayr_2012}. Honest,
  i.e., cross-validated, estimates of the empirical risk
  for different stopping parameters \code{mstop} are computed by
  this function which can be utilized to choose an appropriate
  number of boosting iterations to be applied.

  Different forms of cross-validation can be applied, for example
  10-fold cross-validation or bootstrapping. The weights (zero weights
  correspond to test cases) are defined via the \code{folds} matrix.

  \code{cvrisk} runs in parallel on OSes where forking is possible
  (i.e., not on Windows) and multiple cores/processors are available.
  The scheduling
  can be changed by the corresponding arguments of
  \code{\link[parallel]{mclapply}} (via the dot arguments).

  The function \code{cv} can be used to build an appropriate
  weight matrix to be used with \code{cvrisk}. If \code{strata} is defined
  sampling is performed in each stratum separately thus preserving
  the distribution of the \code{strata} variable in each fold.
  
  There exist various functions to display and work with 
  cross-validation results. One can \code{print} and \code{plot} (see above)
  results and extract the optimal iteration via \code{\link{mstop}}.
}
\value{
  An object of class \code{cvrisk} (when \code{fun} wasn't specified), basically a matrix
  containing estimates of the empirical risk for a varying number
  of bootstrap iterations. \code{plot} and \code{print} methods
  are available as well as a \code{mstop} method.
}
\references{\bibshow{*}}
\seealso{\code{\link{AIC.mboost}} for
  \code{AIC} based selection of the stopping iteration. Use \code{mstop}
  to extract the optimal stopping iteration from \code{cvrisk}
  object.}
\examples{

  data("bodyfat", package = "TH.data")

  ### fit linear model to data
  model <- glmboost(DEXfat ~ ., data = bodyfat, center = TRUE)

  ### AIC-based selection of number of boosting iterations
  maic <- AIC(model)
  maic

  ### inspect coefficient path and AIC-based stopping criterion
  par(mai = par("mai") * c(1, 1, 1, 1.8))
  plot(model)
  abline(v = mstop(maic), col = "lightgray")

  ### 10-fold cross-validation
  cv10f <- cv(model.weights(model), type = "kfold")
  cvm <- cvrisk(model, folds = cv10f, papply = lapply)
  print(cvm)
  mstop(cvm)
  plot(cvm)

  ### 25 bootstrap iterations (manually)
  set.seed(290875)
  n <- nrow(bodyfat)
  bs25 <- rmultinom(25, n, rep(1, n)/n)
  cvm <- cvrisk(model, folds = bs25, papply = lapply)
  print(cvm)
  mstop(cvm)
  plot(cvm)

  ### same by default
  set.seed(290875)
  cvrisk(model, papply = lapply)

  ### 25 bootstrap iterations (using cv)
  set.seed(290875)
  bs25_2 <- cv(model.weights(model), type="bootstrap")
  all(bs25 == bs25_2)

\dontrun{
############################################################
## Do not run this example automatically as it takes
## some time (~ 5 seconds depending on the system)

  ### trees
  blackbox <- blackboost(DEXfat ~ ., data = bodyfat)
  cvtree <- cvrisk(blackbox, papply = lapply)
  plot(cvtree)
  
## End(Not run this automatically)  
}


### cvrisk in parallel modes:

\dontrun{
## at least not automatically

## parallel::mclapply() which is used here for parallelization only runs 
## on unix systems (here we use 2 cores)

    cvrisk(model, mc.cores = 2)

## infrastructure needs to be set up in advance

    cl <- makeCluster(25) # e.g. to run cvrisk on 25 nodes via PVM
    myApply <- function(X, FUN, ...) {
      myFun <- function(...) {
          library("mboost") # load mboost on nodes
          FUN(...)
      }
      ## further set up steps as required
      parLapply(cl = cl, X, myFun, ...)
    }
    cvrisk(model, papply = myApply)
    stopCluster(cl)
}

}
\keyword{models}
\keyword{regression}

\name{Family}
\alias{Family}
\alias{AdaExp}
\alias{Binomial}
\alias{GaussClass}
\alias{GaussReg}
\alias{Gaussian}
\alias{Huber}
\alias{Laplace}
\alias{Poisson}
\alias{GammaReg}
\alias{CoxPH}
\alias{QuantReg}
\alias{ExpectReg}
\alias{NBinomial}
\alias{PropOdds}
\alias{Weibull}
\alias{Loglog}
\alias{Lognormal}
\alias{AUC}
\alias{Gehan}
\alias{Hurdle}
\alias{Multinomial}
\alias{Cindex}
\alias{RCG}

\title{ Gradient Boosting Families }
\description{
    \code{boost_family} objects provide a convenient way to specify loss functions
    and corresponding risk functions to be optimized by one of the boosting
    algorithms implemented in this package.
}
\usage{
Family(ngradient, loss = NULL, risk = NULL,
       offset = function(y, w)
           optimize(risk, interval = range(y),
                    y = y, w = w)$minimum,
       check_y = function(y) y,
       weights = c("any", "none", "zeroone", "case"),
       nuisance = function() return(NA),
       name = "user-specified", fW = NULL,
       response = function(f) NA,
       rclass = function(f) NA)
AdaExp()
AUC()
Binomial(type = c("adaboost", "glm"),
         link = c("logit", "probit", "cloglog", "cauchit", "log"), ...)
GaussClass()
GaussReg()
Gaussian()
Huber(d = NULL)
Laplace()
Poisson()
GammaReg(nuirange = c(0, 100))
CoxPH()
QuantReg(tau = 0.5, qoffset = 0.5)
ExpectReg(tau = 0.5)
NBinomial(nuirange = c(0, 100))
PropOdds(nuirange = c(-0.5, -1), offrange = c(-5, 5))
Weibull(nuirange = c(0, 100))
Loglog(nuirange = c(0, 100))
Lognormal(nuirange = c(0, 100))
Gehan()
Hurdle(nuirange = c(0, 100))
Multinomial()
Cindex(sigma = 0.1, ipcw = 1)
RCG(nuirange = c(0, 1), offrange = c(-5, 5))

}
\arguments{
  \item{ngradient}{ a function with arguments \code{y}, \code{f} and \code{w} implementing the
                    \emph{negative} gradient of the \code{loss} function (which is to be minimized). }
  \item{loss}{ an optional loss function with arguments \code{y} and \code{f}. }
  \item{risk}{ an optional risk function with arguments \code{y}, \code{f} and \code{w} to be minimized (!),
               the weighted mean of the loss function by default. }
  \item{offset}{ a function with argument \code{y} and \code{w} (weights)
                 for computing a \emph{scalar} offset. }
  \item{fW}{ transformation of the fit for the diagonal weights matrix for an
             approximation of the boosting hat matrix for loss functions other than
             squared error.}
  \item{response}{ inverse link function of a GLM or any other transformation
                   on the scale of the response.}
  \item{rclass}{ function to derive class predictions from conditional class
                   probabilities (for models with factor response variable).}
  \item{check_y}{ a function for checking and transforming
                  the class / mode of a response variable.}
  \item{nuisance}{ a function for extracting nuisance parameters from the family.}
  \item{weights}{ a character indicating what type of weights are
    allowed. These can be either arbitrary (non-negative) weights
    \code{"any"}, only zero and one weights \code{"zeroone"},
    (non-negative) interger weights \code{"case"}, or no weights are
    allowed \code{"none"}.}
  \item{name}{ a character giving the name of the loss function for pretty printing. }
  \item{type}{ which parameterization of \code{Binomial} shoule be used?} b
  \item{link}{ link function. For possible values see Usage section.}
  \item{d}{ delta parameter for Huber loss function. If omitted, it is chosen adaptively.}
  \item{tau}{ the quantile or expectile to be estimated, a number strictly between 0 and 1.}
  \item{qoffset}{ quantile of response distribution to be used as offset, i.e., 
      starting values for the intercept. Per default the median of the response is used,
      which is in general a good choice
      \bibcitep{see|mboost::Fenske+Kneib+Hothorn_2011|for details}.}
  \item{nuirange}{ a vector containing the end-points of the interval to be
          searched for the minimum risk w.r.t. the nuisance parameter.
	  In case of \code{PropOdds}, the starting values for
	  the nuisance parameters. }
  \item{offrange}{ interval to search in for offset.}
  \item{sigma}{smoothness parameter for sigmoid functions inside \code{Cindex}.}
  \item{ipcw}{vector containing inverse probability of censoring weights for all observations. If omitted, it is estimated inside \code{Cindex} family.}
  \item{\dots}{ additional arguments to link functions.}
}

\details{
  The boosting algorithm implemented in \code{\link{mboost}} minimizes the
  (weighted) empirical risk function \code{risk(y, f, w)} with respect to \code{f}.
  By default, the risk function is the weighted sum of the loss function \code{loss(y, f)}
  but can be chosen arbitrarily. The \code{ngradient(y, f)} function is the negative
  gradient of \code{loss(y, f)} with respect to \code{f}.

  Pre-fabricated functions for the most commonly used loss functions are
  available as well. \bibcitet{mboost::Buehlmann:2008:StatSci} give a detailed
  overview of the available loss functions. An updated overview can 
  be found in \bibcitet{mboost::Hofner_Mayr_2014}.
  
  The \code{offset} function returns the population minimizers evaluated at the 
  response, i.e., \eqn{1/2 \log(p / (1 - p))} for \code{Binomial()} or \code{AdaExp()}
  and \eqn{(\sum w_i)^{-1} \sum w_i y_i} for \code{Gaussian()} and the
  median for \code{Huber()} and \code{Laplace()}. The offset is used as starting 
  value for the boosting algorithm.
  
  Note that all families are functions and thus need to be specified either with
  empty brackets (e.g., \code{family = Gaussian()} for Gaussian regression) or 
  with additional arguments if these are supported by the respective family 
  (e.g., \code{family = QuantReg(tau = 0.2)} for quantile regression for the 
  20\% quantile).
  
  \strong{A short summary of the available families is given in the following paragraphs:}

\code{AdaExp()}, \code{Binomial()} and \code{AUC()} implement
families for binary classification. \code{AdaExp()} uses the
exponential loss, which essentially leads to the AdaBoost algorithm
of \bibcitet{mboost::nr:freund.schapire:1996}. \code{Binomial()} implements the
negative binomial log-likelihood of a logistic regression model
as loss function. Thus, using \code{Binomial} family closely corresponds
to fitting a logistic model. Alternative link functions
can be specified.

However, the coefficients resulting from boosting with family
\code{Binomial(link = "logit")} are \eqn{1/2} of the coefficients of a logit model
obtained via \code{\link{glm}}.  \bibcitet{mboost::Buehlmann:2008:StatSci} argue that the
family \code{Binomial} is the preferred choice for binary
classification. For binary classification problems the response
\code{y} has to be a \code{factor}. Internally \code{y} is re-coded
to \eqn{-1} and \eqn{+1} (Buehlmann and Hothorn 2007). 

\code{Binomial(type = "glm")} is an alternative to \code{Binomial()} leading to 
coefficients of the same size as coefficients from a classical logit 
model via \code{\link{glm}}. Additionally, it works not only with a 
two-level factor but also with a two-column matrix containing the number 
of successes and number of failures (again, similar to \code{\link{glm}}). 

\code{AUC()} uses \eqn{1-AUC(y, f)} as the loss function.
The area under the ROC curve (AUC) is defined as
\eqn{AUC = (n_{-1} n_1)^{-1} \sum_{i: y_i = 1} \sum_{j: y_j = -1} I(f_i > f_j)}.
Since this is not differentiable in \code{f}, we approximate the jump function
\eqn{I((f_i - f_j) > 0)} by the distribution function of the triangular
distribution on \eqn{[-1, 1]} with mean \eqn{0}, similar to the logistic
distribution approximation used in \bibcitet{mboost::Ma_Huang_2005}.

  \code{Gaussian()} is the default family in \code{\link{mboost}}. It
  implements \eqn{L_2}Boosting for continuous response. Note
  that families \code{GaussReg()} and \code{GaussClass()} (for regression
  and classification) are deprecated now.
  \code{Huber()} implements a robust version for boosting with
  continuous response, where the Huber-loss is used. \code{Laplace()}
  implements another strategy for continuous outcomes and uses the
  \eqn{L_1}-loss instead of the \eqn{L_2}-loss as used by
  \code{Gaussian()}.

  \code{Poisson()} implements a family for fitting count data with
  boosting methods. The implemented loss function is the negative
  Poisson log-likelihood. Note that the natural link function
  \eqn{\log(\mu) = \eta} is assumed. The default step-site \code{nu = 0.1}
  is probably too large for this family (leading to
  infinite residuals) and smaller values are more appropriate.

  \code{GammaReg()} implements a family for fitting nonnegative response
  variables. The implemented loss function is the negative Gamma
  log-likelihood with logarithmic link function (instead of the natural
  link).

  \code{CoxPH()} implements the negative partial log-likelihood for Cox
  models. Hence, survival models can be boosted using this family.

  \code{QuantReg()} implements boosting for quantile regression, which is
  introduced in \bibcitet{mboost::Fenske+Kneib+Hothorn_2011}. \code{ExpectReg} works in analogy,
  only for expectiles, which were introduced to regression by
  \bibcitet{mboost::Newey_Powell_1987}.

  Families with an additional scale parameter can be used for fitting
  models as well: \code{PropOdds()} leads to proportional odds models
  for ordinal outcome variables
  \bibcitep{mboost::Schmid+Hothorn+Maloney_2011}. When using this
  family, an ordered set of threshold parameters is re-estimated in each
  boosting iteration. An example is given below which also shows how to
  obtain the thresholds. \code{NBinomial()} leads to regression models with
  a negative binomial conditional distribution of the response.
  \code{Weibull()}, \code{Loglog()}, and \code{Lognormal()} implement
  the negative log-likelihood functions of accelerated failure time
  models with Weibull, log-logistic, and lognormal distributed outcomes,
  respectively. Hence, parametric survival models can be boosted using
  these families. For details see
  \bibcitet{mboost::schmid:hothorn:aft-boost,mboost::Schmid+Potapov+Pfahlberg:2010}.

  \code{Gehan()} implements rank-based estimation of survival data in an
  accelerated failure time model. The loss function is defined as the sum
  of the pairwise absolute differences of residuals. The response needs to
  be defined as \code{Surv(y, delta)}, where \code{y} is the observed survial
  time (subject to censoring) and \code{delta} is the non-censoring indicator
  (see \code{\link[survival]{Surv}} for details). For details on \code{Gehan()} see
  \bibcitet{mboost::Johnson_2011}.

  \code{Cindex()} optimizes the concordance-index for survival data (often denoted
  as Harrell's C or C-index). The concordance index evaluates the rank-based
  concordance probability between the model and the outcome. The C-index measures
  whether large values of the model are associated with short survival times  and
  vice versa. The interpretation is similar to the AUC: A C-index of 1 represents a
  perfect discrimination while a C-index of 0.5 will be achieved by a completely
  non-informative marker. The \code{Cindex()} family is based on an estimator by
  \bibcitet{mboost::Uno_Cai_2011}, which incorporates inverse probability of censoring weighting
  \code{ipcw}. To make the estimator differentiable, sigmoid functions are applied;
  the corresponding smoothness can be controlled via \code{sigma}. For details on
  \code{Cindex()} see \bibcitet{mboost::Mayr_Schmid_2014}.
  
  Hurdle models for zero-inflated count data can be fitted by using a combination
  of the \code{Binomial()} and \code{Hurdle()} families. While the \code{Binomial()}
  family allows for fitting the zero-generating process of the Hurdle model,
  \code{Hurdle()} fits a negative binomial regression model to the non-zero
  counts. Note that the specification of the Hurdle model allows for using
  \code{Binomial()} and \code{Hurdle()} independently of each other.

  Linear or additive multinomial logit models can be fitted using
  \code{Multinomial()}; although is family requires some extra effort for
  model specification (see example).  More specifically, the predictor must
  be in the form of a linear array model (see \code{\link{\%O\%}}).  Note
  that this family does not work with tree-based base-learners at the
  moment. The class corresponding to the last level of the factor coding
  of the response is used as reference class.
  
  \code{RCG()} implements the ratio of correlated gammas (RCG) model proposed
  by \bibcitet{mboost::Weinhold_2016}.

}
\section{Warning}{
  The coefficients resulting from boosting with family
  \code{Binomial(link = "logit")} are \eqn{1/2} of the coefficients of a logit model
  obtained via \code{\link{glm}} (see above). 

  For \code{AUC()}, variables should be centered and scaled and observations with weight > 0 must not contain missing values.
  The estimated coefficients for \code{AUC()} have no probabilistic interpretation.
}
\value{
  An object of class \code{boost_family}.
}
\references{\bibshow{*}}
\seealso{\code{\link{mboost}} for the usage of \code{Family}s. See
    \code{\link{boost_family-class}} for objects resulting from a call to \code{Family}. }
\author{
    \code{ExpectReg()} was donated by Fabian Sobotka.
    \code{AUC()} was donated by Fabian Scheipl.
}
\examples{
### Define a new family
MyGaussian <- function(){
       Family(ngradient = function(y, f, w = 1) y - f,
       loss = function(y, f) (y - f)^2,
       name = "My Gauss Variant")
}
# Now use the new family
data(bodyfat, package = "TH.data")
mod <- mboost(DEXfat ~ ., data = bodyfat, family = MyGaussian())
# N.B. that the family needs to be called with empty brackets


### Proportional odds model
data(iris)
iris$Species <- factor(iris$Species, ordered = TRUE)
if (require("MASS")) {
    (mod.polr <- polr(Species  ~ Sepal.Length, data = iris))
}
mod.PropOdds <- glmboost(Species  ~ Sepal.Length, data = iris,
                         family = PropOdds(nuirange = c(-0.5, 3)))
mstop(mod.PropOdds) <- 250
## thresholds are treated as nuisance parameters, to extract these use
nuisance(mod.PropOdds)
## effect estimate
coef(mod.PropOdds)["Sepal.Length"]
## make thresholds comparable to a model without intercept
nuisance(mod.PropOdds) - coef(mod.PropOdds)["(Intercept)"] -
    attr(coef(mod.PropOdds), "offset")

### Multinomial logit model via a linear array model
## One needs to convert the data to a list
myiris <- as.list(iris)
## ... and define a dummy vector with one factor level less
## than the outcome, which is used as reference category.
myiris$class <- factor(levels(iris$Species)[-nlevels(iris$Species)])
## Now fit the linear array model
mlm <- mboost(Species ~ bols(Sepal.Length, df = 2) \%O\%
                        bols(class, df = 2, contrasts.arg = "contr.dummy"),
              data = myiris,
              family = Multinomial())
coef(mlm) ## one should use more boosting iterations.
head(round(pred <- predict(mlm, type = "response"), 2))

## Prediction with new data:
newdata <- as.list(iris[1,])
## One always needs to keep the dummy vector class as above!
newdata$class <- factor(levels(iris$Species)[-nlevels(iris$Species)])
pred2 <- predict(mlm, type = "response", newdata = newdata)
## check results
pred[1, ]
pred2

\dontrun{############################################################
## Do not run and check these examples automatically as
## they take some time

## Compare results with nnet::multinom
if (require("nnet")) {
    mlmn <- multinom(Species ~ Sepal.Length, data = iris)
    max(abs(fitted(mlm[1000], type = "response") -
            fitted(mlmn, type = "prob")))
}

## End(Not run and test)
}


### Example for RCG model
## generate covariate values
set.seed(12345)
x1 <- rnorm(500)
x2 <- rnorm(500)
## generate linear predictors
zetaM <- 0.1 + 0.3 * x1 - 0.5 * x2 
zetaU <- 0.1 - 0.1 * x1 + 0.2 * x2
## generate beta values
M <- rgamma(500, shape = 2, rate = exp(zetaM))
U <- rgamma(500, shape = 2, rate = exp(zetaU))
y <- M / (M + U)

## fit RCG model
data <- data.frame(y, x1, x2)
RCGmodel <- glmboost(y ~ x1 + x2, data = data, family = RCG(),
                     control = boost_control(mstop = 250,
                     trace = TRUE, nu = 0.01))
## true coefficients: gamma = (0.0, 0.4, -0.7),
##                    alpha (= shape) = 2,
##                    rho = 0
## compare to coefficient estimates
coef(RCGmodel)
nuisance(RCGmodel)

## compute downstream tests 
## (only suitable without early stopping, i.e., if likelihood based model converged)
downstream.test(RCGmodel)

## compute conditional expectations
predictions <- predict(RCGmodel, type = "response")
plot(predictions, y)
abline(0,1)

}
\keyword{models}

\name{mboost}
\alias{mboost}
\alias{gamboost}
\title{ Gradient Boosting for Additive Models }
\description{
  Gradient boosting for optimizing arbitrary loss functions, where component-wise
  arbitrary base-learners, e.g., smoothing procedures,  are utilized as additive
  base-learners.
}
\usage{
mboost(formula, data = list(), na.action = na.omit, weights = NULL, 
       offset = NULL, family = Gaussian(), control = boost_control(),
       oobweights = NULL, baselearner = c("bbs", "bols", "btree", "bss", "bns"), 
       ...)

gamboost(formula, data = list(), na.action = na.omit, weights = NULL, 
         offset = NULL, family = Gaussian(), control = boost_control(),
         oobweights = NULL, baselearner = c("bbs", "bols", "btree", "bss", "bns"),
         dfbase = 4, ...)
}
\arguments{
  \item{formula}{ a symbolic description of the model to be fit. }
  
  \item{data}{ a data frame containing the variables in the model. }
  \item{na.action}{a function which indicates what should happen when
    the data contain \code{NA}s.}
    \item{weights}{ (optional) a numeric vector of weights to be used in 
    the fitting process.}
  \item{offset}{ a numeric vector to be used as offset (optional).}
  \item{family}{a \code{\link{Family}} object.}
  \item{control}{ a list of parameters controlling the algorithm. For
    more details see \code{\link{boost_control}}. }
  \item{oobweights}{ an additional vector of out-of-bag weights, which is
    used for the out-of-bag risk (i.e., if \code{boost_control(risk =
      "oobag")}). This argument is also used internally by
    \code{cvrisk}. }
  \item{baselearner}{ a character specifying the component-wise base
    learner to be used: \code{\link{bbs}} means P-splines with a
    B-spline basis (see Schmid and Hothorn 2008), \code{\link{bols}}
    linear models and \code{\link{btree}} boosts stumps. 
    \code{bss} and \code{bns} are deprecated.
    Component-wise smoothing splines have been considered in
    \bibcitet{mboost::buehlmannyu2003} and
    \bibcitet{mboost::schmid+hothorn:2008b} investigate P-splines
    with a B-spline basis. \bibcitet{mboost::kneib+hothorn+tutz:2009} also utilize
    P-splines with a B-spline basis, supplement them with their
    bivariate tensor product version to estimate interaction surfaces
    and spatial effects and also consider random effects base
    learners.}
  \item{dfbase}{ a single integer giving the degrees of freedom for P-spline 
    base-learners (\code{\link{bbs}}) globally. }
  \item{\dots}{ additional arguments passed to \code{\link{mboost_fit}}; currently none.}
}
\details{

  A (generalized) additive model is fitted using a boosting algorithm based on
  component-wise base-learners. 
  
  The base-learners can either be specified via the \code{formula} object or via 
  the \code{baselearner} argument. The latter argument is the default base-learner 
  which is used for all variables in the formula, whithout explicit base-learner 
  specification (i.e., if the base-learners are explicitly specified in \code{formula}, 
  the \code{baselearner} argument will be ignored for this variable). 
  
  Of note, \code{"bss"} and \code{"bns"} are deprecated and only in the list for 
  backward compatibility.
  
  Note that more base-learners (i.e., in addition to the ones provided
  via \code{baselearner}) can be specified in \code{formula}. See 
  \code{\link{baselearners}} for details.
  
  The only difference when calling \code{mboost} and \code{gamboost} is that the
  latter function allows one to specify default degrees of freedom for smooth 
  effects specified via \code{baselearner = "bbs"}. In all other cases, 
  degrees of freedom need to be set manually via a specific definition of the 
  corresponding base-learner.
}
\value{
  An object of class \code{mboost} with \code{\link{print}},
  \code{\link{AIC}}, \code{\link{plot}} and \code{\link{predict}}
  methods being available.
}
\references{\bibshow{*}}
\seealso{
  See \code{\link{mboost_fit}} for the generic boosting function, 
  \code{\link{glmboost}} for boosted linear models, and
  \code{\link{blackboost}} for boosted trees. 
  
  See \code{\link{baselearners}} for possible base-learners. 
  
  See \code{\link{cvrisk}} for cross-validated stopping iteration. 
  
  Furthermore see \code{\link{boost_control}}, \code{\link{Family}} and
  \code{\link[mboost]{methods}}.
}
\examples{

    ### a simple two-dimensional example: cars data
    cars.gb <- gamboost(dist ~ speed, data = cars, dfbase = 4,
                        control = boost_control(mstop = 50))
    cars.gb
    AIC(cars.gb, method = "corrected")

    ### plot fit for mstop = 1, ..., 50
    plot(dist ~ speed, data = cars)
    tmp <- sapply(1:mstop(AIC(cars.gb)), function(i)
        lines(cars$speed, predict(cars.gb[i]), col = "red"))
    lines(cars$speed, predict(smooth.spline(cars$speed, cars$dist),
                              cars$speed)$y, col = "green")

    ### artificial example: sinus transformation
    x <- sort(runif(100)) * 10
    y <- sin(x) + rnorm(length(x), sd = 0.25)
    plot(x, y)
    ### linear model
    lines(x, fitted(lm(y ~ sin(x) - 1)), col = "red")
    ### GAM
    lines(x, fitted(gamboost(y ~ x,
                    control = boost_control(mstop = 500))),
          col = "green")

}
\keyword{models}
\keyword{nonlinear}

\name{boost_family-class}
\docType{class}
\alias{boost_family-class}
\alias{show,boost_family-method}
\title{Class "boost_family": Gradient Boosting Family}
\description{Objects of class \code{boost_family} define negative gradients of
             loss functions to be optimized.}
\section{Objects from the Class}{
Objects can be created by calls of the form \code{\link{Family}(...)}
}
\section{Slots}{
	 \describe{
    \item{\code{ngradient}:}{ a function with arguments \code{y} and \code{f}
                              implementing the \emph{negative} gradient of
                              the \code{loss} function. }
    \item{\code{risk}:}{ a risk function with arguments \code{y}, \code{f} and \code{w},
               the weighted mean of the loss function by default. }
    \item{\code{offset}:}{ a function with argument \code{y} and \code{w} (weights)
                 for computing a \emph{scalar} offset. }
    \item{\code{weights}:}{ a logical indicating if weights are allowed. }
    \item{\code{check_y}:}{ a function for checking the class / mode of a response variable.}
    \item{\code{nuisance}:}{ a function for extracting nuisance parameters.}
    \item{\code{response}:}{inverse link function of a GLM or any other transformation
                   on the scale of the response.}
    \item{\code{rclass}:}{function to derive class predictions from conditional class
                   probabilities (for models with factor response variable).}
    \item{\code{name}:}{ a character giving the name of the loss function
                         for pretty printing. }
    \item{\code{charloss}:}{ a character, the deparsed loss function.}
  }
}
\seealso{\code{\link{Family}}}
\examples{

    Laplace()

}
\keyword{classes}

\name{IPCweights}
\alias{IPCweights}
\title{ Inverse Probability of Censoring Weights }
\description{
  Compute weights for censored regression via the inverted 
  probability of censoring principle.
}
\usage{
IPCweights(x, maxweight = 5)
}
\arguments{
  \item{x}{ an object of class \code{Surv}.}
  \item{maxweight}{ the maximal value of the returned weights. }
}
\details{

  Inverse probability of censoring weights are one possibility to fit
  models formulated in the \emph{full data world} in the presence of censoring,
  i.e., the \emph{observed data world} \bibcitep{mboost::Laan_Robins_2003}, see 
  \bibcitet{mboost::Hothorn:2006:Biostatistics:16344280} for an application to
  survival analysis.

}
\value{
  A vector of numeric weights. 
}
\references{\bibshow{*}}
\keyword{survival}

\name{mboost_intern}
\alias{mboost_intern}
\title{
  Call internal functions.
}
\description{
  Call one of the internal mboost functions.
}
\usage{
mboost_intern(..., fun = c("df2lambda", "hyper_bbs", "hyper_ols",
                  "bl_lin", "bl_lin_matrix",
                  "Complete.cases", "get_index", "isMATRIX",
                  "cbs", "bsplines", "model.frame.blg", 
                  "check_newdata", "do_trace", "rescale_weights", "nnls2D"))
}
\arguments{
  \item{\dots}{ Arguments to \code{fun}. }
  \item{fun}{ The name on an internal mboost function. }
}
\details{
  This function must not be called under any circumstances.
}
\keyword{internal}

\name{blackboost}
\alias{blackboost}
\title{ Gradient Boosting with Regression Trees }
\description{
  Gradient boosting for optimizing arbitrary loss functions where regression
  trees are utilized as base-learners.
}
\usage{
blackboost(formula, data = list(),
           weights = NULL, na.action = na.pass,
           offset = NULL, family = Gaussian(), 
           control = boost_control(),
           oobweights = NULL,
           tree_controls = partykit::ctree_control(
               teststat = "quad",
               testtype = "Teststatistic",
               mincriterion = 0,
               minsplit = 10, 
               minbucket = 4,
               maxdepth = 2, 
               saveinfo = FALSE),
           ...)
}
\arguments{
  \item{formula}{ a symbolic description of the model to be fit. }
  \item{data}{ a data frame containing the variables in the model. }
  \item{weights}{ an optional vector of weights to be used in the fitting
    process. }
  \item{na.action}{a function which indicates what should happen when the data
    contain \code{NA}s. }
  \item{offset}{ a numeric vector to be used as offset (optional).}
  \item{family}{a \code{\link{Family}} object.}
  \item{control}{ a list of parameters controlling the algorithm. For
    more details see \code{\link{boost_control}}. }
  \item{oobweights}{ an additional vector of out-of-bag weights, which is
    used for the out-of-bag risk (i.e., if \code{boost_control(risk =
      "oobag")}). This argument is also used internally by
    \code{cvrisk}. }    
  \item{tree_controls}{ an object of class \code{"TreeControl"}, which
    can be obtained using \code{\link[partykit]{ctree_control}}. Defines
    hyper-parameters for the trees which are used as base-learners. It
    is wise to make sure to understand the consequences of altering any
    of its arguments. By default, two-way interactions (but not deeper
    trees) are fitted.}
  \item{\dots}{  additional arguments passed to \code{\link{mboost_fit}},
    including \code{weights}, \code{offset}, \code{family} and
    \code{control}. For default values see \code{\link{mboost_fit}}. }
}
\details{

  This function implements the `classical'
  gradient boosting \bibcitep{mboost::nr:freund.schapire:1996,mboost::friedman2001} 
  as reviewed by \bibcitet{mboost::Buehlmann:2008:StatSci} 
  utilizing unbiased regression trees \bibcitep{mboost::Hothorn:2006:JCGS} as base-learners.
  Essentially, the same algorithm is implemented in package
  \code{\link[gbm]{gbm}} \bibcitet{mboost::ridgew99}. The
  main difference is that arbitrary loss functions to be optimized
  can be specified via the \code{family} argument to \code{blackboost} whereas
  \code{\link[gbm]{gbm}} uses hard-coded loss functions.
  Moreover, the base-learners (conditional
  inference trees, see \code{\link[partykit]{ctree}}) are a little bit more flexible.

  The regression fit is a black box prediction machine and thus
  hardly interpretable.

  Partial dependency plots are not yet available; see example section for
  plotting of additive tree models.

}
\value{
  An object of class \code{mboost} with \code{\link{print}}
  and \code{\link{predict}} methods being available.
}
\references{\bibshow{*}}

\seealso{
  See \code{\link{mboost_fit}} for the generic boosting function, 
  \code{\link{glmboost}} for boosted linear models, and
  \code{\link{gamboost}} for boosted additive models. 
  
  See \code{\link{baselearners}} for possible base-learners. 
  
  See \code{\link{cvrisk}} for cross-validated stopping iteration. 
  
  Furthermore see \code{\link{boost_control}}, \code{\link{Family}} and
  \code{\link[mboost]{methods}}.
}
\examples{

### a simple two-dimensional example: cars data
cars.gb <- blackboost(dist ~ speed, data = cars,
                      control = boost_control(mstop = 50))
cars.gb

### plot fit
plot(dist ~ speed, data = cars)
lines(cars$speed, predict(cars.gb), col = "red")

### set up and plot additive tree model
if (require("partykit")) {
    ctrl <- ctree_control(maxdepth = 3)
    viris <- subset(iris, Species != "setosa")
    viris$Species <- viris$Species[, drop = TRUE]
    imod <- mboost(Species ~ btree(Sepal.Length, tree_controls = ctrl) +
                             btree(Sepal.Width, tree_controls = ctrl) +
                             btree(Petal.Length, tree_controls = ctrl) +
                             btree(Petal.Width, tree_controls = ctrl),
                   data = viris, family = Binomial())[500]
    layout(matrix(1:4, ncol = 2))
    plot(imod)
}
}
\keyword{models}
\keyword{regression}
